@@ -1,7 +1,7 @@
 /*
  * declarations relevant to encapsulation-like operations
  * Copyright (C) 1996, 1997  John Ioannidis.
- * Copyright (C) 1998, 1999  Richard Guy Briggs.
+ * Copyright (C) 1998, 1999, 2000, 2001  Richard Guy Briggs.
  * 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -13,23 +13,34 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: ipsec_encap.h,v 1.10 2000/03/22 16:15:36 rgb Exp $
+ * RCSID $Id: ipsec_encap.h,v 1.16 2001/11/26 09:23:47 rgb Exp $
  */
 
-#define SENT_IP4	0x0008		/* data is two struct in_addr */
+#ifndef _IPSEC_ENCAP_H_
+
+#define SENT_IP4	0x0008	/* data is two struct in_addr */
+			/* (2 * sizeof(struct in_addr)) */
+			/* sizeof(struct sockaddr_encap)
+			   - offsetof(struct sockaddr_encap, Sen.Sip4.Src) */
 
 #define SEN_HDRLEN	(2*sizeof(__u8)+sizeof(__u16))
+			/* offsetof(struct sockaddr_encap, Sen.Sip4.Src) */
 
 #define SEN_IP4_SRCOFF	(0)
 #define SEN_IP4_DSTOFF (sizeof (struct in_addr))
+			/* offsetof(struct sockaddr_encap, Sen.Sip4.Dst)
+			   - offsetof(struct sockaddr_encap, Sen.Sip4.Src) */
 #define SEN_IP4_OPTOFF	(2 * sizeof (struct in_addr))
+			/* sizeof(struct sockaddr_encap)
+			   - offsetof(struct sockaddr_encap, Sen.Sip4.Src) */
 
 #define SEN_IP4_LEN	(SENT_HDRLEN + SENT_IP4_OPTOFF)
+			/* sizeof(struct sockaddr_encap) */
 
-#ifdef DEBUG_IPSEC
+#ifdef CONFIG_IPSEC_DEBUG
 #define DB_ER_PROCFS	0x0001
 #define DB_SP_PROCFS	0x0001
-#endif
+#endif /* CONFIG_IPSEC_DEBUG */
 
 struct sockaddr_encap
 {
@@ -53,59 +64,33 @@ struct sockaddr_encap
 #define AF_ENCAP 26
 #endif /* AF_ENCAP */
 
-/*
- * The "type" is really part of the address as far as the routing
- * system is concerned. By using only one bit in the type field
- * for each type, we sort-of make sure that different types of
- * encapsulation addresses won't be matched against the wrong type.
- */
-
-#ifdef __KERNEL__
-/*
- * An entry in the radix tree 
- */
-
-struct rjtentry
-{
-	struct	radij_node rd_nodes[2];	/* tree glue, and other values */
-#define	rd_key(r)	((struct sockaddr_encap *)((r)->rd_nodes->rj_key))
-#define	rd_mask(r)	((struct sockaddr_encap *)((r)->rd_nodes->rj_mask))
-	short	rd_flags;
-	short	rd_count;
-};
-
-/*
- * An encapsulation route consists of a pointer to a 
- * radix tree entry and a SAID (a destination_address/SPI/protocol triple).
- */
-
-struct
-eroute
-{
-	struct rjtentry er_rjt;
-	struct sa_id er_said;
-	struct sockaddr_encap er_eaddr;
-	struct sockaddr_encap er_emask;
-};
-
-#define er_dst er_said.dst
-#define er_spi er_said.spi
-#define er_proto er_said.proto
-#ifdef DEBUG_IPSEC
-extern int debug_eroute;
-extern int debug_spi;
-#endif
-
-#ifdef NETDEV_23
-#define device net_device
-#define ipsec_dev_get __dev_get_by_name
-#else
-#define ipsec_dev_get dev_get
-#endif /* NETDEV_23 */
-#endif /* __KERNEL__ */
+#define _IPSEC_ENCAP_H_
+#endif /* _IPSEC_ENCAP_H_ */
 
 /*
  * $Log: ipsec_encap.h,v $
+ * Revision 1.16  2001/11/26 09:23:47  rgb
+ * Merge MCR's ipsec_sa, eroute, proc and struct lifetime changes.
+ *
+ * Revision 1.15.2.1  2001/09/25 02:18:54  mcr
+ * 	struct eroute moved to ipsec_eroute.h
+ *
+ * Revision 1.15  2001/09/14 16:58:36  rgb
+ * Added support for storing the first and last packets through a HOLD.
+ *
+ * Revision 1.14  2001/09/08 21:13:31  rgb
+ * Added pfkey ident extension support for ISAKMPd. (NetCelo)
+ *
+ * Revision 1.13  2001/06/14 19:35:08  rgb
+ * Update copyright date.
+ *
+ * Revision 1.12  2001/05/27 06:12:10  rgb
+ * Added structures for pid, packet count and last access time to eroute.
+ * Added packet count to beginning of /proc/net/ipsec_eroute.
+ *
+ * Revision 1.11  2000/09/08 19:12:56  rgb
+ * Change references from DEBUG_IPSEC to CONFIG_IPSEC_DEBUG.
+ *
  * Revision 1.10  2000/03/22 16:15:36  rgb
  * Fixed renaming of dev_get (MB).
  *
