@@ -1,7 +1,7 @@
 /*
  * IPSEC <> netlink interface
  * Copyright (C) 1996, 1997  John Ioannidis.
- * Copyright (C) 1998, 1999  Richard Guy Briggs.
+ * Copyright (C) 1998, 1999, 2000, 2001  Richard Guy Briggs.
  * 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -14,13 +14,19 @@
  * for more details.
  */
 
-char ipsec_netlink_c_version[] = "RCSID $Id: ipsec_netlink.c,v 1.44 2000/03/16 14:01:26 rgb Exp $";
+char ipsec_netlink_c_version[] = "RCSID $Id: ipsec_netlink.c,v 1.56 2002/01/29 17:17:55 mcr Exp $";
 
 #include <linux/config.h>
 #include <linux/version.h>
-
 #include <linux/kernel.h> /* printk() */
-#include <linux/malloc.h> /* kmalloc() */
+
+#include "ipsec_param.h"
+
+#ifdef MALLOC_SLAB
+# include <linux/slab.h> /* kmalloc() */
+#else /* MALLOC_SLAB */
+# include <linux/malloc.h> /* kmalloc() */
+#endif /* MALLOC_SLAB */
 #include <linux/errno.h>  /* error codes */
 #include <linux/types.h>  /* size_t */
 #include <linux/interrupt.h> /* mark_bh */
@@ -31,24 +37,24 @@ char ipsec_netlink_c_version[] = "RCSID $Id: ipsec_netlink.c,v 1.44 2000/03/16 1
 #include <linux/skbuff.h>
 #include <freeswan.h>
 #ifdef SPINLOCK
- #ifdef SPINLOCK_23
-  #include <linux/spinlock.h> /* *lock* */
- #else /* 23_SPINLOCK */
-  #include <asm/spinlock.h> /* *lock* */
- #endif /* 23_SPINLOCK */
+# ifdef SPINLOCK_23
+#  include <linux/spinlock.h> /* *lock* */
+# else /* 23_SPINLOCK */
+#  include <asm/spinlock.h> /* *lock* */
+# endif /* 23_SPINLOCK */
 #endif /* SPINLOCK */
 #ifdef NET_21
- #include <asm/uaccess.h>
- #include <linux/in6.h>
- #define ip_chk_addr inet_addr_type
- #define IS_MYADDR RTN_LOCAL
+# include <asm/uaccess.h>
+# include <linux/in6.h>
+# define ip_chk_addr inet_addr_type
+# define IS_MYADDR RTN_LOCAL
 #endif
 #include <asm/checksum.h>
 #include <net/ip.h>
 #ifdef NETLINK_SOCK
- #include <linux/netlink.h>
+# include <linux/netlink.h>
 #else
- #include <net/netlink.h>
+# include <net/netlink.h>
 #endif
 
 #include "radij.h"
@@ -61,21 +67,20 @@ char ipsec_netlink_c_version[] = "RCSID $Id: ipsec_netlink.c,v 1.44 2000/03/16 1
 #include "ipsec_ah.h"
 #include "ipsec_esp.h"
 
-#ifdef DEBUG_IPSEC
- #include "ipsec_tunnel.h"
-#endif
+#ifdef CONFIG_IPSEC_DEBUG
+# include "ipsec_tunnel.h"
+#endif /* CONFIG_IPSEC_DEBUG */
 
-/* #ifdef CONFIG_IPSEC_PFKEYv2 */
 #include <pfkeyv2.h>
 #include <pfkey.h>
-/* #endif */
 
-#ifdef DEBUG_IPSEC
+#ifdef CONFIG_IPSEC_DEBUG
  int debug_netlink = 0;
-#endif
+#endif /* CONFIG_IPSEC_DEBUG */
 
 #define SENDERR(_x) do { len = -(_x); goto errlab; } while (0)
 
+#if 0
 int 
 #ifdef NETLINK_SOCK
 ipsec_callback(int proto, struct sk_buff *skb)
@@ -91,30 +96,38 @@ ipsec_callback(struct sk_buff *skb)
 	struct encap_msghdr *em	= (struct encap_msghdr *)dat;
 	struct tdb *tdbp, *tprev;
 	int i, nspis, error = 0;
-	unsigned long flags; /* save irq state for spinlock */
-#ifdef DEBUG_IPSEC
+#ifdef CONFIG_IPSEC_DEBUG
 	struct eroute *eret;
 	char sa[SATOA_BUF];
+	size_t sa_len;
+	struct sk_buff *first, *last;
 
 
-	satoa(em->em_said, 0, sa, SATOA_BUF);
+	sa_len = satoa(em->em_said, 0, sa, SATOA_BUF);
 
 	if(debug_netlink) {
-		printk("klips_debug:ipsec_callback: skb=0x%p skblen=%ld em_magic=%d em_type=%d\n",
-		       skb, (unsigned long int)skb->len, em->em_magic, em->em_type);
+		printk("klips_debug:ipsec_callback: "
+		       "skb=0x%p skblen=%ld em_magic=%d em_type=%d\n",
+		       skb,
+		       (unsigned long int)skb->len,
+		       em->em_magic,
+		       em->em_type);
 		switch(em->em_type) {
 		case EMT_SETDEBUG:
-			printk("klips_debug:ipsec_callback: set ipsec_debug level\n");
+			printk("klips_debug:ipsec_callback: "
+			       "set ipsec_debug level\n");
 			break;
 		case EMT_DELEROUTE:
 		case EMT_CLREROUTE:
 		case EMT_CLRSPIS:
 			break;
 		default:
-			printk("klips_debug:ipsec_callback: called for SA:%s\n", sa);
+			printk("klips_debug:ipsec_callback: "
+			       "called for SA:%s\n",
+			       sa_len ? sa : " (error)");
 		}
 	}
-#endif /* DEBUG_IPSEC */
+#endif /* CONFIG_IPSEC_DEBUG */
 
 	/* XXXX Temporarily disable netlink I/F code until it gets permanantly
 	   ripped out in favour of PF_KEYv2 I/F. */
@@ -122,14 +135,15 @@ ipsec_callback(struct sk_buff *skb)
 
 	/*	em = (struct encap_msghdr *)dat; */
 	if (em->em_magic != EM_MAGIC) {
-		printk("klips_debug:ipsec_callback: bad magic=%d failed, should be %d\n",
+		printk("klips_debug:ipsec_callback: "
+		       "bad magic=%d failed, should be %d\n",
 		       em->em_magic,
 		       EM_MAGIC);
 		SENDERR(EINVAL);
 	}
 	switch (em->em_type) {
 	case EMT_SETDEBUG:
-#ifdef DEBUG_IPSEC
+#ifdef CONFIG_IPSEC_DEBUG
 		if(em->em_db_nl >> (sizeof(em->em_db_nl) * 8 - 1)) {
 			em->em_db_nl &= ~(1 << (sizeof(em->em_db_nl) * 8 -1));
 			debug_tunnel  |= em->em_db_tn;
@@ -141,9 +155,7 @@ ipsec_callback(struct sk_buff *skb)
 			debug_esp     |= em->em_db_es;
 			debug_ah      |= em->em_db_ah;
 			debug_rcv     |= em->em_db_rx;
-			/* #ifdef CONFIG_IPSEC_PFKEYv2 */
 			debug_pfkey   |= em->em_db_ky;
-			/* #endif */
 			if(debug_netlink)
 				printk("klips_debug:ipsec_callback: set\n");
 		} else {
@@ -158,52 +170,34 @@ ipsec_callback(struct sk_buff *skb)
 			debug_esp     &= em->em_db_es;
 			debug_ah      &= em->em_db_ah;
 			debug_rcv     &= em->em_db_rx;
-			/* #ifdef CONFIG_IPSEC_PFKEYv2 */
 			debug_pfkey   &= em->em_db_ky;
-			/* #endif */
 		}
-#else /* DEBUG_IPSEC */
-		printk("klips_debug:ipsec_callback: debugging not enabled\n");
+#else /* CONFIG_IPSEC_DEBUG */
+		printk("klips_debug:ipsec_callback: "
+		       "debugging not enabled\n");
 		SENDERR(EINVAL);
-#endif /* DEBUG_IPSEC */
+#endif /* CONFIG_IPSEC_DEBUG */
 		break;
 
-#if 0
-	case EMT_GETDEBUG:
-#ifdef DEBUG_IPSEC
-		if(debug_netlink)
-			printk("klips_debug:ipsec_callback: get\n");
-		em->em_db_tn = debug_tunnel;
-		em->em_db_nl = debug_netlink;
-		em->em_db_xf = debug_xform;
-		em->em_db_er = debug_eroute;
-		em->em_db_sp = debug_spi;
-		em->em_db_rj = debug_radij;
-		em->em_db_es = debug_esp;
-		em->em_db_ah = debug_ah;
-		em->em_db_rx = debug_rcv;
-#else /* DEBUG_IPSEC */
-		printk("klips_debug:ipsec_callback: debugging not enabled\n");
-		SENDERR(EINVAL);
-#endif /* DEBUG_IPSEC */
-		break;
-
-#endif
 	case EMT_SETEROUTE:
-		if ((error = ipsec_makeroute(&(em->em_eaddr), &(em->em_emask), em->em_ersaid)))
+		if ((error = ipsec_makeroute(&(em->em_eaddr), &(em->em_emask), em->em_ersaid, 0, NULL, NULL, NULL)))
 			SENDERR(-error);
 		break;
 
-	case EMT_RPLACEROUTE:
-		if ((error = ipsec_breakroute(&(em->em_eaddr), &(em->em_emask))) == EINVAL) {
-				SENDERR(-error);
+	case EMT_REPLACEROUTE:
+		if ((error = ipsec_breakroute(&(em->em_eaddr), &(em->em_emask), &first, &last)) == EINVAL) {
+			kfree_skb(first);
+			kfree_skb(last);
+			SENDERR(-error);
 		}
-		if ((error = ipsec_makeroute(&(em->em_eaddr), &(em->em_emask), em->em_ersaid)))
+		if ((error = ipsec_makeroute(&(em->em_eaddr), &(em->em_emask), em->em_ersaid, NULL, NULL)))
 			SENDERR(-error);
 		break;
 
 	case EMT_DELEROUTE:
-		if ((error = ipsec_breakroute(&(em->em_eaddr), &(em->em_emask))))
+		if ((error = ipsec_breakroute(&(em->em_eaddr), &(em->em_emask), &first, &last)))
+			kfree_skb(first);
+			kfree_skb(last);
 			SENDERR(-error);
 		break;
 
@@ -212,20 +206,6 @@ ipsec_callback(struct sk_buff *skb)
 			SENDERR(-error);
 		break;
 
-#ifdef DEBUG_IPSEC
-	case EMT_TESTROUTE:
-		      
-		printk("klips_debug:ipsec_callback: testroute: trying to locate 0x%08x->0x%08x", (u_int)em->em_eaddr.sen_ip_src.s_addr, (u_int)em->em_eaddr.sen_ip_dst.s_addr);
-
-		eret = ipsec_findroute(&(em->em_eaddr));
-		
-		if (eret == NULL) {
-			printk(" not found\n");
-		} else {
-			printk(" via SA: %s\n", sa);
-		}
-		break;
-#endif /* DEBUG_IPSEC */
 	case EMT_SETSPI:
 		if (em->em_if >= 5)	/* XXX -- why 5? */
 			SENDERR(ENODEV);
@@ -246,22 +226,26 @@ ipsec_callback(struct sk_buff *skb)
 				tdbp->tdb_flags |= EMT_INBOUND;
 			}
 			KLIPS_PRINT(debug_netlink & DB_NL_TDBCB,
-				    "klips_debug:ipsec_callback: existing Tunnel Descriptor Block not found (this\n"
-				    "klips_debug:                is good) for SA: %s, %s-bound, allocating.\n",
-				    sa, (tdbp->tdb_flags & EMT_INBOUND) ? "in" : "out");
+				    "klips_debug:ipsec_callback: "
+				    "existing Tunnel Descriptor Block not found (this is good) for SA: %s, %s-bound, allocating.\n",
+				    sa_len ? sa : " (error)",
+				    (tdbp->tdb_flags & EMT_INBOUND) ? "in" : "out");
 
 /* XXX  		tdbp->tdb_rcvif = &(enc_softc[em->em_if].enc_if);*/
 			tdbp->tdb_rcvif = NULL;
 		} else {
 			KLIPS_PRINT(debug_netlink & DB_NL_TDBCB,
-				    "klips_debug:ipsec_callback: EMT_SETSPI found an old Tunnel Descriptor Block\n"
-				    "klips_debug:                for SA: %s, delete it first.\n", sa);
+				    "klips_debug:ipsec_callback: "
+				    "EMT_SETSPI found an old Tunnel Descriptor Block for SA: %s, delete it first.\n",
+				    sa_len ? sa : " (error)");
 			SENDERR(EEXIST);
 		}
 		
 		if ((error = tdb_init(tdbp, em))) {
 			KLIPS_PRINT(debug_netlink & DB_NL_TDBCB,
-				    "klips_debug:ipsec_callback: EMT_SETSPI not successful for SA: %s, deleting.\n", sa);
+				    "klips_debug:ipsec_callback: "
+				    "EMT_SETSPI not successful for SA: %s, deleting.\n",
+				    sa_len ? sa : " (error)");
 			ipsec_tdbwipe(tdbp);
 			
 			SENDERR(-error);
@@ -275,37 +259,40 @@ ipsec_callback(struct sk_buff *skb)
 
 		puttdb(tdbp);
 		KLIPS_PRINT(debug_netlink & DB_NL_TDBCB,
-			    "klips_debug:ipsec_callback: EMT_SETSPI successful for SA: %s\n", sa);
+			    "klips_debug:ipsec_callback: "
+			    "EMT_SETSPI successful for SA: %s\n",
+			    sa_len ? sa : " (error)");
 		break;
 		
 	case EMT_DELSPI:
 		if (em->em_if >= 5)	/* XXX -- why 5? */
 			SENDERR(ENODEV);
 		
-		spin_lock_irqsave(&tdb_lock, flags);
+		spin_lock_bh(&tdb_lock);
 		
 		tdbp = gettdb(&(em->em_said));
 		if (tdbp == NULL) {
 			KLIPS_PRINT(debug_netlink & DB_NL_TDBCB,
 				    "klips_debug:ipsec_callback: "
-				    "EMT_DELSPI Tunnel Descriptor Block not found for SA:\n"
-				    "klips_debug:                %s, could not delete.\n", sa);
-			spin_unlock_irqrestore(&tdb_lock, flags);
+				    "EMT_DELSPI Tunnel Descriptor Block not found for SA%s, could not delete.\n",
+				    sa_len ? sa : " (error)");
+			spin_unlock_bh(&tdb_lock);
 			SENDERR(ENXIO);  /* XXX -- wrong error message... */
 		} else {
 			if((error = deltdbchain(tdbp))) {
-				spin_unlock_irqrestore(&tdb_lock, flags);
+				spin_unlock_bh(&tdb_lock);
 				SENDERR(-error);
 			}
 		}
-		spin_unlock_irqrestore(&tdb_lock, flags);
+		spin_unlock_bh(&tdb_lock);
 
 		break;
 		
 	case EMT_GRPSPIS:
 		nspis = (len - EMT_GRPSPIS_FLEN) / sizeof(em->em_rel[0]);
 		if ((nspis * (sizeof(em->em_rel[0]))) != (len - EMT_GRPSPIS_FLEN)) {
-			printk("klips_debug:ipsec_callback: EMT_GRPSPI message size incorrect, expected nspis(%d)*%d, got %d.\n",
+			printk("klips_debug:ipsec_callback: "
+			       "EMT_GRPSPI message size incorrect, expected nspis(%d)*%d, got %d.\n",
 			       nspis,
 			       sizeof(em->em_rel[0]),
 			       (len - EMT_GRPSPIS_FLEN));
@@ -313,26 +300,28 @@ ipsec_callback(struct sk_buff *skb)
 			break;
 		}
 		
-		spin_lock_irqsave(&tdb_lock, flags);
+		spin_lock_bh(&tdb_lock);
 
 		for (i = 0; i < nspis; i++) {
 			KLIPS_PRINT(debug_netlink,
-				    "klips_debug:ipsec_callback: EMT_GRPSPI for SA(%d)\n"
-				    "klips_debug:                %s,\n", i, sa);
+				    "klips_debug:ipsec_callback: "
+				    "EMT_GRPSPI for SA(%d) %s,\n",
+				    i,
+				    sa_len ? sa : " (error)");
 			if ((tdbp = gettdb(&(em->em_rel[i].emr_said))) == NULL) {
 				KLIPS_PRINT(debug_netlink,
 					    "klips_debug:ipsec_callback: "
-					    "EMT_GRPSPI Tunnel Descriptor Block not found for SA:\n"
-					    "klips_debug:                %s, could not group.\n", sa);
-				spin_unlock_irqrestore(&tdb_lock, flags);
+					    "EMT_GRPSPI Tunnel Descriptor Block not found for SA%s, could not group.\n",
+					    sa_len ? sa : " (error)");
+				spin_unlock_bh(&tdb_lock);
 				SENDERR(ENXIO);
 			} else {
 				if(tdbp->tdb_inext || tdbp->tdb_onext) {
 					KLIPS_PRINT(debug_netlink,
 						    "klips_debug:ipsec_callback: "
-						    "EMT_GRPSPI Tunnel Descriptor Block already grouped\n"
-						    "klips_debug:                for SA: %s, can't regroup.\n", sa);
-					spin_unlock_irqrestore(&tdb_lock, flags);
+						    "EMT_GRPSPI Tunnel Descriptor Block already grouped for SA: %s, can't regroup.\n",
+						    sa_len ? sa : " (error)");
+					spin_unlock_bh(&tdb_lock);
 					SENDERR(EBUSY);
 				}
 				em->em_rel[i].emr_tdb = tdbp;
@@ -348,7 +337,7 @@ ipsec_callback(struct sk_buff *skb)
 		}
 		tprev->tdb_onext = NULL;
 
-		spin_unlock_irqrestore(&tdb_lock, flags);
+		spin_unlock_bh(&tdb_lock);
 
 		error = 0;
 		break;
@@ -363,14 +352,14 @@ ipsec_callback(struct sk_buff *skb)
 			break;
 		}
 		
-		spin_lock_irqsave(&tdb_lock, flags);
+		spin_lock_bh(&tdb_lock);
 
 		if ((tdbp = gettdb(&(em->em_rel[0].emr_said))) == NULL) {
 			KLIPS_PRINT(debug_netlink,
 				    "klips_debug:ipsec_callback: "
-				    "EMT_UGRPSPI Tunnel Descriptor Block not found for SA:\n"
-				    "klips_debug:                %s, could not ungroup.\n", sa);
-			spin_unlock_irqrestore(&tdb_lock, flags);
+				    "EMT_UGRPSPI Tunnel Descriptor Block not found for SA%s, could not ungroup.\n",
+				    sa_len ? sa : " (error)");
+			spin_unlock_bh(&tdb_lock);
 			SENDERR(ENXIO);
 		}
 		while(tdbp->tdb_onext) {
@@ -383,20 +372,22 @@ ipsec_callback(struct sk_buff *skb)
 			tdbp->tdb_onext = NULL;
 		}
 
-		spin_unlock_irqrestore(&tdb_lock, flags);
+		spin_unlock_bh(&tdb_lock);
 
 		break;
 		
 	case EMT_CLRSPIS:
 		KLIPS_PRINT(debug_netlink,
-			    "klips_debug:ipsec_callback: spi clear called.\n");
+			    "klips_debug:ipsec_callback: "
+			    "spi clear called.\n");
 		if (em->em_if >= 5)	/* XXX -- why 5? */
 			SENDERR(ENODEV);
 		ipsec_tdbcleanup(0);
 		break;
 	default:
 		KLIPS_PRINT(debug_netlink,
-			    "klips_debug:ipsec_callback: unknown message type\n");
+			    "klips_debug:ipsec_callback: "
+			    "unknown message type\n");
 		SENDERR(EINVAL);
 	}
  errlab:
@@ -407,9 +398,55 @@ ipsec_callback(struct sk_buff *skb)
 #endif /* NET_21 */
 	return len;
 }
+#endif
 
 /*
  * $Log: ipsec_netlink.c,v $
+ * Revision 1.56  2002/01/29 17:17:55  mcr
+ * 	moved include of ipsec_param.h to after include of linux/kernel.h
+ * 	otherwise, it seems that some option that is set in ipsec_param.h
+ * 	screws up something subtle in the include path to kernel.h, and
+ * 	it complains on the snprintf() prototype.
+ *
+ * Revision 1.55  2002/01/29 04:00:51  mcr
+ * 	more excise of kversions.h header.
+ *
+ * Revision 1.54  2001/10/18 04:45:19  rgb
+ * 2.4.9 kernel deprecates linux/malloc.h in favour of linux/slab.h,
+ * lib/freeswan.h version macros moved to lib/kversions.h.
+ * Other compiler directive cleanups.
+ *
+ * Revision 1.53  2001/09/15 16:24:04  rgb
+ * Re-inject first and last HOLD packet when an eroute REPLACE is done.
+ *
+ * Revision 1.52  2001/09/14 16:58:36  rgb
+ * Added support for storing the first and last packets through a HOLD.
+ *
+ * Revision 1.51  2001/09/08 21:13:32  rgb
+ * Added pfkey ident extension support for ISAKMPd. (NetCelo)
+ *
+ * Revision 1.50  2001/07/06 19:49:00  rgb
+ * Renamed EMT_RPLACEROUTE to EMT_REPLACEROUTE for clarity and logical text
+ * searching.
+ *
+ * Revision 1.49  2001/06/14 19:35:08  rgb
+ * Update copyright date.
+ *
+ * Revision 1.48  2001/02/27 22:24:54  rgb
+ * Re-formatting debug output (line-splitting, joining, 1arg/line).
+ * Check for satoa() return codes.
+ *
+ * Revision 1.47  2000/11/06 04:32:08  rgb
+ * Ditched spin_lock_irqsave in favour of spin_lock_bh.
+ *
+ * Revision 1.46  2000/09/08 19:16:50  rgb
+ * Change references from DEBUG_IPSEC to CONFIG_IPSEC_DEBUG.
+ * Removed all references to CONFIG_IPSEC_PFKEYv2.
+ *
+ * Revision 1.45  2000/08/30 05:23:55  rgb
+ * Compiler-define out ipsec_callback() function of ipsec_netlink.c.
+ * Nothing should be using it anyways.
+ *
  * Revision 1.44  2000/03/16 14:01:26  rgb
  * Indented headers for readability.
  *
