@@ -1,7 +1,7 @@
 /*
  * Interface between the IPSEC code and the radix (radij) tree code
  * Copyright (C) 1996, 1997  John Ioannidis.
- * Copyright (C) 1998, 1999  Richard Guy Briggs.
+ * Copyright (C) 1998, 1999, 2000, 2001  Richard Guy Briggs.
  * 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -13,48 +13,65 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: ipsec_radij.c,v 1.37 2000/03/16 14:02:50 rgb Exp $
+ * RCSID $Id: ipsec_radij.c,v 1.60 2002/02/19 23:59:45 rgb Exp $
  */
 
 #include <linux/config.h>
 #include <linux/version.h>
-
 #include <linux/kernel.h> /* printk() */
-#include <linux/malloc.h> /* kmalloc() */
+
+#include "ipsec_param.h"
+
+#ifdef MALLOC_SLAB
+# include <linux/slab.h> /* kmalloc() */
+#else /* MALLOC_SLAB */
+# include <linux/malloc.h> /* kmalloc() */
+#endif /* MALLOC_SLAB */
 #include <linux/errno.h>  /* error codes */
 #include <linux/types.h>  /* size_t */
 #include <linux/interrupt.h> /* mark_bh */
 
-#include <linux/netdevice.h>   /* struct device, and other headers */
+#include <linux/netdevice.h>   /* struct device, struct net_device_stats and other headers */
 #include <linux/etherdevice.h> /* eth_type_trans */
 #include <linux/ip.h>          /* struct iphdr */
 #include <linux/skbuff.h>
 #include <freeswan.h>
 #ifdef SPINLOCK
-#ifdef SPINLOCK_23
-#include <linux/spinlock.h> /* *lock* */
-#else /* 23_SPINLOCK */
-#include <asm/spinlock.h> /* *lock* */
-#endif /* 23_SPINLOCK */
+# ifdef SPINLOCK_23
+#  include <linux/spinlock.h> /* *lock* */
+# else /* 23_SPINLOCK */
+#  include <asm/spinlock.h> /* *lock* */
+# endif /* 23_SPINLOCK */
 #endif /* SPINLOCK */
 #ifdef NET_21
-#include <asm/uaccess.h>
-#include <linux/in6.h>
+# include <asm/uaccess.h>
+# include <linux/in6.h>
 #endif
 #include <asm/checksum.h>
 #include <net/ip.h>
 
+#include "ipsec_eroute.h"
+#include "ipsec_sa.h"
+ 
+#include "radij.h"
+#include "ipsec_encap.h"
 #include "radij.h"
 #include "ipsec_encap.h"
 #include "ipsec_radij.h"
 #include "ipsec_netlink.h"
+#include "ipsec_tunnel.h"	/* struct ipsecpriv */
 #include "ipsec_xform.h"
+ 
+#include <pfkeyv2.h>
+#include <pfkey.h>
 
-#ifdef DEBUG_IPSEC
+#include "ipsec_proto.h"
+
+#ifdef CONFIG_IPSEC_DEBUG
 int debug_radij = 0;
-#endif
+#endif /* CONFIG_IPSEC_DEBUG */
 
-struct radij_node_head *rnh = 0;
+struct radij_node_head *rnh = NULL;
 #ifdef SPINLOCK
 spinlock_t eroute_lock = SPIN_LOCK_UNLOCKED;
 #else /* SPINLOCK */
@@ -68,7 +85,7 @@ ipsec_radijinit(void)
 
 	rj_init();
 	
-	if (rj_inithead((void **)&rnh, 16) == 0) /* 16 is bit offset of sen_type */
+	if (rj_inithead((void **)&rnh, /*16*/offsetof(struct sockaddr_encap, sen_type) * sizeof(__u8)) == 0) /* 16 is bit offset of sen_type */
 		return -1;
 	return 0;
 }
@@ -77,13 +94,12 @@ int
 ipsec_radijcleanup(void)
 {
 	int error;
-	unsigned long flags; /* save irq state for spinlock */
 
-	spin_lock_irqsave(&eroute_lock, flags);
+	spin_lock_bh(&eroute_lock);
 
 	error = radijcleanup();
 
-	spin_unlock_irqrestore(&eroute_lock, flags);
+	spin_unlock_bh(&eroute_lock);
 
 	return error;
 }
@@ -91,101 +107,232 @@ ipsec_radijcleanup(void)
 int
 ipsec_cleareroutes(void)
 {
-	int error;
-	unsigned long flags; /* save irq state for spinlock */
+	int error = 0;
 
-	spin_lock_irqsave(&eroute_lock, flags);
+	spin_lock_bh(&eroute_lock);
 
 	error = radijcleartree();
 
-	spin_unlock_irqrestore(&eroute_lock, flags);
+	spin_unlock_bh(&eroute_lock);
 
 	return error;
 }
 
 int
-ipsec_breakroute(struct sockaddr_encap *eaddr, struct sockaddr_encap *emask)
+ipsec_breakroute(struct sockaddr_encap *eaddr,
+		 struct sockaddr_encap *emask,
+		 struct sk_buff **first,
+		 struct sk_buff **last)
 {
+	struct eroute *ro;
 	struct radij_node *rn;
-	int error;
-	unsigned long flags; /* save irq state for spinlock */
-#ifdef DEBUG_IPSEC
+	int error = 0;
+#ifdef CONFIG_IPSEC_DEBUG
 	char buf1[64], buf2[64];
 	
 	if (debug_eroute) {
 		subnettoa(eaddr->sen_ip_src, emask->sen_ip_src, 0, buf1, sizeof(buf1));
 		subnettoa(eaddr->sen_ip_dst, emask->sen_ip_dst, 0, buf2, sizeof(buf2));
-		printk("klips_debug:ipsec_breakroute: attempting to delete eroute for %s->%s\n",
-			       buf1, buf2);
+		KLIPS_PRINT(debug_eroute, 
+			    "klips_debug:ipsec_breakroute: "
+			    "attempting to delete eroute for %s->%s\n",
+			    buf1,
+			    buf2);
 	}
-#endif
+#endif /* CONFIG_IPSEC_DEBUG */
 
-	spin_lock_irqsave(&eroute_lock, flags);
+	spin_lock_bh(&eroute_lock);
 
 	if ((error = rj_delete(eaddr, emask, rnh, &rn)) != 0) {
-		spin_unlock_irqrestore(&eroute_lock, flags);
-		KLIPS_PRINT(debug_eroute, "klips_debug:ipsec_breakroute: node not found, eroute delete failed.\n");
+		spin_unlock_bh(&eroute_lock);
+		KLIPS_PRINT(debug_eroute, 
+			    "klips_debug:ipsec_breakroute: "
+			    "node not found, eroute delete failed.\n");
 		return error;
 	}
 
-	spin_unlock_irqrestore(&eroute_lock, flags);
+	spin_unlock_bh(&eroute_lock);
+	
+	ro = (struct eroute *)rn;
+	
+	KLIPS_PRINT(debug_eroute, 
+		    "klips_debug:ipsec_breakroute: "
+		    "deleted eroute=%p, ident=%p->%p, first=%p, last=%p\n",
+		    ro,
+		    ro->er_ident_s.data,
+		    ro->er_ident_d.data,
+		    ro->er_first,
+		    ro->er_last);
+	
+	if (ro->er_ident_s.data != NULL) {
+		kfree(ro->er_ident_s.data);
+	}
+	if (ro->er_ident_d.data != NULL) {
+		kfree(ro->er_ident_d.data);
+	}
+	if (ro->er_first != NULL) {
+#if 0
+		struct net_device_stats *stats = (struct net_device_stats *) &(((struct ipsecpriv *)(ro->er_first->dev->priv))->mystats);
+		stats->tx_dropped--;
+#endif
+		*first = ro->er_first;
+	}
+	if (ro->er_last != NULL) {
+#if 0
+		struct net_device_stats *stats = (struct net_device_stats *) &(((struct ipsecpriv *)(ro->er_last->dev->priv))->mystats);
+		stats->tx_dropped--;
+#endif
+		*last = ro->er_last;
+	}
 	
 	if (rn->rj_flags & (RJF_ACTIVE | RJF_ROOT))
-		panic ("ipsec_callback RMT_DELEROUTE root or active node\n");
+		panic ("ipsec_breakroute RMT_DELEROUTE root or active node\n");
 	
 	memset((caddr_t)rn, 0, sizeof (struct eroute));
-	kfree_s(rn, sizeof(struct eroute));
+	kfree(rn);
 	
 	return 0;
 }
 
 int
-ipsec_makeroute(struct sockaddr_encap *eaddr, struct sockaddr_encap *emask, struct sa_id said)
+ipsec_makeroute(struct sockaddr_encap *eaddr,
+		struct sockaddr_encap *emask,
+		struct sa_id said,
+		uint32_t pid,
+		struct sk_buff *skb,
+		struct ident *ident_s,
+		struct ident *ident_d)
 {
 	struct eroute *retrt;
-	int error;
+	int error = 0;
 	char sa[SATOA_BUF];
 	size_t sa_len;
-	unsigned long flags; /* save irq state for spinlock */
-#ifdef DEBUG_IPSEC
+#ifdef CONFIG_IPSEC_DEBUG
 	char buf1[64], buf2[64];
 	
 	if (debug_eroute) {
 		subnettoa(eaddr->sen_ip_src, emask->sen_ip_src, 0, buf1, sizeof(buf1));
 		subnettoa(eaddr->sen_ip_dst, emask->sen_ip_dst, 0, buf2, sizeof(buf2));
 		sa_len = satoa(said, 0, sa, SATOA_BUF);
-		printk("klips_debug:ipsec_makeroute: attempting to insert eroute for %s->%s, SA: %s\n",
-		       buf1, buf2, sa);
+		KLIPS_PRINT(debug_eroute, 
+			    "klips_debug:ipsec_makeroute: "
+			    "attempting to insert eroute for %s->%s, SA: %s, PID:%d, skb=%p, ident:%s->%s\n",
+			    buf1,
+			    buf2,
+			    sa_len ? sa : " (error)",
+			    pid,
+			    skb,
+			    (ident_s ? (ident_s->data ? ident_s->data : "NULL") : "NULL"),
+			    (ident_d ? (ident_d->data ? ident_d->data : "NULL") : "NULL"));
 	}
-#endif
+#endif /* CONFIG_IPSEC_DEBUG */
 
 	retrt = (struct eroute *)kmalloc(sizeof (struct eroute), GFP_ATOMIC);
 	if (retrt == NULL) {
-		printk("klips_error:ipsec_makeroute: not able to allocate kernel memory");
-		return ENOMEM;
+		printk("klips_error:ipsec_makeroute: "
+		       "not able to allocate kernel memory");
+		return -ENOMEM;
 	}
 	memset((caddr_t)retrt, 0, sizeof (struct eroute));
 
 	retrt->er_eaddr = *eaddr;
 	retrt->er_emask = *emask;
 	retrt->er_said = said;
+	retrt->er_pid = pid;
+	retrt->er_count = 0;
+	retrt->er_lasttime = jiffies/HZ;
 	rd_key((&(retrt->er_rjt))) = &(retrt->er_eaddr);
 	
-	spin_lock_irqsave(&eroute_lock, flags);
-
+	if (ident_s && ident_s->type != SADB_IDENTTYPE_RESERVED) {
+		int data_len = ident_s->len * IPSEC_PFKEYv2_ALIGN - sizeof(struct sadb_ident);
+		
+		retrt->er_ident_s.type = ident_s->type;
+		retrt->er_ident_s.id = ident_s->id;
+		retrt->er_ident_s.len = ident_s->len;
+		if(data_len) {
+			if(!(retrt->er_ident_s.data = kmalloc(data_len, GFP_KERNEL))) {
+				kfree(retrt);
+				printk("klips_error:ipsec_makeroute: not able to allocate kernel memory (%d)\n", data_len);
+				return ENOMEM;
+			}
+			memcpy(retrt->er_ident_s.data, ident_s->data, data_len);
+		} else {
+			retrt->er_ident_s.data = NULL;
+		}
+	}
+	
+	if (ident_d && ident_d->type != SADB_IDENTTYPE_RESERVED) {
+		int data_len = ident_d->len  * IPSEC_PFKEYv2_ALIGN - sizeof(struct sadb_ident);
+		
+		retrt->er_ident_d.type = ident_d->type;
+		retrt->er_ident_d.id = ident_d->id;
+		retrt->er_ident_d.len = ident_d->len;
+		if(data_len) {
+			if(!(retrt->er_ident_d.data = kmalloc(data_len, GFP_KERNEL))) {
+				if (retrt->er_ident_s.data)
+					kfree(retrt->er_ident_s.data);
+				kfree(retrt);
+				printk("klips_error:ipsec_makeroute: not able to allocate kernel memory (%d)\n", data_len);
+				return ENOMEM;
+			}
+			memcpy(retrt->er_ident_d.data, ident_d->data, data_len);
+		} else {
+			retrt->er_ident_d.data = NULL;
+		}
+	}
+	retrt->er_first = skb;
+	retrt->er_last = NULL;
+	
+	spin_lock_bh(&eroute_lock);
+	
 	error = rj_addroute(&(retrt->er_eaddr), &(retrt->er_emask), 
 			 rnh, retrt->er_rjt.rd_nodes);
 
-	spin_unlock_irqrestore(&eroute_lock, flags);
+	spin_unlock_bh(&eroute_lock);
 	
 	if(error) {
 		sa_len = satoa(said, 0, sa, SATOA_BUF);
-		printk("klips_debug:ipsec_makeroute: rj_addroute not able to insert eroute for SA:%s\n", sa);
-		kfree(retrt); /* XXX -- should we? */
+		KLIPS_PRINT(debug_eroute, 
+			    "klips_debug:ipsec_makeroute: "
+			    "rj_addroute not able to insert eroute for SA:%s\n",
+			    sa_len ? sa : " (error)");
+		if (retrt->er_ident_s.data)
+			kfree(retrt->er_ident_s.data);
+		if (retrt->er_ident_d.data)
+			kfree(retrt->er_ident_d.data);
+		
+		kfree(retrt);
+		
 		return error;
 	}
+
+#ifdef CONFIG_IPSEC_DEBUG
+	if (debug_eroute && 0) {
+/*
+		subnettoa(eaddr->sen_ip_src, emask->sen_ip_src, 0, buf1, sizeof(buf1));
+		subnettoa(eaddr->sen_ip_dst, emask->sen_ip_dst, 0, buf2, sizeof(buf2));
+*/
+		subnettoa(rd_key((&(retrt->er_rjt)))->sen_ip_src, rd_mask((&(retrt->er_rjt)))->sen_ip_src, 0, buf1, sizeof(buf1));
+		subnettoa(rd_key((&(retrt->er_rjt)))->sen_ip_dst, rd_mask((&(retrt->er_rjt)))->sen_ip_dst, 0, buf2, sizeof(buf2));
+		sa_len = satoa(retrt->er_said, 0, sa, SATOA_BUF);
+		
+		KLIPS_PRINT(debug_eroute,
+			    "klips_debug:ipsec_makeroute: "
+			    "pid=%05d "
+			    "count=%10d "
+			    "lasttime=%6d "
+			    "%-18s -> %-18s => %s\n",
+			    retrt->er_pid,
+			    retrt->er_count,
+			    (int)(jiffies/HZ - retrt->er_lasttime),
+			    buf1,
+			    buf2,
+			    sa_len ? sa : " (error)");
+	}
+#endif /* CONFIG_IPSEC_DEBUG */
 	KLIPS_PRINT(debug_eroute,
-		    "klips_debug:ipsec_makeroute: succeeded, I think...\n");
+		    "klips_debug:ipsec_makeroute: "
+		    "succeeded, I think...\n");
 	return 0;
 }
 
@@ -193,17 +340,28 @@ struct eroute *
 ipsec_findroute(struct sockaddr_encap *eaddr)
 {
 	struct radij_node *rn;
-#ifdef DEBUG_IPSEC
+#ifdef CONFIG_IPSEC_DEBUG
 	char buf1[ADDRTOA_BUF], buf2[ADDRTOA_BUF];
 	
 	if (debug_radij & DB_RJ_FINDROUTE) {
 		addrtoa(eaddr->sen_ip_src, 0, buf1, sizeof(buf1));
 		addrtoa(eaddr->sen_ip_dst, 0, buf2, sizeof(buf2));
-		printk("klips_debug:ipsec_findroute: %s->%s\n",
-		       buf1, buf2);
+		KLIPS_PRINT(debug_eroute,
+			    "klips_debug:ipsec_findroute: "
+			    "%s->%s\n",
+			    buf1,
+			    buf2);
 	}
-#endif
+#endif /* CONFIG_IPSEC_DEBUG */
 	rn = rj_match((caddr_t)eaddr, rnh);
+	if(rn) {
+		KLIPS_PRINT(debug_eroute,
+			    "klips_debug:ipsec_findroute: "
+			    "found, points to proto=%d, spi=%x, dst=%x.\n",
+			    ((struct eroute*)rn)->er_said.proto,
+			    ntohl(((struct eroute*)rn)->er_said.spi),
+			    ntohl(((struct eroute*)rn)->er_said.dst.s_addr));
+	}
 	return (struct eroute *)rn;
 }
 		
@@ -220,8 +378,10 @@ ipsec_rj_walker_procprint(struct radij_node *rn, void *w0)
 	struct sockaddr_encap *key, *mask;
 	
 	KLIPS_PRINT(debug_radij,
-		    "klips_debug:ipsec_rj_walker_procprint: rn=%p, w0=%p\n",
-		    rn, w0);
+		    "klips_debug:ipsec_rj_walker_procprint: "
+		    "rn=%p, w0=%p\n",
+		    rn,
+		    w0);
 	if (rn == NULL)	{
 		return 120;
 	}
@@ -236,13 +396,54 @@ ipsec_rj_walker_procprint(struct radij_node *rn, void *w0)
 	if ((key == 0) || (mask == 0)) {
 		return 0;
 	}
-	
+
 	subnettoa(key->sen_ip_src, mask->sen_ip_src, 0, buf1, sizeof(buf1));
 	subnettoa(key->sen_ip_dst, mask->sen_ip_dst, 0, buf2, sizeof(buf2));
 	sa_len = satoa(ro->er_said, 0, sa, SATOA_BUF);
+#define IPSEC_EROUTE_IDENT_
+#ifndef IPSEC_EROUTE_IDENT
 	w->len += sprintf(w->buffer + w->len,
+/*
+			  "%05d "
+*/
+			  "%-10d "
+/*
+			  "%6d "
+*/
 			  "%-18s -> %-18s => %s\n",
-			  buf1, buf2, sa);
+/*
+			  ro->er_pid,
+*/
+			  ro->er_count,
+/*
+			  jiffies / HZ - ro->er_lasttime,
+*/
+			  buf1,
+			  buf2,
+			  sa_len ? sa : " (error)");
+#else /* IPSEC_EROUTE_IDENT */
+	w->len += sprintf(w->buffer + w->len,
+/*
+			  "%05d "
+*/
+			  "%-10d "
+/*
+			  "%6d "
+*/
+			  "%-18s -> %-18s => %s (%s) (%s)\n",
+/*
+			  ro->er_pid,
+*/
+			  ro->er_count,
+/*
+			  jiffies / HZ - ro->er_lasttime,
+*/
+			  buf1,
+			  buf2,
+			  sa_len ? sa : " (error)",
+			  (ro->er_ident_s.data ? ro->er_ident_s.data : ""),
+			  (ro->er_ident_d.data ? ro->er_ident_d.data : ""));
+#endif /* IPSEC_EROUTE_IDENT */
 	
 	w->pos = w->begin + w->len;
 	if(w->pos < w->offset) {
@@ -259,13 +460,14 @@ ipsec_rj_walker_procprint(struct radij_node *rn, void *w0)
 int
 ipsec_rj_walker_delete(struct radij_node *rn, void *w0)
 {
+	struct eroute *ro;
 	struct rjtentry *rd = (struct rjtentry *)rn;
 	struct radij_node *rn2;
-	int error;
+	int error = 0;
 	struct sockaddr_encap *key, *mask;
-#ifdef DEBUG_IPSEC
+#ifdef CONFIG_IPSEC_DEBUG
 	char buf1[64] = { 0 }, buf2[64] = { 0 };
-#endif
+#endif /* CONFIG_IPSEC_DEBUG */
 
 	if (rn == NULL)	{
 		return 120;
@@ -275,16 +477,19 @@ ipsec_rj_walker_delete(struct radij_node *rn, void *w0)
 	mask = rd_mask(rd);
 	
 	if(!key || !mask) {
-		return -1;
+		return -ENODATA;
 	}
-#ifdef DEBUG_IPSEC
+#ifdef CONFIG_IPSEC_DEBUG
 	if(debug_radij)	{
 		subnettoa(key->sen_ip_src, mask->sen_ip_src, 0, buf1, sizeof(buf1));
 		subnettoa(key->sen_ip_dst, mask->sen_ip_dst, 0, buf2, sizeof(buf2));
-		printk("klips_debug:ipsec_rj_walker_delete: deleting: %s -> %s\n",
-		       buf1, buf2);
+		KLIPS_PRINT(debug_radij, 
+			    "klips_debug:ipsec_rj_walker_delete: "
+			    "deleting: %s -> %s\n",
+			    buf1,
+			    buf2);
 	}
-#endif /* DEBUG_IPSEC */
+#endif /* CONFIG_IPSEC_DEBUG */
 
 	if((error = rj_delete(key, mask, rnh, &rn2))) {
 		KLIPS_PRINT(debug_radij,
@@ -294,17 +499,108 @@ ipsec_rj_walker_delete(struct radij_node *rn, void *w0)
 	}
 
 	if(rn2 != rn) {
-		printk("klips_debug:ipsec_rj_walker_delete: tried to delete a different node?!?"
-		       "This should never happen!\n");
+		printk("klips_debug:ipsec_rj_walker_delete: "
+		       "tried to delete a different node?!?  This should never happen!\n");
 	}
+ 
+	ro = (struct eroute *)rn;
+	
+	if (ro->er_ident_s.data)
+		kfree(ro->er_ident_s.data);
+	if (ro->er_ident_d.data)
+		kfree(ro->er_ident_d.data);
+	
 	memset((caddr_t)rn, 0, sizeof (struct eroute));
-	kfree_s(rn, sizeof(struct eroute));
+	kfree(rn);
 	
 	return 0;
 }
 
 /*
  * $Log: ipsec_radij.c,v $
+ * Revision 1.60  2002/02/19 23:59:45  rgb
+ * Removed redundant compiler directives.
+ *
+ * Revision 1.59  2002/02/06 04:13:47  mcr
+ * 	missing #ifdef CONFIG_IPSEC_DEBUG.
+ *
+ * Revision 1.58  2002/01/29 17:17:56  mcr
+ * 	moved include of ipsec_param.h to after include of linux/kernel.h
+ * 	otherwise, it seems that some option that is set in ipsec_param.h
+ * 	screws up something subtle in the include path to kernel.h, and
+ * 	it complains on the snprintf() prototype.
+ *
+ * Revision 1.57  2002/01/29 04:00:52  mcr
+ * 	more excise of kversions.h header.
+ *
+ * Revision 1.56  2002/01/29 02:13:17  mcr
+ * 	introduction of ipsec_kversion.h means that include of
+ * 	ipsec_param.h must preceed any decisions about what files to
+ * 	include to deal with differences in kernel source.
+ *
+ * Revision 1.55  2001/11/26 09:23:48  rgb
+ * Merge MCR's ipsec_sa, eroute, proc and struct lifetime changes.
+ *
+ * Revision 1.53.2.1  2001/09/25 02:26:32  mcr
+ * 	headers adjusted for new usage.
+ *
+ * Revision 1.54  2001/10/18 04:45:20  rgb
+ * 2.4.9 kernel deprecates linux/malloc.h in favour of linux/slab.h,
+ * lib/freeswan.h version macros moved to lib/kversions.h.
+ * Other compiler directive cleanups.
+ *
+ * Revision 1.53  2001/09/19 17:19:40  rgb
+ * Debug output bugfix for NetCelo's PF_KEY ident patch.
+ *
+ * Revision 1.52  2001/09/19 16:33:37  rgb
+ * Temporarily disable ident fields to /proc/net/ipsec_eroute.
+ *
+ * Revision 1.51  2001/09/15 16:24:04  rgb
+ * Re-inject first and last HOLD packet when an eroute REPLACE is done.
+ *
+ * Revision 1.50  2001/09/14 16:58:36  rgb
+ * Added support for storing the first and last packets through a HOLD.
+ *
+ * Revision 1.49  2001/09/08 21:13:32  rgb
+ * Added pfkey ident extension support for ISAKMPd. (NetCelo)
+ *
+ * Revision 1.48  2001/06/15 04:12:56  rgb
+ * Fixed kernel memory allocation error return code polarity bug.
+ *
+ * Revision 1.47  2001/06/14 19:35:09  rgb
+ * Update copyright date.
+ *
+ * Revision 1.46  2001/06/08 08:47:18  rgb
+ * Fixed for debug disabled.
+ *
+ * Revision 1.45  2001/05/27 06:12:11  rgb
+ * Added structures for pid, packet count and last access time to eroute.
+ * Added packet count to beginning of /proc/net/ipsec_eroute.
+ *
+ * Revision 1.44  2001/05/03 19:41:01  rgb
+ * Initialise error return variable.
+ * Use more appropriate return value for ipsec_rj_walker_delete().
+ *
+ * Revision 1.43  2001/02/27 22:24:54  rgb
+ * Re-formatting debug output (line-splitting, joining, 1arg/line).
+ * Check for satoa() return codes.
+ *
+ * Revision 1.42  2001/02/27 06:21:57  rgb
+ * Added findroute success instrumentation.
+ *
+ * Revision 1.41  2000/11/06 04:32:08  rgb
+ * Ditched spin_lock_irqsave in favour of spin_lock_bh.
+ *
+ * Revision 1.40  2000/09/08 19:12:56  rgb
+ * Change references from DEBUG_IPSEC to CONFIG_IPSEC_DEBUG.
+ *
+ * Revision 1.39  2000/08/30 05:25:20  rgb
+ * Correct debug text in ipsec_breakroute() from incorrect
+ * "ipsec_callback".
+ *
+ * Revision 1.38  2000/07/28 14:58:31  rgb
+ * Changed kfree_s to kfree, eliminating extra arg to fix 2.4.0-test5.
+ *
  * Revision 1.37  2000/03/16 14:02:50  rgb
  * Fixed debug scope to enable compilation with debug off.
  *
