@@ -1,7 +1,7 @@
 /*
  * Common routines for IPSEC transformations.
  * Copyright (C) 1996, 1997  John Ioannidis.
- * Copyright (C) 1998, 1999  Richard Guy Briggs.
+ * Copyright (C) 1998, 1999, 2000, 2001  Richard Guy Briggs.
  * 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -13,14 +13,20 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: ipsec_xform.c,v 1.39 2000/05/10 23:11:09 rgb Exp $
+ * RCSID $Id: ipsec_xform.c,v 1.58 2002/01/29 17:17:57 mcr Exp $
  */
 
 #include <linux/config.h>
 #include <linux/version.h>
-
 #include <linux/kernel.h> /* printk() */
-#include <linux/malloc.h> /* kmalloc() */
+
+#include "ipsec_param.h"
+
+#ifdef MALLOC_SLAB
+# include <linux/slab.h> /* kmalloc() */
+#else /* MALLOC_SLAB */
+# include <linux/malloc.h> /* kmalloc() */
+#endif /* MALLOC_SLAB */
 #include <linux/errno.h>  /* error codes */
 #include <linux/types.h>  /* size_t */
 #include <linux/interrupt.h> /* mark_bh */
@@ -32,15 +38,15 @@
 #include <linux/random.h>	/* get_random_bytes() */
 #include <freeswan.h>
 #ifdef SPINLOCK
-#ifdef SPINLOCK_23
-#include <linux/spinlock.h> /* *lock* */
-#else /* SPINLOCK_23 */
-#include <asm/spinlock.h> /* *lock* */
-#endif /* SPINLOCK_23 */
+# ifdef SPINLOCK_23
+#  include <linux/spinlock.h> /* *lock* */
+# else /* SPINLOCK_23 */
+#  include <asm/spinlock.h> /* *lock* */
+# endif /* SPINLOCK_23 */
 #endif /* SPINLOCK */
 #ifdef NET_21
-#include <asm/uaccess.h>
-#include <linux/in6.h>
+# include <asm/uaccess.h>
+# include <linux/in6.h>
 #endif
 #include <asm/checksum.h>
 #include <net/ip.h>
@@ -54,16 +60,14 @@
 #include "ipsec_ah.h"
 #include "ipsec_esp.h"
 
-/* #ifdef CONFIG_IPSEC_PFKEYv2 */
 #include <pfkeyv2.h>
 #include <pfkey.h>
-/* #endif */
 
-#ifdef DEBUG_IPSEC
+#ifdef CONFIG_IPSEC_DEBUG
 int debug_xform = 0;
-#endif
+#endif /* CONFIG_IPSEC_DEBUG */
 
-#define SENDERR(_x) do { len = -(_x); goto errlab; } while (0)
+#define SENDERR(_x) do { error = -(_x); goto errlab; } while (0)
 
 extern int des_set_key(caddr_t, caddr_t);
 
@@ -71,9 +75,6 @@ struct xformsw xformsw[] = {
 { XF_IP4,		0,		"IPv4_Encapsulation"},
 { XF_AHHMACMD5,		XFT_AUTH,	"HMAC_MD5_Authentication"},
 { XF_AHHMACSHA1,	XFT_AUTH,	"HMAC_SHA-1_Authentication"},
-{ XF_ESPDES,		XFT_CONF,	"DES_Encryption"},
-{ XF_ESPDESMD596,	XFT_CONF,	"DES-MD5-96_Encryption"},
-{ XF_ESPDESSHA196,	XFT_CONF,	"DES-SHA1-96_Encryption"},
 { XF_ESP3DES,		XFT_CONF,	"3DES_Encryption"},
 { XF_ESP3DESMD596,	XFT_CONF,	"3DES-MD5-96_Encryption"},
 { XF_ESP3DESSHA196,	XFT_CONF,	"3DES-SHA1-96_Encryption"},
@@ -100,31 +101,42 @@ ipsec_tdbinit(void)
 	return 0;
 }
 
+#if 0
 struct tdb *
 gettdb(struct sa_id *said)
 {
 	int hashval;
 	struct tdb *tdbp;
         char sa[SATOA_BUF];
+	size_t sa_len;
 
 	if(!said) {
 		KLIPS_PRINT(debug_xform,
-			    "klips_error:gettdb: null pointer passed in!\n");
+			    "klips_error:gettdb: "
+			    "null pointer passed in!\n");
 		return NULL;
 	}
 
-	satoa(*said, 0, sa, SATOA_BUF);
+	sa_len = satoa(*said, 0, sa, SATOA_BUF);
 
 	hashval = (said->spi+said->dst.s_addr+said->proto) % TDB_HASHMOD;
-	
+
+#if 0	
 	KLIPS_PRINT(debug_xform,
-		    "klips_debug:gettdb: linked entry in tdb table for hash=%d of SA:%s requested.\n",
-		    hashval, sa);
+		    "klips_debug:gettdb: "
+		    "linked entry in tdb table for hash=%d of SA:%s requested.\n",
+		    hashval,
+		    sa_len ? sa : " (error)");
+#endif
 
 	if(!(tdbp = tdbh[hashval])) {
+#if 0	
 		KLIPS_PRINT(debug_xform,
-			    "klips_debug:gettdb: no entries in tdb table for hash=%d of SA:%s.\n",
-			    hashval, sa);
+			    "klips_debug:gettdb: "
+			    "no entries in tdb table for hash=%d of SA:%s.\n",
+			    hashval,
+			    sa_len ? sa : " (error)");
+#endif
 		return NULL;
 	}
 
@@ -135,72 +147,84 @@ gettdb(struct sa_id *said)
 			return tdbp;
 		}
 	}
-	
+
+#if 0	
 	KLIPS_PRINT(debug_xform,
-		    "klips_debug:gettdb: no entry in linked list for hash=%d of SA:%s.\n",
-		    hashval, sa);
+		    "klips_debug:gettdb: "
+		    "no entry in linked list for hash=%d of SA:%s.\n",
+		    hashval,
+		    sa_len ? sa : " (error)");
+#endif
 	return NULL;
 }
 
-/* void */
+/*
+  The tdb table better *NOT* be locked before it is handed in, or SMP locks will happen
+*/
 int
 puttdb(struct tdb *tdbp)
 {
 	int error = 0;
 	unsigned int hashval;
-	unsigned long flags; /* save irq state for spinlock */
 
 	if(!tdbp) {
 		KLIPS_PRINT(debug_xform,
-			    "klips_error:puttdb: null pointer passed in!\n");
+			    "klips_error:puttdb: "
+			    "null pointer passed in!\n");
 		return -ENODATA;
 	}
-	hashval = ((tdbp->tdb_spi + tdbp->tdb_dst.s_addr + tdbp->tdb_proto) % TDB_HASHMOD);
+	hashval = ((tdbp->tdb_said.spi + tdbp->tdb_said.dst.s_addr + tdbp->tdb_said.proto) % TDB_HASHMOD);
 
-	spin_lock_irqsave(&tdb_lock, flags);
+	spin_lock_bh(&tdb_lock);
 	
 	tdbp->tdb_hnext = tdbh[hashval];
 	tdbh[hashval] = tdbp;
 	
-	spin_unlock_irqrestore(&tdb_lock, flags);
+	spin_unlock_bh(&tdb_lock);
 
 	return error;
 }
 
-/* This tdb better be locked before it is handed in, or races might
- * happen */
-
-/* void */
+/*
+  The tdb table better be locked before it is handed in, or races might happen
+*/
 int
 deltdb(struct tdb *tdbp)
 {
 	unsigned int hashval;
 	struct tdb *tdbtp;
         char sa[SATOA_BUF];
+	size_t sa_len;
 
 	if(!tdbp) {
 		KLIPS_PRINT(debug_xform,
-			    "klips_error:deltdb: null pointer passed in!\n");
+			    "klips_error:deltdb: "
+			    "null pointer passed in!\n");
 		return -ENODATA;
 	}
 	
-	satoa(tdbp->tdb_said, 0, sa, SATOA_BUF);
+	sa_len = satoa(tdbp->tdb_said, 0, sa, SATOA_BUF);
 	if(tdbp->tdb_inext || tdbp->tdb_onext) {
 		KLIPS_PRINT(debug_xform,
-			    "klips_error:deltdb: SA:%s still linked!\n",
-			    sa);
+			    "klips_error:deltdb: "
+			    "SA:%s still linked!\n",
+			    sa_len ? sa : " (error)");
 		return -EMLINK;
 	}
 	
-	hashval = ((tdbp->tdb_spi + tdbp->tdb_dst.s_addr + tdbp->tdb_proto) % TDB_HASHMOD);
+	hashval = ((tdbp->tdb_said.spi + tdbp->tdb_said.dst.s_addr + tdbp->tdb_said.proto) % TDB_HASHMOD);
 	
 	KLIPS_PRINT(debug_xform,
-		    "klips_debug:deltdb: deleting SA:%s, hashval=%d.\n",
-		    sa, hashval);
+		    "klips_debug:deltdb: "
+		    "deleting SA:%s, hashval=%d.\n",
+		    sa_len ? sa : " (error)",
+		    hashval);
 	if(!tdbh[hashval]) {
 		KLIPS_PRINT(debug_xform,
-			    "klips_debug:deltdb: no entries in tdb table for hash=%d of SA:%s.\n",
-			    hashval, sa);
+			    "klips_debug:deltdb: "
+			    "no entries in tdb table for hash=%d of SA:%s.\n",
+			    hashval,
+			    sa_len ? sa : " (error)");
 		return -ENOENT;
 	}
 	
@@ -208,7 +232,8 @@ deltdb(struct tdb *tdbp)
 		tdbh[hashval] = tdbh[hashval]->tdb_hnext;
 		tdbp->tdb_hnext = NULL;
 		KLIPS_PRINT(debug_xform,
-			    "klips_debug:deltdb: successfully deleted first tdb in chain.\n");
+			    "klips_debug:deltdb: "
+			    "successfully deleted first tdb in chain.\n");
 		return 0;
 	} else {
 		for (tdbtp = tdbh[hashval]; tdbtp; tdbtp = tdbtp->tdb_hnext) {
@@ -216,53 +241,62 @@ deltdb(struct tdb *tdbp)
 				tdbtp->tdb_hnext = tdbp->tdb_hnext;
 				tdbp->tdb_hnext = NULL;
 				KLIPS_PRINT(debug_xform,
-					    "klips_debug:deltdb: successfully "
-					    "deleted link in tdb chain.\n");
+					    "klips_debug:deltdb: "
+					    "successfully deleted link in tdb chain.\n");
 				return 0;
 			}
 		}
 	}
 	
 	KLIPS_PRINT(debug_xform,
-		    "klips_debug:deltdb: no entries in linked list for hash=%d of SA:%s.\n",
-		    hashval, sa);
+		    "klips_debug:deltdb: "
+		    "no entries in linked list for hash=%d of SA:%s.\n",
+		    hashval,
+		    sa_len ? sa : " (error)");
 	return -ENOENT;
 }
 
-/* This tdb better be locked before it is handed in, or races might
- * happen */
-
+/*
+  The tdb table better be locked before it is handed in, or races might happen
+*/
 int
 deltdbchain(struct tdb *tdbp)
 {
 	struct tdb *tdbdel;
 	int error = 0;
         char sa[SATOA_BUF];
+	size_t sa_len;
 
 	if(!tdbp) {
 		KLIPS_PRINT(debug_xform,
-			    "klips_error:deltdbchain: null pointer passed in!\n");
+			    "klips_error:deltdbchain: "
+			    "null pointer passed in!\n");
 		return -ENODATA;
 	}
 
-	satoa(tdbp->tdb_said, 0, sa, SATOA_BUF);
+	sa_len = satoa(tdbp->tdb_said, 0, sa, SATOA_BUF);
 	KLIPS_PRINT(debug_xform,
-		    "klips_debug:deltdbchain: passed SA:%s\n", sa);
+		    "klips_debug:deltdbchain: "
+		    "passed SA:%s\n",
+		    sa_len ? sa : " (error)");
 	while(tdbp->tdb_onext) {
 		tdbp = tdbp->tdb_onext;
 	}
 
 	while(tdbp) {
 		/* XXX send a pfkey message up to advise of deleted TDB */
-		satoa(tdbp->tdb_said, 0, sa, SATOA_BUF);
+		sa_len = satoa(tdbp->tdb_said, 0, sa, SATOA_BUF);
 		KLIPS_PRINT(debug_xform,
-			    "klips_debug:deltdbchain: unlinking and delting SA:%s", sa);
+			    "klips_debug:deltdbchain: "
+			    "unlinking and delting SA:%s",
+			    sa_len ? sa : " (error)");
 		tdbdel = tdbp;
 		tdbp = tdbp->tdb_inext;
 		if(tdbp) {
-			satoa(tdbp->tdb_said, 0, sa, SATOA_BUF);
+			sa_len = satoa(tdbp->tdb_said, 0, sa, SATOA_BUF);
 			KLIPS_PRINT(debug_xform,
-				    ", inext=%s", sa);
+				    ", inext=%s",
+				    sa_len ? sa : " (error)");
 			tdbdel->tdb_inext = NULL;
 			tdbp->tdb_onext = NULL;
 		}
@@ -284,39 +318,45 @@ deltdbchain(struct tdb *tdbp)
 	return error;
 }
 
+#if 0
 int
 tdb_init(struct tdb *tdbp, struct encap_msghdr *em)
 {
 	int alg;
 	struct xformsw *xsp;
-	int len;
+	int error = 0;
         int i;
-#if defined(CONFIG_IPSEC_ENC_DES) || defined(CONFIG_IPSEC_ENC_3DES)
+#if defined(CONFIG_IPSEC_ENC_3DES)
         int error;
-#endif /* CONFIG_IPSEC_ENC_{,3}DES */
+#endif /* CONFIG_IPSEC_ENC_3DES */
 
         char sa[SATOA_BUF];
         size_t sa_len;
 
 	if(!tdbp || !em) {
 		KLIPS_PRINT(debug_xform,
-			    "klips_error:tdb_init: null pointer passed in!\n");
+			    "klips_error:tdb_init: "
+			    "null pointer passed in!\n");
 		SENDERR(ENODATA);
 	}
 
 	sa_len = satoa(em->em_said, 0, sa, SATOA_BUF);
 
         KLIPS_PRINT(debug_esp,
-		    "klips_debug:tdb_init: (algo_switch defined) called for SA:%s\n", sa);
+		    "klips_debug:tdb_init: "
+		    "(algo_switch defined) called for SA:%s\n",
+		    sa_len ? sa : " (error)");
 	alg = em->em_alg;
 	
 	for (xsp = xformsw; xsp < xformswNXFORMSW; xsp++) {
 		if (xsp->xf_type == alg) {
 			KLIPS_PRINT(debug_netlink,
-				    "klips_debug:tdb_init: called with tdbp=0x%p, xsp=0x%p, em=0x%p\n",
+				    "klips_debug:tdb_init: "
+				    "called with tdbp=0x%p, xsp=0x%p, em=0x%p\n",
 				    tdbp, xsp, em);
 			KLIPS_PRINT(debug_netlink,
-				    "klips_debug:tdb_init: calling init routine of %s\n",
+				    "klips_debug:tdb_init: "
+				    "calling init routine of %s\n",
 				    xsp->xf_name);
 			tdbp->tdb_xform = xsp;
 			tdbp->tdb_replaywin_lastseq = 0;
@@ -368,22 +408,23 @@ tdb_init(struct tdb *tdbp, struct encap_msghdr *em)
 				
 				if (ed->ame_klen != AHMD596_KLEN) {
 					KLIPS_PRINT(debug_ah,
-						    "klips_debug:tdb_init: incorrect key size: %d"
-						    "-- must be %d octets (bytes)\n",
+						    "klips_debug:tdb_init: "
+						    "incorrect key size: %d -- must be %d octets (bytes)\n",
 						    ed->ame_klen, AHMD596_KLEN);
 					SENDERR(EINVAL);
 				}
 				
 				if (ed->ame_alen != AHMD596_ALEN) {
 					KLIPS_PRINT(debug_ah,
-						    "klips_debug:tdb_init: authenticator size: %d"
-						    " -- must be %d octets (bytes)\n",
+						    "klips_debug:tdb_init: "
+						    "authenticator size: %d -- must be %d octets (bytes)\n",
 						    ed->ame_alen, AHMD596_ALEN);
 					SENDERR(EINVAL);
 				}
 				
 				KLIPS_PRINT(debug_ah,
-					    "klips_debug:tdb_init: hmac md5-96 key is 0x%08x %08x %08x %08x\n",
+					    "klips_debug:tdb_init: "
+					    "hmac md5-96 key is 0x%08x %08x %08x %08x\n",
 					    (__u32)ntohl(*(((__u32 *)ed->ame_key)+0)),
 					    (__u32)ntohl(*(((__u32 *)ed->ame_key)+1)),
 					    (__u32)ntohl(*(((__u32 *)ed->ame_key)+2)),
@@ -394,8 +435,8 @@ tdb_init(struct tdb *tdbp, struct encap_msghdr *em)
 				
 				if(ed->ame_ooowin > 64) {
 					KLIPS_PRINT(debug_ah,
-						    "klips_debug:tdb_init: replay window size: %d"
-						    " -- must be 0 <= size <= 64\n",
+						    "klips_debug:tdb_init: "
+						    "replay window size: %d -- must be 0 <= size <= 64\n",
 						    ed->ame_ooowin);
 					SENDERR(EINVAL);
 				}
@@ -427,8 +468,8 @@ tdb_init(struct tdb *tdbp, struct encap_msghdr *em)
 				MD5Update(octx, kb, AHMD596_BLKLEN);
 				
 				KLIPS_PRINT(debug_ah,
-					    "klips_debug:tdb_init: MD5 ictx=0x%08x %08x %08x %08x"
-					    " octx=0x%08x %08x %08x %08x\n",
+					    "klips_debug:tdb_init: "
+					    "MD5 ictx=0x%08x %08x %08x %08x octx=0x%08x %08x %08x %08x\n",
 					    ((__u32*)ictx)[0],
 					    ((__u32*)ictx)[1],
 					    ((__u32*)ictx)[2],
@@ -461,22 +502,23 @@ tdb_init(struct tdb *tdbp, struct encap_msghdr *em)
 				
 				if (ed->ame_klen != AHSHA196_KLEN) {
 					KLIPS_PRINT(debug_ah,
-						    "klips_debug:tdb_init: incorrect key size: %d"
-						    "-- must be %d octets (bytes)\n",
+						    "klips_debug:tdb_init: "
+						    "incorrect key size: %d -- must be %d octets (bytes)\n",
 						    ed->ame_klen, AHSHA196_KLEN);
 					SENDERR(EINVAL);
 				}
 				
 				if (ed->ame_alen != AHSHA196_ALEN) {
 					KLIPS_PRINT(debug_ah,
-						    "klips_debug:tdb_init: authenticator size: %d"
-						    " -- must be %d octets (bytes)\n",
+						    "klips_debug:tdb_init: "
+						    "authenticator size: %d -- must be %d octets (bytes)\n",
 						    ed->ame_alen, AHSHA196_ALEN);
 					SENDERR(EINVAL);
 				}
 				
 				KLIPS_PRINT(debug_ah,
-					    "klips_debug:tdb_init: hmac sha1-96 key is 0x%08x %08x %08x %08x\n",
+					    "klips_debug:tdb_init: "
+					    "hmac sha1-96 key is 0x%08x %08x %08x %08x\n",
 					    (__u32)ntohl(*(((__u32 *)ed->ame_key)+0)),
 					    (__u32)ntohl(*(((__u32 *)ed->ame_key)+1)),
 					    (__u32)ntohl(*(((__u32 *)ed->ame_key)+2)),
@@ -487,8 +529,8 @@ tdb_init(struct tdb *tdbp, struct encap_msghdr *em)
 				
 				if(ed->ame_ooowin > 64) {
 					KLIPS_PRINT(debug_ah,
-						    "klips_debug:tdb_init: replay window size: %d"
-						    " -- must be 0 <= size <= 64\n",
+						    "klips_debug:tdb_init: "
+						    "replay window size: %d -- must be 0 <= size <= 64\n",
 						    ed->ame_ooowin);
 					SENDERR(EINVAL);
 				}
@@ -518,8 +560,8 @@ tdb_init(struct tdb *tdbp, struct encap_msghdr *em)
 				SHA1Update(octx, kb, AHSHA196_BLKLEN);
 				
 				KLIPS_PRINT(debug_ah,
-					    "klips_debug:tdb_init: SHA1 ictx=0x%08x %08x %08x %08x"
-					    " octx=0x%08x %08x %08x %08x\n", 
+					    "klips_debug:tdb_init: "
+					    "SHA1 ictx=0x%08x %08x %08x %08x octx=0x%08x %08x %08x %08x\n", 
 					    ((__u32*)ictx)[0],
 					    ((__u32*)ictx)[1],
 					    ((__u32*)ictx)[2],
@@ -539,15 +581,6 @@ tdb_init(struct tdb *tdbp, struct encap_msghdr *em)
 #endif /* CONFIG_IPSEC_AH */
 
 #ifdef CONFIG_IPSEC_ESP
-#ifdef CONFIG_IPSEC_ENC_DES
-			case XF_ESPDES:
-#ifdef CONFIG_IPSEC_AUTH_HMAC_MD5
-			case XF_ESPDESMD596:
-#endif /* CONFIG_IPSEC_AUTH_HMAC_MD5 */
-#ifdef CONFIG_IPSEC_AUTH_HMAC_SHA1
-			case XF_ESPDESSHA196:
-#endif /* CONFIG_IPSEC_AUTH_HMAC_SHA1 */
-#endif /* CONFIG_IPSEC_ENC_DES */
 #ifdef CONFIG_IPSEC_ENC_3DES
 			case XF_ESP3DES:
 #ifdef CONFIG_IPSEC_AUTH_HMAC_MD5
@@ -557,14 +590,6 @@ tdb_init(struct tdb *tdbp, struct encap_msghdr *em)
 			case XF_ESP3DESSHA196:
 #endif /* CONFIG_IPSEC_AUTH_HMAC_SHA1 */
 #endif /* CONFIG_IPSEC_ENC_3DES */
-#ifdef CONFIG_IPSEC_ENC_NULL
-#ifdef CONFIG_IPSEC_AUTH_HMAC_MD5
-			case XF_ESPNULLMD596:
-#endif /* CONFIG_IPSEC_AUTH_HMAC_MD5 */
-#ifdef CONFIG_IPSEC_AUTH_HMAC_SHA1
-			case XF_ESPNULLSHA196:
-#endif /* CONFIG_IPSEC_AUTH_HMAC_SHA1 */
-#endif /* CONFIG_IPSEC_ENC_NULL */
 			{
 				struct espblkrply_edata *ed;
 				unsigned char kb[AHMD596_BLKLEN];
@@ -584,69 +609,28 @@ tdb_init(struct tdb *tdbp, struct encap_msghdr *em)
 
 				if(ed->eme_ooowin > 64) {
 					KLIPS_PRINT(debug_esp,
-						    "klips_debug:tdb_init: replay window size: %d"
-						    "-- must be 0 <= size <= 64\n",
+						    "klips_debug:tdb_init: "
+						    "replay window size: %d -- must be 0 <= size <= 64\n",
 						    ed->eme_ooowin);
 					SENDERR(EINVAL);
 				}
 				tdbp->tdb_replaywin = ed->eme_ooowin;
 
 				switch(alg) {
-				case XF_ESPDES:
 				case XF_ESP3DES:
-				case XF_ESPDESMD596:
 				case XF_ESP3DESMD596:
-				case XF_ESPDESSHA196:
 				case XF_ESP3DESSHA196:
 					if((tdbp->tdb_iv = (caddr_t)
 					   kmalloc((tdbp->tdb_iv_size = EMT_ESPDES_IV_SZ), GFP_ATOMIC)) == NULL) {
 						SENDERR(ENOMEM);
 					}
 					get_random_bytes((void *)tdbp->tdb_iv, EMT_ESPDES_IV_SZ);
+					tdbp->tdb_iv_bits = tdbp->tdb_iv_size * 8;
 					break;
 				default:
 				}
 
 				switch(alg) {
-#ifdef CONFIG_IPSEC_ENC_DES
-				case XF_ESPDES:
-				case XF_ESPDESMD596:
-				case XF_ESPDESSHA196:
-					tdbp->tdb_encalg = ESP_DES;
-				
-					if (ed->eme_klen != EMT_ESPDES_KEY_SZ) {
-						KLIPS_PRINT(debug_esp,
-							    "klips_debug:tdb_init: incorrect encryption "
-							    "key size: %d -- must be %d octets (bytes)\n",
-							    ed->eme_klen, EMT_ESPDES_KEY_SZ);
-						SENDERR(EINVAL);
-					}
-
-					tdbp->tdb_key_bits_e = ed->eme_klen;
-
-					if((tdbp->tdb_key_e = (caddr_t)
-					   kmalloc((tdbp->tdb_key_e_size = sizeof(struct des_eks)),
-						   GFP_ATOMIC)) == NULL) {
-						SENDERR(ENOMEM);
-					}
-#if 0
-					KLIPS_PRINT(debug_esp,
-						    "klips_debug:tdb_init: des key 1 is 0x%08lx%08lx\n",
-						    ntohl(*((__u32 *)ed->eme_key)),
-						    ntohl(*((__u32 *)ed->eme_key + 1)));
-#endif
-					error = des_set_key((caddr_t)(ed->eme_key), (caddr_t)(tdbp->tdb_key_e));
-					if (error == -1)
-						printk("klips_debug:tdb_init: parity error in des key\n");
-					else if (error == -2)
-						printk("klips_debug:tdb_init: illegal weak des key\n");
-					if (error) {
-						memset(tdbp->tdb_key_e, 0, sizeof (struct des_eks));
-						kfree_s(tdbp->tdb_key_e, sizeof(struct des_eks));
-						SENDERR(EINVAL);
-					}
-					break;
-#endif /* CONFIG_IPSEC_ENC_DES */
 #ifdef CONFIG_IPSEC_ENC_3DES
 				case XF_ESP3DES:
 				case XF_ESP3DESMD596:
@@ -655,8 +639,8 @@ tdb_init(struct tdb *tdbp, struct encap_msghdr *em)
 				
 					if (ed->eme_klen != EMT_ESP3DES_KEY_SZ) {
 						KLIPS_PRINT(debug_esp,
-							    "klips_debug:tdb_init: incorrect encryption "
-							    "key size: %d -- must be %d octets (bytes)\n",
+							    "klips_debug:tdb_init: "
+							    "incorrect encryption key size: %d -- must be %d octets (bytes)\n",
 							    ed->eme_klen, EMT_ESP3DES_KEY_SZ);
 						SENDERR(EINVAL);
 					}
@@ -672,7 +656,8 @@ tdb_init(struct tdb *tdbp, struct encap_msghdr *em)
 					for(i = 0; i < 3; i++) {
 #if 0
 						KLIPS_PRINT(debug_esp,
-							    "klips_debug:tdb_init: 3des key %d/3 is 0x%08lx%08lx\n",
+							    "klips_debug:tdb_init: "
+							    "3des key %d/3 is 0x%08lx%08lx\n",
 							    i + 1,
 							    ntohl(*((__u32 *)ed->eme_key + i * 2)),
 							    ntohl(*((__u32 *)ed->eme_key + i * 2 + 1)));
@@ -680,12 +665,14 @@ tdb_init(struct tdb *tdbp, struct encap_msghdr *em)
 						error = des_set_key((caddr_t)(ed->eme_key) + EMT_ESPDES_KEY_SZ * i,
 								    (caddr_t)&((struct des_eks*)(tdbp->tdb_key_e))[i]);
 						if (error == -1)
-							printk("klips_debug:tdb_init: parity error in des key %d/3\n", i + 1);
+							printk("klips_debug:tdb_init: "
+							       "parity error in des key %d/3\n", i + 1);
 						else if (error == -2)
-							printk("klips_debug:tdb_init: illegal weak des key %d/3\n", i + 1);
+							printk("klips_debug:tdb_init: "
+							       "illegal weak des key %d/3\n", i + 1);
 						if (error) {
 							memset(tdbp->tdb_key_e, 0, 3 * sizeof(struct des_eks));
-							kfree_s(tdbp->tdb_key_e, 3 * sizeof(struct des_eks));
+							kfree(tdbp->tdb_key_e);
 							SENDERR(EINVAL);
 						}
 					}
@@ -698,7 +685,6 @@ tdb_init(struct tdb *tdbp, struct encap_msghdr *em)
 
 				switch(alg) {
 #ifdef CONFIG_IPSEC_AUTH_HMAC_MD5
-				case XF_ESPDESMD596:
 				case XF_ESP3DESMD596:
 				case XF_ESPNULLMD596:
 				{
@@ -709,8 +695,8 @@ tdb_init(struct tdb *tdbp, struct encap_msghdr *em)
 				
 					if (ed->ame_klen != AHMD596_KLEN) {
 						KLIPS_PRINT(debug_esp,
-							    "klips_debug:tdb_init: incorrect authorisation "
-							    " key size: %d -- must be %d octets (bytes)\n",
+							    "klips_debug:tdb_init: "
+							    "incorrect authorisation  key size: %d -- must be %d octets (bytes)\n",
 							    ed->ame_klen, AHMD596_KLEN);
 						SENDERR(EINVAL);
 					}
@@ -725,7 +711,8 @@ tdb_init(struct tdb *tdbp, struct encap_msghdr *em)
 						SENDERR(ENOMEM);
 					}
 					KLIPS_PRINT(debug_esp,
-						    "klips_debug:tdb_init: hmac md5-96 key is 0x%08x %08x %08x %08x\n",
+						    "klips_debug:tdb_init: "
+						    "hmac md5-96 key is 0x%08x %08x %08x %08x\n",
 						    (__u32)ntohl(*(((__u32 *)ed->ame_key)+0)),
 						    (__u32)ntohl(*(((__u32 *)ed->ame_key)+1)),
 						    (__u32)ntohl(*(((__u32 *)ed->ame_key)+2)),
@@ -754,8 +741,8 @@ tdb_init(struct tdb *tdbp, struct encap_msghdr *em)
 					MD5Update(octx, kb, AHMD596_BLKLEN);
 					
 					KLIPS_PRINT(debug_esp,
-						    "klips_debug:tdb_init: MD5 ictx=0x%08x %08x %08x %08x"
-						    " octx=0x%08x %08x %08x %08x\n",
+						    "klips_debug:tdb_init: "
+						    "MD5 ictx=0x%08x %08x %08x %08x octx=0x%08x %08x %08x %08x\n",
 						    ((__u32*)ictx)[0],
 						    ((__u32*)ictx)[1],
 						    ((__u32*)ictx)[2],
@@ -773,7 +760,6 @@ tdb_init(struct tdb *tdbp, struct encap_msghdr *em)
 #endif /* CONFIG_IPSEC_AUTH_HMAC_MD5 */
 #ifdef CONFIG_IPSEC_AUTH_HMAC_SHA1
 				case XF_ESPNULLSHA196:
-				case XF_ESPDESSHA196:
 				case XF_ESP3DESSHA196:
 				{
 					SHA1_CTX *ictx;
@@ -783,8 +769,8 @@ tdb_init(struct tdb *tdbp, struct encap_msghdr *em)
 				
 					if (ed->ame_klen != AHSHA196_KLEN) {
 						KLIPS_PRINT(debug_esp,
-							    "klips_debug:tdb_init: incorrect authorisation "
-							    " key size: %d -- must be %d octets (bytes)\n",
+							    "klips_debug:tdb_init: "
+							    "incorrect authorisation key size: %d -- must be %d octets (bytes)\n",
 							    ed->ame_klen, AHSHA196_KLEN);
 						SENDERR(EINVAL);
 					}
@@ -798,7 +784,8 @@ tdb_init(struct tdb *tdbp, struct encap_msghdr *em)
 						SENDERR(ENOMEM);
 					}
 					KLIPS_PRINT(debug_esp,
-						    "klips_debug:tdb_init: hmac sha1-96 key is 0x%08x %08x %08x %08x\n",
+						    "klips_debug:tdb_init: "
+						    "hmac sha1-96 key is 0x%08x %08x %08x %08x\n",
 						    (__u32)ntohl(*(((__u32 *)ed->ame_key)+0)),
 						    (__u32)ntohl(*(((__u32 *)ed->ame_key)+1)),
 						    (__u32)ntohl(*(((__u32 *)ed->ame_key)+2)),
@@ -825,8 +812,8 @@ tdb_init(struct tdb *tdbp, struct encap_msghdr *em)
 					SHA1Update(octx, kb, AHSHA196_BLKLEN);
 				
 					KLIPS_PRINT(debug_esp,
-						    "klips_debug:tdb_init: SHA1 ictx=0x%08x %08x %08x %08x"
-						    " octx=0x%08x %08x %08x %08x\n",
+						    "klips_debug:tdb_init: "
+						    "SHA1 ictx=0x%08x %08x %08x %08x octx=0x%08x %08x %08x %08x\n",
 						    ((__u32*)ictx)[0],
 						    ((__u32*)ictx)[1],
 						    ((__u32*)ictx)[2],
@@ -842,7 +829,6 @@ tdb_init(struct tdb *tdbp, struct encap_msghdr *em)
 					break;
 				}
 #endif /* CONFIG_IPSEC_AUTH_HMAC_SHA1 */
-				case XF_ESPDES:
 				case XF_ESP3DES:
 					tdbp->tdb_authalg = AH_NONE;
 					break;
@@ -853,19 +839,24 @@ tdb_init(struct tdb *tdbp, struct encap_msghdr *em)
 #endif /* !CONFIG_IPSEC_ESP */
 			default:
 				KLIPS_PRINT(debug_xform,
-					    "klips_debug:tdb_init: alg=%d not configured\n", alg);
+					    "klips_debug:tdb_init: "
+					    "alg=%d not configured\n",
+					    alg);
 				SENDERR(ESOCKTNOSUPPORT);
 			}
 			SENDERR(0);
 		}
 	}
 	KLIPS_PRINT(debug_xform & DB_XF_INIT,
-		    "klips_debug:tdb_init: unregistered algorithm %d requested\n"
-		    "klips_debug:   trying to setup SA:%s\n", alg, sa);
+		    "klips_debug:tdb_init: "
+		    "unregistered algorithm %d requested trying to setup SA:%s\n",
+		    alg,
+		    sa_len ? sa : " (error)");
 	SENDERR(EINVAL);
 errlab:
-	return len;
+	return error;
 }
+#endif
 
 int 
 ipsec_tdbcleanup(__u8 proto)
@@ -874,67 +865,80 @@ ipsec_tdbcleanup(__u8 proto)
 	int error = 0;
 	struct tdb *tdbp, **tdbprev, *tdbdel;
         char sa[SATOA_BUF];
-	unsigned long flags; /* save irq state for spinlock */
+	size_t sa_len;
 
 	KLIPS_PRINT(debug_xform,
-		    "klips_debug:ipsec_tdbcleanup: cleaning up proto=%d.\n", proto);
+		    "klips_debug:ipsec_tdbcleanup: "
+		    "cleaning up proto=%d.\n",
+		    proto);
 
-	spin_lock_irqsave(&tdb_lock, flags);
+	spin_lock_bh(&tdb_lock);
 
 	for (i = 0; i < TDB_HASHMOD; i++) {
 		tdbprev = &(tdbh[i]);
 		tdbp = tdbh[i];
 		for(; tdbp;) {
-			satoa(tdbp->tdb_said, 0, sa, SATOA_BUF);
+			sa_len = satoa(tdbp->tdb_said, 0, sa, SATOA_BUF);
 			KLIPS_PRINT(debug_xform,
-				    "klips_debug:ipsec_tdbcleanup: checking SA:%s, hash=%d",
-				    sa, i);
+				    "klips_debug:ipsec_tdbcleanup: "
+				    "checking SA:%s, hash=%d",
+				    sa_len ? sa : " (error)",
+				    i);
 			tdbdel = tdbp;
 			tdbp = tdbdel->tdb_hnext;
 			if(tdbp) {
-				satoa(tdbp->tdb_said, 0, sa, SATOA_BUF);
+				sa_len = satoa(tdbp->tdb_said, 0, sa, SATOA_BUF);
 				KLIPS_PRINT(debug_xform,
-					    ", hnext=%s", sa);
+					    ", hnext=%s",
+					    sa_len ? sa : " (error)");
 			}
 			if(*tdbprev) {
-				satoa((*tdbprev)->tdb_said, 0, sa, SATOA_BUF);
+				sa_len = satoa((*tdbprev)->tdb_said, 0, sa, SATOA_BUF);
 				KLIPS_PRINT(debug_xform,
-					    ", *tdbprev=%s", sa);
+					    ", *tdbprev=%s",
+					    sa_len ? sa : " (error)");
 				if((*tdbprev)->tdb_hnext) {
-					satoa((*tdbprev)->tdb_hnext->tdb_said, 0, sa, SATOA_BUF);
+					sa_len = satoa((*tdbprev)->tdb_hnext->tdb_said, 0, sa, SATOA_BUF);
 					KLIPS_PRINT(debug_xform,
-						    ", *tdbprev->tdb_hnext=%s", sa);
+						    ", *tdbprev->tdb_hnext=%s",
+						    sa_len ? sa : " (error)");
 				}
 			}
 			KLIPS_PRINT(debug_xform,
 				    ".\n");
-			if(!proto || (proto == tdbdel->tdb_proto)) {
-				satoa(tdbdel->tdb_said, 0, sa, SATOA_BUF);
+			if(!proto || (proto == tdbdel->tdb_said.proto)) {
+				sa_len = satoa(tdbdel->tdb_said, 0, sa, SATOA_BUF);
 				KLIPS_PRINT(debug_xform,
-					    "klips_debug:ipsec_tdbcleanup: deleting SA chain:%s.\n",
-					    sa);
-				/* *tdbprev = tdbdel->tdb_hnext; */
+					    "klips_debug:ipsec_tdbcleanup: "
+					    "deleting SA chain:%s.\n",
+					    sa_len ? sa : " (error)");
 				if((error = deltdbchain(tdbdel))) {
-					goto errlab;
+					SENDERR(-error);
 				}
 				tdbprev = &(tdbh[i]);
 				tdbp = tdbh[i];
 
 				KLIPS_PRINT(debug_xform,
-					    "klips_debug:ipsec_tdbcleanup: deleted SA chain:%s", sa);
+					    "klips_debug:ipsec_tdbcleanup: "
+					    "deleted SA chain:%s",
+					    sa_len ? sa : " (error)");
 				if(tdbp) {
-					satoa(tdbp->tdb_said, 0, sa, SATOA_BUF);
+					sa_len = satoa(tdbp->tdb_said, 0, sa, SATOA_BUF);
 					KLIPS_PRINT(debug_xform,
-						    ", tdbh[%d]=%s", i, sa);
+						    ", tdbh[%d]=%s",
+						    i,
+						    sa_len ? sa : " (error)");
 				}
 				if(*tdbprev) {
-					satoa((*tdbprev)->tdb_said, 0, sa, SATOA_BUF);
+					sa_len = satoa((*tdbprev)->tdb_said, 0, sa, SATOA_BUF);
 					KLIPS_PRINT(debug_xform,
-						    ", *tdbprev=%s", sa);
+						    ", *tdbprev=%s",
+						    sa_len ? sa : " (error)");
 					if((*tdbprev)->tdb_hnext) {
-						satoa((*tdbprev)->tdb_hnext->tdb_said, 0, sa, SATOA_BUF);
+						sa_len = satoa((*tdbprev)->tdb_hnext->tdb_said, 0, sa, SATOA_BUF);
 						KLIPS_PRINT(debug_xform,
-							    ", *tdbprev->tdb_hnext=%s", sa);
+							    ", *tdbprev->tdb_hnext=%s",
+							    sa_len ? sa : " (error)");
 					}
 				}
 				KLIPS_PRINT(debug_xform,
@@ -946,63 +950,147 @@ ipsec_tdbcleanup(__u8 proto)
 	}
  errlab:
 
-	spin_unlock_irqrestore(&tdb_lock, flags);
+	spin_unlock_bh(&tdb_lock);
 
-	return(-error);
+	return(error);
 }
 
 int
 ipsec_tdbwipe(struct tdb *tdbp)
 {
 	if(!tdbp) {
-		return -1;
+		return -ENODATA;
 	}
 
 	if(tdbp->tdb_addr_s) {
 		memset((caddr_t)(tdbp->tdb_addr_s), 0, tdbp->tdb_addr_s_size);
-		kfree_s(tdbp->tdb_addr_s, tdbp->tdb_addr_s_size);
+		kfree(tdbp->tdb_addr_s);
 	}
 	tdbp->tdb_addr_s = NULL;
 
 	if(tdbp->tdb_addr_d) {
 		memset((caddr_t)(tdbp->tdb_addr_d), 0, tdbp->tdb_addr_d_size);
-		kfree_s(tdbp->tdb_addr_d, tdbp->tdb_addr_d_size);
+		kfree(tdbp->tdb_addr_d);
 	}
 	tdbp->tdb_addr_d = NULL;
 
 	if(tdbp->tdb_addr_p) {
 		memset((caddr_t)(tdbp->tdb_addr_p), 0, tdbp->tdb_addr_p_size);
-		kfree_s(tdbp->tdb_addr_p, tdbp->tdb_addr_p_size);
+		kfree(tdbp->tdb_addr_p);
 	}
 	tdbp->tdb_addr_p = NULL;
 
 	if(tdbp->tdb_key_a) {
 		memset((caddr_t)(tdbp->tdb_key_a), 0, tdbp->tdb_key_a_size);
-		kfree_s(tdbp->tdb_key_a, tdbp->tdb_key_a_size);
+		kfree(tdbp->tdb_key_a);
 	}
 	tdbp->tdb_key_a = NULL;
 
 	if(tdbp->tdb_key_e) {
 		memset((caddr_t)(tdbp->tdb_key_e), 0, tdbp->tdb_key_e_size);
-		kfree_s(tdbp->tdb_key_e, tdbp->tdb_key_e_size);
+		kfree(tdbp->tdb_key_e);
 	}
 	tdbp->tdb_key_e = NULL;
 
 	if(tdbp->tdb_iv) {
 		memset((caddr_t)(tdbp->tdb_iv), 0, tdbp->tdb_iv_size);
-		kfree_s(tdbp->tdb_iv, tdbp->tdb_iv_size);
+		kfree(tdbp->tdb_iv);
 	}
 	tdbp->tdb_iv = NULL;
 
+	if(tdbp->tdb_ident_s.data) {
+		memset((caddr_t)(tdbp->tdb_ident_s.data),
+                       0,
+		       tdbp->tdb_ident_s.len * IPSEC_PFKEYv2_ALIGN - sizeof(struct sadb_ident));
+		kfree(tdbp->tdb_ident_s.data);
+        }
+	tdbp->tdb_ident_s.data = NULL;
+	
+	if(tdbp->tdb_ident_d.data) {
+		memset((caddr_t)(tdbp->tdb_ident_d.data),
+                       0,
+		       tdbp->tdb_ident_d.len * IPSEC_PFKEYv2_ALIGN - sizeof(struct sadb_ident));
+		kfree(tdbp->tdb_ident_d.data);
+        }
+	tdbp->tdb_ident_d.data = NULL;
+	
 	memset((caddr_t)tdbp, 0, sizeof(*tdbp));
-	kfree_s(tdbp, sizeof(*tdbp));
+	kfree(tdbp);
 	tdbp = NULL;
 
 	return 0;
 }
+#endif
 
 /*
  * $Log: ipsec_xform.c,v $
+ * Revision 1.58  2002/01/29 17:17:57  mcr
+ * 	moved include of ipsec_param.h to after include of linux/kernel.h
+ * 	otherwise, it seems that some option that is set in ipsec_param.h
+ * 	screws up something subtle in the include path to kernel.h, and
+ * 	it complains on the snprintf() prototype.
+ *
+ * Revision 1.57  2002/01/29 04:00:53  mcr
+ * 	more excise of kversions.h header.
+ *
+ * Revision 1.56  2001/11/27 05:17:22  mcr
+ * 	turn off the worst of the per-packet debugging.
+ *
+ * Revision 1.55  2001/11/26 09:23:50  rgb
+ * Merge MCR's ipsec_sa, eroute, proc and struct lifetime changes.
+ *
+ * Revision 1.54  2001/10/18 04:45:21  rgb
+ * 2.4.9 kernel deprecates linux/malloc.h in favour of linux/slab.h,
+ * lib/freeswan.h version macros moved to lib/kversions.h.
+ * Other compiler directive cleanups.
+ *
+ * Revision 1.53  2001/09/08 21:13:34  rgb
+ * Added pfkey ident extension support for ISAKMPd. (NetCelo)
+ *
+ * Revision 1.52  2001/06/14 19:35:11  rgb
+ * Update copyright date.
+ *
+ * Revision 1.51  2001/05/30 08:14:03  rgb
+ * Removed vestiges of esp-null transforms.
+ *
+ * Revision 1.50  2001/05/03 19:43:18  rgb
+ * Initialise error return variable.
+ * Update SENDERR macro.
+ * Fix sign of error return code for ipsec_tdbcleanup().
+ * Use more appropriate return code for ipsec_tdbwipe().
+ *
+ * Revision 1.49  2001/04/19 18:56:17  rgb
+ * Fixed tdb table locking comments.
+ *
+ * Revision 1.48  2001/02/27 22:24:55  rgb
+ * Re-formatting debug output (line-splitting, joining, 1arg/line).
+ * Check for satoa() return codes.
+ *
+ * Revision 1.47  2000/11/06 04:32:08  rgb
+ * Ditched spin_lock_irqsave in favour of spin_lock_bh.
+ *
+ * Revision 1.46  2000/09/20 16:21:57  rgb
+ * Cleaned up ident string alloc/free.
+ *
+ * Revision 1.45  2000/09/08 19:16:51  rgb
+ * Change references from DEBUG_IPSEC to CONFIG_IPSEC_DEBUG.
+ * Removed all references to CONFIG_IPSEC_PFKEYv2.
+ *
+ * Revision 1.44  2000/08/30 05:29:04  rgb
+ * Compiler-define out no longer used tdb_init() in ipsec_xform.c.
+ *
+ * Revision 1.43  2000/08/18 21:30:41  rgb
+ * Purged all tdb_spi, tdb_proto and tdb_dst macros.  They are unclear.
+ *
+ * Revision 1.42  2000/08/01 14:51:51  rgb
+ * Removed _all_ remaining traces of DES.
+ *
+ * Revision 1.41  2000/07/28 14:58:31  rgb
+ * Changed kfree_s to kfree, eliminating extra arg to fix 2.4.0-test5.
+ *
+ * Revision 1.40  2000/06/28 05:50:11  rgb
+ * Actually set iv_bits.
+ *
  * Revision 1.39  2000/05/10 23:11:09  rgb
  * Added netlink debugging output.
  * Added a cast to quiet down the ntohl bug.
