@@ -1,4 +1,4 @@
-char radij_c_version[] = "RCSID $Id: radij.c,v 1.26 2000/05/10 23:11:37 rgb Exp $";
+char radij_c_version[] = "RCSID $Id: radij.c,v 1.40 2002/01/29 17:17:58 mcr Exp $";
 
 /*
  * This file is defived from ${SRC}/sys/net/radix.c of BSD 4.4lite
@@ -57,9 +57,15 @@ char radij_c_version[] = "RCSID $Id: radij.c,v 1.26 2000/05/10 23:11:37 rgb Exp
 
 #include <linux/config.h>
 #include <linux/version.h>
-
 #include <linux/kernel.h> /* printk() */
-#include <linux/malloc.h> /* kmalloc() */
+
+#include "ipsec_param.h"
+
+#ifdef MALLOC_SLAB
+# include <linux/slab.h> /* kmalloc() */
+#else /* MALLOC_SLAB */
+# include <linux/malloc.h> /* kmalloc() */
+#endif /* MALLOC_SLAB */
 #include <linux/errno.h>  /* error codes */
 #include <linux/types.h>  /* size_t */
 #include <linux/interrupt.h> /* mark_bh */
@@ -68,14 +74,15 @@ char radij_c_version[] = "RCSID $Id: radij.c,v 1.26 2000/05/10 23:11:37 rgb Exp
 #include <linux/etherdevice.h> /* eth_type_trans */
 #include <linux/ip.h>          /* struct iphdr */
 #include <linux/skbuff.h>
-#include <freeswan.h>
 #ifdef NET_21
-#include <asm/uaccess.h>
-#include <linux/in6.h>
+# include <asm/uaccess.h>
+# include <linux/in6.h>
 #endif /* NET_21 */
 #include <asm/checksum.h>
 #include <net/ip.h>
 
+#include <freeswan.h>
+
 #include "radij.h"
 #include "ipsec_encap.h"
 #include "ipsec_radij.h"
@@ -90,7 +97,7 @@ static char *rj_zeroes, *rj_ones;
 
 #define rj_masktop (mask_rjhead->rnh_treetop)
 #ifdef Bcmp
-#undef Bcmp
+# undef Bcmp
 #endif /* Bcmp */
 #define Bcmp(a, b, l) (l == 0 ? 0 : memcmp((caddr_t)(b), (caddr_t)(a), (size_t)l))
 /*
@@ -213,7 +220,8 @@ rj_match(v_arg, head)
 	 * See if we match exactly as a host destination
 	 */
 	KLIPS_PRINT(debug_radij,
-		    "klips_debug:rj_match: * See if we match exactly as a host destination\n");
+		    "klips_debug:rj_match: "
+		    "* See if we match exactly as a host destination\n");
 	
 	cp += off; cp2 = t->rj_key + off; cplim = v + vlen;
 	for (; cp < cplim; cp++, cp2++)
@@ -230,7 +238,8 @@ on1:
 	matched_off = cp - v;
 	saved_t = t;
 	KLIPS_PRINT(debug_radij,
-		    "klips_debug:rj_match: ** try to match a leaf, t=0x%p\n", t);
+		    "klips_debug:rj_match: "
+		    "** try to match a leaf, t=0x%p\n", t);
 	do {
 	    if (t->rj_mask) {
 		/*
@@ -251,13 +260,17 @@ on1:
 	t = saved_t;
 	/* start searching up the tree */
 	KLIPS_PRINT(debug_radij,
-		    "klips_debug:rj_match: *** start searching up the tree, t=0x%p\n", t);
+		    "klips_debug:rj_match: "
+		    "*** start searching up the tree, t=0x%p\n",
+		    t);
 	do {
 		register struct radij_mask *m;
 		
 		t = t->rj_p;
 		KLIPS_PRINT(debug_radij,
-			    "klips_debug:rj_match: **** t=0x%p\n", t);
+			    "klips_debug:rj_match: "
+			    "**** t=0x%p\n",
+			    t);
 		if ((m = t->rj_mklist)) {
 			/*
 			 * After doing measurements here, it may
@@ -265,13 +278,18 @@ on1:
 			 * rj_search_m here instead of always
 			 * copying and masking.
 			 */
-			off = min(t->rj_off, matched_off);
+			/* off = min(t->rj_off, matched_off); */
+			off = t->rj_off;
+			if (matched_off < off)
+				off = matched_off;
 			mstart = maskedKey + off;
 			do {
 				cp2 = mstart;
 				cp3 = m->rm_mask + off;
 				KLIPS_PRINT(debug_radij,
-					    "klips_debug:rj_match: ***** cp2=0x%p cp3=0x%p\n", cp2, cp3);
+					    "klips_debug:rj_match: "
+					    "***** cp2=0x%p cp3=0x%p\n",
+					    cp2, cp3);
 				for (cp = v + off; cp < cplim;)
 					*cp2++ =  *cp++ & *cp3++;
 				x = rj_search(maskedKey, t);
@@ -285,7 +303,8 @@ on1:
 		}
 	} while (t != top);
 	KLIPS_PRINT(debug_radij,
-		    "klips_debug:rj_match: ***** not found.\n");
+		    "klips_debug:rj_match: "
+		    "***** not found.\n");
 	return 0;
 };
 		
@@ -293,7 +312,7 @@ on1:
 int	rj_nodenum;
 struct	radij_node *rj_clist;
 int	rj_saveinfo;
-static void traverse(struct radij_node *);
+DEBUG_NO_STATIC void traverse(struct radij_node *);
 #ifdef RJ_DEBUG2
 int	rj_debug =  1;
 #else
@@ -363,7 +382,7 @@ on1:
 	} while (b > (unsigned) x->rj_b); /* x->rj_b < b && x->rj_b >= 0 */
 #ifdef RJ_DEBUG
 	if (rj_debug)
-		printk("klips_debug:Going In:\n"), traverse(p);
+		printk("klips_debug:rj_insert: Going In:\n"), traverse(p);
 #endif /* RJ_DEBUG */
 	t = rj_newpair(v_arg, b, nodes); tt = t->rj_l;
 	if ((cp[p->rj_off] & p->rj_bmask) == 0)
@@ -378,7 +397,7 @@ on1:
 	}
 #ifdef RJ_DEBUG
 	if (rj_debug)
-		printk("klips_debug:Coming out:\n"), traverse(p);
+		printk("klips_debug:rj_insert: Coming out:\n"), traverse(p);
 #endif /* RJ_DEBUG */
     }
 	return (tt);
@@ -460,7 +479,7 @@ rj_addroute(v_arg, n_arg, head, treenodes)
 		if (Bcmp(netmask, x->rj_key, mlen) != 0) {
 			x = rj_addmask(netmask, 0, top->rj_off);
 			if (x == 0)
-				return /* (0) rgb */ ENOMEM;
+				return -ENOMEM; /* (0) rgb */
 		}
 		netmask = x->rj_key;
 		b = -1 - x->rj_b;
@@ -472,7 +491,7 @@ rj_addroute(v_arg, n_arg, head, treenodes)
 	if (keyduplicated) {
 		do {
 			if (tt->rj_mask == netmask)
-				return /* (0) rgb */ ENXIO;
+				return -EEXIST; /* -ENXIO; (0) rgb */
 			t = tt;
 			if (netmask == 0 ||
 			    (tt->rj_mask && rj_refines(netmask, tt->rj_mask)))
@@ -541,7 +560,7 @@ rj_addroute(v_arg, n_arg, head, treenodes)
 	}
 	/* Add new route to highest possible ancestor's list */
 	if ((netmask == 0) || (b > t->rj_b ))
-		return /* tt rgb */ 0; /* can't lift at all */
+		return 0; /* tt rgb */ /* can't lift at all */
 	b_leaf = tt->rj_b;
 	do {
 		x = t;
@@ -562,15 +581,16 @@ rj_addroute(v_arg, n_arg, head, treenodes)
 		if (m->rm_mask == netmask) {
 			m->rm_refs++;
 			tt->rj_mklist = m;
-			return /* tt rgb */ 0;
+			return 0; /* tt rgb */
 		}
 		if (rj_refines(netmask, m->rm_mask))
 			break;
 	}
 	MKGet(m);
 	if (m == 0) {
-		printk("klips_debug:Mask for route not entered\n");
-		return /* (tt) rgb */ 0;
+		printk("klips_debug:rj_addroute: "
+		       "Mask for route not entered\n");
+		return 0; /* (tt) rgb */
 	}
 	Bzero(m, sizeof *m);
 	m->rm_b = b_leaf;
@@ -578,7 +598,7 @@ rj_addroute(v_arg, n_arg, head, treenodes)
 	m->rm_mklist = *mp;
 	*mp = m;
 	tt->rj_mklist = m;
-	return /* tt rgb */ 0;
+	return 0; /* tt rgb */
 }
 
 int
@@ -603,7 +623,7 @@ rj_delete(v_arg, netmask_arg, head, node)
 	top = x;
 	if (tt == 0 ||
 	    Bcmp(v + head_off, tt->rj_key + head_off, vlen - head_off))
-		return /* (0) rgb */ -EFAULT;
+		return -EFAULT; /* (0) rgb */
 	/*
 	 * Delete our route from mask lists.
 	 */
@@ -612,12 +632,13 @@ rj_delete(v_arg, netmask_arg, head, node)
 			netmask = rj_search(netmask, rj_masktop)->rj_key;
 		while (tt->rj_mask != netmask)
 			if ((tt = tt->rj_dupedkey) == 0)
-				return /* (0) rgb */ -ENXIO;
+				return -ENOENT; /* -ENXIO; (0) rgb */
 	}
 	if (tt->rj_mask == 0 || (saved_m = m = tt->rj_mklist) == 0)
 		goto on1;
 	if (m->rm_mask != tt->rj_mask) {
-		printk("klips_debug:rj_delete: inconsistent annotation\n");
+		printk("klips_debug:rj_delete: "
+		       "inconsistent annotation\n");
 		goto on1;
 	}
 	if (--m->rm_refs >= 0)
@@ -637,13 +658,14 @@ rj_delete(v_arg, netmask_arg, head, node)
 			break;
 		}
 	if (m == 0)
-		printk("klips_debug:rj_delete: couldn't find our annotation\n");
+		printk("klips_debug:rj_delete: "
+		       "couldn't find our annotation\n");
 on1:
 	/*
 	 * Eliminate us from tree
 	 */
 	if (tt->rj_flags & RJF_ROOT)
-		return /* (0) rgb */ -EFAULT;
+		return -EFAULT; /* (0) rgb */
 #ifdef RJ_DEBUG
 	/* Get us out of the creation list */
 	for (t = rj_clist; t && t->rj_ybro != tt; t = t->rj_ybro) {}
@@ -658,7 +680,8 @@ on1:
 			for (x = p = saved_tt; p && p->rj_dupedkey != tt;)
 				p = p->rj_dupedkey;
 			if (p) p->rj_dupedkey = tt->rj_dupedkey;
-			else printk("klips_debug:rj_delete: couldn't find us\n");
+			else printk("klips_debug:rj_delete: "
+				       "couldn't find us\n");
 		}
 		t = tt + 1;
 		if  (t->rj_flags & RJF_ACTIVE) {
@@ -691,8 +714,8 @@ on1:
 					x->rj_mklist = 0;
 					MKFree(m);
 				} else 
-					printk("klips_debug:%s %p at %p\n",
-					    "rj_delete: Orphaned Mask", m, x);
+					printk("klips_debug:rj_delete: "
+					    "Orphaned Mask %p at %p\n", m, x);
 				m = mm;
 			}
 		}
@@ -715,7 +738,7 @@ out:
 	tt->rj_flags &= ~RJF_ACTIVE;
 	tt[1].rj_flags &= ~RJF_ACTIVE;
 	*node = tt;
-	return /* (tt) rgb */ 0;
+	return 0; /* (tt) rgb */
 }
 
 int
@@ -729,7 +752,7 @@ rj_walktree(h, f, w)
 	register struct radij_node *rn;
 
 	if(!h || !f /* || !w */) {
-		return -1;
+		return -ENODATA;
 	}
 
 	rn = h->rnh_treetop;
@@ -742,17 +765,22 @@ rj_walktree(h, f, w)
 	while (rn->rj_b >= 0)
 		rn = rn->rj_l;
 	for (;;) {
-#ifdef DEBUG_IPSEC
+#ifdef CONFIG_IPSEC_DEBUG
 		if(debug_radij) {
-			printk("klips_debug:RN_WALKTREE: for: rn=%p rj_b=%d rj_flags=%x", rn, rn->rj_b, rn->rj_flags);
+			printk("klips_debug:rj_walktree: "
+			       "for: rn=%p rj_b=%d rj_flags=%x",
+			       rn,
+			       rn->rj_b,
+			       rn->rj_flags);
 			rn->rj_b >= 0 ?
-				printk(" node off=%x\n", rn->rj_off) :
+				printk(" node off=%x\n",
+				       rn->rj_off) :
 				printk(" leaf key = %08x->%08x\n",
 				       (u_int)ntohl(((struct sockaddr_encap *)rn->rj_key)->sen_ip_src.s_addr),
 				       (u_int)ntohl(((struct sockaddr_encap *)rn->rj_key)->sen_ip_dst.s_addr))
 				;
 		}
-#endif /* DEBUG_IPSEC */
+#endif /* CONFIG_IPSEC_DEBUG */
 		base = rn;
 		/* If at right child go back up, otherwise, go right */
 		while (rn->rj_p->rj_r == rn && (rn->rj_flags & RJF_ROOT) == 0)
@@ -761,32 +789,42 @@ rj_walktree(h, f, w)
 		for (rn = rn->rj_p->rj_r; rn->rj_b >= 0;)
 			rn = rn->rj_l;
 		next = rn;
-#ifdef DEBUG_IPSEC
+#ifdef CONFIG_IPSEC_DEBUG
 		if(debug_radij) {
-			printk("klips_debug:RN_WALKTREE: processing leaves, rn=%p rj_b=%d rj_flags=%x", rn, rn->rj_b, rn->rj_flags);
+			printk("klips_debug:rj_walktree: "
+			       "processing leaves, rn=%p rj_b=%d rj_flags=%x",
+			       rn,
+			       rn->rj_b,
+			       rn->rj_flags);
 			rn->rj_b >= 0 ?
-				printk(" node off=%x\n", rn->rj_off) :
+				printk(" node off=%x\n",
+				       rn->rj_off) :
 				printk(" leaf key = %08x->%08x\n",
 				       (u_int)ntohl(((struct sockaddr_encap *)rn->rj_key)->sen_ip_src.s_addr),
 				       (u_int)ntohl(((struct sockaddr_encap *)rn->rj_key)->sen_ip_dst.s_addr))
 				;
 		}
-#endif /* DEBUG_IPSEC */
+#endif /* CONFIG_IPSEC_DEBUG */
 		/* Process leaves */
 		while ((rn = base)) {
 			base = rn->rj_dupedkey;
-#ifdef DEBUG_IPSEC
+#ifdef CONFIG_IPSEC_DEBUG
 			if(debug_radij) {
-				printk("klips_debug:RN_WALKTREE: while: base=%p rn=%p rj_b=%d rj_flags=%x",
-				       base, rn, rn->rj_b, rn->rj_flags);
+				printk("klips_debug:rj_walktree: "
+				       "while: base=%p rn=%p rj_b=%d rj_flags=%x",
+				       base,
+				       rn,
+				       rn->rj_b,
+				       rn->rj_flags);
 				rn->rj_b >= 0 ?
-					printk(" node off=%x\n", rn->rj_off) :
+					printk(" node off=%x\n",
+					       rn->rj_off) :
 					printk(" leaf key = %08x->%08x\n",
 					       (u_int)ntohl(((struct sockaddr_encap *)rn->rj_key)->sen_ip_src.s_addr),
 					       (u_int)ntohl(((struct sockaddr_encap *)rn->rj_key)->sen_ip_dst.s_addr))
 					;
 			}
-#endif /* DEBUG_IPSEC */
+#endif /* CONFIG_IPSEC_DEBUG */
 			if (!(rn->rj_flags & RJF_ROOT) && (error = (*f)(rn, w)))
 				return (-error);
 		}
@@ -807,7 +845,7 @@ rj_inithead(head, off)
 	if (*head)
 		return (1);
 	R_Malloc(rnh, struct radij_node_head *, sizeof (*rnh));
-	if (rnh == 0)
+	if (rnh == NULL)
 		return (0);
 	Bzero(rnh, sizeof (*rnh));
 	*head = rnh;
@@ -833,11 +871,9 @@ rj_init()
 {
 	char *cp, *cplim;
 
-#if 0
-	KLIPS_PRINT(1, "klips_debug: rj_init: version: %s\n", radij_c_version);
-#endif
 	if (maj_keylen == 0) {
-		printk("klips_debug:rj_init: radij functions require maj_keylen be set\n");
+		printk("klips_debug:rj_init: "
+		       "radij functions require maj_keylen be set\n");
 		return;
 	}
 	R_Malloc(rj_zeroes, char *, 3 * maj_keylen);
@@ -858,7 +894,8 @@ rj_preorder(struct radij_node *rn, int l)
 	int i;
 	
 	if (rn == NULL){
-		printk("klips_debug:rj_preorder: NULL pointer\n");
+		printk("klips_debug:rj_preorder: "
+		       "NULL pointer\n");
 		return;
 	}
 	
@@ -868,34 +905,36 @@ rj_preorder(struct radij_node *rn, int l)
 		printk("klips_debug:");
 		for (i=0; i<l; i++)
 			printk("*");
-		printk(" off = %d\n", rn->rj_off);
+		printk(" off = %d\n",
+		       rn->rj_off);
 	} else {
 		printk("klips_debug:");
 		for (i=0; i<l; i++)
 			printk("@");
-		printk(" flags = %x", (u_int)rn->rj_flags);
+		printk(" flags = %x",
+		       (u_int)rn->rj_flags);
 		if (rn->rj_flags & RJF_ACTIVE) {
-			printk(" @key = %p", rn->rj_key);
-			
+			printk(" @key = %p",
+			       rn->rj_key);
 			printk(" key = %08x->%08x",
 			       (u_int)ntohl(((struct sockaddr_encap *)rn->rj_key)->sen_ip_src.s_addr),
 			       (u_int)ntohl(((struct sockaddr_encap *)rn->rj_key)->sen_ip_dst.s_addr));
-
-			printk(" @mask = %p", rn->rj_mask);
+			printk(" @mask = %p",
+			       rn->rj_mask);
 			if (rn->rj_mask)
 				printk(" mask = %08x->%08x",
 				       (u_int)ntohl(((struct sockaddr_encap *)rn->rj_mask)->sen_ip_src.s_addr),
 				       (u_int)ntohl(((struct sockaddr_encap *)rn->rj_mask)->sen_ip_dst.s_addr));
-		
 			if (rn->rj_dupedkey)
-				printk(" dupedkey = %08x", (u_int)rn->rj_dupedkey);
+				printk(" dupedkey = %08x",
+				       (u_int)rn->rj_dupedkey);
 		}
 		printk("\n");
 	}
 }
 
 #ifdef RJ_DEBUG
-static void traverse(struct radij_node *p)
+DEBUG_NO_STATIC void traverse(struct radij_node *p)
 {
   rj_preorder(p, 0);
 }
@@ -917,7 +956,7 @@ rj_free_mkfreelist(void)
 	{
 		mknp2 = mknp;
 		mknp = mknp->rm_mklist;
-		kfree_s(mknp2, sizeof(struct radij_mask));
+		kfree(mknp2);
 	}
 }
 
@@ -932,25 +971,81 @@ radijcleanup(void)
 {
 	int error = 0;
 
+	error = radijcleartree();
+
 	rj_free_mkfreelist();
 
 /*	rj_walktree(mask_rjhead, ipsec_rj_walker_delete, NULL); */
   	if(mask_rjhead) {
-		kfree_s(mask_rjhead, sizeof(* mask_rjhead));
+		kfree(mask_rjhead);
 	}
 
 	if(rj_zeroes) {
-		kfree_s(rj_zeroes, 3 * sizeof(struct sockaddr_encap));
+		kfree(rj_zeroes);
 	}
 
-	error = radijcleartree();
-	kfree_s(rnh, sizeof(* rnh));
+	if(rnh) {
+		kfree(rnh);
+	}
 
 	return error;
 }
 
 /*
  * $Log: radij.c,v $
+ * Revision 1.40  2002/01/29 17:17:58  mcr
+ * 	moved include of ipsec_param.h to after include of linux/kernel.h
+ * 	otherwise, it seems that some option that is set in ipsec_param.h
+ * 	screws up something subtle in the include path to kernel.h, and
+ * 	it complains on the snprintf() prototype.
+ *
+ * Revision 1.39  2002/01/29 04:00:55  mcr
+ * 	more excise of kversions.h header.
+ *
+ * Revision 1.38  2002/01/29 02:13:19  mcr
+ * 	introduction of ipsec_kversion.h means that include of
+ * 	ipsec_param.h must preceed any decisions about what files to
+ * 	include to deal with differences in kernel source.
+ *
+ * Revision 1.37  2001/10/18 04:45:23  rgb
+ * 2.4.9 kernel deprecates linux/malloc.h in favour of linux/slab.h,
+ * lib/freeswan.h version macros moved to lib/kversions.h.
+ * Other compiler directive cleanups.
+ *
+ * Revision 1.36  2001/08/22 13:43:51  henry
+ * eliminate the single use of min() to avoid problems with Linus changing it
+ *
+ * Revision 1.35  2001/06/15 04:57:29  rgb
+ * Clarified error return codes.
+ * Changed mask add already exists to EEXIST.
+ * Changed mask delete did not exist to ENOENT.
+ *
+ * Revision 1.34  2001/05/03 19:44:26  rgb
+ * Fix sign of error return codes for rj_addroute().
+ *
+ * Revision 1.33  2001/02/27 22:24:56  rgb
+ * Re-formatting debug output (line-splitting, joining, 1arg/line).
+ * Check for satoa() return codes.
+ *
+ * Revision 1.32  2001/02/27 06:23:15  rgb
+ * Debug line splitting.
+ *
+ * Revision 1.31  2000/11/06 04:35:21  rgb
+ * Clear table *before* releasing other items in radijcleanup.
+ *
+ * Revision 1.30  2000/09/20 04:07:40  rgb
+ * Changed static functions to DEBUG_NO_STATIC to reveal function names in
+ * oopsen.
+ *
+ * Revision 1.29  2000/09/12 03:25:02  rgb
+ * Moved radij_c_version printing to ipsec_version_get_info().
+ *
+ * Revision 1.28  2000/09/08 19:12:56  rgb
+ * Change references from DEBUG_IPSEC to CONFIG_IPSEC_DEBUG.
+ *
+ * Revision 1.27  2000/07/28 14:58:32  rgb
+ * Changed kfree_s to kfree, eliminating extra arg to fix 2.4.0-test5.
+ *
  * Revision 1.26  2000/05/10 23:11:37  rgb
  * Comment out most of the startup version information.
  *
