@@ -1,7 +1,7 @@
 /*
  * manipulate eroutes
  * Copyright (C) 1996  John Ioannidis.
- * Copyright (C) 1998, 1999  Richard Guy Briggs.
+ * Copyright (C) 1997, 1998, 1999, 2000, 2001  Richard Guy Briggs.
  * 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -14,7 +14,7 @@
  * for more details.
  */
 
-char eroute_c_version[] = "RCSID $Id: eroute.c,v 1.32 2000/06/21 16:51:27 rgb Exp $";
+char eroute_c_version[] = "RCSID $Id: eroute.c,v 1.48 2002/02/15 19:54:11 rgb Exp $";
 
 
 #include <sys/types.h>
@@ -33,16 +33,9 @@ char eroute_c_version[] = "RCSID $Id: eroute.c,v 1.32 2000/06/21 16:51:27 rgb Ex
 
 #include <unistd.h>
 #include <freeswan.h>
-#include <linux/autoconf.h>	/* CONFIG_IPSEC_PFKEYv2 */
-/* permanently turn it on since netlink support has been disabled */
-#ifndef CONFIG_IPSEC_PFKEYv2
-     #define CONFIG_IPSEC_PFKEYv2
-#endif /* CONFIG_IPSEC_PFKEYv2 */
-#ifdef CONFIG_IPSEC_PFKEYv2
-     #include <signal.h>
-     #include <pfkeyv2.h>
-     #include <pfkey.h>
-#endif /* CONFIG_IPSEC_PFKEYv2 */
+#include <signal.h>
+#include <pfkeyv2.h>
+#include <pfkey.h>
 #include "radij.h"
 #include "ipsec_encap.h"
 #include "ipsec_netlink.h"
@@ -54,66 +47,42 @@ char eroute_c_version[] = "RCSID $Id: eroute.c,v 1.32 2000/06/21 16:51:27 rgb Ex
 char *program_name;
 extern char *optarg;
 extern int optind, opterr, optopt;
-char *edst_opt, *spi_opt, *proto_opt, *said_opt, *dst_opt, *src_opt;
-		
-#ifdef CONFIG_IPSEC_PFKEYv2
+char *eroute_af_opt, *said_af_opt, *edst_opt, *spi_opt, *proto_opt, *said_opt, *dst_opt, *src_opt;
+int action_type = 0;
+
 extern unsigned int pfkey_lib_debug; /* used by libfreeswan/pfkey_v2_build */
 int pfkey_sock;
 fd_set pfkey_socks;
 uint32_t pfkey_seq = 0;
-#endif /* CONFIG_IPSEC_PFKEYv2 */
 
 static void
 usage(char* arg)
 {
-	fprintf(stdout, "usage: %s --{add,replace} --src <src>/<srcmaskbits>|<srcmask> --dst <dst>/<dstmaskbits>|<dstmask> <SA>\n", arg);
-	fprintf(stdout, "            where <SA> is '--edst <edst> --spi <spi> --proto <proto>' OR '--said <said>' OR '--said %%passthrough'.\n");
-	fprintf(stdout, "       %s --del --src <src>/<srcmaskbits>|<srcmask> --dst <dst>/<dstmaskbits>|<dstmask>\n", arg);
+	fprintf(stdout, "usage: %s --{add,addin,replace} --eraf <inet | inet6> --src <src>/<srcmaskbits>|<srcmask> --dst <dst>/<dstmaskbits>|<dstmask> <SA>\n", arg);
+	fprintf(stdout, "            where <SA> is '--af <inet | inet6> --edst <edst> --spi <spi> --proto <proto>'\n");
+	fprintf(stdout, "                       OR '--said <said>'\n");
+	fprintf(stdout, "                       OR '--said <%%passthrough | %%passthrough4 | %%passthrough6 | %%drop | %%reject | %%trap | %%hold | %%pass>'.\n");
+	fprintf(stdout, "       %s --del --eraf <inet | inet6>--src <src>/<srcmaskbits>|<srcmask> --dst <dst>/<dstmaskbits>|<dstmask>\n", arg);
 	fprintf(stdout, "       %s --clear\n", arg);
 	fprintf(stdout, "       %s --help\n", arg);
 	fprintf(stdout, "       %s --version\n", arg);
 	fprintf(stdout, "       %s\n", arg);
 	fprintf(stdout, "        [ --debug ] is optional to any %s command.\n", arg);
+	fprintf(stdout, "        [ --label <label> ] is optional to any %s command.\n", arg);
 exit(1);
 }
 
-#if 0
-static int
-resolve_ip(char *arg, struct in_addr *ip)
-{
-	struct hostent *hostp;
-	struct netent *netp;
-	
-	if (!strcmp(arg, "default")) {/* default is really 0.0.0.0 */
-		ip->s_addr = htonl(INADDR_ANY);
-		return(1);
-	}
-	if (!(strcmp(arg, "255.255.255.255"))) {
-		/* inet_addr() returns -1 for this, as it should, but that means an error */
-		ip->s_addr = htonl(INADDR_BROADCAST);
-		return (1);
-	}
-	if ((netp = getnetbyname(arg)) != (struct netent *)NULL) {
-		ip->s_addr = htonl(netp->n_net);
-		return(1);
-	}
-	if ((hostp = gethostbyname(arg)) == (struct hostent *)NULL) {
-		errno = h_errno;
-		return(-1);
-	}
-	memcpy((char *) ip, (char *) hostp->h_addr_list[0], hostp->h_length);
-	return(0);
-}
-#endif
-
 static struct option const longopts[] =
 {
 	{"dst", 1, 0, 'D'},
 	{"src", 1, 0, 'S'},
+	{"eraf", 1, 0, 'f'},
 	{"add", 0, 0, 'a'},
+	{"addin", 0, 0, 'A'},
 	{"replace", 0, 0, 'r'},
 	{"clear", 0, 0, 'c'},
 	{"del", 0, 0, 'd'},
+	{"af", 1, 0, 'i'},
 	{"edst", 1, 0, 'e'},
 	{"proto", 1, 0, 'p'},
 	{"help", 0, 0, 'h'},
@@ -130,78 +99,80 @@ int
 main(int argc, char **argv)
 {
 /*	int fd; */
-	struct encap_msghdr emh;
 	char *endptr;
 /*	int ret; */
 	int c, previous = -1;
 	const char* error_s;
 	int debug = 0;
 
-#ifdef CONFIG_IPSEC_PFKEYv2
 	int error = 0;
-	int i;
 
-	char ipaddr_txt[ADDRTOA_BUF];                
+	char ipaddr_txt[ADDRTOT_BUF];                
 	struct sadb_ext *extensions[SADB_EXT_MAX + 1];
 	struct sadb_msg *pfkey_msg;
-	struct sockaddr_in pfkey_address_s_ska;
-	struct sockaddr_in pfkey_address_d_ska;
-	struct sockaddr_in pfkey_address_sflow_ska;
-	struct sockaddr_in pfkey_address_dflow_ska;
-	struct sockaddr_in pfkey_address_smask_ska;
-	struct sockaddr_in pfkey_address_dmask_ska;
-#else /* CONFIG_IPSEC_PFKEYv2 */
-	int fd = 0;
-#endif /* CONFIG_IPSEC_PFKEYv2 */
-	
-	if(argc == 1) {
-		system("cat /proc/net/ipsec_eroute");
-		exit(0);
-	}
+	ip_address pfkey_address_s_ska;
+	/*struct sockaddr_in pfkey_address_d_ska;*/
+	ip_address pfkey_address_sflow_ska;
+	ip_address pfkey_address_dflow_ska;
+	ip_address pfkey_address_smask_ska;
+	ip_address pfkey_address_dmask_ska;
+
+	ip_said said;
+	ip_subnet s_subnet, d_subnet;
+ 	int eroute_af = 0;
+ 	int said_af = 0;
+
+	int argcount = argc;
 
 	program_name = argv[0];
-	memset(&emh, 0, sizeof(emh));
+	eroute_af_opt = said_af_opt = edst_opt = spi_opt = proto_opt = said_opt = dst_opt = src_opt = NULL;
 
-	while((c = getopt_long(argc, argv, "acdD:e:hprs:S:vl:+:g", longopts, 0)) != EOF)
-	{
+	while((c = getopt_long(argc, argv, ""/*"acdD:e:i:hprs:S:f:vl:+:g"*/, longopts, 0)) != EOF) {
 		switch(c) {
 		case 'g':
 			debug = 1;
-#ifdef CONFIG_IPSEC_PFKEYv2
 			pfkey_lib_debug = 1;
-#endif /* CONFIG_IPSEC_PFKEYv2 */
+			argcount--;
 			break;
 		case 'a':
-			if(emh.em_type) {
-				fprintf(stderr, "%s: Only one of '--add', '--replace', '--clear', or '--del' options permitted.\n",
+			if(action_type) {
+				fprintf(stderr, "%s: Only one of '--add', '--addin', '--replace', '--clear', or '--del' options permitted.\n",
 					program_name);
 				exit(1);
 			}
-			emh.em_type = EMT_SETEROUTE;
+			action_type = EMT_SETEROUTE;
+			break;
+		case 'A':
+			if(action_type) {
+				fprintf(stderr, "%s: Only one of '--add', '--addin', '--replace', '--clear', or '--del' options permitted.\n",
+					program_name);
+				exit(1);
+			}
+			action_type = EMT_INEROUTE;
 			break;
 		case 'r':
-			if(emh.em_type) {
-				fprintf(stderr, "%s: Only one of '--add', '--replace', '--clear', or '--del' options permitted.\n",
+			if(action_type) {
+				fprintf(stderr, "%s: Only one of '--add', '--addin', '--replace', '--clear', or '--del' options permitted.\n",
 					program_name);
 				exit(1);
 			}
-			emh.em_type = EMT_RPLACEROUTE;
+			action_type = EMT_REPLACEROUTE;
 			break;
 		case 'c':
-			if(emh.em_type) {
-				fprintf(stderr, "%s: Only one of '--add', '--replace', '--clear', or '--del' options permitted.\n",
+			if(action_type) {
+				fprintf(stderr, "%s: Only one of '--add', '--addin', '--replace', '--clear', or '--del' options permitted.\n",
 					program_name);
 				exit(1);
 			}
-			emh.em_type = EMT_CLREROUTE;
+			action_type = EMT_CLREROUTE;
 			break;
 		case 'd':
-			if(emh.em_type) {
-				fprintf(stderr, "%s: Only one of '--add', '--replace', '--clear', or '--del' options permitted.\n",
+			if(action_type) {
+				fprintf(stderr, "%s: Only one of '--add', '--addin', '--replace', '--clear', or '--del' options permitted.\n",
 					program_name);
 				exit(1);
 			}
-			emh.em_type = EMT_DELEROUTE;
+			action_type = EMT_DELEROUTE;
 			break;
 		case 'e':
 			if(said_opt) {
@@ -214,25 +185,11 @@ main(int argc, char **argv)
 					program_name, optarg, edst_opt);
 				exit (1);
 			}				
-#if 0
-			if((ret = resolve_ip(optarg, &(emh.em_erdst))) != 0) {
-				if(ret == 1) {
-					fprintf(stderr, "%s: Destination address cannot be a network: %s\n",
-						program_name, optarg);
-					exit (1);
-				} else {
-					fprintf(stderr, "%s: Invalid formation of destination address: %s\n",
-						program_name, optarg);
-					exit (1);
-				}
-			}
-#endif
-			if((error_s = atoaddr(optarg, 0, &(emh.em_erdst))) != 0) {
-				if(error_s) {
-					fprintf(stderr, "%s: Error, %s converting --edst argument:%s\n",
-						program_name, error_s, optarg);
-					exit (1);
-				}
+			error_s = ttoaddr(optarg, 0, said_af, &said.dst);
+			if(error_s != NULL) {
+				fprintf(stderr, "%s: Error, %s converting --edst argument:%s\n",
+					program_name, error_s, optarg);
+				exit (1);
 			}
 			edst_opt = optarg;
 			break;
@@ -251,15 +208,15 @@ main(int argc, char **argv)
 					program_name, optarg, spi_opt);
 				exit (1);
 			}				
-			emh.em_erspi = htonl(strtoul(optarg, &endptr, 0));
+			said.spi = htonl(strtoul(optarg, &endptr, 0));
 			if(!(endptr == optarg + strlen(optarg))) {
 				fprintf(stderr, "%s: Invalid character in SPI parameter: %s\n",
 					program_name, optarg);
 				exit (1);
 			}
-			if(emh.em_erspi < 0x100) {
+			if(ntohl(said.spi) < 0x100) {
 				fprintf(stderr, "%s: Illegal reserved spi: %s => 0x%x Must be larger than 0x100.\n",
-					program_name, optarg, emh.em_erspi);
+					program_name, optarg, ntohl(said.spi));
 				exit(1);
 			}
 			spi_opt = optarg;
@@ -276,19 +233,21 @@ main(int argc, char **argv)
 				exit (1);
 			}
 #if 0
-			if(emh.em_erproto) {
+			if(said.proto) {
 				fprintf(stderr, "%s: Warning, PROTO parameter redefined:%s\n",
 					program_name, optarg);
 				exit (1);
 			}
 #endif
 			if(!strcmp(optarg, "ah"))
-				emh.em_erproto = SA_AH;
+				said.proto = SA_AH;
 			if(!strcmp(optarg, "esp"))
-				emh.em_erproto = SA_ESP;
+				said.proto = SA_ESP;
 			if(!strcmp(optarg, "tun"))
-				emh.em_erproto = SA_IPIP;
-			if(emh.em_erproto == 0) {
+				said.proto = SA_IPIP;
+			if(!strcmp(optarg, "comp"))
+				said.proto = SA_COMP;
+			if(said.proto == 0) {
 				fprintf(stderr, "%s: Invalid PROTO parameter: %s\n",
 					program_name, optarg);
 				exit (1);
@@ -311,53 +270,102 @@ main(int argc, char **argv)
 					program_name, optarg, edst_opt);
 				exit (1);
 			}
-			if(proto_opt) {
+			if(spi_opt) {
 				fprintf(stderr, "%s: Error, SPI parameter redefined in SA:%s, already defined as:%s\n",
 					program_name, optarg, spi_opt);
 				exit (1);
 			}
-			if((error_s = atosa(optarg, 0, &emh.em_ersaid)) != NULL) {
+			if(said_af_opt) {
+				fprintf(stderr, "%s: Error, address family parameter redefined in SA:%s, already defined as:%s\n",
+					program_name, optarg, said_af_opt);
+				exit (1);
+			}
+			error_s = ttosa(optarg, 0, &said);
+			if(error_s != NULL) {
 				fprintf(stderr, "%s: Error, %s converting --sa argument:%s\n",
 					program_name, error_s, optarg);
 				exit (1);
-			} else if((emh.em_erspi < 0x100) && (emh.em_erspi > 0x0)){
+			} else if(ntohl(said.spi) < 0x100){
 				fprintf(stderr, "%s: Illegal reserved spi: %s => 0x%x Must be larger than or equal to 0x100.\n",
-					program_name, optarg, emh.em_erspi);
+					program_name, optarg, said.spi);
 				exit(1);
 			}
+			said_af = addrtypeof(&said.dst);
 			said_opt = optarg;
 			break;
 		case 'v':
-			printf("%s, %s\n", program_name, eroute_c_version);
+			fprintf(stdout, "%s, %s\n", program_name, eroute_c_version);
 			exit(1);
 		case 'D':
 			if(dst_opt) {
-				fprintf(stderr, "%s: Error, EDST parameter redefined:%s, already defined as:%s\n",
+				fprintf(stderr, "%s: Error, --dst parameter redefined:%s, already defined as:%s\n",
 					program_name, optarg, dst_opt);
 				exit (1);
 			}				
-			if (atosubnet(optarg, 0, &emh.em_eaddr.sen_ip_dst, &emh.em_emask.sen_ip_dst)) {
-				fprintf(stderr, "%s: Invalid formation of destination network/mask: %s\n",
-					program_name, optarg);
+			error_s = ttosubnet(optarg, 0, eroute_af, &d_subnet);
+			if (error_s != NULL) {
+				fprintf(stderr, "%s: Error, %s converting --dst argument: %s\n",
+					program_name, error_s, optarg);
 				exit (1);
 			}
 			dst_opt = optarg;
 			break;
 		case 'S':
 			if(src_opt) {
-				fprintf(stderr, "%s: Error, EDST parameter redefined:%s, already defined as:%s\n",
+				fprintf(stderr, "%s: Error, --src parameter redefined:%s, already defined as:%s\n",
 					program_name, optarg, src_opt);
 				exit (1);
 			}				
-			if (atosubnet(optarg, 0, &emh.em_eaddr.sen_ip_src, &emh.em_emask.sen_ip_src)) {
-				fprintf(stderr, "%s: Invalid formation of source network/mask: %s\n",
-					program_name, optarg);
+			error_s = ttosubnet(optarg, 0, eroute_af, &s_subnet);
+			if (error_s != NULL) {
+				fprintf(stderr, "%s: Error, %s converting --src argument: %s\n",
+					program_name, error_s, optarg);
 				exit (1);
 			}
 			src_opt = optarg;
 			break;
 		case 'l':
-			program_name = optarg;
+			program_name = malloc(strlen(argv[0])
+					      + 10 /* update this when changing the sprintf() */
+					      + strlen(optarg));
+			sprintf(program_name, "%s --label %s",
+				argv[0],
+				optarg);
+			argcount -= 2;
+			break;
+		case 'i': /* specifies the address family of the SAID, stored in said_af */
+			if(said_af_opt) {
+				fprintf(stderr, "%s: Error, address family of SAID redefined:%s, already defined as:%s\n",
+					program_name, optarg, said_af_opt);
+				exit (1);
+			}				
+			if(!strcmp(optarg, "inet"))
+				said_af = AF_INET;
+			if(!strcmp(optarg, "inet6"))
+				said_af = AF_INET6;
+			if(said_af == 0) {
+				fprintf(stderr, "%s: Invalid address family parameter for SAID: %s\n",
+					program_name, optarg);
+				exit (1);
+			}
+			said_af_opt = optarg;
+			break;
+		case 'f': /* specifies the address family of the eroute, stored in eroute_af */
+			if(eroute_af_opt) {
+				fprintf(stderr, "%s: Error, address family of eroute redefined:%s, already defined as:%s\n",
+					program_name, optarg, eroute_af_opt);
+				exit (1);
+			}				
+			if(!strcmp(optarg, "inet"))
+				eroute_af = AF_INET;
+			if(!strcmp(optarg, "inet6"))
+				eroute_af = AF_INET6;
+			if(eroute_af == 0) {
+				fprintf(stderr, "%s: Invalid address family parameter for eroute: %s\n",
+					program_name, optarg);
+				exit (1);
+			}
+			eroute_af_opt = optarg;
 			break;
 		case '+': /* optionsfrom */
 			optionsfrom(optarg, &argc, &argv, optind, stderr);
@@ -369,57 +377,49 @@ main(int argc, char **argv)
 	}
 
 	if(debug) {
-		printf("%s: DEBUG: argc=%d\n", program_name, argc);
+		fprintf(stdout, "%s: DEBUG: argc=%d\n", program_name, argc);
+	}
+	
+	if(argcount == 1) {
+		system("cat /proc/net/ipsec_eroute");
+		exit(0);
 	}
 
 	/* Sanity checks */
 
-	switch(emh.em_type) {
+	if(debug) {
+		fprintf(stdout, "%s: DEBUG: action_type=%d\n", program_name, action_type);
+	}
+
+	switch(action_type) {
 	case EMT_SETEROUTE:
-	case EMT_RPLACEROUTE:
-		if(!(edst_opt && spi_opt && proto_opt) && !(said_opt)) {
-			fprintf(stderr, "%s: add option must have SA specified.\n",
+	case EMT_REPLACEROUTE:
+	case EMT_INEROUTE:
+		if(!(said_af_opt && edst_opt && spi_opt && proto_opt) && !(said_opt)) {
+			fprintf(stderr, "%s: add and addin options must have SA specified.\n",
 				program_name);
 			exit(1);
 		}
 	case EMT_DELEROUTE:
 		if(!src_opt) {
 			fprintf(stderr, "%s: Error -- %s option '--src' is required.\n",
-				program_name, (emh.em_type == EMT_SETEROUTE) ? "add" : "del");
+				program_name, (action_type == EMT_SETEROUTE) ? "add" : "del");
 			exit(1);
 		}
-		if(!emh.em_eaddr.sen_ip_src.s_addr || !emh.em_emask.sen_ip_src.s_addr) {
-			fprintf(stderr, "%s: warning -- %s option '--src' is 0.0.0.0.  If the <default address> was not intentional, then a name lookup failed.\n",
-				program_name, (emh.em_type == EMT_SETEROUTE) ? "add" : "del");
-/*			exit(1); */
-		}
 		if(!dst_opt) {
 			fprintf(stderr, "%s: Error -- %s option '--dst' is required.\n",
-				program_name, (emh.em_type == EMT_SETEROUTE) ? "add" : "del");
+				program_name, (action_type == EMT_SETEROUTE) ? "add" : "del");
 			exit(1);
 		}
-		if(!emh.em_eaddr.sen_ip_dst.s_addr || !emh.em_emask.sen_ip_dst.s_addr) {
-			fprintf(stderr, "%s: warning -- %s option '--dst' is 0.0.0.0.  If the <default address> was not intentional, then a name lookup failed.\n",
-				program_name, (emh.em_type == EMT_SETEROUTE) ? "add" : "del");
-/*			exit(1); */
-		}
 	case EMT_CLREROUTE:
-		emh.em_magic = EM_MAGIC;
-		emh.em_msglen= sizeof emh;
-		emh.em_version = 0;
-		emh.em_eaddr.sen_len = emh.em_emask.sen_len = sizeof (struct sockaddr_encap);
-		emh.em_eaddr.sen_family = emh.em_emask.sen_family = AF_ENCAP;
-		emh.em_eaddr.sen_type = SENT_IP4;
-		emh.em_emask.sen_type = 255;
 		break;
 	default:
-		fprintf(stderr, "%s: exactly one of '--add', '--replace', '--del' or '--clear' options must be specified.\n"
+		fprintf(stderr, "%s: exactly one of '--add', '--addin', '--replace', '--del' or '--clear' options must be specified.\n"
 			"Try %s --help' for usage information.\n",
 			program_name, program_name);
 		exit(1);
 	}
 
-#ifdef CONFIG_IPSEC_PFKEYv2
 	if((pfkey_sock = socket(PF_KEY, SOCK_RAW, PF_KEY_V2) ) < 0) {
 		fprintf(stderr, "%s: Trouble openning PF_KEY family socket with error: ",
 			program_name);
@@ -436,7 +436,7 @@ main(int argc, char **argv)
 			}
 			break;
 		case EUNATCH:
-			fprintf(stderr, "Netlink not enabled OR KLIPS not loaded.\n");
+			fprintf(stderr, "KLIPS not loaded.\n");
 			break;
 		case ENODEV:
 			fprintf(stderr, "KLIPS not loaded or enabled.\n");
@@ -448,16 +448,21 @@ main(int argc, char **argv)
 			fprintf(stderr, "Invalid argument, KLIPS not loaded or check kernel log messages for specifics.\n");
 			break;
 		case ENOBUFS:
-			fprintf(stderr, "No kernel memory to allocate SA.\n");
+		case ENOMEM:
+		case ENFILE:
+			fprintf(stderr, "No kernel memory to allocate socket.\n");
+			break;
+		case EMFILE:
+			fprintf(stderr, "Process file table overflow.\n");
 			break;
 		case ESOCKTNOSUPPORT:
-			fprintf(stderr, "Algorithm support not available in the kernel.  Please compile in support.\n");
+			fprintf(stderr, "Socket type not supported.\n");
 			break;
-		case EEXIST:
-			fprintf(stderr, "SA already in use.  Delete old one first.\n");
+		case EPROTONOSUPPORT:
+			fprintf(stderr, "Protocol version not supported.\n");
 			break;
-		case ENXIO:
-			fprintf(stderr, "SA does not exist.  Cannot delete.\n");
+		case EAFNOSUPPORT:
+			fprintf(stderr, "KLIPS not loaded or enabled.\n");
 			break;
 		default:
 			fprintf(stderr, "Unknown file open error %d.  Please report as much detail as possible to development team.\n", errno);
@@ -465,20 +470,19 @@ main(int argc, char **argv)
 		exit(1);
 	}
 
+	if(debug) {
+		fprintf(stdout, "%s: DEBUG: PFKEYv2 socket successfully openned=%d.\n", program_name, pfkey_sock);
+	}
+
 	/* Build an SADB_X_ADDFLOW or SADB_X_DELFLOW message to send down. */
 	/* It needs <base, SA, address(SD), flow(SD), mask(SD)> minimum. */
 	pfkey_extensions_init(extensions);
 	if((error = pfkey_msg_hdr_build(&extensions[0],
-					(emh.em_type == EMT_SETEROUTE ||
-					 emh.em_type == EMT_RPLACEROUTE) ?
-					SADB_X_ADDFLOW : SADB_X_DELFLOW,
-					proto2satype(emh.em_erproto),
-#if 0
-					(emh.em_erproto == SA_ESP ? SADB_SATYPE_ESP :
-					 (emh.em_erproto == SA_AH ? SADB_SATYPE_AH :
-					  (emh.em_erproto == SA_IPIP ? SADB_X_SATYPE_IPIP :
-					   SADB_SATYPE_UNSPEC))),
-#endif
+					(action_type == EMT_SETEROUTE
+					 || action_type == EMT_REPLACEROUTE
+					 || action_type == EMT_INEROUTE)
+					? SADB_X_ADDFLOW : SADB_X_DELFLOW,
+					proto2satype(said.proto),
 					0,
 					++pfkey_seq,
 					getpid()))) {
@@ -488,143 +492,140 @@ main(int argc, char **argv)
 		exit(1);
 	}
 
-	switch(emh.em_type) {
+	if(debug) {
+		fprintf(stdout, "%s: DEBUG: pfkey_msg_hdr_build successfull.\n", program_name);
+	}
+
+	switch(action_type) {
 	case EMT_SETEROUTE:
-	case EMT_RPLACEROUTE:
+	case EMT_REPLACEROUTE:
+	case EMT_INEROUTE:
 	case EMT_CLREROUTE:
 		if((error = pfkey_sa_build(&extensions[SADB_EXT_SA],
 					   SADB_EXT_SA,
-					   emh.em_erspi, /* in network order */
+					   said.spi, /* in network order */
 					   0,
 					   0,
 					   0,
 					   0,
-					   (emh.em_type == EMT_CLREROUTE) ? SADB_X_SAFLAGS_CLEARFLOW : 0))) {
+					   (action_type == EMT_CLREROUTE) ? SADB_X_SAFLAGS_CLEARFLOW : 0))) {
 			fprintf(stderr, "%s: Trouble building sa extension, error=%d.\n",
 				program_name, error);
 			pfkey_extensions_free(extensions);
 			exit(1);
 		}
+		if(debug) {
+			fprintf(stdout, "%s: DEBUG: pfkey_sa_build successful.\n", program_name);
+		}
+
 	default:
 	}
 
-	switch(emh.em_type) {
+	switch(action_type) {
 	case EMT_SETEROUTE:
-	case EMT_RPLACEROUTE:
-		pfkey_address_s_ska.sin_addr.s_addr = 0; /* esrc */
-		pfkey_address_s_ska.sin_family = AF_INET;
-		pfkey_address_s_ska.sin_port = 0;
-		for(i = 0; i < sizeof(struct sockaddr_in) - offsetof(struct sockaddr_in, sin_zero); i++) {
-			pfkey_address_s_ska.sin_zero[i] = 0;
-		}
+	case EMT_REPLACEROUTE:
+	case EMT_INEROUTE:
+		anyaddr(said_af, &pfkey_address_s_ska);
 		if((error = pfkey_address_build(&extensions[SADB_EXT_ADDRESS_SRC],
 						SADB_EXT_ADDRESS_SRC,
 						0,
 						0,
-						(struct sockaddr*)&pfkey_address_s_ska))) {
-			addrtoa(pfkey_address_s_ska.sin_addr,
-				0, ipaddr_txt, sizeof(ipaddr_txt));
+						sockaddrof(&pfkey_address_s_ska)))) {
+			addrtot(&pfkey_address_s_ska, 0, ipaddr_txt, sizeof(ipaddr_txt));
 			fprintf(stderr, "%s: Trouble building address_s extension (%s), error=%d.\n",
 				program_name, ipaddr_txt, error);
 			pfkey_extensions_free(extensions);
 			exit(1);
 		}
-		
-		pfkey_address_d_ska.sin_addr = emh.em_erdst; /* edst */
-		pfkey_address_d_ska.sin_family = AF_INET;
-		pfkey_address_d_ska.sin_port = 0;
-		for(i = 0; i < sizeof(struct sockaddr_in) - offsetof(struct sockaddr_in, sin_zero); i++) {
-			pfkey_address_d_ska.sin_zero[i] = 0;
+		if(debug) {
+			fprintf(stdout, "%s: DEBUG: pfkey_address_build successful for src.\n", program_name);
 		}
+
 		if((error = pfkey_address_build(&extensions[SADB_EXT_ADDRESS_DST],
 						SADB_EXT_ADDRESS_DST,
 						0,
 						0,
-						(struct sockaddr*)&pfkey_address_d_ska))) {
-			addrtoa(pfkey_address_s_ska.sin_addr,
-				0, ipaddr_txt, sizeof(ipaddr_txt));
+						sockaddrof(&said.dst)))) {
+			addrtot(&said.dst, 0, ipaddr_txt, sizeof(ipaddr_txt));
 			fprintf(stderr, "%s: Trouble building address_d extension (%s), error=%d.\n",
 				program_name, ipaddr_txt, error);
 			pfkey_extensions_free(extensions);
 			exit(1);
 		}
+		if(debug) {
+			fprintf(stdout, "%s: DEBUG: pfkey_address_build successful for dst.\n", program_name);
+		}
 	default:
 	}
 	
-	pfkey_address_sflow_ska.sin_addr/*.s_addr*/ = emh.em_eaddr.sen_ip_src; /* src flow */
-	pfkey_address_sflow_ska.sin_family = AF_INET;
-	pfkey_address_sflow_ska.sin_port = 0;
-	for(i = 0; i < sizeof(struct sockaddr_in) - offsetof(struct sockaddr_in, sin_zero); i++) {
-		pfkey_address_sflow_ska.sin_zero[i] = 0;
-	}
-	if((error = pfkey_address_build(&extensions[SADB_X_EXT_ADDRESS_SRC_FLOW],
-					SADB_X_EXT_ADDRESS_SRC_FLOW,
-					0,
-					0,
-					(struct sockaddr*)&pfkey_address_sflow_ska))) {
-		addrtoa(pfkey_address_sflow_ska.sin_addr,
-			0, ipaddr_txt, sizeof(ipaddr_txt));
-		fprintf(stderr, "%s: Trouble building address_sflow extension (%s), error=%d.\n",
-			program_name, ipaddr_txt, error);
-		pfkey_extensions_free(extensions);
-		exit(1);
-	}
+	switch(action_type) {
+	case EMT_SETEROUTE:
+	case EMT_REPLACEROUTE:
+	case EMT_INEROUTE:
+	case EMT_DELEROUTE:
+		networkof(&s_subnet, &pfkey_address_sflow_ska); /* src flow */
+		if((error = pfkey_address_build(&extensions[SADB_X_EXT_ADDRESS_SRC_FLOW],
+						SADB_X_EXT_ADDRESS_SRC_FLOW,
+						0,
+						0,
+						sockaddrof(&pfkey_address_sflow_ska)))) {
+			addrtot(&pfkey_address_sflow_ska, 0, ipaddr_txt, sizeof(ipaddr_txt));
+			fprintf(stderr, "%s: Trouble building address_sflow extension (%s), error=%d.\n",
+				program_name, ipaddr_txt, error);
+			pfkey_extensions_free(extensions);
+			exit(1);
+		}
+		if(debug) {
+			fprintf(stdout, "%s: DEBUG: pfkey_address_build successful for src flow.\n", program_name);
+		}
 	
-	pfkey_address_dflow_ska.sin_addr = emh.em_eaddr.sen_ip_dst; /* dst flow */
-	pfkey_address_dflow_ska.sin_family = AF_INET;
-	pfkey_address_dflow_ska.sin_port = 0;
-	for(i = 0; i < sizeof(struct sockaddr_in) - offsetof(struct sockaddr_in, sin_zero); i++) {
-		pfkey_address_dflow_ska.sin_zero[i] = 0;
-	}
-	if((error = pfkey_address_build(&extensions[SADB_X_EXT_ADDRESS_DST_FLOW],
-					SADB_X_EXT_ADDRESS_DST_FLOW,
-					0,
-					0,
-					(struct sockaddr*)&pfkey_address_dflow_ska))) {
-		addrtoa(pfkey_address_sflow_ska.sin_addr,
-			0, ipaddr_txt, sizeof(ipaddr_txt));
-		fprintf(stderr, "%s: Trouble building address_dflow extension (%s), error=%d.\n",
-			program_name, ipaddr_txt, error);
-		pfkey_extensions_free(extensions);
-		exit(1);
-	}
-	
-	pfkey_address_smask_ska.sin_addr/*.s_addr*/ = emh.em_emask.sen_ip_src; /* src mask */
-	pfkey_address_smask_ska.sin_family = AF_INET;
-	pfkey_address_smask_ska.sin_port = 0;
-	for(i = 0; i < sizeof(struct sockaddr_in) - offsetof(struct sockaddr_in, sin_zero); i++) {
-		pfkey_address_smask_ska.sin_zero[i] = 0;
-	}
-	if((error = pfkey_address_build(&extensions[SADB_X_EXT_ADDRESS_SRC_MASK],
-					SADB_X_EXT_ADDRESS_SRC_MASK,
-					0,
-					0,
-					(struct sockaddr*)&pfkey_address_smask_ska))) {
-		addrtoa(pfkey_address_smask_ska.sin_addr,
-			0, ipaddr_txt, sizeof(ipaddr_txt));
-		fprintf(stderr, "%s: Trouble building address_smask extension (%s), error=%d.\n",
-			program_name, ipaddr_txt, error);
-		pfkey_extensions_free(extensions);
-		exit(1);
-	}
-	
-	pfkey_address_dmask_ska.sin_addr = emh.em_emask.sen_ip_dst; /* dst mask */
-	pfkey_address_dmask_ska.sin_family = AF_INET;
-	pfkey_address_dmask_ska.sin_port = 0;
-	for(i = 0; i < sizeof(struct sockaddr_in) - offsetof(struct sockaddr_in, sin_zero); i++) {
-		pfkey_address_dmask_ska.sin_zero[i] = 0;
-	}
-	if((error = pfkey_address_build(&extensions[SADB_X_EXT_ADDRESS_DST_MASK],
-					SADB_X_EXT_ADDRESS_DST_MASK,
-					0,
-					0,
-					(struct sockaddr*)&pfkey_address_dmask_ska))) {
-		addrtoa(pfkey_address_smask_ska.sin_addr,
-			0, ipaddr_txt, sizeof(ipaddr_txt));
-		fprintf(stderr, "%s: Trouble building address_dmask extension (%s), error=%d.\n",
-			program_name, ipaddr_txt, error);
-		pfkey_extensions_free(extensions);
-		exit(1);
+		networkof(&d_subnet, &pfkey_address_dflow_ska); /* dst flow */
+		if((error = pfkey_address_build(&extensions[SADB_X_EXT_ADDRESS_DST_FLOW],
+						SADB_X_EXT_ADDRESS_DST_FLOW,
+						0,
+						0,
+						sockaddrof(&pfkey_address_dflow_ska)))) {
+			addrtot(&pfkey_address_dflow_ska, 0, ipaddr_txt, sizeof(ipaddr_txt));
+			fprintf(stderr, "%s: Trouble building address_dflow extension (%s), error=%d.\n",
+				program_name, ipaddr_txt, error);
+			pfkey_extensions_free(extensions);
+			exit(1);
+		}
+		if(debug) {
+			fprintf(stdout, "%s: DEBUG: pfkey_address_build successful for dst flow.\n", program_name);
+		}
+		
+		maskof(&s_subnet, &pfkey_address_smask_ska); /* src mask */
+		if((error = pfkey_address_build(&extensions[SADB_X_EXT_ADDRESS_SRC_MASK],
+						SADB_X_EXT_ADDRESS_SRC_MASK,
+						0,
+						0,
+						sockaddrof(&pfkey_address_smask_ska)))) {
+			addrtot(&pfkey_address_smask_ska, 0, ipaddr_txt, sizeof(ipaddr_txt));
+			fprintf(stderr, "%s: Trouble building address_smask extension (%s), error=%d.\n",
+				program_name, ipaddr_txt, error);
+			pfkey_extensions_free(extensions);
+			exit(1);
+		}
+		if(debug) {
+			fprintf(stdout, "%s: DEBUG: pfkey_address_build successful for src mask.\n", program_name);
+		}
+		
+		maskof(&d_subnet, &pfkey_address_dmask_ska); /* dst mask */
+		if((error = pfkey_address_build(&extensions[SADB_X_EXT_ADDRESS_DST_MASK],
+						SADB_X_EXT_ADDRESS_DST_MASK,
+						0,
+						0,
+						sockaddrof(&pfkey_address_dmask_ska)))) {
+			addrtot(&pfkey_address_dmask_ska, 0, ipaddr_txt, sizeof(ipaddr_txt));
+			fprintf(stderr, "%s: Trouble building address_dmask extension (%s), error=%d.\n",
+				program_name, ipaddr_txt, error);
+			pfkey_extensions_free(extensions);
+			exit(1);
+		}
+		if(debug) {
+			fprintf(stdout, "%s: DEBUG: pfkey_address_build successful for dst mask.\n", program_name);
+		}
 	}
 	
 	if((error = pfkey_msg_build(&pfkey_msg, extensions, EXT_BITS_IN))) {
@@ -634,6 +635,9 @@ main(int argc, char **argv)
 		pfkey_msg_free(&pfkey_msg);
 		exit(1);
 	}
+	if(debug) {
+		fprintf(stdout, "%s: DEBUG: pfkey_msg_build successful.\n", program_name);
+	}
 
 	if((error = write(pfkey_sock,
 			  pfkey_msg,
@@ -648,22 +652,22 @@ main(int argc, char **argv)
 			fprintf(stderr, "Invalid argument, check kernel log messages for specifics.\n");
 			break;
 		case ENXIO:
-			if((emh.em_type == EMT_SETEROUTE) ||
-			   (emh.em_type == EMT_RPLACEROUTE)) {
+			if((action_type == EMT_SETEROUTE) ||
+			   (action_type == EMT_REPLACEROUTE)) {
 				fprintf(stderr, "Invalid mask.\n");
 				break;
 			}
-			if(emh.em_type == EMT_DELEROUTE) {
+			if(action_type == EMT_DELEROUTE) {
 				fprintf(stderr, "Mask not found.\n");
 				break;
 			}
 		case EFAULT:
-			if((emh.em_type == EMT_SETEROUTE) ||
-			   (emh.em_type == EMT_RPLACEROUTE)) {
+			if((action_type == EMT_SETEROUTE) ||
+			   (action_type == EMT_REPLACEROUTE)) {
 				fprintf(stderr, "Invalid address.\n");
 				break;
 			}
-			if(emh.em_type == EMT_DELEROUTE) {
+			if(action_type == EMT_DELEROUTE) {
 				fprintf(stderr, "Address not found.\n");
 				break;
 			}
@@ -676,7 +680,7 @@ main(int argc, char **argv)
 			}
 			break;
 		case EUNATCH:
-			fprintf(stderr, "Netlink not enabled OR KLIPS not loaded.\n");
+			fprintf(stderr, "KLIPS not loaded.\n");
 			break;
 		case EBUSY:
 			fprintf(stderr, "KLIPS is busy.  Most likely a serious internal error occured in a previous command.  Please report as much detail as possible to development team.\n");
@@ -692,10 +696,14 @@ main(int argc, char **argv)
 			fprintf(stderr, "Algorithm support not available in the kernel.  Please compile in support.\n");
 			break;
 		case EEXIST:
-			fprintf(stderr, "SA already in use.  Delete old one first.\n");
+			fprintf(stderr, "eroute already in use.  Delete old one first.\n");
 			break;
 		case ENOENT:
-			fprintf(stderr, "device does not exist.  See FreeS/WAN installation procedure.\n");
+			if((action_type == EMT_INEROUTE)) {
+				fprintf(stderr, "non-existant IPIP SA.\n");
+				break;
+			}
+			fprintf(stderr, "eroute doesn't exist.  Can't delete.\n");
 			break;
 		default:
 			fprintf(stderr, "Unknown socket write error %d.  Please report as much detail as possible to development team.\n", errno);
@@ -704,6 +712,9 @@ main(int argc, char **argv)
 		program_name, errno);*/
 		exit(1);
 	}
+	if(debug) {
+		fprintf(stdout, "%s: DEBUG: pfkey write successful.\n", program_name);
+	}
 
 	if(pfkey_msg) {
 		pfkey_extensions_free(extensions);
@@ -711,87 +722,74 @@ main(int argc, char **argv)
 	}
 
 	(void) close(pfkey_sock);  /* close the socket */
-#else /* CONFIG_IPSEC_PFKEYv2 */
-
-	fd = open("/dev/ipsec", O_RDWR);
-	if (fd < 0) {
-		fprintf(stderr, "%s: Could not open /dev/ipsec -- ", program_name);
-		switch(errno) {
-		case ENOENT:
-			fprintf(stderr, "device does not exist.  See FreeS/WAN installation procedure.\n");
-		case EACCES:
-			fprintf(stderr, "access denied.  ");
-			if(getuid() == 0) {
-				fprintf(stderr, "Check permissions.  Should be 600.\n");
-			} else {
-				fprintf(stderr, "You must be root to open this file.\n");
-			}
-			break;
-		case EUNATCH:
-			fprintf(stderr, "Netlink not enabled OR KLIPS not loaded.\n");
-			break;
-		case ENODEV:
-			fprintf(stderr, "KLIPS not loaded or enabled.\n");
-			break;
-		case EBUSY:
-			fprintf(stderr, "KLIPS is busy.  Most likely a serious internal error occured in a previous command.  Please report as much detail as possible to development team.\n");
-			break;
-		default:
-			fprintf(stderr, "Unknown file open error %d.  Please report as much detail as possible to development team.\n", errno);
-		}
-		exit(1);
-	}
 
 	if(debug) {
-		printf("%s: DEBUG: open ok\n", program_name);
-	}
-
-	if (write(fd, &emh, sizeof emh) != sizeof emh) {
-		fprintf(stderr, "%s: Failed -- ", program_name);
-		switch(errno) {
-		case EINVAL:
-			fprintf(stderr, "Invalid argument, check kernel log messages for specifics.\n");
-			break;
-		case ENXIO:
-			if((emh.em_type == EMT_SETEROUTE) ||
-			   (emh.em_type == EMT_RPLACEROUTE)) {
-				fprintf(stderr, "Invalid mask.\n");
-				break;
-			}
-			if(emh.em_type == EMT_DELEROUTE) {
-				fprintf(stderr, "Mask not found.\n");
-				break;
-			}
-		case EFAULT:
-			if((emh.em_type == EMT_SETEROUTE) ||
-			   (emh.em_type == EMT_RPLACEROUTE)) {
-				fprintf(stderr, "Invalid address.\n");
-				break;
-			}
-			if(emh.em_type == EMT_DELEROUTE) {
-				fprintf(stderr, "Address not found.\n");
-				break;
-			}
-		case ECONNREFUSED:
-			fprintf(stderr, "Connection refused, Is KLIPS, the ipsec module, loaded?\n");
-			break;
-		default:
-			fprintf(stderr, "Unknown file write error %d.  Please report as much detail as possible to development team.\n", errno);
-		}
-		close(fd);
-		exit(1);
-	}
-	close(fd);
-#endif /* CONFIG_IPSEC_PFKEYv2 */
-
-	if(debug) {
-		printf("%s: DEBUG: write ok\n", program_name);
+		fprintf(stdout, "%s: DEBUG: write ok\n", program_name);
 	}
 
 	exit(0);
 }
 /*
  * $Log: eroute.c,v $
+ * Revision 1.48  2002/02/15 19:54:11  rgb
+ * Purged dead code.
+ *
+ * Revision 1.47  2001/11/09 01:42:36  rgb
+ * Re-formatted usage text for clarity.
+ *
+ * Revision 1.46  2001/10/02 17:03:45  rgb
+ * Check error return for all "tto*" calls and report errors.  This, in
+ * conjuction with the fix to "tto*" will detect AF not set.
+ *
+ * Revision 1.45  2001/09/07 22:12:27  rgb
+ * Added EAFNOSUPPORT error return explanation for KLIPS not loaded.
+ *
+ * Revision 1.44  2001/07/06 19:49:33  rgb
+ * Renamed EMT_RPLACEROUTE to EMT_REPLACEROUTE for clarity and logical text
+ * searching.
+ * Added EMT_INEROUTE for supporting incoming policy checks.
+ * Added inbound policy checking code for IPIP SAs.
+ *
+ * Revision 1.43  2001/06/15 05:02:05  rgb
+ * Fixed error return messages and codes.
+ *
+ * Revision 1.42  2001/06/14 19:35:14  rgb
+ * Update copyright date.
+ *
+ * Revision 1.41  2001/05/21 02:02:54  rgb
+ * Eliminate 1-letter options.
+ *
+ * Revision 1.40  2001/05/16 04:39:57  rgb
+ * Fix --label option to add to command name rather than replace it.
+ * Fix 'print table' option to ignore --label and --debug options.
+ *
+ * Revision 1.39  2001/02/26 19:59:03  rgb
+ * Added a number of missing ntohl() conversions for debug output.
+ * Implement magic SAs %drop, %reject, %trap, %hold, %pass as part
+ * of the new SPD and to support opportunistic.
+ * Enforced spi > 0x100 requirement, now that pass uses a magic SA.
+ *
+ * Revision 1.38  2000/09/17 18:56:48  rgb
+ * Added IPCOMP support.
+ *
+ * Revision 1.37  2000/09/12 22:36:08  rgb
+ * Gerhard's IPv6 support.
+ * Restructured to remove unused extensions from CLEARFLOW messages.
+ * Added debugging.
+ *
+ * Revision 1.36  2000/09/08 19:17:31  rgb
+ * Removed all references to CONFIG_IPSEC_PFKEYv2.
+ *
+ * Revision 1.35  2000/08/27 01:46:52  rgb
+ * Update copyright dates and remove no longer used resolve_ip().
+ *
+ * Revision 1.34  2000/07/26 03:41:45  rgb
+ * Changed all printf's to fprintf's.  Fixed tncfg's usage to stderr.
+ *
+ * Revision 1.33  2000/07/13 21:54:49  rgb
+ * Remove old cruft from a time when libfreeswan didn't exist and I checked
+ * name lookup errors with the default address.
+ *
  * Revision 1.32  2000/06/21 16:51:27  rgb
  * Added no additional argument option to usage text.
  *
