@@ -1,7 +1,8 @@
 /*
  * control KLIPS debugging options
  * Copyright (C) 1996  John Ioannidis.
- * Copyright (C) 1998, 1999  Richard Guy Briggs.
+ * Copyright (C) 1998, 1999, 2000, 2001  Richard Guy Briggs <rgb@freeswan.org>
+ *                                 2001  Michael Richardson <mcr@freeswan.org>
  * 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -14,7 +15,7 @@
  * for more details.
  */
 
-char klipsdebug_c_version[] = "RCSID $Id: klipsdebug.c,v 1.32 2000/06/28 05:53:09 rgb Exp $";
+char klipsdebug_c_version[] = "RCSID $Id: klipsdebug.c,v 1.44 2001/11/23 07:23:14 mcr Exp $";
 
 
 #include <sys/types.h>
@@ -34,24 +35,20 @@ char klipsdebug_c_version[] = "RCSID $Id: klipsdebug.c,v 1.32 2000/06/28 05:53:0
 
 #include <unistd.h>
 #include <freeswan.h>
+#if 0
 #include <linux/autoconf.h>	/* CONFIG_IPSEC_PFKEYv2 */
+#endif
 /* permanently turn it on since netlink support has been disabled */
-#ifndef CONFIG_IPSEC_PFKEYv2
-     #define CONFIG_IPSEC_PFKEYv2
-#endif /* CONFIG_IPSEC_PFKEYv2 */
-#ifdef CONFIG_IPSEC_PFKEYv2
      #include <signal.h>
      #include <pfkeyv2.h>
      #include <pfkey.h>
-#endif /* CONFIG_IPSEC_PFKEYv2 */
 #include "radij.h"
 #include "ipsec_encap.h"
-#ifndef DEBUG_IPSEC
-#define DEBUG_IPSEC
-#endif /* DEBUG_IPSEC */
+#ifndef CONFIG_IPSEC_DEBUG
+#define CONFIG_IPSEC_DEBUG
+#endif /* CONFIG_IPSEC_DEBUG */
 #include "ipsec_netlink.h"
 #include "ipsec_tunnel.h"
-#include "version.c"
 
 #include <stdio.h>
 #include <getopt.h>
@@ -59,12 +56,10 @@ char klipsdebug_c_version[] = "RCSID $Id: klipsdebug.c,v 1.32 2000/06/28 05:53:0
 __u32 bigbuf[1024];
 char *program_name;
 
-#ifdef CONFIG_IPSEC_PFKEYv2
 extern unsigned int pfkey_lib_debug; /* used by libfreeswan/pfkey_v2_build */
 int pfkey_sock;
 fd_set pfkey_socks;
 uint32_t pfkey_seq = 0;
-#endif /* CONFIG_IPSEC_PFKEYv2 */
 
 char copyright[] =
 "Copyright (C) 1999 Henry Spencer, Richard Guy Briggs, D. Hugh Redelmeier,\n\
@@ -83,12 +78,13 @@ char copyright[] =
 static void
 usage(char * arg)
 {
-	fprintf(stdout, "usage: %s {--set|--clear} {tunnel|tunnel-xmit|xform|eroute|spi|radij|esp|ah|rcv}\n", arg);
+	fprintf(stdout, "usage: %s {--set|--clear} {tunnel|tunnel-xmit|netlink|xform|eroute|spi|radij|esp|ah|rcv|pfkey|ipcomp|verbose}\n", arg);
 	fprintf(stdout, "       %s {--all|--none}\n", arg);
 	fprintf(stdout, "       %s --help\n", arg);
 	fprintf(stdout, "       %s --version\n", arg);
 	fprintf(stdout, "       %s\n", arg);
 	fprintf(stdout, "        [ --debug ] is optional to any %s command\n", arg);
+	fprintf(stdout, "        [ --label <label> ] is optional to any %s command.\n", arg);
 	exit(1);
 }
 
@@ -117,31 +113,22 @@ main(int argc, char **argv)
 	
 	struct encap_msghdr *em = (struct encap_msghdr *)bigbuf;
 	int debug = 0;
-#ifdef CONFIG_IPSEC_PFKEYv2
 	int error = 0;
+	int argcount = argc;
 
 	struct sadb_ext *extensions[SADB_EXT_MAX + 1];
 	struct sadb_msg *pfkey_msg;
-#else /* CONFIG_IPSEC_PFKEYv2 */
-	int fd = 0;
-#endif /* CONFIG_IPSEC_PFKEYv2 */
 	
 	bitfieldp = (int *)&em->em_db_tn;
 
-	if(argc == 1) {
-		system("cat /proc/net/ipsec_klipsdebug");
-		exit(0);
-	}
-
 	program_name = argv[0];
 
-	while((c = getopt_long(argc, argv, "s:c:anhvl:+:d", longopts, 0)) != EOF) {
+	while((c = getopt_long(argc, argv, ""/*"s:c:anhvl:+:d"*/, longopts, 0)) != EOF) {
 		switch(c) {
 		case 'd':
 			debug = 1;
-#ifdef CONFIG_IPSEC_PFKEYv2
 			pfkey_lib_debug = 1;
-#endif /* CONFIG_IPSEC_PFKEYv2 */
+			argcount--;
 			break;
 		case 's':
 			if(action) {
@@ -173,10 +160,12 @@ main(int argc, char **argv)
 				em->em_db_ah = -1L;
 			} else if(strcmp(optarg, "rcv") == 0) {
 				em->em_db_rx = -1L;
-#ifdef CONFIG_IPSEC_PFKEYv2
 			} else if(strcmp(optarg, "pfkey") == 0) {
 				em->em_db_ky = -1L;
-#endif
+			} else if(strcmp(optarg, "comp") == 0) {
+				em->em_db_gz = -1L;
+			} else if(strcmp(optarg, "verbose") == 0) {
+				em->em_db_vb = -1L;
 			} else {
 				usage(program_name);
 			}
@@ -192,31 +181,33 @@ main(int argc, char **argv)
 			for (i = 0; i < (sizeof(em->Eu.Dbg)/sizeof(int)); i++) {
 				*(bitfieldp++) = -1;
 			}
-			if(strcmp(optarg, "tunnel") == 0){
+			if(strcmp(optarg, "tunnel") == 0) {
 				em->em_db_tn = 0;
-			}else if(strcmp(optarg, "tunnel-xmit") == 0){
+			} else if(strcmp(optarg, "tunnel-xmit") == 0) {
 				em->em_db_tn = ~DB_TN_XMIT;
-			}else if(strcmp(optarg, "netlink") == 0){
+			} else if(strcmp(optarg, "netlink") == 0) {
 				em->em_db_nl = 0;
-			}else if(strcmp(optarg, "xform") == 0){
+			} else if(strcmp(optarg, "xform") == 0) {
 				em->em_db_xf = 0;
-			}else if(strcmp(optarg, "eroute") == 0){
+			} else if(strcmp(optarg, "eroute") == 0) {
 				em->em_db_er = 0;
-			}else if(strcmp(optarg, "spi") == 0){
+			} else if(strcmp(optarg, "spi") == 0) {
 				em->em_db_sp = 0;
-			}else if(strcmp(optarg, "radij") == 0){
+			} else if(strcmp(optarg, "radij") == 0) {
 				em->em_db_rj = 0;
-			}else if(strcmp(optarg, "esp") == 0){
+			} else if(strcmp(optarg, "esp") == 0) {
 				em->em_db_es = 0;
-			}else if(strcmp(optarg, "ah") == 0){
+			} else if(strcmp(optarg, "ah") == 0) {
 				em->em_db_ah = 0;
-			}else if(strcmp(optarg, "rcv") == 0){
+			} else if(strcmp(optarg, "rcv") == 0) {
 				em->em_db_rx = 0;
-#ifdef CONFIG_IPSEC_PFKEYv2
-			}else if(strcmp(optarg, "pfkey") == 0){
+			} else if(strcmp(optarg, "pfkey") == 0) {
 				em->em_db_ky = 0;
-#endif
-			}else{
+			} else if(strcmp(optarg, "comp") == 0) {
+				em->em_db_gz = 0;
+			} else if(strcmp(optarg, "verbose") == 0) {
+				em->em_db_vb = 0;
+			} else {
 				usage(program_name);
 			}
 			em->em_db_nl &= ~(1 << (sizeof(em->em_db_nl) * 8 -1));
@@ -228,9 +219,10 @@ main(int argc, char **argv)
 				exit(1);
 			}
 			action = 'a';
-			for (i = 0; i < (sizeof(em->Eu.Dbg)/sizeof(int)); i++) {
+			for (i = 0; i < (sizeof(em->Eu.Dbg)/sizeof(int)) - 1; i++) {
 				*(bitfieldp++) = -1;
 			}
+			*(bitfieldp++) = 0;
 			break;
 		case 'n':
 			if(action) {
@@ -248,12 +240,18 @@ main(int argc, char **argv)
 			usage(program_name);
 			exit(1);
 		case 'v':
-			printf("klipsdebug (Linux FreeS/WAN %s) %s\n",
-				freeswan_version, klipsdebug_c_version);
+			fprintf(stdout, "klipsdebug (Linux FreeS/WAN %s) %s\n",
+				ipsec_version_code(), klipsdebug_c_version);
 			fputs(copyright, stdout);
 			exit(0);
 		case 'l':
-			program_name = optarg;
+			program_name = malloc(strlen(argv[0])
+					      + 10 /* update this when changing the sprintf() */
+					      + strlen(optarg));
+			sprintf(program_name, "%s --label %s",
+				argv[0],
+				optarg);
+			argcount -= 2;
 			break;
 		case '+': /* optionsfrom */
 			optionsfrom(optarg, &argc, &argv, optind, stderr);
@@ -264,6 +262,11 @@ main(int argc, char **argv)
 		previous = c;
 	}
 
+	if(argcount == 1) {
+		system("cat /proc/net/ipsec_klipsdebug");
+		exit(0);
+	}
+
  	em->em_magic = EM_MAGIC;
 	em->em_version = 0;
 	if(action) {
@@ -277,7 +280,6 @@ main(int argc, char **argv)
 	}
 	em->em_msglen = EMT_SETDEBUG_FLEN;
 
-#ifdef CONFIG_IPSEC_PFKEYv2
 	if((pfkey_sock = socket(PF_KEY, SOCK_RAW, PF_KEY_V2) ) < 0) {
 		fprintf(stderr, "%s: Trouble openning PF_KEY family socket with error: ",
 			program_name);
@@ -317,6 +319,9 @@ main(int argc, char **argv)
 		case ENXIO:
 			fprintf(stderr, "SA does not exist.  Cannot delete.\n");
 			break;
+		case EAFNOSUPPORT:
+			fprintf(stderr, "KLIPS not loaded or enabled.\n");
+			break;
 		default:
 			fprintf(stderr, "Unknown file open error %d.  Please report as much detail as possible to development team.\n", errno);
 		}
@@ -347,7 +352,9 @@ main(int argc, char **argv)
 					em->em_db_es,
 					em->em_db_ah,
 					em->em_db_rx,
-					em->em_db_ky))) {
+					em->em_db_ky,
+					em->em_db_gz,
+					em->em_db_vb))) {
 		fprintf(stderr, "%s: Trouble building message header, error=%d.\n",
 			program_name, error);
 		pfkey_extensions_free(extensions);
@@ -419,106 +426,53 @@ main(int argc, char **argv)
 	}
 
 	(void) close(pfkey_sock);  /* close the socket */
-#else /* CONFIG_IPSEC_PFKEYv2 */
-
-	fd = open("/dev/ipsec", 2);
-	if (fd < 0) {
-		fprintf(stderr, "%s: Could not open /dev/ipsec -- ", argv[0]);
-		switch(errno){
-		case ENOENT:
-			fprintf(stderr, "device does not exist.  See FreeS/WAN installation procedure.\n");
-		case EACCES:
-			fprintf(stderr, "access denied.  ");
-			if(getuid() == 0){
-				fprintf(stderr, "Check permissions.  Should be 600.\n");
-			}else{
-				fprintf(stderr, "You must be root to open this file.\n");
-			}
-			break;
-		case EUNATCH:
-			fprintf(stderr, "Netlink not enabled OR KLIPS not loaded.\n");
-			break;
-		case ENODEV:
-			fprintf(stderr, "KLIPS not loaded or enabled.\n");
-			break;
-		case EBUSY:
-			fprintf(stderr, "KLIPS is busy.  Most likely a serious internal error occured in a previous command.  Please report as much detail as possible to development team.\n");
-		default:
-			fprintf(stderr, "Unknown file open error %d\n", errno);
-		}
-		exit(1);
-	}
-	if(action) {
-		if (write(fd, (caddr_t)em, em->em_msglen) != em->em_msglen){
-			fprintf(stderr, "%s: Had trouble writing to /dev/ipsec -- ", argv[0]);
-			switch(errno){
-			case EINVAL:
-				fprintf(stderr, "Invalid argument, check kernel log messages for specifics.\n");
-				break;
-			case ECONNREFUSED:
-				fprintf(stderr, "Connection refused, Is KLIPS, the ipsec module, loaded?.\n");
-				break;
-			default:
-				fprintf(stderr, "Unknown file write error %d\n", errno);
-			}
-			close(fd);
-			exit(1);
-		}
-	} else {
-		if (read(fd, (caddr_t)em, em->em_msglen) != em->em_msglen){
-			fprintf(stderr, "%s: Had trouble reading from /dev/ipsec -- ", argv[0]);
-			switch(errno){
-			case EINVAL:
-				fprintf(stderr, "Invalid argument, check kernel log messages for specifics.\n");
-				break;
-			case ECONNREFUSED:
-				fprintf(stderr, "Connection refused, check kernel log messages for specifics.\n");
-				break;
-			default:
-				fprintf(stderr, "Unknown file write error %d\n", errno);
-			}
-			close(fd);
-			exit(1);
-		}
-		if(em->em_db_tn) {
-			printf("tunnel=0x%08x\n", em->em_db_tn);
-		}
-		if(em->em_db_nl) {
-			printf("netlink=0x%08x\n", em->em_db_nl);
-		}
-		if(em->em_db_xf) {
-			printf("xform=0x%08x\n", em->em_db_xf);
-		}
-		if(em->em_db_er) {
-			printf("eroute=0x%08x\n", em->em_db_er);
-		}
-		if(em->em_db_sp) {
-			printf("spi=0x%08x\n", em->em_db_sp);
-		}
-		if(em->em_db_rj) {
-			printf("radij=0x%08x\n", em->em_db_rj);
-		}
-		if(em->em_db_es) {
-			printf("esp=0x%08x\n", em->em_db_es);
-		}
-		if(em->em_db_ah) {
-			printf("ah=0x%08x\n", em->em_db_ah);
-		}
-		if(em->em_db_rx) {
-			printf("rcv=0x%08x\n", em->em_db_rx);
-		}
-#ifdef CONFIG_IPSEC_PFKEYv2
-		if(em->em_db_ky) {
-			printf("pfkey=0x%08x\n", em->em_db_ky);
-		}
-#endif /* CONFIG_IPSEC_PFKEYv2 */
-	}
-	close(fd);
-#endif /* CONFIG_IPSEC_PFKEYv2 */
 	exit(0);
 }
 /*
  * $Log: klipsdebug.c,v $
+ * Revision 1.44  2001/11/23 07:23:14  mcr
+ * 	pulled up klips2 Makefile and pf_key code.
+ *
+ * Revision 1.43  2001/11/22 05:44:01  henry
+ * new version stuff
+ *
+ * Revision 1.42.2.1  2001/10/13 18:22:21  mcr
+ * 	usage string was missing "netlink" and "pf_key" debug options.
+ *
+ * Revision 1.42  2001/09/07 22:24:07  rgb
+ * Added EAFNOSUPPORT socket open error code in case KLIPS is not loaded.
+ *
+ * Revision 1.41  2001/06/14 19:35:14  rgb
+ * Update copyright date.
+ *
+ * Revision 1.40  2001/05/21 02:02:54  rgb
+ * Eliminate 1-letter options.
+ *
+ * Revision 1.39  2001/05/16 05:07:19  rgb
+ * Fixed --label option in KLIPS manual utils to add the label to the
+ * command name rather than replace it in error text.
+ * Fix 'print table' non-option in KLIPS manual utils to deal with --label
+ * and --debug options.
+ *
+ * Revision 1.38  2000/10/11 03:56:54  rgb
+ * Initialise verbose field to zero on --all.
+ *
+ * Revision 1.37  2000/10/11 03:48:44  henry
+ * add a couple of overlooked parameters to a call
+ *
+ * Revision 1.36  2000/10/10 20:10:19  rgb
+ * Added support for debug_ipcomp and debug_verbose to klipsdebug.
+ *
+ * Revision 1.35  2000/09/08 19:16:51  rgb
+ * Change references from DEBUG_IPSEC to CONFIG_IPSEC_DEBUG.
+ * Removed all references to CONFIG_IPSEC_PFKEYv2.
+ *
+ * Revision 1.34  2000/08/27 01:48:30  rgb
+ * Update copyright.
+ *
+ * Revision 1.33  2000/07/26 03:41:46  rgb
+ * Changed all printf's to fprintf's.  Fixed tncfg's usage to stderr.
+ *
  * Revision 1.32  2000/06/28 05:53:09  rgb
  * Mention that netlink is obsolete.
  *
