@@ -1,7 +1,7 @@
 /*
  * SA grouping
  * Copyright (C) 1996  John Ioannidis.
- * Copyright (C) 1998, 1999  Richard Guy Briggs.
+ * Copyright (C) 1997, 1998, 1999, 2000, 2001  Richard Guy Briggs.
  * 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -14,7 +14,7 @@
  * for more details.
  */
 
-char spigrp_c_version[] = "RCSID $Id: spigrp.c,v 1.30 2000/06/21 16:51:27 rgb Exp $";
+char spigrp_c_version[] = "RCSID $Id: spigrp.c,v 1.40 2001/10/02 17:17:17 rgb Exp $";
 
 
 #include <sys/types.h>
@@ -35,102 +35,73 @@ char spigrp_c_version[] = "RCSID $Id: spigrp.c,v 1.30 2000/06/21 16:51:27 rgb Ex
 #include <stdio.h>
 #include <netdb.h>
 #include <freeswan.h>
+#if 0
 #include <linux/autoconf.h>	/* CONFIG_IPSEC_PFKEYv2 */
-/* permanently turn it on since netlink support has been disabled */
-#ifndef CONFIG_IPSEC_PFKEYv2
-     #define CONFIG_IPSEC_PFKEYv2
-#endif /* CONFIG_IPSEC_PFKEYv2 */
-#ifdef CONFIG_IPSEC_PFKEYv2
+#endif
      #include <signal.h>
      #include <pfkeyv2.h>
      #include <pfkey.h>
-#endif /* CONFIG_IPSEC_PFKEYv2 */
 #include "radij.h"
 #include "ipsec_encap.h"
 #include "ipsec_netlink.h"
 #include "ipsec_ah.h"
 
 
-__u32 bigbuf[1024];
 char *program_name;
 
-#ifdef CONFIG_IPSEC_PFKEYv2
 extern unsigned int pfkey_lib_debug; /* used by libfreeswan/pfkey_v2_build */
 int pfkey_sock;
 fd_set pfkey_socks;
 uint32_t pfkey_seq = 0;
-#endif /* CONFIG_IPSEC_PFKEYv2 */
-
-#if 0
-static int
-resolve_ip(char *arg, struct in_addr *ip)
-{
-	struct hostent *hostp;
-	struct netent *netp;
-	
-	if (!strcmp(arg, "default")) {/* default is really 0.0.0.0 */
-		ip->s_addr = htonl(INADDR_ANY);
-		return(1);
-	}
-	if (!(strcmp(arg, "255.255.255.255"))) {
-		/* inet_addr() returns -1 for this, as it should, but that means an error */
-		ip->s_addr = htonl(INADDR_BROADCAST);
-		return (1);
-	}
-	if ((netp = getnetbyname(arg)) != (struct netent *)NULL) {
-		ip->s_addr = htonl(netp->n_net);
-		return(1);
-	}
-	if ((hostp = gethostbyname(arg)) == (struct hostent *)NULL) {
-		errno = h_errno;
-		return(-1);
-	}
-	memcpy((char *) ip, (char *) hostp->h_addr_list[0], (unsigned int)hostp->h_length);
-	return(0);
-}
-#endif
+ 
+struct said_af {
+ 	int af;
+ 	ip_said said;
+}; /* to store the given saids and their address families in an array */
+ /* XXX: Note that we do *not* check if the address families of all SAID?s are the same.
+  *      This can make it possible to group SAs for IPv4 addresses with SAs for
+  *      IPv6 addresses (perhaps some kind of IPv4-over-secIPv6 or vice versa).
+  *      Do not know, if this is a bug or feature */
 
 static void
 usage(char *s)
 {
 	fprintf(stdout, "usage: Note: position of options and arguments is important!\n");
-	fprintf(stdout, "usage: %s [ --debug ] [ --label <label> ] dst1 spi1 proto1 [ dst2 spi2 proto2 [ dst3 spi3 proto3 [ dst4 spi4 proto4 ] ] ]\n", s);
+	fprintf(stdout, "usage: %s [ --debug ] [ --label <label> ] af1 dst1 spi1 proto1 [ af2 dst2 spi2 proto2 [ af3 dst3 spi3 proto3 [ af4 dst4 spi4 proto4 ] ] ]\n", s);
 	fprintf(stdout, "usage: %s [ --debug ] [ --label <label> ] --said <SA1> [ <SA2> [ <SA3> [ <SA4> ] ] ]\n", s);
 	fprintf(stdout, "usage: %s --help\n", s);
 	fprintf(stdout, "usage: %s --version\n", s);
 	fprintf(stdout, "usage: %s\n", s);
+	fprintf(stdout, "        [ --debug ] is optional to any %s command.\n", s);
+	fprintf(stdout, "        [ --label <label> ] is optional to any %s command.\n", s);
 }
 
 	
 int
 main(int argc, char **argv)
 {
-/*	int fd; */
-/*	int ret; */
 	int i, nspis;
 	char *endptr;
 	int said_opt = 0;
 
-	struct encap_msghdr *em = (struct encap_msghdr *)bigbuf;
-	const char* error_s;
-	char ipaddr_txt[ADDRTOA_BUF];                
+	const char* error_s = NULL;
+	char ipaddr_txt[ADDRTOT_BUF];
 	int debug = 0;
 	int j;
+	struct said_af said_af_array[4];
 
-#ifdef CONFIG_IPSEC_PFKEYv2
 	int error = 0;
 
 	struct sadb_ext *extensions[SADB_EXT_MAX + 1];
 	struct sadb_msg *pfkey_msg;
 #if 0
-	struct sockaddr_in pfkey_address_s_ska;
+	ip_address pfkey_address_s_ska;
 #endif
-	struct sockaddr_in pfkey_address_d_ska;
-#else /* CONFIG_IPSEC_PFKEYv2 */
-	int fd = 0;
-#endif /* CONFIG_IPSEC_PFKEYv2 */
 	
 	program_name = argv[0];
+	for(i = 0; i < 4; i++) {
+		memset(&said_af_array[i], 0, sizeof(struct said_af));
+	}
 
         if(argc > 1 && strcmp(argv[1], "--debug") == 0) {
 		debug = 1;
@@ -139,9 +110,7 @@ main(int argc, char **argv)
 		}
 		argv += 1;
 		argc -= 1;
-#ifdef CONFIG_IPSEC_PFKEYv2
 		pfkey_lib_debug = 1;
-#endif /* CONFIG_IPSEC_PFKEYv2 */
         }
 
 	if(debug) {
@@ -152,7 +121,12 @@ main(int argc, char **argv)
 
         if(argc > 1 && strcmp(argv[1], "--label") == 0) {
 		if(argc > 2) {
-			program_name = argv[2];
+			program_name = malloc(strlen(argv[0])
+					      + 10 /* update this when changing the sprintf() */
+					      + strlen(argv[2]));
+			sprintf(program_name, "%s --label %s",
+				argv[0],
+				argv[2]);
 			if(debug) {
 				fprintf(stdout, "using \"%s\" as a label.\n", program_name);
 			}
@@ -221,106 +195,93 @@ main(int argc, char **argv)
 		}
 		nspis = argc - 2;
 	} else {
-		if ((argc < 4) || (argc > 13) || ((argc % 3) != 1)) {
-			fprintf(stderr, "expecting 4 or more args without --said, got %d.\n", argc);
+		if ((argc < 5) || (argc > 17) || ((argc % 4) != 1)) {
+			fprintf(stderr, "expecting 5 or more args without --said, got %d.\n", argc);
 			usage(program_name);
                 	exit(1);
 		}
-		nspis = argc / 3;
+		nspis = argc / 4;
 	}
 
 	if(debug) {
 		fprintf(stdout, "processing %d nspis.\n", nspis);
 	}
 
-	em->em_magic = EM_MAGIC;
-	em->em_version = 0;
-	if(nspis == 1)
-		em->em_type = EMT_UNGRPSPIS;
-	else
-		em->em_type = EMT_GRPSPIS;
-	em->em_msglen = EMT_GRPSPIS_FLEN +
-		nspis *
-		(sizeof(em->em_rel[0]));
 	for(i = 0; i < nspis; i++) {
 		if(debug) {
 			fprintf(stdout, "processing spi #%d.\n", i);
 		}
 
 		if(said_opt) {
-			if((error_s = atosa(argv[i+2], 0, &(em->em_rel[i].emr_said))) != NULL) {
+			error_s = ttosa((const char *)argv[i+2], 0, (ip_said*)&(said_af_array[i].said));
+			if(error_s != NULL) {
 				fprintf(stderr, "%s: Error, %s converting --sa argument:%s\n",
 					program_name, error_s, argv[i+2]);
 				exit (1);
 			}
+			said_af_array[i].af = addrtypeof(&(said_af_array[i].said.dst));
 			if(debug) {
-				addrtoa(em->em_rel[i].emr_said.dst, 0, ipaddr_txt, sizeof(ipaddr_txt));
+				addrtot(&said_af_array[i].said.dst, 0, ipaddr_txt, sizeof(ipaddr_txt));
 				fprintf(stdout, "said[%d].dst=%s.\n", i, ipaddr_txt);
 			}
 		} else {
-			if(!strcmp(argv[i*3+3], "ah")) {
-				em->em_rel[i].emr_proto = SA_AH;
+			if(!strcmp(argv[i*4+4], "ah")) {
+				said_af_array[i].said.proto = SA_AH;
 			}
-			if(!strcmp(argv[i*3+3], "esp")) {
-				em->em_rel[i].emr_proto = SA_ESP;
+			if(!strcmp(argv[i*4+4], "esp")) {
+				said_af_array[i].said.proto = SA_ESP;
 			}
-			if(!strcmp(argv[i*3+3], "tun")) {
-				em->em_rel[i].emr_proto = SA_IPIP;
+			if(!strcmp(argv[i*4+4], "tun")) {
+				said_af_array[i].said.proto = SA_IPIP;
 			}
-			if(em->em_rel[i].emr_proto == 0) {
+			if(!strcmp(argv[i*4+4], "comp")) {
+				said_af_array[i].said.proto = SA_COMP;
+			}
+			if(said_af_array[i].said.proto == 0) {
 				fprintf(stderr, "%s: Badly formed proto: %s\n",
-					program_name, argv[i*3+3]);
+					program_name, argv[i*4+4]);
 				exit(1);
 			}
-			em->em_rel[i].emr_spi = htonl(strtoul(argv[i*3+2], &endptr, 0));
-			if(!(endptr == argv[i*3+2] + strlen(argv[i*3+2]))) {
+			said_af_array[i].said.spi = htonl(strtoul(argv[i*4+3], &endptr, 0));
+			if(!(endptr == argv[i*4+3] + strlen(argv[i*4+3]))) {
 				fprintf(stderr, "%s: Badly formed spi: %s\n",
-					program_name, argv[i*3+2]);
+					program_name, argv[i*4+3]);
 				exit(1);
 			}
-			if((error_s = atoaddr(argv[i*3+1], 0, &(em->em_rel[i].emr_dst))) != 0) {
-				if(error_s) {
-					fprintf(stderr, "%s: Error, %s converting %dth address argument:%s\n",
-						program_name, error_s, i, argv[i*3+1]);
-					exit (1);
-				}
+			if(!strcmp(argv[i*4+1], "inet")) {
+				said_af_array[i].af = AF_INET;
 			}
-#if 0
-			if((ret = resolve_ip(argv[i*3+1] /*optarg*/, &(em->em_rel[i].emr_dst))) != 0) {
-				if(ret == 1) {
-					fprintf(stderr, "%s: Destination address cannot be a network: %s\n",
-						program_name, argv[i*3+1]/*optarg*/);
-					exit (1);
-				} else {
-					fprintf(stderr, "%s: Invalid formation of destination address: %s\n",
-						program_name, argv[i*3+1]/*optarg*/);
-					exit (1);
-				}
+			if(!strcmp(argv[i*4+1], "inet6")) {
+				said_af_array[i].af = AF_INET6;
+			}
+			if((said_af_array[i].af != AF_INET) && (said_af_array[i].af != AF_INET6)) {
+				fprintf(stderr, "%s: Address family %s not supported\n",
+					program_name, argv[i*4+1]);
+				exit(1);
+			}
+			error_s = ttoaddr(argv[i*4+2], 0, said_af_array[i].af, &(said_af_array[i].said.dst));
+			if(error_s != NULL) {
+				fprintf(stderr, "%s: Error, %s converting %dth address argument:%s\n",
+					program_name, error_s, i, argv[i*4+2]);
+				exit (1);
 			}
-#endif
 		}
 		if(debug) {
-			fprintf(stdout, "SA (size:%d) %d contains: ",
-				(sizeof(struct sa_id) + sizeof(struct tdb*)), i+1);
-			for(j = 0; j < sizeof(struct sa_id); j++) {
-				fprintf(stdout, " %02x",
-					((char*)(em))
-					[8 + i * (sizeof(struct sa_id) + sizeof(struct tdb*)) + j]);
-			}
+			fprintf(stdout, "SA %d contains: ", i+1);
 			fprintf(stdout, "\n");
-			fprintf(stdout, "proto = %d\n", em->em_rel[i].emr_said.proto);
-			fprintf(stdout, "spi = %08x\n", em->em_rel[i].emr_said.spi);
-			fprintf(stdout, "edst = %08x\n", em->em_rel[i].emr_said.dst.s_addr);
+			fprintf(stdout, "proto = %d\n", said_af_array[i].said.proto);
+			fprintf(stdout, "spi = %08x\n", said_af_array[i].said.spi);
+			addrtot(&said_af_array[i].said.dst, 0, ipaddr_txt, sizeof(ipaddr_txt));
+			fprintf(stdout, "edst = %s\n", ipaddr_txt);
 		}
 	}	
 
-#ifdef CONFIG_IPSEC_PFKEYv2
 	if(debug) {
-		fprintf(stdout, "Openning pfkey socket.\n");
+		fprintf(stdout, "Opening pfkey socket.\n");
 	}
 
 	if((pfkey_sock = socket(PF_KEY, SOCK_RAW, PF_KEY_V2) ) < 0) {
-		fprintf(stderr, "%s: Trouble openning PF_KEY family socket with error: ",
+		fprintf(stderr, "%s: Trouble opening PF_KEY family socket with error: ",
 			program_name);
 		switch(errno) {
 		case ENOENT:
@@ -358,6 +319,9 @@ main(int argc, char **argv)
 		case ENXIO:
 			fprintf(stderr, "SA does not exist.  Cannot delete.\n");
 			break;
+		case EAFNOSUPPORT:
+			fprintf(stderr, "KLIPS not loaded or enabled.\n");
+			break;
 		default:
 			fprintf(stderr, "Unknown file open error %d.  Please report as much detail as possible to development team.\n", errno);
 		}
@@ -376,17 +340,11 @@ main(int argc, char **argv)
 			}
 
 			/* Build an SADB_X_GRPSA message to send down. */
-			/* It needs <base, SA, SA2, address(S,D,D2) > minimum. */
+			/* It needs <base, SA, SA2, address(D,D2) > minimum. */
 			if(!j) {
 				if((error = pfkey_msg_hdr_build(&extensions[0],
 								SADB_X_GRPSA,
-								proto2satype(em->em_rel[i].emr_said.proto),
-#if 0
-								(em->em_rel[i].emr_said.proto == SA_ESP ? SADB_SATYPE_ESP :
-								 (em->em_rel[i].emr_said.proto == SA_AH ? SADB_SATYPE_AH :
-								  (em->em_rel[i].emr_said.proto == SA_IPIP ? SADB_X_SATYPE_IPIP :
-								   SADB_SATYPE_UNSPEC))),
-#endif
+								proto2satype(said_af_array[i].said.proto),
 								0,
 								++pfkey_seq,
 								getpid()))) {
@@ -398,25 +356,13 @@ main(int argc, char **argv)
 			} else {
 				if(debug) {
 					fprintf(stdout, "setting x_satype proto=%d satype=%d\n",
-						em->em_rel[i+j].emr_said.proto,
-						proto2satype(em->em_rel[i+j].emr_said.proto)
-#if 0
-						(em->em_rel[i+j].emr_said.proto == SA_ESP ? SADB_SATYPE_ESP :
-						 (em->em_rel[i+j].emr_said.proto == SA_AH ? SADB_SATYPE_AH :
-						  (em->em_rel[i+j].emr_said.proto == SA_IPIP ? SADB_X_SATYPE_IPIP :
-						   SADB_SATYPE_UNSPEC)))
-#endif
+						said_af_array[i+j].said.proto,
+						proto2satype(said_af_array[i+j].said.proto)
 						);
 				}
 
 				if((error = pfkey_x_satype_build(&extensions[SADB_X_EXT_SATYPE2],
-								 proto2satype(em->em_rel[i+j].emr_said.proto)
-#if 0
-								 (em->em_rel[i+j].emr_said.proto == SA_ESP ? SADB_SATYPE_ESP :
-								  (em->em_rel[i+j].emr_said.proto == SA_AH ? SADB_SATYPE_AH :
-								   (em->em_rel[i+j].emr_said.proto == SA_IPIP ? SADB_X_SATYPE_IPIP :
-								    SADB_SATYPE_UNSPEC)))
-#endif
+								 proto2satype(said_af_array[i+j].said.proto)
 					))) {
 					fprintf(stderr, "%s: Trouble building message header, error=%d.\n",
 						program_name, error);
@@ -427,7 +373,7 @@ main(int argc, char **argv)
 
 			if((error = pfkey_sa_build(&extensions[!j ? SADB_EXT_SA : SADB_X_EXT_SA2],
 						   !j ? SADB_EXT_SA : SADB_X_EXT_SA2,
-						   em->em_rel[i+j].emr_said.spi, /* in network order */
+						   said_af_array[i+j].said.spi, /* in network order */
 						   0,
 						   0,
 						   0,
@@ -441,26 +387,13 @@ main(int argc, char **argv)
 			
 #if 0
 			if(!j) {
-				pfkey_address_s_ska.sin_addr.s_addr = 0;
-				pfkey_address_s_ska.sin_family = AF_INET;
-				pfkey_address_s_ska.sin_port = 0;
-				{
-					int k;
-					
-					for(k = 0;
-					    k < sizeof(struct sockaddr_in) -
-						    offsetof(struct sockaddr_in, sin_zero);
-					    k++) {
-						pfkey_address_s_ska.sin_zero[k] = 0;
-					}
-				}
+				anyaddr(said_af_array[i].af, &pfkey_address_s_ska); /* Is the address family correct ?? */
 				if((error = pfkey_address_build(&extensions[SADB_EXT_ADDRESS_SRC],
 								SADB_EXT_ADDRESS_SRC,
 								0,
 								0,
-								(struct sockaddr*)&pfkey_address_s_ska))) {
-					addrtoa(pfkey_address_s_ska.sin_addr,
-						0, ipaddr_txt, sizeof(ipaddr_txt));
+								sockaddrof(&pfkey_address_s_ska)))) {
+					addrtot(&pfkey_address_s_ska, 0, ipaddr_txt, sizeof(ipaddr_txt));
 					fprintf(stderr, "%s: Trouble building address_s extension (%s), error=%d.\n",
 						program_name, ipaddr_txt, error);
 					pfkey_extensions_free(extensions);
@@ -468,24 +401,12 @@ main(int argc, char **argv)
 				}
 			}
 #endif			
-			pfkey_address_d_ska.sin_addr = em->em_rel[i+j].emr_said.dst; /* dst */
-			pfkey_address_d_ska.sin_family = AF_INET;
-			pfkey_address_d_ska.sin_port = 0;
-			{
-				int k;
-				for(k = 0;
-				    k < sizeof(struct sockaddr_in) -
-					    offsetof(struct sockaddr_in, sin_zero);
-				    k++) {
-					pfkey_address_d_ska.sin_zero[k] = 0;
-				}
-			}
 			if((error = pfkey_address_build(&extensions[!j ? SADB_EXT_ADDRESS_DST : SADB_X_EXT_ADDRESS_DST2],
 							!j ? SADB_EXT_ADDRESS_DST : SADB_X_EXT_ADDRESS_DST2,
 							0,
 							0,
-							(struct sockaddr*)&pfkey_address_d_ska))) {
-				addrtoa(pfkey_address_d_ska.sin_addr,
+							sockaddrof(&said_af_array[i+j].said.dst)))) {
+				addrtot(&said_af_array[i+j].said.dst,
 					0, ipaddr_txt, sizeof(ipaddr_txt));
 				fprintf(stderr, "%s: Trouble building address_d extension (%s), error=%d.\n",
 					program_name, ipaddr_txt, error);
@@ -560,60 +481,44 @@ main(int argc, char **argv)
 	}
 
 	(void) close(pfkey_sock);  /* close the socket */
-#else /* CONFIG_IPSEC_PFKEYv2 */
-
-	fd = open("/dev/ipsec", 2);
-	if (fd < 0) {
-		fprintf(stderr, "%s: Could not open /dev/ipsec -- ", program_name);
-		switch(errno) {
-		case ENOENT:
-			fprintf(stderr, "device does not exist.  See FreeS/WAN installation procedure.\n");
-		case EACCES:
-			fprintf(stderr, "access denied.  ");
-			if(getuid() == 0) {
-				fprintf(stderr, "Check permissions.  Should be 600.\n");
-			} else {
-				fprintf(stderr, "You must be root to open this file.\n");
-			}
-			break;
-		case EUNATCH:
-			fprintf(stderr, "Netlink not enabled OR KLIPS not loaded.\n");
-			break;
-		case ENODEV:
-			fprintf(stderr, "KLIPS not loaded or enabled.\n");
-			break;
-		case EBUSY:
-			fprintf(stderr, "KLIPS is busy.  Most likely a serious internal error occured in a previous command.  Please report as much detail as possible to development team.\n");
-		default:
-			fprintf(stderr, "Unknown file open error %d\n", errno);
-		}
-		exit(1);
-	}
-	if (write(fd, (caddr_t)em, em->em_msglen) != em->em_msglen) {
-		close(fd);
-		fprintf(stderr, "%s: Had trouble writing to /dev/ipsec -- ", program_name);
-		switch(errno) {
-		case EINVAL:
-			fprintf(stderr, "Invalid argument, check kernel log messages for specifics.\n");
-			break;
-		case ENXIO:
-			fprintf(stderr, "SA not found.  Check kernel log messages for specifics.\n");
-			break;
-		case EBUSY:
-			fprintf(stderr, "SA in use.  Check kernel log messages for specifics.\n");
-			break;
-		default:
-			fprintf(stderr, "Unknown file write error %d\n", errno);
-		}
-		close(fd);
-		exit(1);
-	}
-	close(fd);
-#endif /* CONFIG_IPSEC_PFKEYv2 */
 	exit(0);
 }
 /*
  * $Log: spigrp.c,v $
+ * Revision 1.40  2001/10/02 17:17:17  rgb
+ * Check error return for all "tto*" calls and report errors.  This, in
+ * conjuction with the fix to "tto*" will detect AF not set.
+ *
+ * Revision 1.39  2001/09/07 22:24:42  rgb
+ * Added EAFNOSUPPORT socket open error code in case KLIPS is not loaded.
+ *
+ * Revision 1.38  2001/06/14 19:35:15  rgb
+ * Update copyright date.
+ *
+ * Revision 1.37  2001/05/16 05:07:20  rgb
+ * Fixed --label option in KLIPS manual utils to add the label to the
+ * command name rather than replace it in error text.
+ * Fix 'print table' non-option in KLIPS manual utils to deal with --label
+ * and --debug options.
+ *
+ * Revision 1.36  2001/01/23 20:24:12  rgb
+ * Fix comment to reflect reality that src is not needed for grouping.
+ *
+ * Revision 1.35  2000/09/17 18:56:48  rgb
+ * Added IPCOMP support.
+ *
+ * Revision 1.34  2000/09/16 04:56:32  rgb
+ * Added Svenning's ipcomp patch.
+ *
+ * Revision 1.33  2000/09/12 22:36:45  rgb
+ * Gerhard's IPv6 support.
+ *
+ * Revision 1.32  2000/09/08 19:17:31  rgb
+ * Removed all references to CONFIG_IPSEC_PFKEYv2.
+ *
+ * Revision 1.31  2000/08/27 01:46:52  rgb
+ * Update copyright dates and remove no longer used resolve_ip().
+ *
  * Revision 1.30  2000/06/21 16:51:27  rgb
  * Added no additional argument option to usage text.
  *
