@@ -1,7 +1,7 @@
 /*
  * IPSEC interface configuration
  * Copyright (C) 1996  John Ioannidis.
- * Copyright (C) 1998, 1999  Richard Guy Briggs.
+ * Copyright (C) 1998, 1999, 2000, 2001  Richard Guy Briggs.
  * 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -14,7 +14,7 @@
  * for more details.
  */
 
-char tncfg_c_version[] = "RCSID $Id: tncfg.c,v 1.21 2000/06/21 16:51:27 rgb Exp $";
+char tncfg_c_version[] = "RCSID $Id: tncfg.c,v 1.27 2001/06/14 19:35:15 rgb Exp $";
 
 
 #include <stdio.h>
@@ -43,7 +43,7 @@ char tncfg_c_version[] = "RCSID $Id: tncfg.c,v 1.21 2000/06/21 16:51:27 rgb Exp
 static void
 usage(char *name)
 {	
-	fprintf(stdout,"%s --attach --virtual <virtual-device> --physical <real-device>\n",
+	fprintf(stdout,"%s --attach --virtual <virtual-device> --physical <physical-device>\n",
 		name);
 	fprintf(stdout,"%s --detach --virtual <virtual-device>\n",
 		name);
@@ -55,6 +55,8 @@ usage(char *name)
 		name);
 	fprintf(stdout,"%s\n",
 		name);
+	fprintf(stdout, "        [ --debug ] is optional to any %s command.\n", name);
+	fprintf(stdout, "        [ --label <label> ] is optional to any %s command.\n", name);
 	exit(1);
 }
 
@@ -82,19 +84,16 @@ main(int argc, char *argv[])
 	int c, previous = -1;
 	char *program_name;
 	int debug = 0;
+	int argcount = argc;
      
-	if(argc == 1) {
-		system("cat /proc/net/ipsec_tncfg");
-		exit(0);
-	}
-
 	memset(&ifr, 0, sizeof(ifr));
 	program_name = argv[0];
 
-	while((c = getopt_long_only(argc, argv, "adchvV:P:l:+:", longopts, 0)) != EOF) {
+	while((c = getopt_long_only(argc, argv, ""/*"adchvV:P:l:+:"*/, longopts, 0)) != EOF) {
 		switch(c) {
 		case 'g':
 			debug = 1;
+			argcount--;
 			break;
 		case 'a':
 			if(shc->cf_cmd) {
@@ -125,7 +124,7 @@ main(int argc, char *argv[])
 				fprintf(stderr, "%s: warning; '-v' and '--version' options don't expect arguments, arg '%s' found, perhaps unintended.\n",
 					program_name, optarg);
 			}
-			printf("%s, %s\n", program_name, tncfg_c_version);
+			fprintf(stdout, "%s, %s\n", program_name, tncfg_c_version);
 			exit(1);
 			break;
 		case 'V':
@@ -135,7 +134,13 @@ main(int argc, char *argv[])
 			strcpy(shc->cf_name, optarg);
 			break;
 		case 'l':
-			program_name = optarg;
+			program_name = malloc(strlen(argv[0])
+					      + 10 /* update this when changing the sprintf() */
+					      + strlen(optarg));
+			sprintf(program_name, "%s --label %s",
+				argv[0],
+				optarg);
+			argcount -= 2;
 			break;
 		case '+': /* optionsfrom */
 			optionsfrom(optarg, &argc, &argv, optind, stderr);
@@ -148,6 +153,11 @@ main(int argc, char *argv[])
 		previous = c;
 	}
 
+	if(argcount == 1) {
+		system("cat /proc/net/ipsec_tncfg");
+		exit(0);
+	}
+
 	switch(shc->cf_cmd) {
 	case IPSEC_SET_DEV:
 		if(!shc->cf_name) {
@@ -175,85 +185,85 @@ main(int argc, char *argv[])
 	s=socket(AF_INET, SOCK_DGRAM,0);
 	if(s==-1)
 	{
-		printf("%s: Socket creation failed -- ", program_name);
+		fprintf(stderr, "%s: Socket creation failed -- ", program_name);
 		switch(errno)
 		{
 		case EACCES:
 			if(getuid()==0)
-				printf("Root denied permission!?!\n");
+				fprintf(stderr, "Root denied permission!?!\n");
 			else
-				printf("Run as root user.\n");
+				fprintf(stderr, "Run as root user.\n");
 			break;
 		case EPROTONOSUPPORT:
-			printf("Internet Protocol not enabled");
+			fprintf(stderr, "Internet Protocol not enabled");
 			break;
 		case EMFILE:
 		case ENFILE:
 		case ENOBUFS:
-			printf("Insufficient system resources.\n");
+			fprintf(stderr, "Insufficient system resources.\n");
 			break;
 		case ENODEV:
-			printf("No such device.  Is the virtual device valid?  Is the ipsec module linked into the kernel or loaded as a module?\n");
+			fprintf(stderr, "No such device.  Is the virtual device valid?  Is the ipsec module linked into the kernel or loaded as a module?\n");
 			break;
 		default:
-			printf("Unknown socket error %d.\n", errno);
+			fprintf(stderr, "Unknown socket error %d.\n", errno);
 		}
 		exit(1);
 	}
 	if(ioctl(s, shc->cf_cmd, &ifr)==-1)
 	{
 		if(shc->cf_cmd == IPSEC_SET_DEV) {
-			printf("%s: Socket ioctl failed on attach -- ", program_name);
+			fprintf(stderr, "%s: Socket ioctl failed on attach -- ", program_name);
 			switch(errno)
 			{
 			case EINVAL:
-				printf("Invalid argument, check kernel log messages for specifics.\n");
+				fprintf(stderr, "Invalid argument, check kernel log messages for specifics.\n");
 				break;
 			case ENODEV:
-				printf("No such device.  Is the virtual device valid?  Is the ipsec module linked into the kernel or loaded as a module?\n");
+				fprintf(stderr, "No such device.  Is the virtual device valid?  Is the ipsec module linked into the kernel or loaded as a module?\n");
 				break;
 			case ENXIO:
-				printf("No such device.  Is the physical device valid?\n");
+				fprintf(stderr, "No such device.  Is the physical device valid?\n");
 				break;
 			case EBUSY:
-				printf("Device busy.  Virtual device %s is already attached to a physical device -- Use detach first.\n",
+				fprintf(stderr, "Device busy.  Virtual device %s is already attached to a physical device -- Use detach first.\n",
 				       ifr.ifr_name);
 				break;
 			default:
-				printf("Unknown socket error %d.\n", errno);
+				fprintf(stderr, "Unknown socket error %d.\n", errno);
 			}
 			exit(1);
 		}
 		if(shc->cf_cmd == IPSEC_DEL_DEV) {
-			printf("%s: Socket ioctl failed on detach -- ", program_name);
+			fprintf(stderr, "%s: Socket ioctl failed on detach -- ", program_name);
 			switch(errno)
 			{
 			case EINVAL:
-				printf("Invalid argument, check kernel log messages for specifics.\n");
+				fprintf(stderr, "Invalid argument, check kernel log messages for specifics.\n");
 				break;
 			case ENODEV:
-				printf("No such device.  Is the virtual device valid?  The ipsec module may not be linked into the kernel or loaded as a module.\n");
+				fprintf(stderr, "No such device.  Is the virtual device valid?  The ipsec module may not be linked into the kernel or loaded as a module.\n");
 				break;
 			case ENXIO:
-				printf("Device requested is not linked to any physical device.\n");
+				fprintf(stderr, "Device requested is not linked to any physical device.\n");
 				break;
 			default:
-				printf("Unknown socket error %d.\n", errno);
+				fprintf(stderr, "Unknown socket error %d.\n", errno);
 			}
 			exit(1);
 		}
 		if(shc->cf_cmd == IPSEC_CLR_DEV) {
-			printf("%s: Socket ioctl failed on clear -- ", program_name);
+			fprintf(stderr, "%s: Socket ioctl failed on clear -- ", program_name);
 			switch(errno)
 			{
 			case EINVAL:
-				printf("Invalid argument, check kernel log messages for specifics.\n");
+				fprintf(stderr, "Invalid argument, check kernel log messages for specifics.\n");
 				break;
 			case ENODEV:
-				printf("Failed.  Is the ipsec module linked into the kernel or loaded as a module?.\n");
+				fprintf(stderr, "Failed.  Is the ipsec module linked into the kernel or loaded as a module?.\n");
 				break;
 			default:
-				printf("Unknown socket error %d.\n", errno);
+				fprintf(stderr, "Unknown socket error %d.\n", errno);
 			}
 			exit(1);
 		}
@@ -263,6 +273,27 @@ main(int argc, char *argv[])
 	
 /*
  * $Log: tncfg.c,v $
+ * Revision 1.27  2001/06/14 19:35:15  rgb
+ * Update copyright date.
+ *
+ * Revision 1.26  2001/05/21 02:02:55  rgb
+ * Eliminate 1-letter options.
+ *
+ * Revision 1.25  2001/05/16 05:07:20  rgb
+ * Fixed --label option in KLIPS manual utils to add the label to the
+ * command name rather than replace it in error text.
+ * Fix 'print table' non-option in KLIPS manual utils to deal with --label
+ * and --debug options.
+ *
+ * Revision 1.24  2000/09/12 13:09:05  rgb
+ * Fixed real/physical discrepancy between tncfg.8 and tncfg.c.
+ *
+ * Revision 1.23  2000/08/27 01:48:30  rgb
+ * Update copyright.
+ *
+ * Revision 1.22  2000/07/26 03:41:46  rgb
+ * Changed all printf's to fprintf's.  Fixed tncfg's usage to stderr.
+ *
  * Revision 1.21  2000/06/21 16:51:27  rgb
  * Added no additional argument option to usage text.
  *
