@@ -12,7 +12,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
  * License for more details.
  *
- * RCSID $Id: atosa.c,v 1.8 2000/01/21 09:43:26 rgb Exp $
+ * RCSID $Id: atosa.c,v 1.11 2000/09/17 18:55:37 henry Exp $
  */
 #include "internal.h"
 #include "freeswan.h"
@@ -25,6 +25,7 @@ static struct satype {
 	{ "ah",		2,	SA_AH	},
 	{ "esp",	3,	SA_ESP	},
 	{ "tun",	3,	SA_IPIP },
+	{ "comp",	4,	SA_COMP },
 	{ NULL,		0,	0,	}
 };
 
@@ -70,7 +71,7 @@ struct sa_id *sa;
 			break;			/* NOTE BREAK OUT */
 		}
 	if (sat->prefix == NULL)
-		return "SA specifier must begin with `ah', `esp', or `tun'";
+		return "SA specifier lacks valid protocol prefix";
 
 	if (spi >= at)
 		return "no SPI in SA specifier";
