@@ -1,6 +1,6 @@
 /*
  * FreeS/WAN specific PF_KEY headers
- * Copyright (C) 1999  Richard Guy Briggs.
+ * Copyright (C) 1999, 2000, 2001  Richard Guy Briggs.
  * 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -12,7 +12,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: pfkey.h,v 1.20 2000/05/10 20:14:19 rgb Exp $
+ * RCSID $Id: pfkey.h,v 1.36 2002/01/20 20:34:49 mcr Exp $
  */
 
 #ifndef __NET_IPSEC_PF_KEY_H
@@ -32,32 +32,57 @@ struct socket_list
 	struct socket *socketp;
 	struct socket_list *next;
 };
-
-extern void pfkey_list_insert_socket(struct socket*, struct socket_list*);
-extern void pfkey_list_remove_socket(struct socket*, struct socket_list*);
+extern int pfkey_list_insert_socket(struct socket*, struct socket_list**);
+extern int pfkey_list_remove_socket(struct socket*, struct socket_list**);
 extern struct socket_list *pfkey_open_sockets;
-extern struct socket_list *pfkey_registered_sockets;
-extern uint8_t sadb_satype2proto[];
+extern struct socket_list *pfkey_registered_sockets[SADB_SATYPE_MAX+1];
 
-/* #if defined(__KERNEL__) || !defined(__GLIBC__) || (__GLIBC__ < 2) */
+struct supported
+{
+	uint16_t supported_alg_exttype;
+	uint8_t supported_alg_id;
+	uint8_t supported_alg_ivlen;
+	uint16_t supported_alg_minbits;
+	uint16_t supported_alg_maxbits;
+};
+
+extern struct supported_list *pfkey_supported_list[SADB_SATYPE_MAX+1];
+struct supported_list
+{
+	struct supported *supportedp;
+	struct supported_list *next;
+};
+extern int pfkey_list_insert_supported(struct supported*, struct supported_list**);
+extern int pfkey_list_remove_supported(struct supported*, struct supported_list**);
+
+/*
+#if defined(__KERNEL__) || !defined(__GLIBC__) || (__GLIBC__ < 2)
+ */
 struct sockaddr_key
 {
 	uint16_t	key_family;	/* PF_KEY */
 	uint16_t	key_pad;	/* not used */
 	uint32_t	key_pid;	/* process ID */
 };
-/* #endif */ /* defined(__KERNEL__) || !defined(__GLIBC__) || (__GLIBC__ < 2) */
+/*
+#endif */ /* defined(__KERNEL__) || !defined(__GLIBC__) || (__GLIBC__ < 2)
+ */
 
 struct pfkey_extracted_data
 {
-	struct tdb* tdb;
-	struct tdb* tdb2;
+	struct ipsec_sa* tdb;
+	struct ipsec_sa* tdb2;
 	struct eroute *eroute;
 };
+
+extern int pfkey_upmsg(struct socket *, struct sadb_msg *);
+extern int pfkey_expire(struct ipsec_sa *, int);
+extern int pfkey_acquire(struct ipsec_sa *);
 #endif /* __KERNEL__ */
 
 extern uint8_t satype2proto(uint8_t satype);
 extern uint8_t proto2satype(uint8_t proto);
+extern char* satype2name(uint8_t satype);
 extern char* proto2name(uint8_t proto);
 
 struct key_opt
@@ -70,6 +95,8 @@ struct key_opt
 
 #define IPSEC_PFKEYv2_ALIGN (sizeof(uint64_t)/sizeof(uint8_t))
 #define BITS_PER_OCTET 8
+#define OCTETBITS 8
+#define PFKEYBITS 64
 #define DIVUP(x,y) ((x + y -1) / y) /* divide, rounding upwards */
 #define ALIGN_N(x,y) (DIVUP(x,y) * y) /* align on y boundary */
 
@@ -78,6 +105,11 @@ struct key_opt
 /*
  * PF_KEYv2 permitted and required extensions in and out bitmaps
  */
+struct pf_key_ext_parsers_def {
+	int  (*parser)(struct sadb_ext*);
+	char  *parser_name;
+};
+
 
 extern unsigned int extensions_bitmaps[2/*in/out*/][2/*perm/req*/][SADB_MAX + 1/*ext*/];
 #define EXT_BITS_IN 0
@@ -90,7 +122,7 @@ extern void pfkey_extensions_free(struct sadb_ext *extensions[SADB_EXT_MAX + 1])
 extern void pfkey_msg_free(struct sadb_msg **pfkey_msg);
 
 extern int pfkey_msg_parse(struct sadb_msg *pfkey_msg,
-			   int (*ext_parsers[])(struct sadb_ext* ),
+			   struct pf_key_ext_parsers_def *ext_parsers[],
 			   struct sadb_ext **extensions,
 			   int dir);
 
@@ -122,7 +154,8 @@ pfkey_lifetime_build(struct sadb_ext **	pfkey_ext,
 		     uint32_t		allocations,
 		     uint64_t		bytes,
 		     uint64_t		addtime,
-		     uint64_t		usetime);
+		     uint64_t		usetime,
+		     uint32_t		packets);
 
 int
 pfkey_address_build(struct sadb_ext**	pfkey_ext,
@@ -142,6 +175,7 @@ pfkey_ident_build(struct sadb_ext**	pfkey_ext,
 		  uint16_t		exttype,
 		  uint16_t		ident_type,
 		  uint64_t		ident_id,
+		  uint8_t               ident_len,
 		  char*			ident_string);
 
 int
@@ -190,17 +224,80 @@ pfkey_x_debug_build(struct sadb_ext**	pfkey_ext,
 		    uint32_t		esp,
 		    uint32_t		ah,
 		    uint32_t		rcv,
-		    uint32_t            pfkey);
+		    uint32_t            pfkey,
+		    uint32_t            ipcomp,
+		    uint32_t            verbose);
 
 int
 pfkey_msg_build(struct sadb_msg**	pfkey_msg,
 		struct sadb_ext*	extensions[],
 		int			dir);
 
+/* in pfkey_v2_debug.c - routines to decode numbers -> strings */
+const char *
+pfkey_v2_sadb_ext_string(int extnum);
+
+const char *
+pfkey_v2_sadb_type_string(int sadb_type);
+
+
 #endif /* __NET_IPSEC_PF_KEY_H */
 
 /*
  * $Log: pfkey.h,v $
+ * Revision 1.36  2002/01/20 20:34:49  mcr
+ * 	added pfkey_v2_sadb_type_string to decode sadb_type to string.
+ *
+ * Revision 1.35  2001/11/27 05:27:47  mcr
+ * 	pfkey parses are now maintained by a structure
+ * 	that includes their name for debug purposes.
+ *
+ * Revision 1.34  2001/11/26 09:23:53  rgb
+ * Merge MCR's ipsec_sa, eroute, proc and struct lifetime changes.
+ *
+ * Revision 1.33  2001/11/06 19:47:47  rgb
+ * Added packet parameter to lifetime and comb structures.
+ *
+ * Revision 1.32  2001/09/08 21:13:34  rgb
+ * Added pfkey ident extension support for ISAKMPd. (NetCelo)
+ *
+ * Revision 1.31  2001/06/14 19:35:16  rgb
+ * Update copyright date.
+ *
+ * Revision 1.30  2001/02/27 07:04:52  rgb
+ * Added satype2name prototype.
+ *
+ * Revision 1.29  2001/02/26 19:59:33  rgb
+ * Ditch unused sadb_satype2proto[], replaced by satype2proto().
+ *
+ * Revision 1.28  2000/10/10 20:10:19  rgb
+ * Added support for debug_ipcomp and debug_verbose to klipsdebug.
+ *
+ * Revision 1.27  2000/09/21 04:20:45  rgb
+ * Fixed array size off-by-one error.  (Thanks Svenning!)
+ *
+ * Revision 1.26  2000/09/12 03:26:05  rgb
+ * Added pfkey_acquire prototype.
+ *
+ * Revision 1.25  2000/09/08 19:21:28  rgb
+ * Fix pfkey_prop_build() parameter to be only single indirection.
+ *
+ * Revision 1.24  2000/09/01 18:46:42  rgb
+ * Added a supported algorithms array lists, one per satype and registered
+ * existing algorithms.
+ * Fixed pfkey_list_{insert,remove}_{socket,support}() to allow change to
+ * list.
+ *
+ * Revision 1.23  2000/08/27 01:55:26  rgb
+ * Define OCTETBITS and PFKEYBITS to avoid using 'magic' numbers in code.
+ *
+ * Revision 1.22  2000/08/20 21:39:23  rgb
+ * Added kernel prototypes for kernel funcitions pfkey_upmsg() and
+ * pfkey_expire().
+ *
+ * Revision 1.21  2000/08/15 17:29:23  rgb
+ * Fixes from SZI to untested pfkey_prop_build().
+ *
  * Revision 1.20  2000/05/10 20:14:19  rgb
  * Fleshed out sensitivity, proposal and supported extensions.
  *
