@@ -1,6 +1,6 @@
 /*
  * RFC2367 PF_KEYv2 Key management API message parser
- * Copyright (C) 1999  Richard Guy Briggs.
+ * Copyright (C) 1999, 2000, 2001  Richard Guy Briggs.
  * 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -12,14 +12,14 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: pfkey_v2_build.c,v 1.13 2000/06/02 22:54:14 rgb Exp $
+ * RCSID $Id: pfkey_v2_build.c,v 1.31 2002/01/29 22:25:35 rgb Exp $
  */
 
 /*
  *		Template from klips/net/ipsec/ipsec/ipsec_parser.c.
  */
 
-char pfkey_v2_build_c_version[] = "$Id: pfkey_v2_build.c,v 1.13 2000/06/02 22:54:14 rgb Exp $";
+char pfkey_v2_build_c_version[] = "$Id: pfkey_v2_build.c,v 1.31 2002/01/29 22:25:35 rgb Exp $";
 
 /*
  * Some ugly stuff to allow consistent debugging code for use in the
@@ -30,7 +30,12 @@ char pfkey_v2_build_c_version[] = "$Id: pfkey_v2_build.c,v 1.13 2000/06/02 22:54
 
 # include <linux/kernel.h>  /* for printk */
 
-# include <linux/malloc.h> /* kmalloc() */
+# include "../ipsec_kversion.h" /* for malloc switch */
+# ifdef MALLOC_SLAB
+#  include <linux/slab.h> /* kmalloc() */
+# else /* MALLOC_SLAB */
+#  include <linux/malloc.h> /* kmalloc() */
+# endif /* MALLOC_SLAB */
 # include <linux/errno.h>  /* error codes */
 # include <linux/types.h>  /* size_t */
 # include <linux/interrupt.h> /* mark_bh */
@@ -41,12 +46,10 @@ char pfkey_v2_build_c_version[] = "$Id: pfkey_v2_build.c,v 1.13 2000/06/02 22:54
 # if defined(CONFIG_IPV6) || defined(CONFIG_IPV6_MODULE)
 #  include <linux/ipv6.h>        /* struct ipv6hdr */
 # endif /* if defined(CONFIG_IPV6) || defined(CONFIG_IPV6_MODULE) */
-extern int debug_pfkey;
 
-# define DEBUGGING(format,args...) \
-         ((debug_pfkey) ? printk(KERN_INFO "klips_debug:" format , ## args) : 0)
 # define MALLOC(size) kmalloc(size, GFP_ATOMIC)
 # define FREE(obj) kfree(obj)
+# include <freeswan.h>
 #else /* __KERNEL__ */
 
 # include <sys/types.h>
@@ -55,6 +58,7 @@ extern int debug_pfkey;
 # include <malloc.h>
 # include <string.h> /* memset */
 
+# include <freeswan.h>
 # include "../pluto/constants.h" 
 # include "../pluto/defs.h"  /* for PRINTF_LIKE */
 # include "../pluto/log.h"  /* for debugging and DBG_log */
@@ -65,18 +69,27 @@ unsigned int pfkey_lib_debug = 0;
 /* #define PLUTO */
 
 # ifdef PLUTO
-#  define DEBUGGING(format,args...)  { DBG_log("pfkey_lib_debug:" format, ## args);  }
+#  define DEBUGGING(args...)  { DBG_log("pfkey_lib_debug:" args);  }
 # else
-#  define DEBUGGING(format,args...)  if(pfkey_lib_debug) { printf("pfkey_lib_debug:" format, ## args); } else { ; }
+#  define DEBUGGING(args...)  if(pfkey_lib_debug) { printf("pfkey_lib_debug:" args); } else { ; }
 # endif
 # define MALLOC(size) malloc(size)
 # define FREE(obj) free(obj)
 #endif /* __KERNEL__ */
 
-#include <freeswan.h>
 #include <pfkeyv2.h>
 #include <pfkey.h>
 
+#ifdef __KERNEL__
+# include "../radij.h"  /* rd_nodes */
+# include "../ipsec_encap.h"  /* sockaddr_encap */
+# include "../ipsec_netlink.h"  /* KLIPS_PRINT */
+# define DEBUGGING(args...) \
+         KLIPS_PRINT(debug_pfkey, "klips_debug:" args)
+/*         ((debug_pfkey) ? printk(KERN_INFO "klips_debug:" format , ## args) : 0) */
+#endif /* __KERNEL__ */
+
+
 #define SENDERR(_x) do { error = -(_x); goto errlab; } while (0)
 
 void
@@ -139,33 +152,38 @@ pfkey_msg_hdr_build(struct sadb_ext**	pfkey_ext,
 	DEBUGGING(
 		"pfkey_msg_hdr_build:\n");
 	DEBUGGING(
-		"pfkey_msg_hdr_build: on_entry &pfkey_ext=%p pfkey_ext=%p *pfkey_ext=%p.\n",
+		"pfkey_msg_hdr_build: "
+		"on_entry &pfkey_ext=%p pfkey_ext=%p *pfkey_ext=%p.\n",
 		&pfkey_ext,
 		pfkey_ext,
 		*pfkey_ext);
 	/* sanity checks... */
 	if(pfkey_msg) {
 		DEBUGGING(
-			"pfkey_msg_hdr_build:why is pfkey_msg already pointing to something?\n");
+			"pfkey_msg_hdr_build: "
+			"why is pfkey_msg already pointing to something?\n");
 		SENDERR(EINVAL);
 	}
 
 	if(!msg_type) {
 		DEBUGGING(
-			"pfkey_msg_hdr_build: msg type not set, must be non-zero..\n");
+			"pfkey_msg_hdr_build: "
+			"msg type not set, must be non-zero..\n");
 		SENDERR(EINVAL);
 	}
 
 	if(msg_type > SADB_MAX) {
 		DEBUGGING(
-			"pfkey_msg_hdr_build: msg type too large:%d.\n",
+			"pfkey_msg_hdr_build: "
+			"msg type too large:%d.\n",
 			msg_type);
 		SENDERR(EINVAL);
 	}
 
 	if(satype > SADB_SATYPE_MAX) {
 		DEBUGGING(
-			"pfkey_msg_hdr_build: satype %d > max %d\n", 
+			"pfkey_msg_hdr_build: "
+			"satype %d > max %d\n", 
 			satype, SADB_SATYPE_MAX);
 		SENDERR(EINVAL);
 	}
@@ -174,7 +192,8 @@ pfkey_msg_hdr_build(struct sadb_ext**	pfkey_ext,
 	     pfkey_msg = (struct sadb_msg*)
 	     MALLOC(sizeof(struct sadb_msg)))) {
 		DEBUGGING(
-			"pfkey_msg_hdr_build: memory allocation failed\n");
+			"pfkey_msg_hdr_build: "
+			"memory allocation failed\n");
 		SENDERR(ENOMEM);
 	}
 	memset(pfkey_msg, 0, sizeof(struct sadb_msg));
@@ -190,7 +209,8 @@ pfkey_msg_hdr_build(struct sadb_ext**	pfkey_ext,
 	pfkey_msg->sadb_msg_seq = seq;
 	pfkey_msg->sadb_msg_pid = pid;
 	DEBUGGING(
-		"pfkey_msg_hdr_build: on_exit &pfkey_ext=%p pfkey_ext=%p *pfkey_ext=%p.\n",
+		"pfkey_msg_hdr_build: "
+		"on_exit &pfkey_ext=%p pfkey_ext=%p *pfkey_ext=%p.\n",
 		&pfkey_ext,
 		pfkey_ext,
 		*pfkey_ext);
@@ -212,8 +232,9 @@ pfkey_sa_build(struct sadb_ext **	pfkey_ext,
 	struct sadb_sa *pfkey_sa = (struct sadb_sa *)*pfkey_ext;
 
 	DEBUGGING(
-		    "pfkey_sa_build: spi=%x replay=%d sa_state=%d auth=%d encrypt=%d flags=%d\n",
-		    spi, /* in host order */
+		    "pfkey_sa_build: "
+		    "spi=%08x replay=%d sa_state=%d auth=%d encrypt=%d flags=%d\n",
+		    ntohl(spi), /* in network order */
 		    replay_window,
 		    sa_state,
 		    auth,
@@ -222,29 +243,32 @@ pfkey_sa_build(struct sadb_ext **	pfkey_ext,
 	/* sanity checks... */
 	if(pfkey_sa) {
 		DEBUGGING(
-			"pfkey_sa_build:why is pfkey_sa already pointing to something?\n");
+			"pfkey_sa_build: "
+			"why is pfkey_sa already pointing to something?\n");
 		SENDERR(EINVAL);
 	}
 
 	if(exttype != SADB_EXT_SA &&
 	   exttype != SADB_X_EXT_SA2) {
 		DEBUGGING(
-			"pfkey_sa_build: invalid exttype=%d.\n",
+			"pfkey_sa_build: "
+			"invalid exttype=%d.\n",
 			exttype);
 		SENDERR(EINVAL);
 	}
 
 	if(replay_window > 64) {
 		DEBUGGING(
-			"pfkey_sa_build: replay window size: %d"
-			" -- must be 0 <= size <= 64\n",
+			"pfkey_sa_build: "
+			"replay window size: %d -- must be 0 <= size <= 64\n",
 			replay_window);
 		SENDERR(EINVAL);
 	}
 
 	if(auth > SADB_AALG_MAX) {
 		DEBUGGING(
-			"pfkey_sa_build: auth=%d > SADB_AALG_MAX=%d.\n",
+			"pfkey_sa_build: "
+			"auth=%d > SADB_AALG_MAX=%d.\n",
 			auth,
 			SADB_AALG_MAX);
 		SENDERR(EINVAL);
@@ -252,7 +276,8 @@ pfkey_sa_build(struct sadb_ext **	pfkey_ext,
 
 	if(encrypt > SADB_EALG_MAX) {
 		DEBUGGING(
-			"pfkey_sa_build: encrypt=%d > SADB_EALG_MAX=%d.\n",
+			"pfkey_sa_build: "
+			"encrypt=%d > SADB_EALG_MAX=%d.\n",
 			encrypt,
 			SADB_EALG_MAX);
 		SENDERR(EINVAL);
@@ -260,7 +285,8 @@ pfkey_sa_build(struct sadb_ext **	pfkey_ext,
 
 	if(sa_state > SADB_SASTATE_MAX) {
 		DEBUGGING(
-			"pfkey_sa_build: sa_state=%d exceeds MAX=%d.\n",
+			"pfkey_sa_build: "
+			"sa_state=%d exceeds MAX=%d.\n",
 			sa_state,
 			SADB_SASTATE_MAX);
 		SENDERR(EINVAL);
@@ -268,7 +294,8 @@ pfkey_sa_build(struct sadb_ext **	pfkey_ext,
 
 	if(sa_state == SADB_SASTATE_DEAD) {
 		DEBUGGING(
-			"pfkey_sa_build: sa_state=%d is DEAD=%d is not allowed.\n",
+			"pfkey_sa_build: "
+			"sa_state=%d is DEAD=%d is not allowed.\n",
 			sa_state,
 			SADB_SASTATE_DEAD);
 		SENDERR(EINVAL);
@@ -278,7 +305,8 @@ pfkey_sa_build(struct sadb_ext **	pfkey_ext,
 	     pfkey_sa = (struct sadb_sa*)
 	     MALLOC(sizeof(struct sadb_sa)))) {
 		DEBUGGING(
-			"pfkey_sa_build: memory allocation failed\n");
+			"pfkey_sa_build: "
+			"memory allocation failed\n");
 		SENDERR(ENOMEM);
 	}
 	memset(pfkey_sa, 0, sizeof(struct sadb_sa));
@@ -302,7 +330,8 @@ pfkey_lifetime_build(struct sadb_ext **	pfkey_ext,
 		     uint32_t		allocations,
 		     uint64_t		bytes,
 		     uint64_t		addtime,
-		     uint64_t		usetime)
+		     uint64_t		usetime,
+		     uint32_t		packets)
 {
 	int error = 0;
 	struct sadb_lifetime *pfkey_lifetime = (struct sadb_lifetime *)*pfkey_ext;
@@ -312,7 +341,8 @@ pfkey_lifetime_build(struct sadb_ext **	pfkey_ext,
 	/* sanity checks... */
 	if(pfkey_lifetime) {
 		DEBUGGING(
-			"pfkey_lifetime_build:why is pfkey_lifetime already pointing to something?\n");
+			"pfkey_lifetime_build: "
+			"why is pfkey_lifetime already pointing to something?\n");
 		SENDERR(EINVAL);
 	}
 
@@ -320,7 +350,8 @@ pfkey_lifetime_build(struct sadb_ext **	pfkey_ext,
 	   exttype != SADB_EXT_LIFETIME_HARD &&
 	   exttype != SADB_EXT_LIFETIME_SOFT) {
 		DEBUGGING(
-			"pfkey_lifetime_build: invalid exttype=%d.\n",
+			"pfkey_lifetime_build: "
+			"invalid exttype=%d.\n",
 			exttype);
 		SENDERR(EINVAL);
 	}
@@ -329,7 +360,8 @@ pfkey_lifetime_build(struct sadb_ext **	pfkey_ext,
 	     pfkey_lifetime = (struct sadb_lifetime*)
 	     MALLOC(sizeof(struct sadb_lifetime)))) {
 		DEBUGGING(
-			"pfkey_lifetime_build: memory allocation failed\n");
+			"pfkey_lifetime_build: "
+			"memory allocation failed\n");
 		SENDERR(ENOMEM);
 	}
 	memset(pfkey_lifetime, 0, sizeof(struct sadb_lifetime));
@@ -340,6 +372,7 @@ pfkey_lifetime_build(struct sadb_ext **	pfkey_ext,
 	pfkey_lifetime->sadb_lifetime_bytes = bytes;
 	pfkey_lifetime->sadb_lifetime_addtime = addtime;
 	pfkey_lifetime->sadb_lifetime_usetime = usetime;
+	pfkey_lifetime->sadb_x_lifetime_packets = packets;
 
 errlab:
 	return error;
@@ -354,18 +387,28 @@ pfkey_address_build(struct sadb_ext**	pfkey_ext,
 {
 	int error = 0;
 	int saddr_len = 0;
-	char *saddr;
-	char ipaddr_txt[ADDRTOA_BUF];
+	char ipaddr_txt[ADDRTOT_BUF + 6/*extra for port number*/];
 	struct sadb_address *pfkey_address = (struct sadb_address *)*pfkey_ext;
 	
 	DEBUGGING(
-		"pfkey_address_build: exttype=%d proto=%d prefixlen=%d\n", exttype, proto, prefixlen);
+		"pfkey_address_build: "
+		"exttype=%d proto=%d prefixlen=%d\n",
+		exttype,
+		proto,
+		prefixlen);
 	/* sanity checks... */
 	if(pfkey_address) {
 		DEBUGGING(
-			"pfkey_address_build:why is pfkey_address already pointing to something?\n");
+			"pfkey_address_build: "
+			"why is pfkey_address already pointing to something?\n");
 		SENDERR(EINVAL);
 	}
+
+	if (!address)  {
+			DEBUGGING("pfkey_address_build: "
+				  "address is NULL\n");
+			SENDERR(EINVAL);
+	}
 	
 	switch(exttype) {	
 	case SADB_EXT_ADDRESS_SRC:
@@ -379,7 +422,8 @@ pfkey_address_build(struct sadb_ext**	pfkey_ext,
 		break;
 	default:
 		DEBUGGING( 
-			"pfkey_address_build: unrecognised ext_type=%d.\n", 
+			"pfkey_address_build: "
+			"unrecognised ext_type=%d.\n", 
 			exttype); 
 		SENDERR(EINVAL); 
 	}
@@ -387,46 +431,48 @@ pfkey_address_build(struct sadb_ext**	pfkey_ext,
 	switch(address->sa_family) {
 	case AF_INET:
 		DEBUGGING(
-			"pfkey_address_build: found address family AF_INET.\n");
+			"pfkey_address_build: "
+			"found address family AF_INET.\n");
 		saddr_len = sizeof(struct sockaddr_in);
-#ifdef WITH_SOCKADDR_STORAGE
-		saddr = (char*)&((struct sockaddr*)address);
-#else
-		saddr = (char*)&((struct sockaddr_in*)address)->sin_addr;
-#endif /* WITH_SOCKADDR_STORAGE */
+		sprintf(ipaddr_txt, "%d.%d.%d.%d:%d"
+			, (((struct sockaddr_in*)address)->sin_addr.s_addr >>  0) & 0xFF
+			, (((struct sockaddr_in*)address)->sin_addr.s_addr >>  8) & 0xFF
+			, (((struct sockaddr_in*)address)->sin_addr.s_addr >> 16) & 0xFF
+			, (((struct sockaddr_in*)address)->sin_addr.s_addr >> 24) & 0xFF
+			, ntohs(((struct sockaddr_in*)address)->sin_port));
 		break;
-#ifdef WITH_SOCKADDR_STORAGE
-#if defined(WITH_IPV6) || defined(CONFIG_IPV6) || defined(CONFIG_IPV6_MODULE)
 	case AF_INET6:
 		DEBUGGING(
-			"pfkey_address_build: found address family AF_INET6.\n");
+			"pfkey_address_build: "
+			"found address family AF_INET6.\n");
 		saddr_len = sizeof(struct sockaddr_in6);
-		saddr = (char*)&((struct sockaddr*)address);
+		sprintf(ipaddr_txt, "%x:%x:%x:%x:%x:%x:%x:%x-%x"
+			, ntohs(((struct sockaddr_in6*)address)->sin6_addr.s6_addr16[0])
+			, ntohs(((struct sockaddr_in6*)address)->sin6_addr.s6_addr16[1])
+			, ntohs(((struct sockaddr_in6*)address)->sin6_addr.s6_addr16[2])
+			, ntohs(((struct sockaddr_in6*)address)->sin6_addr.s6_addr16[3])
+			, ntohs(((struct sockaddr_in6*)address)->sin6_addr.s6_addr16[4])
+			, ntohs(((struct sockaddr_in6*)address)->sin6_addr.s6_addr16[5])
+			, ntohs(((struct sockaddr_in6*)address)->sin6_addr.s6_addr16[6])
+			, ntohs(((struct sockaddr_in6*)address)->sin6_addr.s6_addr16[7])
+			, ntohs(((struct sockaddr_in6*)address)->sin6_port));
 		break;
-#endif /* defined(WITH_IPV6) || defined(CONFIG_IPV6) || defined(CONFIG_IPV6_MODULE) */
-#endif /* WITH_SOCKADDR_STORAGE */
 	default:
 		DEBUGGING(
-			"pfkey_address_build: address->sa_family=%d not supported.\n",
+			"pfkey_address_build: "
+			"address->sa_family=%d not supported.\n",
 			address->sa_family);
 		SENDERR(EPFNOSUPPORT);
 	}
 
-#ifdef WITH_SOCKADDR_STORAGE	
-	addrtoa(*((struct sockaddr_storage*)saddr),
-		0,
-		ipaddr_txt, sizeof(ipaddr_txt));
-#else
-	addrtoa(*((struct in_addr*)saddr),
-		0,
-		ipaddr_txt, sizeof(ipaddr_txt));
-#endif /* WITH_SOCKADDR_STORAGE */
 	DEBUGGING(
-		"pfkey_address_build: found address=%s.\n",
+		"pfkey_address_build: "
+		"found address=%s.\n",
 		ipaddr_txt);
 	if(prefixlen != 0) {
 		DEBUGGING(
-			"pfkey_address_build: address prefixes not supported yet.\n");
+			"pfkey_address_build: "
+			"address prefixes not supported yet.\n");
 		SENDERR(EAFNOSUPPORT); /* not supported yet */
 	}
 
@@ -434,7 +480,8 @@ pfkey_address_build(struct sadb_ext**	pfkey_ext,
 	     pfkey_address = (struct sadb_address*)
 	     MALLOC(ALIGN_N(sizeof(struct sadb_address) + saddr_len, IPSEC_PFKEYv2_ALIGN) ))) {
 		DEBUGGING(
-			"pfkey_lifetime_build: memory allocation failed\n");
+			"pfkey_lifetime_build: "
+			"memory allocation failed\n");
 		SENDERR(ENOMEM);
 	}
 	memset(pfkey_address,
@@ -455,16 +502,13 @@ pfkey_address_build(struct sadb_ext**	pfkey_ext,
 	       saddr_len);
 
 #if 0
-	/* XXX check if port!=0 */
-	
-	pfkey_address_s_ska.sin_port = 0;
 	for(i = 0; i < sizeof(struct sockaddr_in) - offsetof(struct sockaddr_in, sin_zero); i++) {
 		pfkey_address_s_ska.sin_zero[i] = 0;
 	}
 #endif
-
 	DEBUGGING(
-		"pfkey_address_build: successful.\n");
+		"pfkey_address_build: "
+		"successful.\n");
 
  errlab:
 	return error;
@@ -484,19 +528,22 @@ pfkey_key_build(struct sadb_ext**	pfkey_ext,
 	/* sanity checks... */
 	if(pfkey_key) {
 		DEBUGGING(
-			"pfkey_key_build:why is pfkey_key already pointing to something?\n");
+			"pfkey_key_build: "
+			"why is pfkey_key already pointing to something?\n");
 		SENDERR(EINVAL);
 	}
 
 	if(!key_bits) {
 		DEBUGGING(
-			"pfkey_key_build: key_bits is zero, it must be non-zero.\n");
+			"pfkey_key_build: "
+			"key_bits is zero, it must be non-zero.\n");
 		SENDERR(EINVAL);
 	}
 
 	if( !((exttype == SADB_EXT_KEY_AUTH) || (exttype == SADB_EXT_KEY_ENCRYPT))) {
 		DEBUGGING(
-			"pfkey_key_build: unsupported extension type=%d.\n",
+			"pfkey_key_build: "
+			"unsupported extension type=%d.\n",
 			exttype);
 		SENDERR(EINVAL);
 	}
@@ -506,7 +553,8 @@ pfkey_key_build(struct sadb_ext**	pfkey_ext,
 	     MALLOC(sizeof(struct sadb_key) +
 				    DIVUP(key_bits, 64) * IPSEC_PFKEYv2_ALIGN))) {
 		DEBUGGING(
-			"pfkey_key_build: memory allocation failed\n");
+			"pfkey_key_build: "
+			"memory allocation failed\n");
 		SENDERR(ENOMEM);
 	}
 	memset(pfkey_key,
@@ -532,35 +580,43 @@ pfkey_ident_build(struct sadb_ext**	pfkey_ext,
 		  uint16_t		exttype,
 		  uint16_t		ident_type,
 		  uint64_t		ident_id,
+		  uint8_t               ident_len,
 		  char*			ident_string)
 {
 	int error = 0;
 	struct sadb_ident *pfkey_ident = (struct sadb_ident *)*pfkey_ext;
+	int data_len = ident_len * IPSEC_PFKEYv2_ALIGN - sizeof(struct sadb_ident);
 
+	DEBUGGING(
+		"pfkey_ident_build:\n");
 	/* sanity checks... */
 	if(pfkey_ident) {
 		DEBUGGING(
-			"pfkey_ident_build:why is pfkey_ident already pointing to something?\n");
+			"pfkey_ident_build: "
+			"why is pfkey_ident already pointing to something?\n");
 		SENDERR(EINVAL);
 	}
 
 	if( ! ((exttype == SADB_EXT_IDENTITY_SRC) ||
 	       (exttype == SADB_EXT_IDENTITY_DST))) {
 		DEBUGGING(
-			"pfkey_ident_build: unsupported extension type=%d.\n",
+			"pfkey_ident_build: "
+			"unsupported extension type=%d.\n",
 			exttype);
 		SENDERR(EINVAL);
 	}
 
 	if((ident_type == SADB_IDENTTYPE_RESERVED)) {
 		DEBUGGING(
-			"pfkey_ident_build: ident_type must be non-zero.\n");
+			"pfkey_ident_build: "
+			"ident_type must be non-zero.\n");
 		SENDERR(EINVAL);
 	}
 
 	if(ident_type > SADB_IDENTTYPE_MAX) {
 		DEBUGGING(
-			"pfkey_ident_build: identtype=%d out of range.\n",
+			"pfkey_ident_build: "
+			"identtype=%d out of range.\n",
 			ident_type);
 		SENDERR(EINVAL);
 	}
@@ -569,7 +625,8 @@ pfkey_ident_build(struct sadb_ext**	pfkey_ext,
 	    (ident_type == SADB_IDENTTYPE_FQDN)) &&
 	   !ident_string) {
 		DEBUGGING(
-			"pfkey_ident_build: string required to allocate size of extension.\n");
+			"pfkey_ident_build: "
+			"string required to allocate size of extension.\n");
 		SENDERR(EINVAL);
 	}
 	
@@ -580,41 +637,22 @@ pfkey_ident_build(struct sadb_ext**	pfkey_ext,
 	    
 	if(!(*pfkey_ext = (struct sadb_ext*)
 	     pfkey_ident = (struct sadb_ident*)
-	     MALLOC(ALIGN_N(sizeof(struct sadb_key) + strlen(ident_string), IPSEC_PFKEYv2_ALIGN)))) {
+	     MALLOC(ident_len * IPSEC_PFKEYv2_ALIGN))) {
 		DEBUGGING(
-			"pfkey_ident_build: memory allocation failed\n");
+			"pfkey_ident_build: "
+			"memory allocation failed\n");
 		SENDERR(ENOMEM);
 	}
-	memset(pfkey_ident,
-	       0,
-	       ALIGN_N(sizeof(struct sadb_ident) + strlen(ident_string),
-		     IPSEC_PFKEYv2_ALIGN));
-	
-	pfkey_ident->sadb_ident_len = DIVUP(sizeof(struct sadb_ident) + strlen(ident_string),
-					    IPSEC_PFKEYv2_ALIGN);
+	memset(pfkey_ident, 0, ident_len * IPSEC_PFKEYv2_ALIGN);
 	
+	pfkey_ident->sadb_ident_len = ident_len;
 	pfkey_ident->sadb_ident_exttype = exttype;
 	pfkey_ident->sadb_ident_type = ident_type;
 	pfkey_ident->sadb_ident_reserved = 0;
 	pfkey_ident->sadb_ident_id = ident_id;
 	memcpy((char*)pfkey_ident + sizeof(struct sadb_ident),
 	       ident_string,
-	       strlen(ident_string));
-
-	/* string terminator/padding must be zero */
-	/* Which one is better, I don't know... */
-#if 0
-	for(i = strlen(ident_string);
-	    i < ALIGN_N(sizeof(struct sadb_ident) + strlen(ident_string), IPSEC_PFKEYv2_ALIGN);
-	    i++) {
-		((char*)pfkey_ident)[i] = 0;
-	}
-#else
-	memset(((char*)pfkey_ident) + sizeof(struct sadb_ident) + strlen(ident_string),
-	       0,
-	       ALIGN_N(sizeof(struct sadb_ident) + strlen(ident_string), IPSEC_PFKEYv2_ALIGN) -
-	       sizeof(struct sadb_ident) + strlen(ident_string));
-#endif
+	       data_len);
 
 errlab:
 	return error;
@@ -635,15 +673,19 @@ pfkey_sens_build(struct sadb_ext**	pfkey_ext,
 	int i;
 	uint64_t* bitmap;
 
+	DEBUGGING(
+		"pfkey_sens_build:\n");
 	/* sanity checks... */
 	if(pfkey_sens) {
 		DEBUGGING(
-			"pfkey_sens_build:why is pfkey_sens already pointing to something?\n");
+			"pfkey_sens_build: "
+			"why is pfkey_sens already pointing to something?\n");
 		SENDERR(EINVAL);
 	}
 
 	DEBUGGING(
-		"pfkey_sens_build: Sorry, I can't build exttype=%d yet.\n",
+		"pfkey_sens_build: "
+		"Sorry, I can't build exttype=%d yet.\n",
 		(*pfkey_ext)->sadb_ext_type);
 	SENDERR(EINVAL); /* don't process these yet */
 
@@ -652,7 +694,8 @@ pfkey_sens_build(struct sadb_ext**	pfkey_ext,
 	     MALLOC(sizeof(struct sadb_sens) +
 		    (sens_len + integ_len) * sizeof(uint64_t)))) {
 		DEBUGGING(
-			"pfkey_sens_build: memory allocation failed\n");
+			"pfkey_sens_build: "
+			"memory allocation failed\n");
 		SENDERR(ENOMEM);
 	}
 	memset(pfkey_sens,
@@ -695,24 +738,23 @@ pfkey_prop_build(struct sadb_ext**	pfkey_ext,
 	struct sadb_prop *pfkey_prop = (struct sadb_prop *)*pfkey_ext;
 	struct sadb_comb *combp;
 
+	DEBUGGING(
+		"pfkey_prop_build:\n");
 	/* sanity checks... */
 	if(pfkey_prop) {
 		DEBUGGING(
-			"pfkey_prop_build:why is pfkey_prop already pointing to something?\n");
+			"pfkey_prop_build: "
+			"why is pfkey_prop already pointing to something?\n");
 		SENDERR(EINVAL);
 	}
 
-	DEBUGGING(
-		"pfkey_prop_build: Sorry, I can't build exttype=%d yet.\n",
-		(*pfkey_ext)->sadb_ext_type);
-	SENDERR(EINVAL); /* don't process these yet */
-
 	if(!(*pfkey_ext = (struct sadb_ext*)
 	     pfkey_prop = (struct sadb_prop*)
 	     MALLOC(sizeof(struct sadb_prop) +
 		    comb_num * sizeof(struct sadb_comb)))) {
 		DEBUGGING(
-			"pfkey_prop_build: memory allocation failed\n");
+			"pfkey_prop_build: "
+			"memory allocation failed\n");
 		SENDERR(ENOMEM);
 	}
 	memset(pfkey_prop,
@@ -723,16 +765,16 @@ pfkey_prop_build(struct sadb_ext**	pfkey_ext,
 	pfkey_prop->sadb_prop_len = (sizeof(struct sadb_prop) +
 		    comb_num * sizeof(struct sadb_comb)) / IPSEC_PFKEYv2_ALIGN;
 
-	pfkey_prop->sadb_prop_exttype = SADB_EXT_SUPPORTED_AUTH;
+	pfkey_prop->sadb_prop_exttype = SADB_EXT_PROPOSAL;
 	pfkey_prop->sadb_prop_replay = replay;
 
 	for(i=0; i<3; i++) {
 		pfkey_prop->sadb_prop_reserved[i] = 0;
 	}
 
-	combp = (struct sadb_comb*)((char*)pfkey_ext + sizeof(struct sadb_prop));
+	combp = (struct sadb_comb*)((char*)*pfkey_ext + sizeof(struct sadb_prop));
 	for(i = 0; i < comb_num; i++) {
-		*combp = comb[i];
+		memcpy (combp, &(comb[i]), sizeof(struct sadb_comb));
 		combp++;
 	}
 
@@ -753,6 +795,8 @@ pfkey_prop_build(struct sadb_ext**	pfkey_ext,
   uint64_t sadb_comb_hard_addtime;
   uint64_t sadb_comb_soft_usetime;
   uint64_t sadb_comb_hard_usetime;
+  uint32_t sadb_comb_soft_packets;
+  uint32_t sadb_comb_hard_packets;
 #endif
 errlab:
 	return error;
@@ -772,13 +816,15 @@ pfkey_supported_build(struct sadb_ext**	pfkey_ext,
 	/* sanity checks... */
 	if(pfkey_supported) {
 		DEBUGGING(
-			"pfkey_supported_build:why is pfkey_supported already pointing to something?\n");
+			"pfkey_supported_build: "
+			"why is pfkey_supported already pointing to something?\n");
 		SENDERR(EINVAL);
 	}
 
 	if( !((exttype == SADB_EXT_SUPPORTED_AUTH) || (exttype == SADB_EXT_SUPPORTED_ENCRYPT))) {
 		DEBUGGING(
-			"pfkey_supported_build: unsupported extension type=%d.\n",
+			"pfkey_supported_build: "
+			"unsupported extension type=%d.\n",
 			exttype);
 		SENDERR(EINVAL);
 	}
@@ -789,7 +835,8 @@ pfkey_supported_build(struct sadb_ext**	pfkey_ext,
 					       alg_num *
 					       sizeof(struct sadb_alg)))) {
 		DEBUGGING(
-			"pfkey_supported_build: memory allocation failed\n");
+			"pfkey_supported_build: "
+			"memory allocation failed\n");
 		SENDERR(ENOMEM);
 	}
 	memset(pfkey_supported,
@@ -807,15 +854,15 @@ pfkey_supported_build(struct sadb_ext**	pfkey_ext,
 
 	pfkey_alg = (struct sadb_alg*)((char*)pfkey_supported + sizeof(struct sadb_supported));
 	for(i = 0; i < alg_num; i++) {
-		*pfkey_alg = alg[i];
-		/* process algo description */
+		memcpy (pfkey_alg, &(alg[i]), sizeof(struct sadb_alg));
 		pfkey_alg->sadb_alg_reserved = 0;
 		pfkey_alg++;
 	}
 	
 #if 0
 	DEBUGGING(
-		"pfkey_supported_build: Sorry, I can't build exttype=%d yet.\n",
+		"pfkey_supported_build: "
+		"Sorry, I can't build exttype=%d yet.\n",
 		(*pfkey_ext)->sadb_ext_type);
 	SENDERR(EINVAL); /* don't process these yet */
 
@@ -832,8 +879,8 @@ errlab:
 int
 pfkey_spirange_build(struct sadb_ext**	pfkey_ext,
 		     uint16_t		exttype,
-		     uint32_t		min,
-		     uint32_t		max)
+		     uint32_t		min, /* in network order */
+		     uint32_t		max) /* in network order */
 {
 	int error = 0;
 	struct sadb_spirange *pfkey_spirange = (struct sadb_spirange *)*pfkey_ext;
@@ -841,22 +888,25 @@ pfkey_spirange_build(struct sadb_ext**	pfkey_ext,
 	/* sanity checks... */
 	if(pfkey_spirange) {
 		DEBUGGING(
-			"pfkey_spirange_build:why is pfkey_spirange already pointing to something?\n");
+			"pfkey_spirange_build: "
+			"why is pfkey_spirange already pointing to something?\n");
 		SENDERR(EINVAL);
 	}
 	
-        if(max < min) {
+        if(ntohl(max) < ntohl(min)) {
 		DEBUGGING(
-			"pfkey_spirange_build: minspi=%d must be < maxspi=%d.\n",
-			min,
-			max);
+			"pfkey_spirange_build: "
+			"minspi=%08x must be < maxspi=%08x.\n",
+			ntohl(min),
+			ntohl(max));
                 SENDERR(EINVAL);
         }
 	
-	if(min <= 255) {
+	if(ntohl(min) <= 255) {
 		DEBUGGING(
-			"pfkey_spirange_build: minspi=%d must be > 255.\n",
-			min);
+			"pfkey_spirange_build: "
+			"minspi=%08x must be > 255.\n",
+			ntohl(min));
 		SENDERR(EEXIST);
 	}
 	
@@ -864,7 +914,8 @@ pfkey_spirange_build(struct sadb_ext**	pfkey_ext,
 	     pfkey_spirange = (struct sadb_spirange*)
 	     MALLOC(sizeof(struct sadb_spirange)))) {
 		DEBUGGING(
-			"pfkey_spirange_build: memory allocation failed\n");
+			"pfkey_spirange_build: "
+			"memory allocation failed\n");
 		SENDERR(ENOMEM);
 	}
 	memset(pfkey_spirange,
@@ -890,14 +941,16 @@ pfkey_x_kmprivate_build(struct sadb_ext**	pfkey_ext)
 	/* sanity checks... */
 	if(pfkey_x_kmprivate) {
 		DEBUGGING(
-			"pfkey_x_kmprivate_build:why is pfkey_x_kmprivate already pointing to something?\n");
+			"pfkey_x_kmprivate_build: "
+			"why is pfkey_x_kmprivate already pointing to something?\n");
 		SENDERR(EINVAL);
 	}
 	
 	pfkey_x_kmprivate->sadb_x_kmprivate_reserved = 0;
 
 	DEBUGGING(
-		"pfkey_x_kmprivate_build: Sorry, I can't build exttype=%d yet.\n",
+		"pfkey_x_kmprivate_build: "
+		"Sorry, I can't build exttype=%d yet.\n",
 		(*pfkey_ext)->sadb_ext_type);
 	SENDERR(EINVAL); /* don't process these yet */
 
@@ -905,7 +958,8 @@ pfkey_x_kmprivate_build(struct sadb_ext**	pfkey_ext)
 	     pfkey_x_kmprivate = (struct sadb_x_kmprivate*)
 	     MALLOC(sizeof(struct sadb_x_kmprivate)))) {
 		DEBUGGING(
-			"pfkey_x_kmprivate_build: memory allocation failed\n");
+			"pfkey_x_kmprivate_build: "
+			"memory allocation failed\n");
 		SENDERR(ENOMEM);
 	}
 	memset(pfkey_x_kmprivate,
@@ -934,19 +988,22 @@ pfkey_x_satype_build(struct sadb_ext**	pfkey_ext,
 	/* sanity checks... */
 	if(pfkey_x_satype) {
 		DEBUGGING(
-			"pfkey_x_satype_build:why is pfkey_x_satype already pointing to something?\n");
+			"pfkey_x_satype_build: "
+			"why is pfkey_x_satype already pointing to something?\n");
 		SENDERR(EINVAL);
 	}
 	
 	if(!satype) {
 		DEBUGGING(
-			"pfkey_x_satype_build: SA type not set, must be non-zero.\n");
+			"pfkey_x_satype_build: "
+			"SA type not set, must be non-zero.\n");
 		SENDERR(EINVAL);
 	}
 
 	if(satype > SADB_SATYPE_MAX) {
 		DEBUGGING(
-			"pfkey_x_satype_build: satype %d > max %d\n", 
+			"pfkey_x_satype_build: "
+			"satype %d > max %d\n", 
 			satype, SADB_SATYPE_MAX);
 		SENDERR(EINVAL);
 	}
@@ -954,7 +1011,8 @@ pfkey_x_satype_build(struct sadb_ext**	pfkey_ext,
 	if(!(*pfkey_ext = (struct sadb_ext*)pfkey_x_satype = (struct sadb_x_satype*)
 	     MALLOC(sizeof(struct sadb_x_satype)))) {
 		DEBUGGING(
-			"pfkey_x_satype_build: memory allocation failed\n");
+			"pfkey_x_satype_build: "
+			"memory allocation failed\n");
 		SENDERR(ENOMEM);
 	}
 	memset(pfkey_x_satype,
@@ -984,7 +1042,9 @@ pfkey_x_debug_build(struct sadb_ext**	pfkey_ext,
 		    uint32_t		esp,
 		    uint32_t		ah,
 		    uint32_t		rcv,
-		    uint32_t            pfkey)
+		    uint32_t            pfkey,
+		    uint32_t            ipcomp,
+		    uint32_t            verbose)
 {
 	int error = 0;
 	int i;
@@ -995,18 +1055,21 @@ pfkey_x_debug_build(struct sadb_ext**	pfkey_ext,
 	/* sanity checks... */
 	if(pfkey_x_debug) {
 		DEBUGGING(
-			"pfkey_x_debug_build:why is pfkey_x_debug already pointing to something?\n");
+			"pfkey_x_debug_build: "
+			"why is pfkey_x_debug already pointing to something?\n");
 		SENDERR(EINVAL);
 	}
 	
 	DEBUGGING(
-		"pfkey_x_debug_build:tunnel=%x netlink=%x xform=%x eroute=%x spi=%x radij=%x esp=%x ah=%x rcv=%x pfkey=%x?\n",
-		tunnel, netlink, xform, eroute, spi, radij, esp, ah, rcv, pfkey);
+		"pfkey_x_debug_build: "
+		"tunnel=%x netlink=%x xform=%x eroute=%x spi=%x radij=%x esp=%x ah=%x rcv=%x pfkey=%x ipcomp=%x verbose=%x?\n",
+		tunnel, netlink, xform, eroute, spi, radij, esp, ah, rcv, pfkey, ipcomp, verbose);
 
 	if(!(*pfkey_ext = (struct sadb_ext*)pfkey_x_debug = (struct sadb_x_debug*)
 	     MALLOC(sizeof(struct sadb_x_debug)))) {
 		DEBUGGING(
-			"pfkey_x_debug_build: memory allocation failed\n");
+			"pfkey_x_debug_build: "
+			"memory allocation failed\n");
 		SENDERR(ENOMEM);
 	}
 #if 0
@@ -1028,6 +1091,8 @@ pfkey_x_debug_build(struct sadb_ext**	pfkey_ext,
 	pfkey_x_debug->sadb_x_debug_ah = ah;
 	pfkey_x_debug->sadb_x_debug_rcv = rcv;
 	pfkey_x_debug->sadb_x_debug_pfkey = pfkey;
+	pfkey_x_debug->sadb_x_debug_ipcomp = ipcomp;
+	pfkey_x_debug->sadb_x_debug_verbose = verbose;
 
 	for(i=0; i<4; i++) {
 		pfkey_x_debug->sadb_x_debug_reserved[i] = 0;
@@ -1082,7 +1147,8 @@ pfkey_msg_build(struct sadb_msg **pfkey_msg, struct sadb_ext *extensions[], int
 	
 	if(!extensions[0]) {
 		DEBUGGING(
-			"pfkey_msg_build: extensions[0] must be specified (struct sadb_msg).\n");
+			"pfkey_msg_build: "
+			"extensions[0] must be specified (struct sadb_msg).\n");
 		SENDERR(EINVAL);
 	}
 
@@ -1095,12 +1161,14 @@ pfkey_msg_build(struct sadb_msg **pfkey_msg, struct sadb_ext *extensions[], int
 
 	if(!(*pfkey_msg = (struct sadb_msg*)MALLOC(total_size * IPSEC_PFKEYv2_ALIGN))) {
 		DEBUGGING(
-			"pfkey_msg_build: memory allocation failed\n");
+			"pfkey_msg_build: "
+			"memory allocation failed\n");
 		SENDERR(ENOMEM);
 	}
 
 	DEBUGGING(
-		"pfkey_msg_build: pfkey_msg=%p allocated %d bytes, &(extensions[0])=%p\n",
+		"pfkey_msg_build: "
+		"pfkey_msg=%p allocated %d bytes, &(extensions[0])=%p\n",
 		*pfkey_msg,
 		total_size * IPSEC_PFKEYv2_ALIGN,
 		&(extensions[0]));
@@ -1128,7 +1196,8 @@ pfkey_msg_build(struct sadb_msg **pfkey_msg, struct sadb_ext *extensions[], int
 				SENDERR(EINVAL);
 			}
 			DEBUGGING(
-				"pfkey_msg_build: copying %d bytes from extensions[%d]=%p to=%p\n",
+				"pfkey_msg_build: "
+				"copying %d bytes from extensions[%d]=%p to=%p\n",
 				(extensions[ext])->sadb_ext_len * IPSEC_PFKEYv2_ALIGN,
 				ext,
 				(extensions[ext]),
@@ -1144,8 +1213,8 @@ pfkey_msg_build(struct sadb_msg **pfkey_msg, struct sadb_ext *extensions[], int
 
 	/* check required extensions */
 	DEBUGGING(
-		"pfkey_msg_build: extensions "
-		"permitted=%08x, seen=%08x, required=%08x.\n",
+		"pfkey_msg_build: "
+		"extensions permitted=%08x, seen=%08x, required=%08x.\n",
 		extensions_bitmaps[dir][EXT_BITS_PERM][(*pfkey_msg)->sadb_msg_type],
 		extensions_seen,
 		extensions_bitmaps[dir][EXT_BITS_REQ][(*pfkey_msg)->sadb_msg_type]);
@@ -1154,7 +1223,8 @@ pfkey_msg_build(struct sadb_msg **pfkey_msg, struct sadb_ext *extensions[], int
 	    extensions_bitmaps[dir][EXT_BITS_REQ][(*pfkey_msg)->sadb_msg_type]) !=
 	   extensions_bitmaps[dir][EXT_BITS_REQ][(*pfkey_msg)->sadb_msg_type]) {
 		DEBUGGING(
-			"pfkey_msg_build: required extensions missing:%08x.\n",
+			"pfkey_msg_build: "
+			"required extensions missing:%08x.\n",
 			extensions_bitmaps[dir][EXT_BITS_REQ][(*pfkey_msg)->sadb_msg_type] -
 			(extensions_seen &
 			 extensions_bitmaps[dir][EXT_BITS_REQ][(*pfkey_msg)->sadb_msg_type]) );
@@ -1163,7 +1233,8 @@ pfkey_msg_build(struct sadb_msg **pfkey_msg, struct sadb_ext *extensions[], int
 	
 	if((error = pfkey_msg_parse(*pfkey_msg, NULL, extensions_check, dir))) {
 		DEBUGGING(
-			"pfkey_msg_build: Trouble parsing newly built pfkey message, error=%d.\n",
+			"pfkey_msg_build: "
+			"Trouble parsing newly built pfkey message, error=%d.\n",
 			error);
 		SENDERR(-error);
 	}
@@ -1175,6 +1246,69 @@ errlab:
 
 /*
  * $Log: pfkey_v2_build.c,v $
+ * Revision 1.31  2002/01/29 22:25:35  rgb
+ * Re-add ipsec_kversion.h to keep MALLOC happy.
+ *
+ * Revision 1.30  2002/01/29 01:59:09  mcr
+ * 	removal of kversions.h - sources that needed it now use ipsec_param.h.
+ * 	updating of IPv6 structures to match latest in6.h version.
+ * 	removed dead code from freeswan.h that also duplicated kversions.h
+ * 	code.
+ *
+ * Revision 1.29  2001/12/19 21:06:09  rgb
+ * Added port numbers to pfkey_address_build() debugging.
+ *
+ * Revision 1.28  2001/11/06 19:47:47  rgb
+ * Added packet parameter to lifetime and comb structures.
+ *
+ * Revision 1.27  2001/10/18 04:45:24  rgb
+ * 2.4.9 kernel deprecates linux/malloc.h in favour of linux/slab.h,
+ * lib/freeswan.h version macros moved to lib/kversions.h.
+ * Other compiler directive cleanups.
+ *
+ * Revision 1.26  2001/09/08 21:13:34  rgb
+ * Added pfkey ident extension support for ISAKMPd. (NetCelo)
+ *
+ * Revision 1.25  2001/06/14 19:35:16  rgb
+ * Update copyright date.
+ *
+ * Revision 1.24  2001/03/20 03:49:45  rgb
+ * Ditch superfluous debug_pfkey declaration.
+ * Move misplaced freeswan.h inclusion for kernel case.
+ *
+ * Revision 1.23  2001/03/16 07:41:50  rgb
+ * Put freeswan.h include before pluto includes.
+ *
+ * Revision 1.22  2001/02/27 22:24:56  rgb
+ * Re-formatting debug output (line-splitting, joining, 1arg/line).
+ * Check for satoa() return codes.
+ *
+ * Revision 1.21  2000/11/17 18:10:30  rgb
+ * Fixed bugs mostly relating to spirange, to treat all spi variables as
+ * network byte order since this is the way PF_KEYv2 stored spis.
+ *
+ * Revision 1.20  2000/10/12 00:02:39  rgb
+ * Removed 'format, ##' nonsense from debug macros for RH7.0.
+ *
+ * Revision 1.19  2000/10/10 20:10:20  rgb
+ * Added support for debug_ipcomp and debug_verbose to klipsdebug.
+ *
+ * Revision 1.18  2000/09/12 18:59:54  rgb
+ * Added Gerhard's IPv6 support to pfkey parts of libfreeswan.
+ *
+ * Revision 1.17  2000/09/12 03:27:00  rgb
+ * Moved DEBUGGING definition to compile kernel with debug off.
+ *
+ * Revision 1.16  2000/09/08 19:22:12  rgb
+ * Fixed pfkey_prop_build() parameter to be only single indirection.
+ * Fixed struct alg copy.
+ *
+ * Revision 1.15  2000/08/20 21:40:01  rgb
+ * Added an address parameter sanity check to pfkey_address_build().
+ *
+ * Revision 1.14  2000/08/15 17:29:23  rgb
+ * Fixes from SZI to untested pfkey_prop_build().
+ *
  * Revision 1.13  2000/06/02 22:54:14  rgb
  * Added Gerhard Gessler's struct sockaddr_storage mods for IPv6 support.
  *
