@@ -1,5 +1,5 @@
 /* information about connections between hosts and clients
- * Copyright (C) 1998, 1999  D. Hugh Redelmeier
+ * Copyright (C) 1998-2001  D. Hugh Redelmeier
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -11,7 +11,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: connections.h,v 1.38 2000/06/21 18:24:31 dhr Exp $
+ * RCSID $Id: connections.h,v 1.58 2002/01/30 06:36:47 dhr Exp $
  */
 
 /* There are two kinds of connections:
@@ -54,13 +54,16 @@
  *   gateway can get traffic for a specific (client) subnet.
  *   Furthermore, if the routing isn't in place, packets will
  *   be sent in the clear.
- *   "routed" indicates whether the routing has been done for
- *   this connection.
+ *   "routing" indicates whether the routing has been done for
+ *   this connection.  Note that several connections may claim
+ *   the same routing, as long as they agree about where the
+ *   packets are to be sent.
  * - With the current KLIPS, only one outbound IPsec SA bundle can be
  *   used for a particular client.  This is due to a limitation
  *   of using only routing for selection.  So only one IPsec state (SA)
  *   may "own" the eroute.  "eroute_owner" is the serial number of
- *   this state, or SOS_NOBODY if there is none.
+ *   this state, SOS_NOBODY if there is none.  "routing" indicates
+ *   what kind of erouting has been done for this connection, if any.
  *
  * Operations on Connections:
  *
@@ -70,42 +73,32 @@
  * - find a connection (by IP addresses of hosts)
  *   [response to peer request; finding ISAKMP connection for IPsec connection]
  *
- * When a connection is referenced by a state object, it must have
- * the IP addresses of the hosts filled in.  Some connections are
- * missing one or the other IP addresses (essentially "wildcarded",
- * denoted by a value of 0.0.0.0).  When one must be filled in (to allow
- * it to be used by a state object), a new connection is derived from
- * the old one.  It is deleted when no longer in use.
+ * Some connections are templates, missing the address of the peer
+ * (represented by INADDR_ANY).  These are always arranged so that the
+ * missing end is "that" (there can only be one missing end).  These can
+ * be instantiated (turned into real connections) by Pluto in one of two
+ * different ways: Road Warrior Instantiation or Opportunistic
+ * Instantiation.  A template connection is marked for Opportunistic
+ * Instantiation by specifying the peer client as 0.0.0.0/32 (or the IPV6
+ * equivalent).  Otherwise, it is suitable for Road Warrior Instantiation.
+ *
+ * Instantiation creates a new temporary connection, with the missing
+ * details filled in.  The resulting template lasts only as long as there
+ * is a stat that uses it.
  */
 
 struct end {
     struct id id;
-    struct in_addr	/* network order */
+    ip_address
 	host_addr,
-	host_nexthop,
-	client_net, client_mask;
+	host_nexthop;
+    ip_subnet client;
 
     bool has_client;
     char *updown;
     u_int16_t host_port;	/* host order */
 };
 
-/* test if IPv4 address a is inside end's client subnet
- * Warning: uses tricky bit twiddling.
- */
-#define inside_client(a, end) \
-    (0 == (((a).s_addr ^ (end).client_net.s_addr) & (end).client_mask.s_addr))
-
-/* test if client end matches subnet */
-#define client_is(end, addr, mask) \
-    (same_subnet((end).client_net, (end).client_mask, (addr), (mask)))
-
-/* test if ends a and b have the same client subnets */
-#define same_client(a, b)  (client_is((a), (b).client_net, (b).client_mask))
-
-/* test if end's client subnet is just its host */
-#define self_client(end)  (client_is((end), (end).host_addr, mask32.sin_addr))
-
 struct connection {
     char *name;
     lset_t policy;
@@ -120,62 +113,117 @@ struct connection {
 
     /* internal fields: */
 
+    enum connection_kind kind;
     const struct iface *interface;	/* filled in iff oriented */
-    bool routed;	/* is routing in place for peer's client? */
+    enum routing_t routing;	/* level of routing in place */
+
     so_serial_t	/* state object serial number */
 	newest_isakmp_sa,
 	newest_ipsec_sa,
 	eroute_owner;
 
-    enum rwcs {
-	rwcs_permanent,	/* normal connection */
-	rwcs_instance,	/* instance created for a particular attempt */
-	rwcs_going_away	/* being deleted -- don't delete again */
-    };
-    enum rwcs rw_state;
 
 #ifdef DEBUG
     unsigned int extra_debugging;
 #endif
 
-    struct host_pair *host_pair;
-    struct connection *hp_next;
+    /* note: if the client is the gateway, the following must be equal */
+    sa_family_t addr_family;	/* between gateways */
+    sa_family_t tunnel_addr_family;	/* between clients */
 
-    struct connection *next;
+    struct gw_info *gw_info;
+
+    struct host_pair *host_pair;
+    struct connection *hp_next;	/* host pair list link */
+
+    struct connection *ac_next;	/* all connections list link */
 };
 
 #define oriented(c) ((c).interface != NULL)
-extern bool orient(struct connection *c, bool must);
+extern bool orient(struct connection *c);
 
 extern bool same_peer_ids(const struct connection *c
     , const struct connection *d, const struct id *his_id);
 
 extern size_t format_end(char *buf, size_t buf_len
-    , struct end *this, struct end *that, bool is_left);
+    , const struct end *this, const struct end *that, bool is_left);
 
 struct whack_message;	/* forward declaration of tag whack_msg */
 extern void add_connection(const struct whack_message *wm);
 extern void initiate_connection(const char *name, int whackfd);
+extern void initiate_opportunistic(const ip_address *our_client
+    , const ip_address *peer_client, bool held, int whackfd);
 extern void terminate_connection(const char *nm);
-extern void unorient_connection(struct connection *c);
+extern void release_connection(struct connection *c);
 extern void delete_connection(struct connection *c);
 extern void delete_every_connection(void);
-extern void release_interface(struct iface *i);
-extern struct connection *route_owner(struct connection *c, bool eroute);
+extern void release_dead_interfaces(void);
+extern void check_orientations(void);
+extern struct connection *route_owner(struct connection *c
+    , struct connection **erop);
+extern struct connection *shunt_owner(const ip_subnet *ours
+    , const ip_subnet *his);
 
-#define HasWildcardIP(c) (is_NO_IP((c).that.host_addr))
-extern struct connection
-    *rw_connection(const struct connection *c, struct in_addr him);
+extern bool uniqueIDs;	/* --uniqueids? */
+extern void ISAKMP_SA_established(struct connection *c, so_serial_t serial);
+
+#define his_id_was_instantiated(c) ((c)->kind == CK_INSTANCE \
+    && id_is_ipaddr(&(c)->that.id) \
+    && sameaddr(&(c)->that.id.ip_addr, &(c)->that.host_addr))
+
+/* for Opportunism */
+extern bool HasWildcardClient(const struct connection *c);
+extern const ip_subnet *EffectivePeerClient(const struct connection *c);
 
 struct state;	/* forward declaration of tag (defined in state.h) */
 extern struct connection
     *con_by_name(const char *nm, bool strict),
-    *find_host_connection(struct in_addr me, u_int16_t my_port
-	, struct in_addr him, u_int16_t his_port),
+    *find_host_connection(const ip_address *me, u_int16_t my_port
+	, const ip_address *him, u_int16_t his_port),
     *refine_host_connection(const struct state *st, const struct id *id
 	, bool initiator),
     *find_client_connection(struct connection *c
-	, struct in_addr our_net, struct in_addr our_mask
-	, struct in_addr peer_net, struct in_addr peer_mask);
+	, const ip_subnet *our_net
+	, const ip_subnet *peer_net);
+
+/* instantiating routines
+ * Note: connection_discard() is in state.h because all its work
+ * is looking through state objects.
+ */
+struct gw_info;	/* forward declaration of tag (defined in dnskey.h) */
+extern struct connection
+    *rw_instantiate(const struct connection *c, const ip_address *him
+	, const struct id *his_id),
+    *oppo_instantiate(const struct connection *c, const ip_address *him
+	, const struct id *his_id, struct gw_info *gw
+	, const ip_address *our_client, const ip_address *peer_client),
+    *build_outgoing_opportunistic_connection(struct gw_info *gw
+	, const ip_address *our_client, const ip_address *peer_client);
+
+#define CONN_INST_BUF	(ADDRTOT_BUF+1 + SUBNETTOT_BUF+1)
+extern void fmt_conn_instance(const struct connection *c
+    , char buf[CONN_INST_BUF]);
+
+/* operations on "pending", the structure representing Quick Mode
+ * negotiations delayed until a Keying Channel has been negotiated.
+ */
+
+struct pending;	/* forward declaration (opaque outside connections.c) */
+
+extern void add_pending(int whack_sock
+    , struct state *isakmp_sa
+    , struct connection *c
+    , lset_t policy
+    , unsigned long try
+    , so_serial_t replacing);
+
+extern void release_pending_whacks(struct state *st, err_t story);
+extern void unpend(struct state *st);
+extern void update_pending(struct state *os, struct state *ns);
+extern void flush_pending_by_state(struct state *st);
+
+extern void connection_discard(struct connection *c);
+
+/* print connection status */
 
 extern void show_connections_status(void);
