@@ -1,5 +1,5 @@
 /* tables of names for values defined in constants.h
- * Copyright (C) 1998, 1999  D. Hugh Redelmeier.
+ * Copyright (C) 1998-2002  D. Hugh Redelmeier.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -11,7 +11,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: constants.c,v 1.31 2000/06/18 19:15:34 dhr Exp $
+ * RCSID $Id: constants.c,v 1.56 2002/01/29 04:23:37 dhr Exp $
  */
 
 /*
@@ -20,10 +20,14 @@
  */
 
 #include <stddef.h>
+#include <string.h>
 #include <stdio.h>
 #include <netinet/in.h>
 
+#include <freeswan.h>
+
 #include "constants.h"
+#include "defs.h"
 #include "packet.h"
 
 struct enum_names {
@@ -33,7 +37,6 @@ struct enum_names {
     const struct enum_names *en_next_range;	/* descriptor of next range */
 };
 
-
 /* version */
 
 static const char *const version_name[] = {
@@ -50,6 +53,7 @@ enum_names version_names =
 static const char *const timer_event_name[] = {
 	"EVENT_NULL",
 	"EVENT_REINIT_SECRET",
+	"EVENT_SHUNT_SCAN",
 	"EVENT_SO_DISCARD",
 	"EVENT_RETRANSMIT",
 	"EVENT_SA_REPLACE",
@@ -79,7 +83,9 @@ const char *const debug_bit_names[] = {
 	"debug-parsing",
 	"debug-emitting",
 	"debug-control",
+	"debug-lifecycle",
 	"debug-klips",
+	"debug-dns",
 	"debug-private",
 	NULL
     };
@@ -132,6 +138,32 @@ const char *const state_story[] = {
 	"got encrypted Informational Message",	/* STATE_INFO_PROTECTED */
     };
 
+/* kind of struct connection */
+
+static const char *const connection_kind_name[] = {
+    "CK_TEMPLATE",	/* abstract connection, with wildcard */
+    "CK_PERMANENT",	/* normal connection */
+    "CK_INSTANCE",	/* instance of template, created for a particular attempt */
+    "CK_GOING_AWAY"	/* instance being deleted -- don't delete again */
+};
+
+enum_names connection_kind_names =
+    { CK_TEMPLATE, CK_GOING_AWAY, connection_kind_name, NULL };
+
+/* routing status names */
+
+static const char *const routing_story_strings[] = {
+    "unrouted",	/* RT_UNROUTED: unrouted */
+    "unrouted HOLD",	/* RT_UNROUTED_HOLD: unrouted, but HOLD eroute installed */
+    "trap erouted",	/* RT_ROUTED_PROSPECTIVE: routed, and TRAP eroute installed */
+    "routed HOLD",	/* RT_ROUTED_HOLD: routed, and HOLD eroute installed */
+    "fail erouted",	/* RT_ROUTED_FAILURE: routed, and failure-context shunt eroute installed */
+    "erouted",	/* RT_ROUTED_TUNNEL: routed, and erouted to an IPSEC SA group */
+    };
+
+enum_names routing_story =
+    { RT_UNROUTED, RT_ROUTED_TUNNEL, routing_story_strings, NULL};
+
 /* Payload types (draft-ietf-ipsec-isakmp-09.txt section 3.1) */
 
 const char *const payload_name[] = {
@@ -222,10 +254,13 @@ static const char *const ah_transform_name[] = {
 	"AH_MD5",
 	"AH_SHA",
 	"AH_DES",
+	"AH_SHA2_256",
+	"AH_SHA2_384",
+	"AH_SHA2_512",
     };
 
 enum_names ah_transformid_names =
-    { AH_MD5, AH_DES, ah_transform_name, NULL };
+    { AH_MD5, AH_SHA2_512, ah_transform_name, NULL };
 
 /* IPsec ESP transform values */
 
@@ -241,22 +276,23 @@ static const char *const esp_transform_name[] = {
 	"ESP_DES_IV32",
 	"ESP_RC4",
 	"ESP_NULL",
+	"ESP_AES",
     };
 
 enum_names esp_transformid_names =
-    { ESP_DES_IV64, ESP_NULL, esp_transform_name, NULL };
+    { ESP_DES_IV64, ESP_AES, esp_transform_name, NULL };
 
 /* IPCOMP transform values */
 
-static const char *const ipcomp_name[] = {
+static const char *const ipcomp_transform_name[] = {
 	"IPCOMP_OUI",
 	"IPCOMP_DEFLAT",
 	"IPCOMP_LZS",
 	"IPCOMP_V42BIS",
     };
 
-enum_names ipcomp_names =
-    { IPCOMP_OUI, IPCOMP_V42BIS, ipcomp_name, NULL };
+enum_names ipcomp_transformid_names =
+    { IPCOMP_OUI, IPCOMP_V42BIS, ipcomp_transform_name, NULL };
 
 /* Identification type values */
 
@@ -277,15 +313,23 @@ static const char *const ident_name[] = {
 enum_names ident_names =
     { ID_IPV4_ADDR, ID_KEY_ID, ident_name, NULL };
 
-/* Goal BITs for establishing an SA */
+/* Goal BITs for establishing an SA
+ * Note: we drop the POLICY_ prefix so that logs are more concise.
+ */
 
 const char *const sa_policy_bit_names[] = {
-	"POLICY_PSK",
-	"POLICY_RSASIG",
-	"POLICY_ENCRYPT",
-	"POLICY_AUTHENTICATE",
-	"POLICY_TUNNEL",
-	"POLICY_PFS",
+	"PSK",
+	"RSASIG",
+	"ENCRYPT",
+	"AUTHENTICATE",
+	"COMPRESS",
+	"TUNNEL",
+	"PFS",
+	"DISABLEARRIVALCHECK",
+	"PASS",
+	"DROP",
+	"DONTREKEY",
+	"OPPORTUNISTIC",
 	NULL
     };
 
@@ -466,10 +510,11 @@ static const char *const oakley_enc_name[] = {
 	"OAKLEY_RC5_R16_B64_CBC",
 	"OAKLEY_3DES_CBC",
 	"OAKLEY_CAST_CBC",
+	"OAKLEY_AES_CBC",
     };
 
 enum_names oakley_enc_names =
-    { OAKLEY_DES_CBC, OAKLEY_CAST_CBC, oakley_enc_name, NULL };
+    { OAKLEY_DES_CBC, OAKLEY_AES_CBC, oakley_enc_name, NULL };
 
 /* Oakley Hash Algorithm attribute */
 
@@ -477,14 +522,17 @@ static const char *const oakley_hash_name[] = {
 	"OAKLEY_MD5",
 	"OAKLEY_SHA",
 	"OAKLEY_TIGER",
+	"OAKLEY_SHA2_256",
+	"OAKLEY_SHA2_384",
+	"OAKLEY_SHA2_512",
     };
 
 enum_names oakley_hash_names =
-    { OAKLEY_MD5, OAKLEY_TIGER, oakley_hash_name, NULL };
+    { OAKLEY_MD5, OAKLEY_SHA2_512, oakley_hash_name, NULL };
 
 /* Oakley Authentication Method attribute */
 
-static const char *const oakley_auth_name[] = {
+static const char *const oakley_auth_name1[] = {
 	"OAKLEY_PRESHARED_KEY",
 	"OAKLEY_DSS_SIG",
 	"OAKLEY_RSA_SIG",
@@ -493,9 +541,24 @@ static const char *const oakley_auth_name[] = {
 	"OAKLEY_ELGAMAL_ENC",
 	"OAKLEY_ELGAMAL_ENC_REV",
     };
+static const char *const oakley_auth_name2[] = {
+	"XAUTHInitPreShared",
+	"XAUTHRespPreShared",
+	"XAUTHInitDSS",
+	"XAUTHRespDSS",
+	"XAUTHInitRSA",
+	"XAUTHRespRSA",
+	"XAUTHInitRSAEncryption",
+	"XAUTHRespRSAEncryption",
+	"XAUTHInitRSARevisedEncryption",
+	"XAUTHRespRSARevisedEncryption",
+    };
 
-enum_names oakley_auth_names =
-    { OAKLEY_PRESHARED_KEY, OAKLEY_ELGAMAL_ENC_REV, oakley_auth_name, NULL };
+static enum_names oakley_auth_names1 =
+    { OAKLEY_PRESHARED_KEY, OAKLEY_ELGAMAL_ENC_REV, oakley_auth_name1, NULL };
+
+enum_names oakley_auth_names = { XAUTHInitPreShared
+    , XAUTHRespRSARevisedEncryption, oakley_auth_name2, &oakley_auth_names1 };
 
 /* Oakley Group Description attribute */
 
@@ -580,6 +643,58 @@ enum_names ipsec_notification_names =
 	ipsec_notification_name, &notification_status_names };
 
 
+/* socket address family info */
+
+static const char *const af_inet_name[] = {
+	"AF_INET",
+    };
+
+static const char *const af_inet6_name[] = {
+	"AF_INET6",
+    };
+
+static enum_names af_names6 = { AF_INET6, AF_INET6, af_inet6_name, NULL };
+
+enum_names af_names = { AF_INET, AF_INET, af_inet_name, &af_names6 };
+
+static ip_address ipv4_any, ipv6_any;
+static ip_subnet ipv4_wildcard, ipv6_wildcard;
+static ip_subnet ipv4_all, ipv6_all;
+
+const struct af_info af_inet4_info = {
+	AF_INET,
+	"AF_INET",
+	sizeof(struct in_addr),
+	sizeof(struct sockaddr_in),
+	32,
+	ID_IPV4_ADDR, ID_IPV4_ADDR_SUBNET, ID_IPV4_ADDR_RANGE,
+	&ipv4_any, &ipv4_wildcard, &ipv4_all,
+    };
+
+const struct af_info af_inet6_info = {
+	AF_INET6,
+	"AF_INET6",
+	sizeof(struct in6_addr),
+	sizeof(struct sockaddr_in6),
+	128,
+	ID_IPV6_ADDR, ID_IPV6_ADDR_SUBNET, ID_IPV6_ADDR_RANGE,
+	&ipv6_any, &ipv6_wildcard, &ipv6_all,
+    };
+
+const struct af_info *
+aftoinfo(int af)
+{
+    switch (af)
+    {
+	case AF_INET:
+	    return &af_inet4_info;
+	case AF_INET6:
+	    return &af_inet6_info;
+	default:
+	    return NULL;
+    }
+}
+
 /* BIND enumerated types */
 
 #include <arpa/nameser.h>
@@ -647,7 +762,8 @@ enum_names rr_class_names = { C_IN, C_IN, rr_class_name, NULL };
 
 /* look up enum names in an enum_names */
 
-const char *enum_name(enum_names *ed, unsigned long val)
+const char *
+enum_name(enum_names *ed, unsigned long val)
 {
     enum_names	*p;
 
@@ -697,7 +813,7 @@ bitnamesof(const char *const table[], lset_t val)
 	    if (n == NULL || *n == '\0')
 	    {
 		/* no name for this bit, so use hex */
-		static char flagbuf[10];
+		static char flagbuf[sizeof("0x80000000")];
 
 		snprintf(flagbuf, sizeof(flagbuf), "0x%lx", bit);
 		n = flagbuf;
@@ -705,7 +821,7 @@ bitnamesof(const char *const table[], lset_t val)
 
 	    nl = strlen(n);
 
-	    if (p != buf && buf < buf+sizeof(buf) - 1)
+	    if (p != buf && p < buf+sizeof(buf) - 1)
 		*p++ = '+';
 
 	    if (buf+sizeof(buf) - p > (ptrdiff_t)nl) {
@@ -738,3 +854,55 @@ testset(const char *const table[], lset_t val)
     }
     return TRUE;
 }
+
+
+const char sparse_end[] = "end of sparse names";
+
+/* look up enum names in a sparse_names */
+const char *sparse_name(sparse_names sd, unsigned long val)
+{
+    const struct sparse_name *p;
+
+    for (p = sd; p->name != sparse_end; p++)
+	if (p->val == val)
+	    return p->name;
+    return NULL;
+}
+
+/* find or construct a string to describe an sparse value
+ * Result may be in STATIC buffer!
+ */
+const char *
+sparse_val_show(sparse_names sd, unsigned long val)
+{
+    const char *p = sparse_name(sd, val);
+
+    if (p == NULL)
+    {
+	static char buf[12];	/* only one!  I hope that it is big enough */
+
+	snprintf(buf, sizeof(buf), "%lu??", val);
+	p = buf;
+    }
+    return p;
+}
+
+void init_constants(void)
+{
+    err_t ugh;
+
+    ugh = anyaddr(AF_INET, &ipv4_any);
+    passert(ugh == NULL);
+    ugh = anyaddr(AF_INET, &ipv6_any);
+    passert(ugh == NULL);
+
+    ugh = rangetosubnet(&ipv4_any, &ipv4_any, &ipv4_wildcard);
+    passert(ugh == NULL);
+    ugh = rangetosubnet(&ipv6_any, &ipv6_any, &ipv6_wildcard);
+    passert(ugh == NULL);
+
+    ugh = initsubnet(&ipv4_any, 0, '0', &ipv4_all);
+    passert(ugh == NULL);
+    ugh = initsubnet(&ipv6_any, 0, '0', &ipv6_all);
+    passert(ugh == NULL);
+}
