@@ -1,6 +1,6 @@
 /* manifest constants
  * Copyright (C) 1997 Angelos D. Keromytis.
- * Copyright (C) 1998, 1999  D. Hugh Redelmeier.
+ * Copyright (C) 1998-2002  D. Hugh Redelmeier.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -12,9 +12,11 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: constants.h,v 1.51 2000/06/20 20:09:45 dhr Exp $
+ * RCSID $Id: constants.h,v 1.79 2002/02/05 21:50:19 dhr Exp $
  */
 
+extern void init_constants(void);
+
 /*
  * NOTE:For debugging purposes, constants.c has tables to map numbers back to names.
  * Any changes here should be reflected there.
@@ -28,8 +30,8 @@
  * The fact that the string is a constant is limiting, but it
  * avoids storage management issues: the recipient is allowed to assume
  * that the string will live "long enough" (usually forever).
+ * <freeswan.h> defines err_t for this return type.
  */
-typedef const char *complaint_t;	/* NULL iff happy */
 
 typedef int bool;
 #define FALSE	0
@@ -40,12 +42,15 @@ typedef int bool;
 
 #define BITS_PER_BYTE	8
 
+#define streq(a, b) (strcmp((a), (b)) == 0)	/* clearer shorthand */
+
 /* set type with room for at least 32 elements */
 
 typedef unsigned long lset_t;
 #define LEMPTY 0UL
 #define LELEM(opt) (1UL << (opt))
-#define LRANGE(lwb, upb) (LELEM(upb) + LELEM(upb) - LELEM(lwb))
+#define LRANGE(lwb, upb) LRANGES(LELEM(lwb), LELEM(upb))
+#define LRANGES(first, last) (last - first + last)
 #define LALLIN(set, probe)	(((set) & (probe)) == (probe))
 
 /* Control and lock pathnames */
@@ -75,6 +80,21 @@ extern const char *enum_show(enum_names *ed, unsigned long val);
 extern bool testset(const char *const table[], lset_t val);
 extern const char *bitnamesof(const char *const table[], lset_t val);
 
+/* sparse_names is much like enum_names, except values are
+ * not known to be contiguous or ordered.
+ * The array of names is ended with one with the name sparse_end
+ * (this avoids having to reserve a value to signify the end).
+ * Often appropriate for enums defined by others.
+ */
+struct sparse_name {
+    unsigned long val;
+    const char *const name;
+};
+typedef const struct sparse_name sparse_names[];
+
+extern const char *sparse_name(sparse_names sd, unsigned long val);
+extern const char *sparse_val_show(sparse_names sd, unsigned long val);
+extern const char sparse_end[];
 
 #define FULL_INET_ADDRESS_SIZE    6
 
@@ -137,33 +157,30 @@ extern const char *bitnamesof(const char *const table[], lset_t val);
 
 #define IKE_UDP_PORT	500
 
-#ifndef PORT
-# define PORT IKE_UDP_PORT
-#endif
-
 /* Timer events */
 
 extern enum_names timer_event_names;
 
-#define EVENT_NULL				0 /* non-event */
-#define EVENT_REINIT_SECRET			1 /* Refresh cookie secret */
-#define EVENT_SO_DISCARD			2 /* discard unfinished state object */
-#define EVENT_RETRANSMIT			3 /* Retransmit packet */
-#define EVENT_SA_REPLACE			4 /* SA replacement event */
-#define EVENT_SA_EXPIRE				5 /* SA expiration event */
+enum event_type {
+    EVENT_NULL,	/* non-event */
+    EVENT_REINIT_SECRET,	/* Refresh cookie secret */
+    EVENT_SHUNT_SCAN,	/* scan shunt eroutes known to kernel */
+    EVENT_SO_DISCARD,	/* discard unfinished state object */
+    EVENT_RETRANSMIT,	/* Retransmit packet */
+    EVENT_SA_REPLACE,	/* SA replacement event */
+    EVENT_SA_EXPIRE	/* SA expiration event */
+};
 
 #define EVENT_REINIT_SECRET_DELAY		3600 /* 1 hour */
 #define EVENT_RETRANSMIT_DELAY_0		10   /* 10 seconds */
 
 /* Misc. stuff */
 
-#define ISAKMP_RESPONDER     0
-#define ISAKMP_INITIATOR     1
-
 #define MAXIMUM_RETRANSMISSIONS              2
 #define MAXIMUM_RETRANSMISSIONS_INITIAL      20
 
-#define UDP_SIZE             65536
+#define MAX_INPUT_UDP_SIZE             65536
+#define MAX_OUTPUT_UDP_SIZE            65536
 
 /* Version numbers */
 
@@ -186,8 +203,9 @@ extern enum_names doi_names;
 #define IPSEC_DOI_SPI_SIZE         4
 
 /* SPI value 0 is invalid and values 1-255 are reserved to IANA.
- * RFC 2402 (ESP) 2.4, RFC 2406 (AH) 2.1
- * IPCOMP???
+ * ESP: RFC 2402 2.4; AH: RFC 2406 2.1
+ * IPComp RFC 2393 substitutes a CPI in the place of an SPI.
+ * see also draft-shacham-ippcp-rfc2393bis-05.txt.
  * We (FreeS/WAN) reserve 0x100 to 0xFFF for manual keying, so
  * Pluto won't generate these values.
  */
@@ -206,70 +224,76 @@ extern const char *const debug_bit_names[];
 #define DBG_PARSING	0x04	/* show decoding of messages */
 #define DBG_EMITTING	0x08	/* show encoding of messages */
 #define DBG_CONTROL	0x10	/* control flow within Pluto */
-#define DBG_KLIPS	0x20	/* messages to KLIPS */
-#define DBG_PRIVATE	0x40	/* private information: DANGER! */
+#define DBG_LIFECYCLE	0x20	/* SA lifecycle */
+#define DBG_KLIPS	0x40	/* messages to KLIPS */
+#define DBG_DNS		0x80	/* DNS activity */
+#define DBG_PRIVATE	0x100	/* private information: DANGER! */
 
 #define DBG_NONE	0	/* no options on */
-#define DBG_ALL		0x3f	/* all options on EXCEPT DBG_PRIVATE */
+#define DBG_ALL		LRANGES(DBG_RAW, DBG_DNS)	/* all options on EXCEPT DBG_PRIVATE */
 #endif
 
 /* State of exchanges
  *
  * The name of the state describes the last message sent, not the
  * message currently being input or output (except during retry).
+ * In effect, the state represents the last completed action.
  *
- * STATE_MAIN_R0 and STATE_QUICK_R0 are intermediate state (not
- * retained between messages) representing the state when the first
- * message of an exchange has been read but not processed.  The name
- * reflects the fiction that a zeroth responder message has been sent.
+ * Messages are named [MQ][IR]n where
+ * - M stands for Main Mode (Phase 1);
+ *   Q stands for Quick Mode (Phase 2)
+ * - I stands for Initiator;
+ *   R stands for Responder
+ * - n, a digit, stands for the number of the message
  *
- * STATE_MAIN_I4 is an exception because there is no such message.
- * It is best to think of this as a notional empty message following
- * the receipt of STATE_MAIN_R3 message.
+ * It would be more convenient if each state accepted a message
+ * and produced one.  This is the case for states at the start
+ * or end of an exchange.  To fix this, we pretend that there are
+ * MR0 and QR0 messages before the MI1 and QR1 messages.  Similarly,
+ * we pretend that there are MR4 and QR2 messages.
  *
- * Similarly, there is no STATE_QUICK_R2 message, but the state
- * represents having received the STATE_QUICK_I2 message.
+ * STATE_MAIN_R0 and STATE_QUICK_R0 are intermediate states (not
+ * retained between messages) representing the state that accepts the
+ * first message of an exchange has been read but not processed.
  *
- * state                 routine that sets it
- * =====                 ====================
- * STATE_MAIN_R0       comm_handle: Main Mode packet but no state
- * STATE_MAIN_I1       main_outI1
- * STATE_MAIN_R1       main_inI1_outR1
- * STATE_MAIN_I2       main_inR1_outI2
- * STATE_MAIN_R2       main_inI2_outR2
- * STATE_MAIN_I3       main_inR2_outI3
- * STATE_MAIN_R3       main_inI3_outR3
- * STATE_MAIN_I4       main_inR3
- *
- * STATE_QUICK_R0      comm_handle: Quick Mode packet no Quick Mode state
- * STATE_QUICK_I1      quick_outI1
- * STATE_QUICK_R1      quick_inI1_outR1
- * STATE_QUICK_I2      quick_inR1_outI2
- * STATE_QUICK_R2      quick_inI2
+ * state_microcode state_microcode_table in demux.c describes
+ * other important details.
  */
 
 extern enum_names state_names;
 extern const char *const state_story[];
 
-#define STATE_MAIN_R0       1
-#define STATE_MAIN_I1       2
-#define STATE_MAIN_R1       3
-#define STATE_MAIN_I2       4
-#define STATE_MAIN_R2       5
-#define STATE_MAIN_I3       6
-#define STATE_MAIN_R3       7
-#define STATE_MAIN_I4       8
+enum state_kind {
+    STATE_UNDEFINED,	/* 0 -- most likely accident */
 
-#define STATE_QUICK_R0      9
-#define STATE_QUICK_I1      10
-#define STATE_QUICK_R1      11
-#define STATE_QUICK_I2      12
-#define STATE_QUICK_R2      13
+    /*  Opportunism states: see "Opportunistic Encryption" 2.2 */
 
-#define STATE_INFO           14
-#define STATE_INFO_PROTECTED 15
+    OPPO_ACQUIRE,	/* got an ACQUIRE message for this pair */
+    OPPO_GW_DISCOVERED,	/* got TXT specifying gateway */
 
-#define STATE_ROOF          16
+    /* IKE states */
+
+    STATE_MAIN_R0,
+    STATE_MAIN_I1,
+    STATE_MAIN_R1,
+    STATE_MAIN_I2,
+    STATE_MAIN_R2,
+    STATE_MAIN_I3,
+    STATE_MAIN_R3,
+    STATE_MAIN_I4,
+
+    STATE_QUICK_R0,
+    STATE_QUICK_I1,
+    STATE_QUICK_R1,
+    STATE_QUICK_I2,
+    STATE_QUICK_R2,
+
+    STATE_INFO,
+    STATE_INFO_PROTECTED
+};
+
+#define STATE_IKE_FLOOR	STATE_MAIN_R0
+#define STATE_IKE_ROOF	(STATE_INFO_PROTECTED + 1)
 
 #define IS_PHASE1(s) (STATE_MAIN_R0 <= (s) && (s) <= STATE_MAIN_I4)
 #define IS_QUICK(s) (STATE_QUICK_R0 <= (s) && (s) <= STATE_QUICK_R2)
@@ -277,6 +301,37 @@ extern const char *const state_story[];
 #define IS_IPSEC_SA_ESTABLISHED(s) ((s) == STATE_QUICK_I2 || (s) == STATE_QUICK_R2)
 #define IS_ONLY_INBOUND_IPSEC_SA_ESTABLISHED(s) ((s) == STATE_QUICK_R1)
 
+/* kind of struct connection */
+
+extern enum_names connection_kind_names;
+
+enum connection_kind {
+    CK_TEMPLATE,	/* abstract connection, with wildcard */
+    CK_PERMANENT,	/* normal connection */
+    CK_INSTANCE,	/* instance of template, created for a particular attempt */
+    CK_GOING_AWAY	/* instance being deleted -- don't delete again */
+};
+
+
+/* routing status.
+ * Note: routing ignores source address, but erouting does not!
+ */
+
+extern enum_names routing_story;
+
+enum routing_t {
+    RT_UNROUTED,	/* unrouted */
+    RT_UNROUTED_HOLD,	/* unrouted, but HOLD shunt installed */
+    RT_ROUTED_PROSPECTIVE,	/* routed, and TRAP shunt installed */
+    RT_ROUTED_HOLD,	/* routed, and HOLD shunt installed */
+    RT_ROUTED_FAILURE,	/* routed, and failure-context shunt installed */
+    RT_ROUTED_TUNNEL	/* routed, and erouted to an IPSEC SA group */
+};
+
+#define routed(rs) ((rs) > RT_UNROUTED_HOLD)
+#define erouted(rs) ((rs) != RT_UNROUTED)
+#define shunt_erouted(rs) (erouted(rs) && (rs) != RT_ROUTED_TUNNEL)
+
 /* Payload types
  * RFC2408 Internet Security Association and Key Management Protocol (ISAKMP)
  * section 3.1
@@ -364,6 +419,7 @@ extern enum_names protocol_names;
 #define trans_show(p, t) \
     ((p)==PROTO_IPSEC_AH ? enum_show(&ah_transformid_names, (t)) \
     : (p)==PROTO_IPSEC_ESP ? enum_show(&esp_transformid_names, (t)) \
+    : (p)==PROTO_IPCOMP ? enum_show(&ipcomp_transformid_names, (t)) \
     : "??")
 
 /* IPsec ISAKMP transform values
@@ -376,6 +432,7 @@ extern enum_names isakmp_transformid_names;
 
 /* IPsec AH transform values
  * RFC2407 The Internet IP security Domain of Interpretation for ISAKMP 4.4.3
+ * and in http://www.isi.edu/in-notes/iana/assignments/isakmp-registry
  */
 
 extern enum_names ah_transformid_names;
@@ -383,9 +440,13 @@ extern enum_names ah_transformid_names;
 #define AH_MD5                   2
 #define AH_SHA                   3
 #define AH_DES                   4
+#define AH_SHA2_256              5
+#define AH_SHA2_384              6
+#define AH_SHA2_512              7
 
 /* IPsec ESP transform values
  * RFC2407 The Internet IP security Domain of Interpretation for ISAKMP 4.4.4
+ * and from http://www.isi.edu/in-notes/iana/assignments/isakmp-registry
  */
 
 extern enum_names esp_transformid_names;
@@ -402,18 +463,25 @@ extern enum_names esp_transformid_names;
 #define ESP_DES_IV32             9
 #define ESP_RC4                 10
 #define ESP_NULL                11
+#define ESP_AES                 12
 
 /* IPCOMP transform values
  * RFC2407 The Internet IP security Domain of Interpretation for ISAKMP 4.4.5
  */
 
-extern enum_names ipcomp_names;
+extern enum_names ipcomp_transformid_names;
 
 #define IPCOMP_OUI               1
-#define IPCOMP_DEFLAT            2
+#define IPCOMP_DEFLATE           2
 #define IPCOMP_LZS               3
 #define IPCOMP_V42BIS            4
 
+/* the following are from RFC 2393/draft-shacham-ippcp-rfc2393bis-05.txt 3.3 */
+typedef u_int16_t cpi_t;
+#define IPCOMP_CPI_SIZE          2
+#define IPCOMP_FIRST_NEGOTIATED  256
+#define IPCOMP_LAST_NEGOTIATED   61439
+
 /* Identification type values
  * RFC 2407 The Internet IP security Domain of Interpretation for ISAKMP 4.6.2.1
  */
@@ -435,24 +503,50 @@ extern enum_names ident_names;
 
 /* Policies for establishing an SA
  *
- * These are used to specify attributes (eg. encryption) and techniques for
- * (eg PFS) required of an SA.  POLICY_PSK and POLICY_RSASIG are for
- * ISAKMP SAs; the rest are about IPsec SAs
+ * These are used to specify attributes (eg. encryption) and techniques
+ * (eg PFS) for an SA.
  */
 
 extern const char *const sa_policy_bit_names[];
 
+/* ISAKMP auth techniques */
 #define POLICY_PSK           LELEM(0)
 #define POLICY_RSASIG        LELEM(1)
 
-#define POLICY_ENCRYPT       LELEM(2)
-#define POLICY_AUTHENTICATE  LELEM(3)
-#define POLICY_TUNNEL        LELEM(4)
-#define POLICY_PFS           LELEM(5)
-
 #define POLICY_ISAKMP_SHIFT	0	/* log2(POLICY_PSK) */
-#define POLICY_ISAKMP_MASK	(POLICY_PSK | POLICY_RSASIG)
+#define POLICY_ID_AUTH_MASK	LRANGES(POLICY_PSK, POLICY_RSASIG)
+#define POLICY_ISAKMP_MASK	POLICY_ID_AUTH_MASK	/* all so far */
+
+/* Quick Mode (IPSEC) attributes */
+#define POLICY_ENCRYPT       LELEM(2)	/* must be first of IPSEC policies */
+#define POLICY_AUTHENTICATE  LELEM(3)	/* must be second */
+#define POLICY_COMPRESS      LELEM(4)	/* must be third */
+#define POLICY_TUNNEL        LELEM(5)
+#define POLICY_PFS           LELEM(6)
+#define POLICY_DISABLEARRIVALCHECK  LELEM(7)	/* supress tunnel egress address checking */
+
 #define POLICY_IPSEC_SHIFT	2	/* log2(POLICY_ENCRYPT) */
+#define POLICY_IPSEC_MASK	LRANGES(POLICY_ENCRYPT, POLICY_DISABLEARRIVALCHECK)
+
+/* opportunistic attributes: what to do with a packet without a tunnel */
+#define POLICY_PASS     LELEM(8)
+#define POLICY_DROP     LELEM(9)
+/* PASS | DROP is construed as REJECT */
+
+#define POLICY_OPPO_SHIFT	8	/* log2(POLICY_PASS) */
+#define POLICY_OPPO_MASK	(POLICY_PASS | POLICY_DROP)
+
+/* connection policy
+ * Other policies could vary per state object.  These live in connection.
+ */
+#define POLICY_DONT_REKEY     LELEM(10)	/* don't rekey state either Phase */
+#define POLICY_OPPO     LELEM(11)	/* is this opportunistic? */
+
+
+/* Any IPsec policy?  If not, a connection description
+ * is only for ISAKMP SA, not IPSEC SA.  (A pun, I admit.)
+ */
+#define HAS_IPSEC_POLICY(p) (((p) & POLICY_IPSEC_MASK) != 0)
 
 /* Oakley transform attributes
  * draft-ietf-ipsec-ike-01.txt appendix A
@@ -504,6 +598,15 @@ extern enum_names *ipsec_attr_val_descs[];
 /* SA Lifetime Type attribute
  * RFC2407 The Internet IP security Domain of Interpretation for ISAKMP 4.5
  * Default time specified in 4.5
+ *
+ * There are two defaults for IPSEC SA lifetime, SA_LIFE_DURATION_DEFAULT,
+ * and PLUTO_SA_LIFE_DURATION_DEFAULT.
+ * SA_LIFE_DURATION_DEFAULT is specified in RFC2407 "The Internet IP
+ * Security Domain of Interpretation for ISAKMP" 4.5.  It applies when
+ * an ISAKMP negotiation does not explicitly specify a life duration.
+ * PLUTO_SA_LIFE_DURATION_DEFAULT is specified in pluto(8).  It applies
+ * when a connection description does not specify --ipseclifetime.
+ * The value of SA_LIFE_DURATION_MAXIMUM is our local policy.
  */
 
 extern enum_names sa_lifetime_names;
@@ -511,8 +614,9 @@ extern enum_names sa_lifetime_names;
 #define SA_LIFE_TYPE_SECONDS   1
 #define SA_LIFE_TYPE_KBYTES    2
 
-#define SA_LIFE_DURATION_DEFAULT    28800 /* Eight hours */
-#define SA_LIFE_DURATION_MAXIMUM    86400 /* One day */
+#define SA_LIFE_DURATION_DEFAULT    28800 /* eight hours (RFC2407 4.5) */
+#define PLUTO_SA_LIFE_DURATION_DEFAULT    28800 /* eight hours (pluto(8)) */
+#define SA_LIFE_DURATION_MAXIMUM    86400 /* one day */
 
 #define SA_REPLACEMENT_MARGIN_DEFAULT	    540	  /* (IPSEC & IKE) nine minutes */
 #define SA_REPLACEMENT_FUZZ_DEFAULT	    100	  /* (IPSEC & IKE) 100% of MARGIN */
@@ -540,14 +644,18 @@ extern enum_names auth_alg_names, extended_auth_alg_names;
 
 /* Oakley Lifetime Type attribute
  * draft-ietf-ipsec-ike-01.txt appendix A
+ * As far as I can see, there is not specification for
+ * OAKLEY_ISAKMP_SA_LIFETIME_DEFAULT.  This could lead to interop problems!
+ * For no particular reason, we chose one hour.
+ * The value of OAKLEY_ISAKMP_SA_LIFETIME_MAXIMUM is our local policy.
  */
 extern enum_names oakley_lifetime_names;
 
 #define OAKLEY_LIFE_SECONDS   1
 #define OAKLEY_LIFE_KILOBYTES 2
 
-#define OAKLEY_ISAKMP_SA_LIFETIME_DEFAULT 3600    /* One hour */
-#define OAKLEY_ISAKMP_SA_LIFETIME_MAXIMUM 28800   /* 8 hours */
+#define OAKLEY_ISAKMP_SA_LIFETIME_DEFAULT 3600    /* one hour */
+#define OAKLEY_ISAKMP_SA_LIFETIME_MAXIMUM 28800   /* eight hours */
 
 /* Oakley PRF attribute (none defined)
  * draft-ietf-ipsec-ike-01.txt appendix A
@@ -562,6 +670,7 @@ extern enum_names oakley_prf_names;
 
 /* Oakley Encryption Algorithm attribute
  * draft-ietf-ipsec-ike-01.txt appendix A
+ * and from http://www.isi.edu/in-notes/iana/assignments/ipsec-registry
  */
 
 extern enum_names oakley_enc_names;
@@ -572,9 +681,11 @@ extern enum_names oakley_enc_names;
 #define OAKLEY_RC5_R16_B64_CBC  4
 #define OAKLEY_3DES_CBC         5
 #define OAKLEY_CAST_CBC         6
+#define OAKLEY_AES_CBC          7
 
 /* Oakley Hash Algorithm attribute
  * draft-ietf-ipsec-ike-01.txt appendix A
+ * and from http://www.isi.edu/in-notes/iana/assignments/ipsec-registry
  */
 
 extern enum_names oakley_hash_names;
@@ -582,9 +693,13 @@ extern enum_names oakley_hash_names;
 #define OAKLEY_MD5      1
 #define OAKLEY_SHA      2
 #define OAKLEY_TIGER    3
+#define OAKLEY_SHA2_256        4
+#define OAKLEY_SHA2_384        5
+#define OAKLEY_SHA2_512        6
 
 /* Oakley Authentication Method attribute
  * draft-ietf-ipsec-ike-01.txt appendix A
+ * Goofy XAUTH extensions from draft-ietf-ipsec-isakmp-xauth-06.txt
  */
 
 extern enum_names oakley_auth_names;
@@ -597,7 +712,18 @@ extern enum_names oakley_auth_names;
 #define OAKLEY_ELGAMAL_ENC         6
 #define OAKLEY_ELGAMAL_ENC_REV     7
 
-#define OAKLEY_AUTH_ROOF           8	/* roof on auth values */
+#define OAKLEY_AUTH_ROOF           8	/* roof on auth values THAT WE SUPPORT */
+
+#define XAUTHInitPreShared                                65001
+#define XAUTHRespPreShared                                65002
+#define XAUTHInitDSS                                      65003
+#define XAUTHRespDSS                                      65004
+#define XAUTHInitRSA                                      65005
+#define XAUTHRespRSA                                      65006
+#define XAUTHInitRSAEncryption                            65007
+#define XAUTHRespRSAEncryption                            65008
+#define XAUTHInitRSARevisedEncryption                     65009
+#define XAUTHRespRSARevisedEncryption                     65010
 
 /* Oakley Group Description attribute
  * draft-ietf-ipsec-ike-01.txt appendix A
@@ -700,6 +826,29 @@ enum pubkey_alg
 #define RSA_MAX_OCTETS	(4096 / BITS_PER_BYTE)
 #define RSA_MAX_OCTETS_UGH	"RSA modulus too large: more than 4096 bits"
 
+/* socket address family info */
+
+struct af_info
+{
+    int af;
+    const char *name;
+    size_t ia_sz;
+    size_t sa_sz;
+    int mask_cnt;
+    u_int8_t id_addr, id_subnet, id_range;
+    const ip_address *any;
+    const ip_subnet *wildcard;	/* 0.0.0.0/32 or IPv6 equivalent */
+    const ip_subnet *all;	/* 0.0.0.0/0 or IPv6 equivalent */
+};
+
+extern const struct af_info
+    af_inet4_info,
+    af_inet6_info;
+
+extern const struct af_info *aftoinfo(int af);
+
+extern enum_names af_names;
+
 /* BIND enumerated types */
 
 extern enum_names
