@@ -1,5 +1,5 @@
 /* cookie generation/verification routines.
- * Copyright (C) 1998, 1999  D. Hugh Redelmeier.
+ * Copyright (C) 1998-2002  D. Hugh Redelmeier.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -11,13 +11,15 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: cookie.h,v 1.7 1999/12/13 00:40:50 dhr Exp $
+ * RCSID $Id: cookie.h,v 1.11 2002/01/21 03:14:15 dhr Exp $
  */
 
+#include <freeswan.h>
+
 extern const u_char zero_cookie[COOKIE_SIZE];	/* guaranteed 0 */
 
-extern void get_cookie(int initiator, u_int8_t *cookie, int length
-    , struct in_addr a);
+extern void get_cookie(bool initiator, u_int8_t *cookie, int length
+    , const ip_address *addr);
 
-extern bool is_zero_cookie(const void *obj);
+#define is_zero_cookie(cookie) all_zero((cookie), COOKIE_SIZE)
 
