@@ -1,5 +1,5 @@
 /* misc. universal things
- * Copyright (C) 1998, 1999  D. Hugh Redelmeier.
+ * Copyright (C) 1998-2001  D. Hugh Redelmeier.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -11,7 +11,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: defs.c,v 1.16 2000/06/21 18:24:31 dhr Exp $
+ * RCSID $Id: defs.c,v 1.19 2002/01/21 03:14:15 dhr Exp $
  */
 
 #include <stdlib.h>
@@ -19,6 +19,8 @@
 #include <stdio.h>
 #include <sys/types.h>
 
+#include <freeswan.h>
+
 #include "constants.h"
 #include "defs.h"
 #include "log.h"
@@ -34,6 +36,16 @@ passert_fail(const char *pred_str, const char *file_str, unsigned long line_no)
     abort();	/* exiting correctly doesn't always work */
 }
 
+bool
+all_zero(const unsigned char *m, size_t len)
+{
+    size_t i;
+
+    for (i = 0; i != len; i++)
+	if (m[i] != '\0')
+	    return FALSE;
+    return TRUE;
+}
 
 /* Convert MP_INT to network form (binary octets, big-endian).
  * We do the malloc; caller must eventually do free.
