@@ -1,6 +1,6 @@
 /* misc. universal things
  * Copyright (C) 1997 Angelos D. Keromytis.
- * Copyright (C) 1998, 1999  D. Hugh Redelmeier.
+ * Copyright (C) 1998-2001  D. Hugh Redelmeier.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -12,7 +12,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: defs.h,v 1.21 2000/02/25 22:37:10 dhr Exp $
+ * RCSID $Id: defs.h,v 1.27 2002/01/21 03:14:15 dhr Exp $
  */
 
 /* GCC magic! */
@@ -37,8 +37,8 @@ extern void passert_fail(const char *pred_str
  * Needed in connections.h and state.h; here to simplify dependencies.
  */
 typedef unsigned long so_serial_t;
-#define SOS_NOBODY  0	/* null serial number */
-
+#define SOS_NOBODY	0	/* null serial number */
+#define SOS_FIRST	1	/* first normal serial number */
 
 /* memory allocation */
 
@@ -78,31 +78,18 @@ typedef struct chunk chunk_t;
 extern const chunk_t empty_chunk;
 
 
-/* IP stuff */
-
-#define NO_IP	0	/* our s_addr value signifying no IPv4 address */
-
-#define is_NO_IP(a) ((a).s_addr == NO_IP)
-
-#define same_ip(a, b) ((a).s_addr == (b).s_addr)
-
-#define same_subnet(aa, am, ba, bm) (same_ip((aa), (ba)) && same_ip((am), (bm)))
-
-/* make a struct sockaddr_in */
-
-#define mksin(/*struct sockaddr_in*/ sin, /*in_addr_t*/ addr, /*u_int16_t*/ port) { \
-	memset(&(sin), '\0', sizeof(sin)); \
-	(sin).sin_family = AF_INET; \
-	(sin).sin_addr.s_addr = (addr); \
-	(sin).sin_port = htons(port); \
-    }
-
-
 /* cleanly exit Pluto */
 
 extern void exit_pluto(int /*status*/) NEVER_RETURNS;
 
 
+/* zero all bytes */
+#define zero(x) memset((x), '\0', sizeof(*(x)))
+
+/* are all bytes 0? */
+extern bool all_zero(const unsigned char *m, size_t len);
+
+
 /* some MP utilities */
 
 #include <gmp.h>
