@@ -1,5 +1,5 @@
 /* identity representation, as in IKE ID Payloads (RFC 2407 DOI 4.6.2.1)
- * Copyright (C) 1999  D. Hugh Redelmeier
+ * Copyright (C) 1999-2001  D. Hugh Redelmeier
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -11,11 +11,12 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: id.c,v 1.6 2000/06/18 21:20:33 dhr Exp $
+ * RCSID $Id: id.c,v 1.17 2002/01/21 03:14:17 dhr Exp $
  */
 
 #include <stdlib.h>
 #include <string.h>
+#include <ctype.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
@@ -30,21 +31,26 @@
 
 const struct id empty_id;	/* all zeros and NULLs */
 
-/* convert textual form of id into a (temporary) struct id */
-
-complaint_t
+/* Convert textual form of id into a (temporary) struct id.
+ * Note that if the id is to be kept, unshare_id_content will be necessary.
+ */
+err_t
 atoid(char *src, struct id *id)
 {
-    complaint_t ugh = NULL;
+    err_t ugh = NULL;
 
-    memset(id, 0, sizeof(*id));
-    id->name.ptr = NULL;
-    id->next = NULL;
+    *id = empty_id;
 
     if (strchr(src, '@') == NULL)
     {
-	id->kind = ID_IPV4_ADDR;
-	ugh = atoaddr(src, 0, &id->ip_addr);
+	/* !!! this test is not sufficient for distinguishing address families.
+	 * We need a notation to specify that a FQDN is to be resolved to IPv6.
+	 */
+	const struct af_info *afi = strchr(src, ':') == NULL
+	    ? &af_inet4_info: &af_inet6_info;
+
+	id->kind = afi->id_addr;
+	ugh = ttoaddr(src, 0, afi->af, &id->ip_addr);
     }
     else
     {
@@ -66,40 +72,68 @@ atoid(char *src, struct id *id)
     return ugh;
 }
 
-int
-idtoa(struct id *id, char *dst, size_t dstlen)
+void
+iptoid(const ip_address *ip, struct id *id)
 {
+    *id = empty_id;
+
+    switch (addrtypeof(ip))
+    {
+    case AF_INET:
+	id->kind = ID_IPV4_ADDR;
+	break;
+    case AF_INET6:
+	id->kind = ID_IPV6_ADDR;
+	break;
+    default:
+	passert(FALSE);
+    }
+    id->ip_addr = *ip;
+}
+
+int
+idtoa(const struct id *id, char *dst, size_t dstlen)
+{
+    int n;
+
     switch (id->kind)
     {
     case ID_NONE:
-	return snprintf(dst, dstlen, "(none)");
+	n = snprintf(dst, dstlen, "(none)");
+	break;
     case ID_IPV4_ADDR:
-	return snprintf(dst, dstlen, "%s", inet_ntoa(id->ip_addr));
+    case ID_IPV6_ADDR:
+	n = (int)addrtot(&id->ip_addr, 0, dst, dstlen) - 1;
+	break;
     case ID_FQDN:
-	return snprintf(dst, dstlen, "@%.*s", (int)id->name.len, id->name.ptr);
+	n = snprintf(dst, dstlen, "@%.*s", (int)id->name.len, id->name.ptr);
+	break;
     case ID_USER_FQDN:
-	return snprintf(dst, dstlen, "%.*s", (int)id->name.len, id->name.ptr);
+	n = snprintf(dst, dstlen, "%.*s", (int)id->name.len, id->name.ptr);
+	break;
     default:
-	return snprintf(dst, dstlen, "unknown id kind %d", id->kind);
+	n = snprintf(dst, dstlen, "unknown id kind %d", id->kind);
+	break;
     }
+
+    /* "Sanitize" string so that log isn't endangered:
+     * replace unprintable characters with '?'.
+     */
+    if (n > 0)
+    {
+	for ( ; *dst != '\0'; dst++)
+	    if (!isprint(*dst))
+		*dst = '?';
+    }
+
+    return n;
 }
 
-/* deep-copy a struct id into a new heap object
+/* Make private copy of string in struct id.
  * This is needed if the result of atoid is to be kept.
  */
-
-struct id *
-clone_id(const struct id *src)
-{
-    struct id *dst = clone_thing(*src, "keep id");
-
-    clone_id_content(dst);
-    dst->next = NULL;
-    return dst;
-}
-
 void
-clone_id_content(struct id *id)
+unshare_id_content(struct id *id)
 {
     switch (id->kind)
     {
@@ -109,20 +143,13 @@ clone_id_content(struct id *id)
 	break;
     case ID_NONE:
     case ID_IPV4_ADDR:
+    case ID_IPV6_ADDR:
 	break;
     default:
 	passert(FALSE);
     }
 }
 
-/* free a heap struct id */
-void
-free_id(struct id *id)
-{
-    free_id_content(id);
-    pfree(id);
-}
-
 void
 free_id_content(struct id *id)
 {
@@ -134,6 +161,7 @@ free_id_content(struct id *id)
 	break;
     case ID_NONE:
     case ID_IPV4_ADDR:
+    case ID_IPV6_ADDR:
 	break;
     default:
 	passert(FALSE);
@@ -142,7 +170,7 @@ free_id_content(struct id *id)
 
 /* compare two struct id values */
 bool
-id_same(const struct id *a, const struct id *b)
+same_id(const struct id *a, const struct id *b)
 {
     if (a->kind != b->kind)
 	return FALSE;
@@ -152,7 +180,8 @@ id_same(const struct id *a, const struct id *b)
 	return TRUE;	/* kind of vacuous */
 
     case ID_IPV4_ADDR:
-	return same_ip(a->ip_addr, b->ip_addr);
+    case ID_IPV6_ADDR:
+	return sameaddr(&a->ip_addr, &b->ip_addr);
 
     case ID_FQDN:
     case ID_USER_FQDN:
@@ -165,25 +194,31 @@ id_same(const struct id *a, const struct id *b)
     }
 }
 
+/* build an ID payload
+ * Note: no memory is allocated for the body of the payload (tl->ptr).
+ * We assume it will end up being a pointer into a sufficiently
+ * stable datastructure.  It only needs to last a short time.
+ */
 void
 build_id_payload(struct isakmp_ipsec_id *hd, chunk_t *tl, struct end *end)
 {
-    memset(hd, '\0', sizeof(*hd));
+    zero(hd);
     hd->isaiid_idtype = end->id.kind;
     switch (end->id.kind)
     {
     case ID_NONE:
-	hd->isaiid_idtype = ID_IPV4_ADDR;
-	tl->ptr = (void *)&end->host_addr;
-	tl->len = sizeof(end->host_addr);
+	hd->isaiid_idtype = aftoinfo(addrtypeof(&end->host_addr))->id_addr;
+	tl->len = addrbytesptr(&end->host_addr
+	    , (const unsigned char **)&tl->ptr);	/* sets tl->ptr too */
 	break;
     case ID_FQDN:
     case ID_USER_FQDN:
 	*tl = end->id.name;
 	break;
     case ID_IPV4_ADDR:
-	tl->ptr = (void *)&end->id.ip_addr;
-	tl->len = sizeof(end->id.ip_addr);
+    case ID_IPV6_ADDR:
+	tl->len = addrbytesptr(&end->id.ip_addr
+	    , (const unsigned char **)&tl->ptr);	/* sets tl->ptr too */
 	break;
     default:
 	passert(FALSE);
