@@ -1,5 +1,5 @@
 /* identity representation, as in IKE ID Payloads (RFC 2407 DOI 4.6.2.1)
- * Copyright (C) 1999  D. Hugh Redelmeier
+ * Copyright (C) 1999-2001  D. Hugh Redelmeier
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -11,26 +11,26 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: id.h,v 1.6 2000/06/18 21:20:34 dhr Exp $
+ * RCSID $Id: id.h,v 1.16 2002/01/21 03:14:17 dhr Exp $
  */
 
 struct id {
     int kind;	/* ID_* value */
-    struct in_addr ip_addr;	/* (network order) ID_IPV4_ADDR */
+    ip_address ip_addr;	/* ID_IPV4_ADDR, ID_IPV6_ADDR */
     chunk_t name;	/* ID_FQDN, ID_USER_FQDN (with @) */
-    struct id *next;
 };
 
 extern const struct id empty_id;
 
-extern complaint_t atoid(char *src, struct id *id);
-extern int idtoa(struct id *id, char *dst, size_t dstlen);
+extern err_t atoid(char *src, struct id *id);
+extern void iptoid(const ip_address *ip, struct id *id);
+extern int idtoa(const struct id *id, char *dst, size_t dstlen);
+#define IDTOA_BUF 256
 struct end;	/* forward declaration of tag (defined in connections.h) */
-extern struct id *clone_id(const struct id *src);
-extern void clone_id_content(struct id *id);
-extern void free_id(struct id *id);
+extern void unshare_id_content(struct id *id);
 extern void free_id_content(struct id *id);
-extern bool id_same(const struct id *a, const struct id *b);
+extern bool same_id(const struct id *a, const struct id *b);
+#define id_is_ipaddr(id) ((id)->kind == ID_IPV4_ADDR || (id)->kind == ID_IPV6_ADDR)
 
 struct isakmp_ipsec_id;	/* forward declaration of tag (defined in packet.h) */
 extern void
