@@ -1,5 +1,5 @@
 /* IPsec DOI and Oakley resolution routines
- * Copyright (C) 1998, 1999  D. Hugh Redelmeier.
+ * Copyright (C) 1998-2002  D. Hugh Redelmeier.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -11,18 +11,24 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: ipsec_doi.h,v 1.22 2000/06/21 02:27:28 dhr Exp $
+ * RCSID $Id: ipsec_doi.h,v 1.26 2002/01/29 04:23:39 dhr Exp $
  */
 
+extern void echo_hdr(struct msg_digest *md, bool enc, u_int8_t np);
+
 extern void ipsecdoi_initiate(int whack_sock, struct connection *c
-    , bool pending_quick, lset_t policy, unsigned long try);
+    , lset_t policy, unsigned long try, so_serial_t replacing);
 
 extern void ipsecdoi_replace(struct state *st, unsigned long try);
 
 extern void init_phase2_iv(struct state *st, const msgid_t *msgid);
 
-extern stf_status quick_outI1(int whack_sock, struct state *isakmp_sa
-    , struct connection *c, lset_t policy, unsigned long try);
+extern stf_status quick_outI1(int whack_sock
+    , struct state *isakmp_sa
+    , struct connection *c
+    , lset_t policy
+    , unsigned long try
+    , so_serial_t replacing);
 
 extern state_transition_fn
     main_inI1_outR1,
