@@ -1,5 +1,5 @@
 /* declarations of routines that interface with the kernel's IPsec mechanism
- * Copyright (C) 1998, 1999  D. Hugh Redelmeier.
+ * Copyright (C) 1998-2001  D. Hugh Redelmeier.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -11,20 +11,39 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: kernel.h,v 1.13 2000/01/17 19:50:22 dhr Exp $
+ * RCSID $Id: kernel.h,v 1.25 2002/01/21 03:14:19 dhr Exp $
  */
 
 extern bool no_klips;	/* don't actually use KLIPS */
+extern bool can_do_IPcomp;  /* can system actually perform IPCOMP? */
 
-extern void pfkey_handle(void);
+#ifdef KLIPS
+extern int pfkeyfd;
+extern void pfkey_dequeue(void);
+extern void pfkey_event(void);
+#endif
+
+extern void init_kernel(void);
+
+extern void scan_proc_shunts(void);
+
+extern void pfkey_event(void);
 
 struct connection;	/* forward declaration of tag */
-extern bool route_connection(struct connection *c, bool doit);
+extern bool trap_connection(struct connection *c);
 extern void unroute_connection(struct connection *c);
 
+extern bool replace_bare_shunt(const ip_address *src, const ip_address *dst
+    , ipsec_spi_t shunt_spi	/* in host order! */
+    , bool repl, const char *opname);
+
+extern bool assign_hold(struct connection *c
+    , const ip_address *src, const ip_address *dst);
+
 struct state;	/* forward declaration of tag */
 extern ipsec_spi_t get_ipsec_spi(ipsec_spi_t avoid);
+extern ipsec_spi_t get_my_cpi(void);
 
 extern bool install_inbound_ipsec_sa(struct state *st);
 extern bool install_ipsec_sa(struct state *st, bool inbound_also);
-extern bool delete_ipsec_sa(struct state *st, bool inbound_only);
+extern void delete_ipsec_sa(struct state *st, bool inbound_only);
