@@ -1,6 +1,6 @@
 /* whack communicating routines
  * Copyright (C) 1997 Angelos D. Keromytis.
- * Copyright (C) 1998, 1999  D. Hugh Redelmeier.
+ * Copyright (C) 1998-2001  D. Hugh Redelmeier.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -12,7 +12,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: kernel_comm.c,v 1.49 2000/06/21 18:24:33 dhr Exp $
+ * RCSID $Id: kernel_comm.c,v 1.66 2002/01/30 09:42:35 dhr Exp $
  */
 
 #include <stdio.h>
@@ -44,16 +44,10 @@
 #include "dnskey.h"
 #include "server.h"
 
-/*
- * XXX Eventually it should use PF_KEY or what kernel API is available
- * XXX but for now this will have to do.
- */
-
 /* helper variables and function to decode strings from whack message */
 
-static char
-    *next_str,
-    *str_roof;
+static char *next_str
+    , *str_roof;
 
 static bool
 unpack_str(char **p)
@@ -101,22 +95,19 @@ whack_handle(int whackctlfd)
 
     /* sanity check message */
     {
-	int ughno = RC_BADWHACKMESSAGE;
-	char ugh[200];
+	err_t ugh = NULL;
 
 	next_str = msg.string;
 	str_roof = (char *)&msg + n;
 
 	if (next_str > str_roof)
 	{
-	    snprintf(ugh, sizeof(ugh)
-		, "truncated message from whack: got %d bytes; expected %d.  Message ignored."
+	    ugh = builddiag("truncated message from whack: got %d bytes; expected %d.  Message ignored."
 		, n, (int) sizeof(msg));
 	}
 	else if (msg.magic != WHACK_MAGIC)
 	{
-	    snprintf(ugh, sizeof(ugh)
-		, "message from whack has bad magic %d; should be %d; probably wrong version.  Message ignored"
+	    ugh = builddiag("message from whack has bad magic %d; should be %d; probably wrong version.  Message ignored"
 		, msg.magic, WHACK_MAGIC);
 	}
 	else if (!unpack_str(&msg.name)	/* string 1 */
@@ -127,18 +118,16 @@ whack_handle(int whackctlfd)
 	|| !unpack_str(&msg.keyid)	/* string 6 */
 	|| str_roof - next_str != (ptrdiff_t)msg.keyval.len)	/* check chunk */
 	{
-	    snprintf(ugh, sizeof(ugh)
-		, "message from whack contains bad string");
+	    ugh = "message from whack contains bad string";
 	}
 	else
 	{
 	    msg.keyval.ptr = next_str;	/* grab chunk */
-	    ughno = 0;	/* ran through the gauntlet -- success */
 	}
 
-	if (ughno != 0)
+	if (ugh != NULL)
 	{
-	    loglog(ughno, "%s", ugh);
+	    loglog(RC_BADWHACKMESSAGE, "%s", ugh);
 	    whack_log_fd = NULL_FD;
 	    close(whackfd);
 	    return;
@@ -154,8 +143,8 @@ whack_handle(int whackctlfd)
 	     * cause the message to print, it will be printed.
 	     */
 	    cur_debugging |= msg.debugging;
-	    DBG(DBG_CONTROL,
-		DBG_log("base debugging = %s"
+	    DBG(DBG_CONTROL
+		, DBG_log("base debugging = %s"
 		    , bitnamesof(debug_bit_names, msg.debugging)));
 	    cur_debugging = base_debugging = msg.debugging;
 	}
@@ -166,8 +155,8 @@ whack_handle(int whackctlfd)
 	    if (c != NULL)
 	    {
 		c->extra_debugging = msg.debugging;
-		DBG(DBG_CONTROL,
-		    DBG_log("\"%s\" extra_debugging = %s"
+		DBG(DBG_CONTROL
+		    , DBG_log("\"%s\" extra_debugging = %s"
 			, c->name
 			, bitnamesof(debug_bit_names, c->extra_debugging)));
 	    }
@@ -190,6 +179,21 @@ whack_handle(int whackctlfd)
 	    delete_connection(c);
     }
 
+    if (msg.whack_deletestate)
+    {
+	struct state *st = state_with_serialno(msg.whack_deletestateno);
+
+	if (st == NULL)
+	{
+	    loglog(RC_UNKNOWN_NAME, "no state #%lu to delete"
+		, msg.whack_deletestateno);
+	}
+	else
+	{
+	    delete_state(st);
+	}
+    }
+
     if (msg.whack_connection)
 	add_connection(&msg);
 
@@ -211,25 +215,32 @@ whack_handle(int whackctlfd)
     {
 	/* add a public key */
 	struct id keyid;
-	complaint_t ugh = atoid(msg.keyid, &keyid);
+	err_t ugh = atoid(msg.keyid, &keyid);
 
 	if (ugh != NULL)
 	{
-	    loglog(RC_BADID, "bad --keyid: %s", ugh);
+	    loglog(RC_BADID, "bad --keyid \"%s\": %s", msg.keyid, ugh);
 	}
 	else
 	{
+	    if (!msg.whack_addkey)
+		delete_public_keys(&keyid, msg.pubkey_alg);
+
 	    if (msg.keyval.len == 0)
 	    {
 		ugh = fetch_public_key(&keyid);
 		if (ugh != NULL)
 		{
-		    char name[100];
+		    char name[100];	/* longer IDs will be truncated in message */
 
 		    (void)idtoa(&keyid, name, sizeof(name));
 		    loglog(RC_NOKEY
 			, "failure to fetch key for %s from DNS: %s", name, ugh);
 		}
+		else
+		{
+		    remember_keys_from_dns();
+		}
 	    }
 	    else
 	    {
@@ -248,12 +259,13 @@ whack_handle(int whackctlfd)
 
 	    if (c != NULL)
 	    {
-		SET_CUR_CONNECTION(c);
-		if (!orient(c, TRUE))
-		    whack_log(RC_ORIENT, "could not orient connection");
-		else if (!route_connection(c, TRUE))
+		set_cur_connection(c);
+		if (!oriented(*c))
+		    whack_log(RC_ORIENT
+			, "we have no ipsecN interface for either end of this connection");
+		else if (!trap_connection(c))
 		    whack_log(RC_ROUTE, "could not route");
-		UNSET_CUR_CONNECTION();
+		reset_cur_connection();
 	    }
 	}
     }
@@ -264,12 +276,12 @@ whack_handle(int whackctlfd)
 
 	if (c != NULL)
 	{
-	    SET_CUR_CONNECTION(c);
-	    if (c->eroute_owner != SOS_NOBODY)
+	    set_cur_connection(c);
+	    if (c->routing >= RT_ROUTED_TUNNEL)
 		whack_log(RC_RTBUSY, "cannot unroute: route busy");
 	    else
 		unroute_connection(c);
-	    UNSET_CUR_CONNECTION();
+	    reset_cur_connection();
 	}
     }
 
@@ -282,11 +294,23 @@ whack_handle(int whackctlfd)
 		, msg.whack_async? NULL_FD : dup_any(whackfd));
     }
 
+    if (msg.whack_oppo_initiate)
+    {
+	if (!listening)
+	    whack_log(RC_DEAF, "need --listen before opportunistic initiation");
+	else
+	    initiate_opportunistic(&msg.oppo_my_client, &msg.oppo_peer_client
+		, FALSE
+		, msg.whack_async? NULL_FD : dup_any(whackfd));
+    }
+
     if (msg.whack_terminate)
 	terminate_connection(msg.name);
 
     if (msg.whack_status)
     {
+	show_ifaces_status();
+	whack_log(RC_COMMENT, BLANK_FORMAT);	/* spacer */
 	show_connections_status();
 	whack_log(RC_COMMENT, BLANK_FORMAT);	/* spacer */
 	show_states_status();
