@@ -1,5 +1,5 @@
 /* logging definitions
- * Copyright (C) 1998, 1999  D. Hugh Redelmeier.
+ * Copyright (C) 1998-2001  D. Hugh Redelmeier.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -11,68 +11,85 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: log.h,v 1.22 2000/06/21 18:24:34 dhr Exp $
+ * RCSID $Id: log.h,v 1.28 2002/01/21 03:14:20 dhr Exp $
  */
 
+#include <freeswan.h>
+
 extern bool
     log_to_stderr,	/* should log go to stderr? */
     log_to_syslog;	/* should log go to syslog? */
 
 /* Context for logging.
- * 
+ *
  * Global variables: must be carefully adjusted at transaction boundaries!
+ * All are to be left in RESET condition and will be checked.
+ * There are several pairs of routines to set and reset them.
  * If the context provides a whack file descriptor, messages
  * should be copied to it -- see whack_log()
  */
 extern int whack_log_fd;	/* only set during whack_handle() */
 extern struct state *cur_state;	/* current state, for diagnostics */
 extern struct connection *cur_connection;	/* current connection, for diagnostics */
-extern const struct sockaddr_in *cur_from;	/* source of current current message */
+extern const ip_address *cur_from;	/* source of current current message */
+extern u_int16_t cur_from_port;	/* host order */
 
 #ifdef DEBUG
 
-extern unsigned int cur_debugging;	/* current debugging level */
+  extern unsigned int cur_debugging;	/* current debugging level */
 
-#define GLOBALS_ARE_RESET() (whack_log_fd == NULL_FD \
+  extern void extra_debugging(const struct connection *c);
+
+# define reset_debugging() { cur_debugging = base_debugging; }
+
+# define GLOBALS_ARE_RESET() (whack_log_fd == NULL_FD \
     && cur_state == NULL \
     && cur_connection == NULL \
     && cur_from == NULL \
     && cur_debugging == base_debugging)
 
-#define RESET_GLOBALS() { whack_log_fd = NULL_FD; \
-    cur_state = NULL; \
-    cur_connection = NULL; \
-    cur_from = NULL; \
-    cur_debugging = base_debugging; }
-
-#define SET_CUR_CONNECTION(c) { \
-    cur_connection = (c); \
-    extra_debugging(c); \
-    }
-
-#define UNSET_CUR_CONNECTION() { \
-    cur_connection = NULL; \
-    cur_debugging = base_debugging; \
-    }
-
 #else
 
-#define GLOBALS_ARE_RESET() (whack_log_fd == NULL_FD \
+# define extra_debugging(c)  { }
+
+# define reset_debugging() { }
+
+# define GLOBALS_ARE_RESET() (whack_log_fd == NULL_FD \
     && cur_state == NULL \
     && cur_connection == NULL \
     && cur_from == NULL)
 
-#define RESET_GLOBALS() { whack_log_fd = NULL_FD; \
-    cur_state = NULL; \
-    cur_connection = NULL; \
-    cur_from = NULL; }
-
-#define SET_CUR_CONNECTION(c) { cur_connection = (c); }
-
-#define UNSET_CUR_CONNECTION() {cur_connection = NULL; }
-
 #endif
 
+#define reset_globals() { \
+    whack_log_fd = NULL_FD; \
+    cur_state = NULL; \
+    cur_from = NULL; \
+    reset_cur_connection(); \
+    }
+
+
+#define set_cur_connection(c) { \
+    cur_connection = (c); \
+    extra_debugging(c); \
+    }
+
+#define reset_cur_connection() { \
+    cur_connection = NULL; \
+    reset_debugging(); \
+    }
+
+
+#define set_cur_state(s) { \
+    cur_state = (s); \
+    extra_debugging((s)->st_connection); \
+    }
+
+#define reset_cur_state() { \
+    cur_state = NULL; \
+    reset_debugging(); \
+    }
+
 extern void init_log(void);
 extern void close_log(void);
 extern void log(const char *message, ...) PRINTF_LIKE(1);
@@ -94,35 +111,21 @@ extern void whack_log(int mess_no, const char *message, ...) PRINTF_LIKE(2);
  */
 extern void loglog(int mess_no, const char *message, ...) PRINTF_LIKE(2);
 
-/* Convert struct sockaddr or struct sockaddr_in to a string for presentation.
- * The resulting string is stored in a statically allocated variable,
- * so the next call will overwrite the string.
- * The prototypes use struct sockaddr and struct sockaddr.
- * To ensure that these types are not given prototype scope,
- * we need to declare them.  According to ANSI, the following should work:
- *  struct sockaddr;
- *  struct sockaddr_in;
- * Unfortunately, this isn't good enough for gcc, so we include these
- * headers instead (overkill).
+/* Build up a diagnostic in a static buffer.
+ * Although this would be a generally useful function, it is very
+ * hard to come up with a discipline that prevents different uses
+ * from interfering.  It is intended that by limiting it to building
+ * diagnostics, we will avoid this problem.
+ * Juggling is performed to allow an argument to be a previous
+ * result: the new string may safely depend on the old one.  This
+ * restriction is not checked in any way: violators will produce
+ * confusing results (without crashing!).
  */
-#include <sys/socket.h>
-#include <netinet/in.h>
-
-#define SOCKADDR_STRING_SIZE 50	/* should be enough */
-
-extern char *get_address(struct sockaddr sa);
-extern char *get_address_in(struct sockaddr_in sin);
-
-extern int get_port(struct sockaddr sa);
-extern int get_port_in(struct sockaddr_in sin);
-
-extern const char *show_sa(const struct sockaddr *sa);
-extern const char *show_sa_in(const struct sockaddr_in *sin);
+extern err_t builddiag(const char *fmt, ...) PRINTF_LIKE(1);
 
 #ifdef DEBUG
 
 extern unsigned int base_debugging;	/* bits selecting what to report */
-extern void extra_debugging(const struct connection *c);
 #define DBG(cond, action)   { if (cur_debugging & (cond)) { action ; } }
 
 extern void DBG_log(const char *message, ...) PRINTF_LIKE(1);
@@ -137,3 +140,12 @@ extern void DBG_dump(const char *label, const void *p, size_t len);
 
 #define DBG_cond_dump(cond, label, p, len) DBG(cond, DBG_dump(label, p, len))
 #define DBG_cond_dump_chunk(cond, label, ch) DBG(cond, DBG_dump_chunk(label, ch))
+
+
+/* ip_str: a simple to use variant of addrtot.
+ * It stores its result in a static buffer.
+ * This means that newer calls overwrite the storage of older calls.
+ * Note: this is not used in any of the logging functions, so their
+ * callers may use it.
+ */
+extern const char *ip_str(const ip_address *src);
