@@ -1,6 +1,6 @@
 /* parsing packets: formats and tools
  * Copyright (C) 1997 Angelos D. Keromytis.
- * Copyright (C) 1998, 1999  D. Hugh Redelmeier.
+ * Copyright (C) 1998-2001  D. Hugh Redelmeier.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -12,7 +12,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: packet.c,v 1.22 2000/06/21 18:24:34 dhr Exp $
+ * RCSID $Id: packet.c,v 1.27 2002/01/21 03:14:20 dhr Exp $
  */
 
 #include <stdio.h>
@@ -20,6 +20,8 @@
 #include <netinet/in.h>
 #include <string.h>
 
+#include <freeswan.h>
+
 #include "constants.h"
 #include "defs.h"
 #include "log.h"
@@ -239,6 +241,22 @@ struct_desc isakmp_esp_transform_desc = {
     "ISAKMP Transform Payload (ESP)",
     isat_fields_esp, sizeof(struct isakmp_transform) };
 
+/* PROTO_IPCOMP */
+static field_desc isat_fields_ipcomp[] = {
+    { ft_enum, 8/BITS_PER_BYTE, "next payload type", &payload_names },
+    { ft_mbz, 8/BITS_PER_BYTE, NULL, NULL },
+    { ft_len, 16/BITS_PER_BYTE, "length", NULL },
+    { ft_nat, 8/BITS_PER_BYTE, "transform number", NULL },
+    { ft_enum, 8/BITS_PER_BYTE, "transform ID", &ipcomp_transformid_names },
+    { ft_mbz, 16/BITS_PER_BYTE, NULL, NULL },
+    { ft_end, 0, NULL, NULL }
+};
+
+struct_desc isakmp_ipcomp_transform_desc = {
+    "ISAKMP Transform Payload (COMP)",
+    isat_fields_ipcomp, sizeof(struct isakmp_transform) };
+
+
 /* ISAKMP Key Exchange Payload: no fixed fields beyond the generic ones.
  * layout from draft-ietf-ipsec-isakmp-09.txt section 3.7
  * Variable Key Exchange Data follow the generic fields.
@@ -585,7 +603,7 @@ DBG_print_struct(const char *label, const void *struct_ptr
 
 	case ft_raw:	/* bytes to be left in network-order */
 	    {
-		char m[50];
+		char m[50];	/* arbitrary limit on name width in log */
 
 		snprintf(m, sizeof(m), "   %s:", fp->name);
 		DBG_dump(m, inp, i);
@@ -606,7 +624,7 @@ static void DBG_prefix_print_struct(const pb_stream *pbs
     /* print out a title, with a prefix of asterisks to show
      * the nesting level.
      */
-    char space[40];
+    char space[40];	/* arbitrary limit on label+flock-of-* */
     size_t len = strlen(label);
 
     if (sizeof(space) <= len)
@@ -647,23 +665,21 @@ static void DBG_prefix_print_struct(const pb_stream *pbs
  */
 
 bool
-in_struct(void *struct_ptr, struct_desc *sd,
-    pb_stream *ins, pb_stream *obj_pbs)
+in_struct(void *struct_ptr, struct_desc *sd
+, pb_stream *ins, pb_stream *obj_pbs)
 {
-    char ugh[100];
+    err_t ugh = NULL;
     u_int8_t *cur = ins->cur;
 
-    ugh[0] = '\0';
-
     if (ins->roof - cur < (ptrdiff_t)sd->size) {
-	snprintf(ugh, sizeof(ugh), "not enough room in input packet for %s", sd->name);
+	ugh = builddiag("not enough room in input packet for %s", sd->name);
     } else {
 	u_int8_t *roof = cur + sd->size;    /* may be changed by a length field */
 	u_int8_t *outp = struct_ptr;
 	bool immediate = FALSE;
 	field_desc *fp;
 
-	for (fp = sd->fields; ugh[0] == '\0'; fp++) {
+	for (fp = sd->fields; ugh == NULL; fp++) {
 	    size_t i = fp->size;
 
 	    passert(ins->roof - cur >= (ptrdiff_t)i);
@@ -671,15 +687,15 @@ in_struct(void *struct_ptr, struct_desc *sd,
 	    passert(outp - (cur - ins->cur) == struct_ptr);
 
 #if 0
-	    DBG(DBG_PARSING, DBG_log("%d %s",
-		(int) (cur - ins->cur), fp->name == NULL? "" : fp->name));
+	    DBG(DBG_PARSING, DBG_log("%d %s"
+		, (int) (cur - ins->cur), fp->name == NULL? "" : fp->name));
 #endif
 	    switch (fp->field_type) {
 	    case ft_mbz:	/* must be zero */
 		for (; i != 0; i--) {
 		    if (*cur++ != 0) {
-			snprintf(ugh, sizeof(ugh), "byte %d of %s must be zero, but is not",
-			    (int) (cur - ins->cur), sd->name);
+			ugh = builddiag("byte %d of %s must be zero, but is not"
+			    , (int) (cur - ins->cur), sd->name);
 			break;
 		    }
 		    *outp++ = '\0';	/* probably redundant */
@@ -707,13 +723,11 @@ in_struct(void *struct_ptr, struct_desc *sd,
 			: immediate? sd->size : n + sd->size;
 
 		    if (len < sd->size) {
-			snprintf(ugh, sizeof(ugh),
-			    "%s of %s is smaller than minimum",
-			    fp->name, sd->name);
+			ugh = builddiag("%s of %s is smaller than minimum"
+			    , fp->name, sd->name);
 		    } else if (pbs_left(ins) < len) {
-			snprintf(ugh, sizeof(ugh),
-			    "%s of %s is larger than can fit",
-			    fp->name, sd->name);
+			ugh = builddiag("%s of %s is larger than can fit"
+			    , fp->name, sd->name);
 		    } else {
 			roof = ins->cur + len;
 		    }
@@ -725,18 +739,16 @@ in_struct(void *struct_ptr, struct_desc *sd,
 		    /* FALL THROUGH */
 		case ft_enum:	/* value from an enumeration */
 		    if (enum_name(fp->desc, n) == NULL) {
-			snprintf(ugh, sizeof(ugh),
-			    "%s of %s has an unknown value: %lu",
-			    fp->name, sd->name, (unsigned long)n);
+			ugh = builddiag("%s of %s has an unknown value: %lu"
+			    , fp->name, sd->name, (unsigned long)n);
 		    }
 		    /* FALL THROUGH */
 		case ft_loose_enum:	/* value from an enumeration with only some names known */
 		    break;
 		case ft_set:	/* bits representing set */
 		    if (!testset(fp->desc, n)) {
-			snprintf(ugh, sizeof(ugh),
-			    "bitset %s of %s has unknown member(s): %s",
-			    fp->name, sd->name, bitnamesof(fp->desc, n));
+			ugh = builddiag("bitset %s of %s has unknown member(s): %s"
+			    , fp->name, sd->name, bitnamesof(fp->desc, n));
 		    }
 		    break;
 		default:
@@ -775,8 +787,8 @@ in_struct(void *struct_ptr, struct_desc *sd,
 		    obj_pbs->cur = cur;
 		}
 		ins->cur = roof;
-		DBG(DBG_PARSING,
-		    DBG_prefix_print_struct(ins, "parse ", struct_ptr, sd, TRUE));
+		DBG(DBG_PARSING
+		    , DBG_prefix_print_struct(ins, "parse ", struct_ptr, sd, TRUE));
 		return TRUE;
 
 	    default:
@@ -802,18 +814,18 @@ in_raw(void *bytes, size_t len, pb_stream *ins, const char *name)
     {
 	if (bytes == NULL)
 	{
-	    DBG(DBG_PARSING,
-		DBG_log("skipping %u raw bytes of %s (%s)",
-		    (unsigned) len, ins->name, name);
-		DBG_dump(name, ins->cur, len));
+	    DBG(DBG_PARSING
+		, DBG_log("skipping %u raw bytes of %s (%s)"
+		    , (unsigned) len, ins->name, name);
+		  DBG_dump(name, ins->cur, len));
 	}
 	else
 	{
 	    memcpy(bytes, ins->cur, len);
-	    DBG(DBG_PARSING,
-		DBG_log("parsing %u raw bytes of %s into %s",
-		    (unsigned) len, ins->name, name);
-		DBG_dump(name, bytes, len));
+	    DBG(DBG_PARSING
+		, DBG_log("parsing %u raw bytes of %s into %s"
+		    , (unsigned) len, ins->name, name);
+		  DBG_dump(name, bytes, len));
 	}
 	ins->cur += len;
 	return TRUE;
@@ -835,20 +847,19 @@ in_raw(void *bytes, size_t len, pb_stream *ins, const char *name)
  */
 
 bool
-out_struct(const void *struct_ptr, struct_desc *sd,
-    pb_stream *outs, pb_stream *obj_pbs)
+out_struct(const void *struct_ptr, struct_desc *sd
+, pb_stream *outs, pb_stream *obj_pbs)
 {
-    char ugh[100];
+    err_t ugh = NULL;
     const u_int8_t *inp = struct_ptr;
     u_int8_t *cur = outs->cur;
 
-    DBG(DBG_EMITTING,
-	DBG_prefix_print_struct(outs, "emit ", struct_ptr, sd, obj_pbs==NULL));
+    DBG(DBG_EMITTING
+	, DBG_prefix_print_struct(outs, "emit ", struct_ptr, sd, obj_pbs==NULL));
 
-    ugh[0] = '\0';
     if (outs->roof - cur < (ptrdiff_t)sd->size) {
-	snprintf(ugh, sizeof(ugh)
-	    , "not enough room left in output packet to place %s", sd->name);
+	ugh = builddiag("not enough room left in output packet to place %s"
+	    , sd->name);
     } else {
 	bool immediate = FALSE;
 	pb_stream obj;
@@ -857,7 +868,7 @@ out_struct(const void *struct_ptr, struct_desc *sd,
 	obj.lenfld = NULL;  /* until a length field is discovered */
 	obj.lenfld_desc = NULL;
 
-	for (fp = sd->fields; ugh[0] == '\0'; fp++) {
+	for (fp = sd->fields; ugh == NULL; fp++) {
 	    size_t i = fp->size;
 
 	    passert(outs->roof - cur >= (ptrdiff_t)i);
@@ -865,8 +876,8 @@ out_struct(const void *struct_ptr, struct_desc *sd,
 	    passert(inp - (cur - outs->cur) == struct_ptr);
 
 #if 0
-	    DBG(DBG_EMITTING, DBG_log("%d %s",
-		(int) (cur - outs->cur), fp->name == NULL? "" : fp->name);
+	    DBG(DBG_EMITTING, DBG_log("%d %s"
+		, (int) (cur - outs->cur), fp->name == NULL? "" : fp->name);
 #endif
 	    switch (fp->field_type) {
 	    case ft_mbz:	/* must be zero */
@@ -918,18 +929,16 @@ out_struct(const void *struct_ptr, struct_desc *sd,
 		    /* FALL THROUGH */
 		case ft_enum:	/* value from an enumeration */
 		    if (enum_name(fp->desc, n) == NULL) {
-			snprintf(ugh, sizeof(ugh),
-			    "%s of %s has an unknown value: %lu",
-			    fp->name, sd->name, (unsigned long)n);
+			ugh = builddiag("%s of %s has an unknown value: %lu"
+			    , fp->name, sd->name, (unsigned long)n);
 		    }
 		    /* FALL THROUGH */
 		case ft_loose_enum:	/* value from an enumeration with only some names known */
 		    break;
 		case ft_set:	/* bits representing set */
 		    if (!testset(fp->desc, n)) {
-			snprintf(ugh, sizeof(ugh),
-			    "bitset %s of %s has unknown member(s): %s",
-			    fp->name, sd->name, bitnamesof(fp->desc, n));
+			ugh = builddiag("bitset %s of %s has unknown member(s): %s"
+			    , fp->name, sd->name, bitnamesof(fp->desc, n));
 		    }
 		    break;
 		default:
@@ -986,8 +995,8 @@ out_struct(const void *struct_ptr, struct_desc *sd,
 }
 
 bool
-out_generic(u_int8_t np, struct_desc *sd,
-    pb_stream *outs, pb_stream *obj_pbs)
+out_generic(u_int8_t np, struct_desc *sd
+, pb_stream *outs, pb_stream *obj_pbs)
 {
     struct isakmp_generic gen;
 
@@ -997,8 +1006,8 @@ out_generic(u_int8_t np, struct_desc *sd,
 }
 
 bool
-out_generic_raw(u_int8_t np, struct_desc *sd,
-    pb_stream *outs, void *bytes, size_t len, const char *name)
+out_generic_raw(u_int8_t np, struct_desc *sd
+, pb_stream *outs, void *bytes, size_t len, const char *name)
 {
     pb_stream pbs;
 
@@ -1014,16 +1023,16 @@ out_raw(const void *bytes, size_t len, pb_stream *outs, const char *name)
 {
     if (pbs_left(outs) < len)
     {
-	loglog(RC_LOG_SERIOUS, "not enough room left to place %lu bytes of %s in %s",
-	    (unsigned long) len, name, outs->name);
+	loglog(RC_LOG_SERIOUS, "not enough room left to place %lu bytes of %s in %s"
+	    , (unsigned long) len, name, outs->name);
 	return FALSE;
     }
     else
     {
-	DBG(DBG_EMITTING,
-	    DBG_log("emitting %u raw bytes of %s into %s",
-		(unsigned) len, name, outs->name);
-	    DBG_dump(name, bytes, len));
+	DBG(DBG_EMITTING
+	    , DBG_log("emitting %u raw bytes of %s into %s"
+		, (unsigned) len, name, outs->name);
+	      DBG_dump(name, bytes, len));
 	memcpy(outs->cur, bytes, len);
 	outs->cur += len;
 	return TRUE;
@@ -1040,8 +1049,8 @@ out_zero(size_t len, pb_stream *outs, const char *name)
     }
     else
     {
-	DBG(DBG_EMITTING, DBG_log("emitting %u zero bytes of %s into %s",
-	    (unsigned) len, name, outs->name));
+	DBG(DBG_EMITTING, DBG_log("emitting %u zero bytes of %s into %s"
+	    , (unsigned) len, name, outs->name));
 	memset(outs->cur, 0x00, len);
 	outs->cur += len;
 	return TRUE;
@@ -1062,8 +1071,8 @@ close_output_pbs(pb_stream *pbs)
 
 	if (pbs->lenfld_desc->field_type == ft_lv)
 	    len -= sizeof(struct isakmp_attribute);
-	DBG(DBG_EMITTING, DBG_log("emitting length of %s: %lu",
-	    pbs->name, (unsigned long) len));
+	DBG(DBG_EMITTING, DBG_log("emitting length of %s: %lu"
+	    , pbs->name, (unsigned long) len));
 	while (i-- != 0) {
 	    pbs->lenfld[i] = (u_int8_t)len;
 	    len >>= BITS_PER_BYTE;
