@@ -1,6 +1,6 @@
 /* parsing packets: formats and tools
  * Copyright (C) 1997 Angelos D. Keromytis.
- * Copyright (C) 1998, 1999  D. Hugh Redelmeier.
+ * Copyright (C) 1998-2001  D. Hugh Redelmeier.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -12,7 +12,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: packet.h,v 1.15 2000/06/18 16:18:01 dhr Exp $
+ * RCSID $Id: packet.h,v 1.18 2002/01/21 03:14:20 dhr Exp $
  */
 
 /* a struct_desc describes a structure for the struct I/O routines.
@@ -122,14 +122,36 @@ extern void DBG_print_struct(const char *label, const void *struct_ptr,
  * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
  * !                            Length                             !
  * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
- * Although the drafts are a little unclear, all messages should
- * be padded with enough 0x00 bytes to make the length a multiple of
- * (a) a multiple of 4 octets in length (source:
- *     draft-ietf-ipsec-isakmp-09.txt 3, but ??? no padding value is
- *     specified), and
- * (b) the encryption block size (if encrypted) (source:
- *     draft-ietf-ipsec-isakmp-oakley-07, 5.3 and Appendix B)
- * ??? It is not clear that if (b) applies, (a) always makes sense.
+ * Although the drafts are a little unclear, there are a few
+ * places that specify that messages should be padded with 0x00
+ * octets (bytes) to make the length a multiple of something.
+ *
+ * RFC 2408 "ISAKMP" 3.6 specifies that all messages will be
+ * padded to be a multiple of 4 octets in length.
+ * ??? This looks vestigial, and we ignore this requirement.
+ *
+ * RFC 2409 "IKE" Appedix B specifies:
+ *     Each message should be padded up to the nearest block size
+ *     using bytes containing 0x00.
+ * ??? This does not appear to be limited to encrypted messages,
+ * but it surely must be: the block size is meant to be the encryption
+ * block size, and that is meaningless for a non-encrypted message.
+ *
+ * RFC 2409 "IKE" 5.3 specifies:
+ *     Encrypted payloads are padded up to the nearest block size.
+ *     All padding bytes, except for the last one, contain 0x00. The
+ *     last byte of the padding contains the number of the padding
+ *     bytes used, excluding the last one. Note that this means there
+ *     will always be padding.
+ * ??? This is nuts since payloads are not padded, messages are.
+ * It also contradicts Appendix B.  So we ignore it.
+ *
+ * Summary: we pad encrypted output messages with 0x00 to bring them
+ * up to a multiple of the encryption block size.  On input, we require
+ * that any encrypted portion of a message be a multiple of the encryption
+ * block size.   After any decryption, we ignore padding (any bytes after
+ * the first payload that specifies a next payload of none; we don't
+ * require them to be zero).
  */
 
 struct isakmp_hdr
@@ -290,7 +312,8 @@ struct isakmp_transform
 extern struct_desc
     isakmp_isakmp_transform_desc,
     isakmp_ah_transform_desc,
-    isakmp_esp_transform_desc;
+    isakmp_esp_transform_desc,
+    isakmp_ipcomp_transform_desc;
 
 /* ISAKMP Key Exchange Payload: no fixed fields beyond the generic ones.
  * layout from draft-ietf-ipsec-isakmp-09.txt section 3.7
