@@ -1,5 +1,5 @@
 /* mechanisms for preshared keys (public, private, and preshared secrets)
- * Copyright (C) 1998, 1999  D. Hugh Redelmeier.
+ * Copyright (C) 1998-2001  D. Hugh Redelmeier.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -11,7 +11,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: preshared.c,v 1.33 2000/06/21 18:24:34 dhr Exp $
+ * RCSID $Id: preshared.c,v 1.53 2002/01/21 03:14:21 dhr Exp $
  */
 
 #include <stdio.h>
@@ -38,6 +38,7 @@
 #include "connections.h"	/* needs id.h */
 #include "state.h"
 #include "preshared.h"
+#include "dnskey.h"
 #include "log.h"
 #include "whack.h"	/* for RC_LOG_SERIOUS */
 
@@ -99,8 +100,8 @@ static const char *
 RSA_private_key_sanity(struct RSA_private_key *k)
 {
     /* note that the *last* error found is reported */
-    complaint_t ugh = NULL;
-    mpz_t t, u;
+    err_t ugh = NULL;
+    mpz_t t, u, q1;
 
 #ifdef DEBUG	/* debugging info that compromises security */
     DBG(DBG_PRIVATE, RSA_show_private_key(k));
@@ -118,6 +119,7 @@ RSA_private_key_sanity(struct RSA_private_key *k)
 
     mpz_init(t);
     mpz_init(u);
+    mpz_init(q1);
 
     /* check that n == p * q */
     mpz_mul(u, &k->p, &k->q);
@@ -135,15 +137,18 @@ RSA_private_key_sanity(struct RSA_private_key *k)
     if (mpz_cmp_ui(t, 0) == 0)
 	ugh = "e divides q-1";
 
-    /* check that d is e^-1 (mod (p-1) * (q-1)) */
-    mpz_sub(u, u, &k->p);  /* u := p*q - p - q +1 */
-    mpz_sub(u, u, &k->q);
-    mpz_add_ui(u, u, 1);
+    /* check that d is e^-1 (mod lcm(p-1, q-1)) */
+    /* see PKCS#1v2, aka RFC 2437, for the "lcm" */
+    mpz_sub_ui(q1, &k->q, 1);
+    mpz_sub_ui(u, &k->p, 1);
+    mpz_gcd(t, u, q1);		/* t := gcd(p-1, q-1) */
+    mpz_mul(u, u, q1);		/* u := (p-1) * (q-1) */
+    mpz_divexact(u, u, t);	/* u := lcm(p-1, q-1) */
 
     mpz_mul(t, &k->d, &k->pub.e);
     mpz_mod(t, t, u);
     if (mpz_cmp_ui(t, 1) != 0)
-	ugh = "(d * e) mod ((p-1) * (q-1)) != 1";
+	ugh = "(d * e) mod (lcm(p-1, q-1)) != 1";
 
     /* check that dP is d mod (p-1) */
     mpz_sub_ui(u, &k->p, 1);
@@ -165,13 +170,19 @@ RSA_private_key_sanity(struct RSA_private_key *k)
 
     mpz_clear(t);
     mpz_clear(u);
+    mpz_clear(q1);
     return ugh;
 }
 
 const char *shared_secrets_file = SHARED_SECRETS_FILE;
 
+struct id_list {
+    struct id id;
+    struct id_list *next;
+};
+
 struct secret {
-    struct id *ids;
+    struct id_list *ids;
     enum PrivateKeyKind kind;
     union {
 	chunk_t preshared_secret;
@@ -197,18 +208,15 @@ get_secret(struct connection *c, enum PrivateKeyKind kind, bool asym)
     unsigned int best_match = 0;
     struct secret *best = NULL;
     struct secret *s;
-    struct id
-	*my_id = &c->this.id,
-	rw_id,
-	*his_id = &c->that.id;
+    struct id *my_id = &c->this.id
+	, rw_id
+	, *his_id = &c->that.id;
 
-    if (c->rw_state == rwcs_instance
-    && c->that.id.kind == ID_IPV4_ADDR
-    && same_ip(c->that.id.ip_addr, c->that.host_addr))
+    if (his_id_was_instantiated(c))
     {
 	/* roadwarrior: replace him with 0.0.0.0 */
-	rw_id.kind = ID_IPV4_ADDR;
-	rw_id.ip_addr.s_addr = NO_IP;
+	rw_id.kind = c->that.id.kind;
+	(void)anyaddr(addrtypeof(&c->that.host_addr), &rw_id.ip_addr);
 	his_id = &rw_id;
     }
 
@@ -228,14 +236,14 @@ get_secret(struct connection *c, enum PrivateKeyKind kind, bool asym)
 	    else
 	    {
 		/* check if both ends match ids */
-		struct id *i;
+		struct id_list *i;
 
 		for (i = s->ids; i != NULL; i = i->next)
 		{
-		    if (id_same(my_id, i))
+		    if (same_id(my_id, &i->id))
 			match |= match_me;
 
-		    if (id_same(his_id, i))
+		    if (same_id(his_id, &i->id))
 			match |= match_him;
 		}
 
@@ -264,14 +272,31 @@ get_secret(struct connection *c, enum PrivateKeyKind kind, bool asym)
 		if (match == best_match)
 		{
 		    /* two good matches are equally good:
-		     * do they agree?  Assume they do not
-		     * unless they are preshared secrets.
+		     * do they agree?
 		     */
-		    if (!(s->kind == PPK_PSK && best->kind == PPK_PSK
-		    && s->u.preshared_secret.len == best->u.preshared_secret.len
-		    && memcmp(s->u.preshared_secret.ptr, best->u.preshared_secret.ptr, s->u.preshared_secret.len) == 0))
+		    bool same;
+
+		    switch (kind)
 		    {
-			loglog(RC_LOG_SERIOUS, "multiple ipsec.secrets entries match endpoints:"
+		    case PPK_PSK:
+			same = s->u.preshared_secret.len == best->u.preshared_secret.len
+			    && memcmp(s->u.preshared_secret.ptr, best->u.preshared_secret.ptr, s->u.preshared_secret.len) == 0;
+			break;
+		    case PPK_RSA:
+			/* Dirty trick: since we have code to compare
+			 * RSA public keys, but not private keys, we
+			 * make the assumption that equal public keys
+			 * mean equal private keys.  This ought to work.
+			 */
+			same = same_RSA_public_key(&s->u.RSA_private_key.pub
+			    , &best->u.RSA_private_key.pub);
+			break;
+		    default:
+			passert(FALSE);
+		    }
+		    if (!same)
+		    {
+			loglog(RC_LOG_SERIOUS, "multiple ipsec.secrets entries with distinct secrets match endpoints:"
 			    " first secret used");
 			best = s;	/* list is backwards: take latest in list */
 		    }
@@ -386,7 +411,7 @@ static struct secrets_file_position *sfp = NULL;
 static void process_secrets_file(const char *file_pat);
 
 static char *tok;
-#define tokeq(s) (strcmp(tok, (s)) == 0)
+#define tokeq(s) (streq(tok, (s)))
 #define tokeqword(s) (strcasecmp(tok, (s)) == 0)
 
 static bool
@@ -519,10 +544,8 @@ flushline(const char *m)
 static void
 process_key(struct secret *s)
 {
-    char buf[2048];
-    complaint_t ugh = NULL;
-    complaint_t ugh_extra = "";
-    char ugh_space[100];	/* perhaps enough space */
+    char buf[2048];	/* limit on size of binary representation of key */
+    err_t ugh = NULL;
 
     if (*tok == '"' || *tok == '\'')
     {
@@ -550,10 +573,7 @@ process_key(struct secret *s)
 	    if (ugh != NULL)
 	    {
 		/* atobytes didn't like PSK data */
-		snprintf(ugh_space, sizeof(ugh_space)
-		    , "PSK data malformed (%s): ", ugh);
-		ugh = ugh_space;
-		ugh_extra = tok;
+		ugh = builddiag("PSK data malformed (%s): %s", ugh, tok);
 	    }
 	    else
 	    {
@@ -590,8 +610,8 @@ process_key(struct secret *s)
 		}
 		else if (!tokeqword(p->name))
 		{
-		    ugh = p->name;
-		    ugh_extra = " keyword not found where expected in RSA key";
+		    ugh = builddiag("%s keyword not found where expected in RSA key"
+			, p->name);
 		}
 		else if (!(shift()
 		&& (!tokeq(":") || shift())))	/* ignore optional ":" */
@@ -601,10 +621,7 @@ process_key(struct secret *s)
 		else if (NULL != (ugh = atobytes(tok, sfp->cur - tok, buf, sizeof(buf), &sz)))
 		{
 		    /* in RSA key, atobytes didn't like */
-		    snprintf(ugh_space, sizeof(ugh_space)
-			, "RSA data malformed (%s): ", ugh);
-		    ugh = ugh_space;
-		    ugh_extra = tok;
+		    ugh = builddiag("RSA data malformed (%s): %s", ugh, tok);
 		}
 		else
 		{
@@ -627,10 +644,18 @@ process_key(struct secret *s)
 
 	    if (ugh == NULL)
 	    {
-		/* note: the following requires a boundary after "}" */
-		if (!(shift() && tokeq("}") && !shift()))
+		/* We require an (indented) '}' and the end of the record.
+		 * We break down the test so that the diagnostic will be
+		 * more helpful.  Some people don't seem to wish to indent
+		 * the brace!
+		 */
+		if (!shift() || !tokeq("}"))
 		{
-		    ugh = "malformed end of RSA private key";
+		    ugh = "malformed end of RSA private key -- indented '}' required";
+		}
+		else if (shift())
+		{
+		    ugh = "malformed end of RSA private key -- unexpected token after '}'";
 		}
 		else
 		{
@@ -644,14 +669,13 @@ process_key(struct secret *s)
     }
     else
     {
-	ugh = "unrecognized key format: ";
-	ugh_extra = tok;
+	ugh = builddiag("unrecognized key format: %s", tok);
     }
 
     if (ugh != NULL)
     {
-	loglog(RC_LOG_SERIOUS, "\"%s\" line %d: %s%s"
-	    , sfp->filename, sfp->lino, ugh, ugh_extra);
+	loglog(RC_LOG_SERIOUS, "\"%s\" line %d: %s"
+	    , sfp->filename, sfp->lino, ugh);
     }
     else if (flushline("expected record boundary in key"))
     {
@@ -748,7 +772,24 @@ process_secret_records(void)
 		     * See RFC2407 IPsec Domain of Interpretation 4.6.2
 		     */
 		    struct id id;
-		    complaint_t ugh = atoid(tok, &id);
+		    err_t ugh;
+
+		    if (tokeq("%any"))
+		    {
+			id = empty_id;
+			id.kind = ID_IPV4_ADDR;
+			ugh = anyaddr(AF_INET, &id.ip_addr);
+		    }
+		    else if (tokeq("%any6"))
+		    {
+			id = empty_id;
+			id.kind = ID_IPV6_ADDR;
+			ugh = anyaddr(AF_INET6, &id.ip_addr);
+		    }
+		    else
+		    {
+			ugh = atoid(tok, &id);
+		    }
 
 		    if (ugh != NULL)
 		    {
@@ -757,11 +798,14 @@ process_secret_records(void)
 		    }
 		    else
 		    {
-			struct id *i = clone_id(&id);
+			struct id_list *i = alloc_thing(struct id_list
+			    , "id_list");
 
+			i->id = id;
+			unshare_id_content(&i->id);
 			i->next = s->ids;
 			s->ids = i;
-			/* DBG_log("id type %d: %s %.*s", i->id.kind, inet_ntoa(i->id.ip_addr), (int)i->id.name.len, i->id.name.ptr); */
+			/* DBG_log("id type %d: %s %.*s", i->kind, ip_str(&i->ip_addr), (int)i->name.len, i->name.ptr); */
 		    }
 		    if (!shift())
 		    {
@@ -846,7 +890,9 @@ process_secrets_file(const char *file_pat)
 	pos.under = *pos.cur = '\0';
 
 	(void) shift();	/* prime tok */
+	(void) flushline("file starts with indentation (continuation notation)");
 	process_secret_records();
+	fclose(pos.fp);
     }
 
     sfp = pos.previous;	/* restore old state */
@@ -863,13 +909,14 @@ free_preshared_secrets(void)
 
 	for (s = secrets; s != NULL; s = ns)
 	{
-	    struct id *i, *ni;
+	    struct id_list *i, *ni;
 
 	    ns = s->next;	/* grab before freeing s */
 	    for (i = s->ids; i != NULL; i = ni)
 	    {
 		ni = i->next;	/* grab before freeing i */
-		free_id(i);
+		free_id_content(&i->id);
+		pfree(i);
 	    }
 	    switch (s->kind)
 	    {
@@ -877,8 +924,7 @@ free_preshared_secrets(void)
 		pfree(s->u.preshared_secret.ptr);
 		break;
 	    case PPK_RSA:
-		mpz_clear(&s->u.RSA_private_key.pub.n);
-		mpz_clear(&s->u.RSA_private_key.pub.e);
+		free_RSA_public_content(&s->u.RSA_private_key.pub);
 		mpz_clear(&s->u.RSA_private_key.d);
 		mpz_clear(&s->u.RSA_private_key.p);
 		mpz_clear(&s->u.RSA_private_key.q);
@@ -904,46 +950,57 @@ load_preshared_secrets(void)
 
 /* public key machinery */
 
-struct pubkeyrec {
-    struct id *id;
-    enum pubkey_alg alg;
-    union {
-	struct RSA_public_key rsa;
-    } u;
-    struct pubkeyrec *next;
-};
-
-static struct pubkeyrec *pubkeys = NULL;
-
-static void
-free_first_public_key(void)
+void free_RSA_public_content(struct RSA_public_key *rsa)
 {
-    struct pubkeyrec *p = pubkeys;
+    mpz_clear(&rsa->n);
+    mpz_clear(&rsa->e);
+}
 
-    pubkeys = p->next;
+/* Free a public key record.
+ * As a convenience, this returns a pointer to next.
+ */
+struct pubkeyrec *
+free_public_key(struct pubkeyrec *p)
+{
+    struct pubkeyrec *nxt = p->next;
 
-    free_id(p->id);
+    free_id_content(&p->id);
 
     /* algorithm-specific freeing */
     switch (p->alg)
     {
     case PUBKEY_ALG_RSA:
-	mpz_clear(&p->u.rsa.n);
-	mpz_clear(&p->u.rsa.e);
+	free_RSA_public_content(&p->u.rsa);
 	break;
     default:
 	passert(FALSE);
     }
 
     pfree(p);
+    return nxt;
 }
 
 
+struct pubkeyrec *pubkeys = NULL;	/* keys from ipsec.conf */
+
 void
 free_public_keys(void)
 {
     while (pubkeys != NULL)
-	free_first_public_key();
+	pubkeys = free_public_key(pubkeys);
+}
+
+/* steal keys_from_dns list and place at front of pubkeys */
+void
+remember_keys_from_dns(void)
+{
+    struct pubkeyrec **pp = &keys_from_dns;
+
+    while (*pp != NULL)
+	pp = &(*pp)->next;
+    *pp = pubkeys;
+    pubkeys = keys_from_dns;
+    keys_from_dns = NULL;
 }
 
 /* decode of RSA pubkey chunk
@@ -954,7 +1011,7 @@ free_public_keys(void)
  * - exponent
  * - modulus
  */
-static const char *
+err_t
 unpack_RSA_public_key(struct RSA_public_key *rsa, chunk_t *pubkey)
 {
     chunk_t exp;
@@ -1006,36 +1063,30 @@ unpack_RSA_public_key(struct RSA_public_key *rsa, chunk_t *pubkey)
     return NULL;
 }
 
-static struct pubkeyrec *
-get_public_key(struct id *id, enum pubkey_alg alg)
+bool
+same_RSA_public_key(const struct RSA_public_key *a
+    , const struct RSA_public_key *b)
 {
-    struct pubkeyrec *p, **pp;
-
-    for (pp = &pubkeys; (p = *pp) != NULL; pp = &p->next)
-    {
-	if (id_same(id, p->id) && p->alg == alg)
-	{
-	    /* we have a winner: bring it to front, return key */
-
-	    *pp = p->next;	/* remove p from list */
-	    p->next = pubkeys;	/* add p to front of list */
-	    pubkeys = p;
-	    return p;
-	}
-    }
-    return NULL;	/* failure */
-}
-
-const struct RSA_public_key *
-get_RSA_public_key(struct id *id)
-{
-    struct pubkeyrec *p = get_public_key(id, PUBKEY_ALG_RSA);
-
-    return p == NULL? NULL : &p->u.rsa;
+    return a == b
+    || (a->k == b->k && mpz_cmp(&a->n, &b->n) == 0 && mpz_cmp(&a->e, &b->e) == 0);
 }
 
 void
-add_public_key(struct id *id, enum pubkey_alg alg, chunk_t *key)
+delete_public_keys(const struct id *id, enum pubkey_alg alg)
+{
+    struct pubkeyrec **pp, *p;
+
+    for (pp = &pubkeys; (p = *pp) != NULL; )
+    {
+	if (same_id(id, &p->id) && p->alg == alg)
+	    *pp = free_public_key(p);
+	else
+	    pp = &p->next;
+    }
+}
+
+void
+add_public_key(const struct id *id, enum pubkey_alg alg, chunk_t *key)
 {
     struct pubkeyrec *p = alloc_thing(struct pubkeyrec, "pubkeyrec");
 
@@ -1044,7 +1095,7 @@ add_public_key(struct id *id, enum pubkey_alg alg, chunk_t *key)
     {
     case PUBKEY_ALG_RSA:
 	{
-	    complaint_t ugh = unpack_RSA_public_key(&p->u.rsa, key);
+	    err_t ugh = unpack_RSA_public_key(&p->u.rsa, key);
 
 	    if (ugh != NULL)
 	    {
@@ -1058,22 +1109,12 @@ add_public_key(struct id *id, enum pubkey_alg alg, chunk_t *key)
 	passert(FALSE);
     }
 
-    p->id = clone_id(id);
+    p->id = *id;
+    unshare_id_content(&p->id);
+
     p->alg = alg;
 
-    /* just before installing, delete any previous key of this type
-     * for this id.
-     */
-    {
-	struct pubkeyrec *q = get_public_key(id, alg);
-
-	if (q != NULL)
-	{
-	    passert(q == pubkeys);
-	    free_first_public_key();
-	}
-    }
-
+    /* install new key at front */
     p->next = pubkeys;
     pubkeys = p;
 }
