@@ -1,5 +1,5 @@
 /* get-next-event loop
- * Copyright (C) 1998, 1999  D. Hugh Redelmeier.
+ * Copyright (C) 1998-2001  D. Hugh Redelmeier.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -11,27 +11,37 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: server.h,v 1.10 1999/12/12 22:27:25 dhr Exp $
+ * RCSID $Id: server.h,v 1.19 2002/01/21 03:14:22 dhr Exp $
  */
 
+extern int ctl_fd;	/* file descriptor of control (whack) socket */
 extern struct sockaddr_un ctl_addr;	/* address of control (whack) socket */
+extern err_t init_ctl_socket(void);
 extern void delete_ctl_socket(void);
-extern int pfkeyfd;
 
 extern bool listening;	/* should we pay attention to IKE messages? */
 
 
-/* IP interface (eg. ipsec0/eth0) */
+/* interface: a terminal point for IKE traffic, IPsec transport mode
+ * and IPsec tunnels.
+ * Essentially:
+ * - an IP device (eg. eth1), and
+ * - its partner, an ipsec device (eg. ipsec0), and
+ * - their shared IP address (eg. 10.7.3.2)
+ * Note: the port for IKE is always implicitly UDP/pluto_port.
+ */
 struct iface {
-    char *vname;	/* virtual device name */
+    char *vname;	/* virtual (ipsec) device name */
     char *rname;	/* real device name */
-    struct in_addr addr;	/* interface IP address (network order) */
+    ip_address addr;	/* interface IP address */
     int fd;	/* file descriptor of socket for IKE UDP messages */
     struct iface *next;
+    enum { IFN_ADD, IFN_KEEP, IFN_DELETE } change;
 };
 
 extern struct iface *interfaces;	/* public interfaces */
 
 extern void find_ifaces(void);
+extern void show_ifaces_status(void);
 extern void free_ifaces(void);
 extern void call_server(void);
