@@ -1,6 +1,6 @@
 /* routines for state objects
  * Copyright (C) 1997 Angelos D. Keromytis.
- * Copyright (C) 1998, 1999  D. Hugh Redelmeier.
+ * Copyright (C) 1998-2001  D. Hugh Redelmeier.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -12,7 +12,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: state.c,v 1.69 2000/06/21 18:24:35 dhr Exp $
+ * RCSID $Id: state.c,v 1.93 2002/02/18 18:12:09 dhr Exp $
  */
 
 #include <stdio.h>
@@ -34,6 +34,8 @@
 #include "state.h"
 #include "kernel.h"
 #include "log.h"
+#include "packet.h"	/* so we can calculate sizeof(struct isakmp_hdr) */
+#include "preshared.h"	/* for free_public_key */
 #include "rnd.h"
 #include "timer.h"
 #include "whack.h"
@@ -46,18 +48,13 @@
  * Global variables: had to go somewhere, might as well be this file.
  */
 
-u_int16_t pluto_port = PORT;	/* Pluto's port */
-struct sockaddr_in mask32;	/* 255.255.255.255 */
-struct sockaddr_in mask0;	/* 0.0.0.0 */
+u_int16_t pluto_port = IKE_UDP_PORT;	/* Pluto's port */
 
 /*
  * This file has the functions that handle the
  * state hash table and the Message ID list.
  */
 
-#define SA_EQUAL(x, y)   ((x).sa_family == (y).sa_family && \
-    memcmp(&((x).sa_data), &((y).sa_data), FULL_INET_ADDRESS_SIZE) == 0)
-
 /* Message-IDs
  *
  * A Message ID is contained in each IKE message header.
@@ -134,19 +131,26 @@ generate_msgid(struct state *isakmp_sa)
 static struct state *statetable[STATE_TABLE_SIZE];
 
 static struct state **
-state_hash(const u_char *icookie, const u_char *rcookie, struct in_addr peer)
+state_hash(const u_char *icookie, const u_char *rcookie, const ip_address *peer)
 {
     u_int i = 0, j;
+    const unsigned char *byte_ptr;
+    size_t length = addrbytesptr(peer, &byte_ptr);
 
     DBG(DBG_RAW | DBG_CONTROL,
 	DBG_dump("ICOOKIE:", icookie, COOKIE_SIZE);
 	DBG_dump("RCOOKIE:", rcookie, COOKIE_SIZE);
-	DBG_dump("peer:", &peer, sizeof(peer)));
+	DBG_dump("peer:", byte_ptr, length));
+
+    /* XXX the following hash is pretty pathetic */
 
     for (j = 0; j < COOKIE_SIZE; j++)
-	i += icookie[j] + rcookie[j];
+	i = i * 407 + icookie[j] + rcookie[j];
 
-    i = (i + peer.s_addr) % STATE_TABLE_SIZE;
+    for (j = 0; j < length; j++)
+	i = i * 613 + byte_ptr[j];
+
+    i = i % STATE_TABLE_SIZE;
 
     DBG(DBG_CONTROL, DBG_log("state hash entry %d", i));
 
@@ -160,12 +164,13 @@ state_hash(const u_char *icookie, const u_char *rcookie, struct in_addr peer)
 struct state *
 new_state(void)
 {
-    static struct state blank_state;	/* initialized all to zero & NULL */
+    static const struct state blank_state;	/* initialized all to zero & NULL */
+    static so_serial_t next_so = SOS_FIRST;
     struct state *st;
 
-    blank_state.st_serialno++;
-    passert(blank_state.st_serialno != SOS_NOBODY);	/* overflow can't happen! */
     st = clone_thing(blank_state, "struct state in new_state()");
+    st->st_serialno = next_so++;
+    passert(next_so > SOS_FIRST);	/* overflow can't happen! */
     st->st_whack_sock = NULL_FD;
     DBG(DBG_CONTROL, DBG_log("creating state object #%lu at %p",
 	st->st_serialno, (void *) st));
@@ -180,16 +185,33 @@ init_states(void)
 {
     int i;
 
-    /* initialize mask32 */
-    mksin(mask32, inet_addr("255.255.255.255"), 0);
-
-    /* initialize mask0 */
-    mksin(mask0, inet_addr("0.0.0.0"), 0);
-
     for (i = 0; i < STATE_TABLE_SIZE; i++)
 	statetable[i] = (struct state *) NULL;
 }
 
+/* Find the state object with this serial number.
+ * This allows state object references that don't turn into dangerous
+ * dangling pointers: reference a state by its serial number.
+ * Returns NULL if there is no such state.
+ * If this turns out to be a significant CPU hog, it could be
+ * improved to use a hash table rather than sequential seartch.
+ */
+struct state *
+state_with_serialno(so_serial_t sn)
+{
+    if (sn >= SOS_FIRST)
+    {
+	struct state *st;
+	int i;
+
+	for (i = 0; i < STATE_TABLE_SIZE; i++)
+	    for (st = statetable[i]; st != NULL; st = st->st_hashchain_next)
+		if (st->st_serialno == sn)
+		    return st;
+    }
+    return NULL;
+}
+
 /* Insert a state object in the hash table. The object is inserted
  * at the begining of list.
  * Needs cookies, connection, and msgid.
@@ -198,7 +220,7 @@ void
 insert_state(struct state *st)
 {
     struct state **p = state_hash(st->st_icookie, st->st_rcookie
-	, st->st_connection->that.host_addr);
+	, &st->st_connection->that.host_addr);
 
     passert(st->st_hashchain_prev == NULL && st->st_hashchain_next == NULL);
 
@@ -209,6 +231,14 @@ insert_state(struct state *st)
     }
     st->st_hashchain_next = *p;
     *p = st;
+
+    /* Ensure that somebody is in charge of killing this state:
+     * if no event is scheduled for it, schedule one to discard the state.
+     * If nothing goes wrong, this event will be replaced by
+     * a more appropriate one.
+     */
+    if (st->st_event == NULL)
+	event_schedule(EVENT_SO_DISCARD, 0, st);
 }
 
 /* unlink a state object from the hash table, but don't free it
@@ -218,7 +248,7 @@ unhash_state(struct state *st)
 {
     /* unlink from forward chain */
     struct state **p = st->st_hashchain_prev == NULL
-	? state_hash(st->st_icookie, st->st_rcookie, st->st_connection->that.host_addr)
+	? state_hash(st->st_icookie, st->st_rcookie, &st->st_connection->that.host_addr)
 	: &st->st_hashchain_prev->st_hashchain_next;
 
     /* unlink from forward chain */
@@ -248,24 +278,24 @@ release_whack(struct state *st)
     }
 }
 
-/*
- * delete a state object
- */
+/* delete a state object */
+
 void
 delete_state(struct state *st)
 {
-    /* Check that no timer event is left dangling.
-     * We could actually delete it here, but in most
-     * cases this is a "can't happen".
-     */
     struct connection *const c = st->st_connection;
     struct state *old_cur_state = cur_state == st? NULL : cur_state;
 
-    cur_state = st;
-#ifdef DEBUG
-    extra_debugging(c);
-#endif
-    passert(st->st_event == NULL);
+    set_cur_state(st);
+
+    delete_event(st);	/* delete any pending timer event */
+
+    /* Ditch anything pending on ISAKMP SA being established.
+     * Note: this must be done before the unhash_state to prevent
+     * flush_pending_by_state inadvertently and prematurely
+     * deleting our connection.
+     */
+    flush_pending_by_state(st);
 
     /* effectively, this deletes any ISAKMP SA that this state represents */
     unhash_state(st);
@@ -274,9 +304,9 @@ delete_state(struct state *st)
      * ??? we ought to tell peer to delete IPSEC SAs
      */
     if (IS_IPSEC_SA_ESTABLISHED(st->st_state))
-	(void) delete_ipsec_sa(st, FALSE);
+	delete_ipsec_sa(st, FALSE);
     else if (IS_ONLY_INBOUND_IPSEC_SA_ESTABLISHED(st->st_state))
-	(void) delete_ipsec_sa(st, TRUE);
+	delete_ipsec_sa(st, TRUE);
 
     if (c->newest_ipsec_sa == st->st_serialno)
 	c->newest_ipsec_sa = SOS_NOBODY;
@@ -286,7 +316,7 @@ delete_state(struct state *st)
 
     st->st_connection = NULL;	/* we might be about to free it */
     cur_state = old_cur_state;	/* without st_connection, st isn't complete */
-    rw_connection_discard(c);
+    connection_discard(c);
 
     release_whack(st);
 
@@ -303,6 +333,12 @@ delete_state(struct state *st)
 	}
     }
 
+    if (st->st_peer_pubkey != NULL)
+    {
+	st->st_peer_pubkey = free_public_key(st->st_peer_pubkey);
+	passert(st->st_peer_pubkey == NULL);
+    }
+
     if (st->st_sec_in_use)
 	mpz_clear(&(st->st_sec));
 
@@ -327,28 +363,20 @@ delete_state(struct state *st)
     pfree(st);
 }
 
-
-/* If a connection will no longer be used, and it is temporary, delete it.
- * We must be careful to avoid circularity:
- * we don't touch it if it is rwcs_going_away.
- */
-void
-rw_connection_discard(struct connection *c)
+/* Is a connection in use by some state? */
+bool
+states_use_connection(struct connection *c)
 {
-    if (c->rw_state == rwcs_instance)
-    {
-	/* are there any states still using it? */
-	struct state *st = NULL;
-	int i;
+    /* are there any states still using it? */
+    struct state *st = NULL;
+    int i;
 
-	for (i = 0; st == NULL && i < STATE_TABLE_SIZE; i++)
-	    for (st = statetable[i]
-	    ; st != NULL && st->st_connection != c
-	    ; st = st->st_hashchain_next)
-		;
-	if (st == NULL)
-	    delete_connection(c);
-    }
+    for (i = 0; st == NULL && i < STATE_TABLE_SIZE; i++)
+	for (st = statetable[i]; st != NULL; st = st->st_hashchain_next)
+	    if (st->st_connection == c)
+		return TRUE;
+
+    return FALSE;
 }
 
 void
@@ -357,10 +385,10 @@ delete_states_by_connection(struct connection *c)
     int i;
 
     /* this kludge avoids an n^2 algorithm */
-    enum rwcs rws = c->rw_state;
+    enum connection_kind ck = c->kind;
 
-    if (rws == rwcs_instance)
-	c->rw_state = rwcs_going_away;
+    if (ck == CK_INSTANCE)
+	c->kind = CK_GOING_AWAY;
 
     for (i = 0; i < STATE_TABLE_SIZE; i++)
     {
@@ -380,14 +408,10 @@ delete_states_by_connection(struct connection *c)
 		unsigned int old_cur_debugging = cur_debugging;
 #endif
 
-		cur_state = this;
-#ifdef DEBUG
-		extra_debugging(this->st_connection);
-#endif
+		set_cur_state(this);
 		log("deleting state (%s)"
 		    , enum_show(&state_names, this->st_state));
 		passert(this->st_event != NULL);
-		delete_event(this);
 		delete_state(this);
 		cur_state = old_cur_state;
 #ifdef DEBUG
@@ -397,9 +421,14 @@ delete_states_by_connection(struct connection *c)
 	}
     }
 
-    if (rws == rwcs_instance)
+    passert(c->newest_ipsec_sa == SOS_NOBODY
+	&& c->newest_isakmp_sa == SOS_NOBODY
+	&& c->eroute_owner == SOS_NOBODY
+	&& c->routing != RT_ROUTED_TUNNEL);
+
+    if (ck == CK_INSTANCE)
     {
-	c->rw_state = rws;
+	c->kind = ck;
 	delete_connection(c);
     }
 }
@@ -447,12 +476,12 @@ duplicate_state(const struct state *st)
  */
 struct state *
 find_state(const u_char *icookie, const u_char *rcookie,
-		const struct in_addr peer, msgid_t /*network order*/  msgid)
+		const ip_address *peer, msgid_t /*network order*/  msgid)
 {
     struct state *st = *state_hash(icookie, rcookie, peer);
 
     while (st != (struct state *) NULL)
-	if (same_ip(peer, st->st_connection->that.host_addr)
+	if (sameaddr(peer, &st->st_connection->that.host_addr)
 	&& memcmp(icookie, st->st_icookie, COOKIE_SIZE) == 0
 	&& memcmp(rcookie, st->st_rcookie, COOKIE_SIZE) == 0
 	&& msgid == st->st_msgid)
@@ -471,18 +500,39 @@ find_state(const u_char *icookie, const u_char *rcookie,
     return st;
 }
 
-/* Find newest ISAKMP SA state object that implements connection c */
+/* Find the state that sent a packet
+ * ??? this could be expensive -- it should be rate-limited to avoid DoS
+ */
 struct state *
-find_isakmp_sa(const struct connection *c)
+find_sender(size_t packet_len, u_char *packet)
 {
-    struct
-	state *st,
+    int i;
+    struct state *st;
+
+    if (packet_len >= sizeof(struct isakmp_hdr))
+	for (i = 0; i < STATE_TABLE_SIZE; i++)
+	    for (st = statetable[i]; st != NULL; st = st->st_hashchain_next)
+		if (st->st_tpacket.ptr != NULL
+		&& st->st_tpacket.len == packet_len
+		&& memcmp(st->st_tpacket.ptr, packet, packet_len) == 0)
+		    return st;
+
+    return NULL;
+}
+
+/* Find newest Phase 1 negotiation state object for suitable for connection c
+ */
+struct state *
+find_phase1_state(const struct connection *c)
+{
+    struct state
+	*st,
 	*best = NULL;
     int i;
 
     for (i = 0; i < STATE_TABLE_SIZE; i++)
 	for (st = statetable[i]; st != NULL; st = st->st_hashchain_next)
-	    if (IS_ISAKMP_SA_ESTABLISHED(st->st_state)
+	    if (IS_PHASE1(st->st_state)
 	    && c->host_pair == st->st_connection->host_pair
 	    && same_peer_ids(c, st->st_connection, NULL)
 	    && (best == NULL || best->st_serialno < st->st_serialno))
@@ -494,7 +544,7 @@ find_isakmp_sa(const struct connection *c)
 void
 show_states_status(void)
 {
-    time_t now = time((time_t *) NULL);
+    time_t n = now();
     int i;
 
     for (i = 0; i < STATE_TABLE_SIZE; i++)
@@ -505,10 +555,10 @@ show_states_status(void)
 	{
 	    /* what the heck is interesting about a state? */
 	    const struct connection *c = st->st_connection;
-	    long delta = st->st_event->ev_time >= now
-		? (long)(st->st_event->ev_time - now)
-		: -(long)(now - st->st_event->ev_time);
-	    char him[ADDRTOA_BUF+1];	/* for RW */
+	    long delta = st->st_event->ev_time >= n
+		? (long)(st->st_event->ev_time - n)
+		: -(long)(n - st->st_event->ev_time);
+	    char inst[CONN_INST_BUF];
 	    const char *np1 = c->newest_isakmp_sa == st->st_serialno
 		? "; newest ISAKMP" : "";
 	    const char *np2 = c->newest_ipsec_sa == st->st_serialno
@@ -518,18 +568,12 @@ show_states_status(void)
 
 	    passert(st->st_event != 0);
 
-	    him[0] = '\0';
-	    if (c->rw_state == rwcs_instance)
-	    {
-		him[0] = ':';
-		addrtoa(c->that.host_addr, 0, him+1, sizeof(him)-1);
-	    }
+	    fmt_conn_instance(c, inst);
 
 	    whack_log(RC_COMMENT
 		, "#%lu: \"%s\"%s %s (%s); %s in %lds%s%s%s"
 		, st->st_serialno
-		, c->name
-		, him
+		, c->name, inst
 		, enum_name(&state_names, st->st_state)
 		, state_story[st->st_state - STATE_MAIN_R0]
 		, enum_name(&timer_event_names, st->st_event->ev_type)
@@ -539,19 +583,17 @@ show_states_status(void)
 	    /* print out SPIs if SAs are established */
 	    if (IS_IPSEC_SA_ESTABLISHED(st->st_state))
 	    {
-		char buf[SATOA_BUF*6 + 1];
+		char buf[SATOT_BUF*6 + 1];
 		char *p = buf;
 
 #		define add_said(adst, aspi, aproto) { \
-		    struct sa_id s; \
+		    ip_said s; \
 		    \
-		    s.dst = (adst); \
-		    s.spi = (aspi); \
-		    s.proto = (aproto); \
+		    initsaid(adst, aspi, aproto, &s); \
 		    if (p < &buf[sizeof(buf)-1]) \
 		    { \
 			*p++ = ' '; \
-			p += satoa(s, 0, p, &buf[sizeof(buf)] - p) - 1; \
+			p += satot(&s, 0, p, &buf[sizeof(buf)] - p) - 1; \
 		    } \
 		}
 
@@ -559,27 +601,32 @@ show_states_status(void)
 		*p = '\0';
 		if (st->st_ah.present)
 		{
-		    add_said(c->that.host_addr, st->st_ah.attrs.spi, SA_AH);
-		    add_said(c->this.host_addr, st->st_ah.our_spi, SA_AH);
+		    add_said(&c->that.host_addr, st->st_ah.attrs.spi, SA_AH);
+		    add_said(&c->this.host_addr, st->st_ah.our_spi, SA_AH);
 		}
 		if (st->st_esp.present)
 		{
-		    add_said(c->that.host_addr, st->st_esp.attrs.spi, SA_ESP);
-		    add_said(c->this.host_addr, st->st_esp.our_spi, SA_ESP);
+		    add_said(&c->that.host_addr, st->st_esp.attrs.spi, SA_ESP);
+		    add_said(&c->this.host_addr, st->st_esp.our_spi, SA_ESP);
+		}
+		if (st->st_ipcomp.present)
+		{
+		    add_said(&c->that.host_addr, st->st_ipcomp.attrs.spi, SA_COMP);
+		    add_said(&c->this.host_addr, st->st_ipcomp.our_spi, SA_COMP);
 		}
 #ifdef KLIPS
 		if (st->st_ah.attrs.encapsulation == ENCAPSULATION_MODE_TUNNEL
-		|| st->st_esp.attrs.encapsulation == ENCAPSULATION_MODE_TUNNEL)
+		|| st->st_esp.attrs.encapsulation == ENCAPSULATION_MODE_TUNNEL
+		|| st->st_ipcomp.attrs.encapsulation == ENCAPSULATION_MODE_TUNNEL)
 		{
-		    add_said(c->that.host_addr, st->st_tunnel_out_spi, SA_IPIP);
-		    add_said(c->this.host_addr, st->st_tunnel_in_spi, SA_IPIP);
+		    add_said(&c->that.host_addr, st->st_tunnel_out_spi, SA_IPIP);
+		    add_said(&c->this.host_addr, st->st_tunnel_in_spi, SA_IPIP);
 		}
 #endif
 		whack_log(RC_COMMENT
 		    , "#%lu: \"%s\"%s%s"
 		    , st->st_serialno
-		    , c->name
-		    , him
+		    , c->name, inst
 		    , buf);
 
 #		undef add_said
@@ -587,3 +634,96 @@ show_states_status(void)
 	}
     }
 }
+
+/* Given that we've used up a range of unused CPI's,
+ * search for a new range of currently unused ones.
+ * Note: this is very expensive when not trivial!
+ * If we can't find one easily, choose 0 (a bad SPI,
+ * no matter what order) indicating failure.
+ */
+void
+find_my_cpi_gap(cpi_t *latest_cpi, cpi_t *first_busy_cpi)
+{
+    int tries = 0;
+    cpi_t base = *latest_cpi;
+    cpi_t closest;
+    int i;
+
+startover:
+    closest = ~0;	/* not close at all */
+    for (i = 0; i < STATE_TABLE_SIZE; i++)
+    {
+	struct state *st;
+
+	for (st = statetable[i]; st != NULL; st = st->st_hashchain_next)
+	{
+	    if (st->st_ipcomp.present)
+	    {
+		cpi_t c = ntohl(st->st_ipcomp.our_spi) - base;
+
+		if (c < closest)
+		{
+		    if (c == 0)
+		    {
+			/* oops: next spot is occupied; start over */
+			if (++tries == 20)
+			{
+			    /* FAILURE */
+			    *latest_cpi = *first_busy_cpi = 0;
+			    return;
+			}
+			base++;
+			if (base > IPCOMP_LAST_NEGOTIATED)
+			    base = IPCOMP_FIRST_NEGOTIATED;
+			goto startover;	/* really a tail call */
+		    }
+		    closest = c;
+		}
+	    }
+	}
+    }
+    *latest_cpi = base;	/* base is first in next free range */
+    *first_busy_cpi = closest + base;	/* and this is the roof */
+}
+
+/* Muck with high-order 16 bits of this SPI in order to make
+ * the corresponding SAID unique.
+ * Its low-order 16 bits hold a well-known IPCOMP CPI.
+ * Oh, and remember that SPIs are stored in network order.
+ * Kludge!!!  So I name it with the non-English word "uniquify".
+ * If we can't find one easily, return 0 (a bad SPI,
+ * no matter what order) indicating failure.
+ */
+ipsec_spi_t
+uniquify_his_cpi(ipsec_spi_t cpi, struct state *st)
+{
+    int tries = 0;
+    int i;
+
+startover:
+
+    /* network order makes first two bytes our target */
+    get_rnd_bytes((u_char *)&cpi, 2);
+
+    /* Make sure that the result is unique.
+     * Hard work.  If there is no unique value, we'll loop forever!
+     */
+    for (i = 0; i < STATE_TABLE_SIZE; i++)
+    {
+	struct state *s;
+
+	for (s = statetable[i]; s != NULL; s = s->st_hashchain_next)
+	{
+	    if (s->st_ipcomp.present
+	    && sameaddr(&s->st_connection->that.host_addr
+	      , &st->st_connection->that.host_addr)
+	    && cpi == s->st_ipcomp.attrs.spi)
+	    {
+		if (++tries == 20)
+		    return 0;	/* FAILURE */
+		goto startover;
+	    }
+	}
+    }
+    return cpi;
+}
