@@ -1,6 +1,6 @@
 /* state and event objects
  * Copyright (C) 1997 Angelos D. Keromytis.
- * Copyright (C) 1998, 1999  D. Hugh Redelmeier.
+ * Copyright (C) 1998-2001  D. Hugh Redelmeier.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -12,7 +12,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: state.h,v 1.44 2000/06/21 18:24:36 dhr Exp $
+ * RCSID $Id: state.h,v 1.54 2002/01/21 03:14:23 dhr Exp $
  */
 
 #include <sys/types.h>
@@ -66,12 +66,12 @@ struct oakley_trans_attrs {
 
 /* IPsec (Phase 2 / Quick Mode) transform and attributes
  * This is a flattened/decoded version of what is represented
- * by a Transaction Payload.  There may be one for AH and one
- * for ESP.
+ * by a Transaction Payload.  There may be one for AH, one
+ * for ESP, and a funny one for IPCOMP.
  */
 struct ipsec_trans_attrs {
     u_int8_t transid;	/* transform id */
-    ipsec_spi_t spi;
+    ipsec_spi_t spi;	/* his SPI */
     time_t life_seconds;		/* When this SA expires */
     u_int32_t life_kilobytes;	/* When this SA expires */
     u_int16_t encapsulation;
@@ -115,6 +115,7 @@ struct state
 
     struct ipsec_proto_info st_ah;
     struct ipsec_proto_info st_esp;
+    struct ipsec_proto_info st_ipcomp;
 #ifdef KLIPS
     ipsec_spi_t        st_tunnel_in_spi;          /* KLUDGE */
     ipsec_spi_t        st_tunnel_out_spi;         /* KLUDGE */
@@ -125,7 +126,6 @@ struct state
     u_int32_t          st_doi;                 /* Domain of Interpretation */
     u_int32_t          st_situation;
 
-    bool               st_pending_quick;       /* need to build IPsec SA after ISAKMP SA */
     lset_t             st_policy;              /* policy for IPsec SA */
 
     msgid_t            st_msgid;               /* MSG-ID from header.  Network Order! */
@@ -179,7 +179,10 @@ struct state
                                                 * selected.
                                                 */
 
-    u_int16_t          st_state;               /* State of exchange */
+    /* In a Phase 1 state, preserve peer's public key after authentication */
+    struct pubkeyrec  *st_peer_pubkey;
+
+    enum state_kind    st_state;               /* State of exchange */
     u_int8_t           st_retransmit;          /* Number of retransmits */
     unsigned long      st_try;                 /* number of times rekeying attempted */
                                                /* 0 means the only time */
@@ -203,10 +206,8 @@ struct state
 /* global variables */
 
 extern u_int16_t pluto_port;	/* Pluto's port */
-extern struct sockaddr_in mask32;	/* 255.255.255.255 */
 
-extern struct sockaddr_in mask0;	/* 0.0.0.0 */
-extern void rw_connection_discard(struct connection *c);
+extern bool states_use_connection(struct connection *c);
 
 /* state functions */
 
@@ -224,8 +225,13 @@ extern struct state
     *find_state(
 	const u_char *icookie,
 	const u_char *rcookie,
-	const struct in_addr peer,
+	const ip_address *peer,
 	msgid_t msgid),
-    *find_isakmp_sa(const struct connection *c);
+    *state_with_serialno(so_serial_t sn),
+    *find_phase1_state(const struct connection *c),
+    *find_sender(size_t packet_len, u_char *packet);
 
 extern void show_states_status(void);
+
+extern void find_my_cpi_gap(cpi_t *latest_cpi, cpi_t *first_busy_cpi);
+extern ipsec_spi_t uniquify_his_cpi(ipsec_spi_t cpi, struct state *st);
