@@ -1,6 +1,6 @@
 /* command interface to Pluto
  * Copyright (C) 1997 Angelos D. Keromytis.
- * Copyright (C) 1998, 1999  D. Hugh Redelmeier.
+ * Copyright (C) 1998-2001  D. Hugh Redelmeier.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -12,7 +12,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: whack.c,v 1.61 2000/06/18 21:20:35 dhr Exp $
+ * RCSID $Id: whack.c,v 1.88 2002/02/11 05:23:05 dhr Exp $
  */
 
 #include <stdio.h>
@@ -34,7 +34,6 @@
 
 #include "constants.h"
 #include "defs.h"
-#include "version.c"
 #include "whack.h"
 
 static void
@@ -54,6 +53,9 @@ help(void)
 	"connection: whack"
 	    " --name <connection_name>"
 	    " \\\n   "
+	    " [--ipv4 | --ipv6 ]"
+	    " [--tunnelipv4 | --tunnelipv6 ]"
+	    " \\\n   "
 	    " (--host <ip-address> | --id <identity>)"
 	    " [--ikeport <port-number>]"
 	    " \\\n   "
@@ -73,8 +75,10 @@ help(void)
 	    " \\\n   "
 	    " [--psk]"
 	    " [--rsasig]"
+	    " \\\n   "
 	    " [--encrypt]"
 	    " [--authenticate]"
+	    " [--compress]"
 	    " [--tunnel]"
 	    " [--pfs]"
 	    " \\\n   "
@@ -83,7 +87,9 @@ help(void)
 	    " \\\n   "
 	    " [--reykeymargin <seconds>]"
 	    " [--reykeyfuzz <percentage>]"
+	    " \\\n   "
 	    " [--keyingtries <count>]"
+	    " [--dontrekey]"
 	    "\n\n"
 	"routing: whack"
 	    " (--route | --unroute)"
@@ -94,16 +100,29 @@ help(void)
 	    " --name <connection_name>"
 	    " [--asynchronous]"
 	    "\n\n"
+	"opportunistic initiation:"
+	    "\n"
+	    " whack"
+	    " [--ipv4 | --ipv6 ]"
+	    " [--tunnelipv4 | --tunnelipv6 ]"
+	    " \\\n   "
+	    " --oppohere <ip-address>"
+	    " --oppothere <ip-address>"
+	    "\n\n"
 	"delete: whack"
 	    " --delete"
 	    " --name <connection_name>"
 	    "\n\n"
+	"deletestate: whack"
+	    " --deletestate <state_object_number>"
+	    "\n\n"
 	"pubkey: whack"
 	    " --keyid <id>"
+	    " [--addkey]"
 	    " [--pubkeyrsa <key>]"
 	    "\n\n"
 #ifdef DEBUG
-	"debug: whack"
+	"debug: whack [--name <connection_name>]"
 	    " \\\n   "
 	    " [--debug-none]"
 	    " [--debug-all]"
@@ -111,10 +130,11 @@ help(void)
 	    " [--debug-raw]"
 	    " [--debug-crypt]"
 	    " [--debug-parsing]"
-	    " \\\n   "
 	    " [--debug-emitting]"
+	    " \\\n   "
 	    " [--debug-control]"
 	    " [--debug-klips]"
+	    " [--debug-dns]"
 	    " [--debug-private]"
 	    "\n\n"
 #endif
@@ -128,7 +148,7 @@ help(void)
 	    " --shutdown"
 	    "\n\n"
 	"FreeS/WAN %s\n",
-	freeswan_version);
+	ipsec_version_code());
 }
 
 static const char *label = NULL;	/* --label operand, saved for diagnostics */
@@ -152,51 +172,41 @@ diag(const char *mess)
     exit(RC_WHACK_PROBLEM);
 }
 
-/* conditially calls diag; prints second arg as quoted string */
+/* conditially calls diag; prints second arg, if non-NULL, as quoted string */
 static void
-diagq(complaint_t ugh, const char *this)
+diagq(err_t ugh, const char *this)
 {
     if (ugh != NULL)
     {
-	char buf[120];
+	if (this == NULL)
+	{
+	    diag(ugh);
+	}
+	else
+	{
+	    char buf[120];	/* arbitrary limit */
 
-	snprintf(buf, sizeof(buf), "%s \"%s\"", ugh, this);
-	diag(buf);
+	    snprintf(buf, sizeof(buf), "%s \"%s\"", ugh, this);
+	    diag(buf);
+	}
     }
 }
 
 /* complex combined operands return one of these enumerated values
- * Note: we are close to the limit of 32 flags in an unsigned long!
+ * Note: these become flags in an lset_t.  Since there are more than
+ * 32, we partition them into:
+ * - OPT_* options (most random options)
+ * - DBGOPT_* option (DEBUG options)
+ * - CD_* options (Connection Description options)
  */
 enum {
     OPT_CTLBASE,
     OPT_NAME,
 
-#   define OPT_CD_FIRST OPT_TO	/* first connection description */
-    OPT_TO,
-    OPT_HOST,	/* first per-end */
-    OPT_ID,
-    OPT_IKEPORT,
-    OPT_NEXTHOP,
-    OPT_CLIENT,
-    OPT_UPDOWN,	/* last per-end */
-
-#   define OPT_POLICY_FIRST  OPT_PSK
-    OPT_PSK,	/* same order as POLICY_* */
-    OPT_RSASIG,	/* same order as POLICY_* */
-    OPT_ENCRYPT,	/* same order as POLICY_* */
-    OPT_AUTHENTICATE,	/* same order as POLICY_* */
-    OPT_TUNNEL,	/* same order as POLICY_* */
-    OPT_PFS,	/* same order as POLICY_* */
-
-    OPT_IKELIFETIME,
-    OPT_IPSECLIFETIME,
-    OPT_RKMARGIN,
-    OPT_RKFUZZ,
-    OPT_KTRIES,
-#   define OPT_CD_LAST OPT_KTRIES	/* last connection description */
+    OPT_CD,
 
     OPT_KEYID,
+    OPT_ADDKEY,
     OPT_PUBKEYRSA,
 
     OPT_ROUTE,
@@ -205,26 +215,72 @@ enum {
     OPT_INITIATE,
     OPT_TERMINATE,
     OPT_DELETE,
+    OPT_DELETESTATE,
     OPT_LISTEN,
     OPT_UNLISTEN,
     OPT_STATUS,
     OPT_SHUTDOWN,
 
-    OPT_ASYNC
+    OPT_OPPO_HERE,
+    OPT_OPPO_THERE,
+
+    OPT_ASYNC,
+
+#   define OPT_LAST OPT_ASYNC	/* last "normal" option */
+
+/* Connection Description options -- segregated */
+
+#   define CD_FIRST CD_TO	/* first connection description */
+    CD_TO,
+    CD_HOST,	/* first per-end */
+    CD_ID,
+    CD_IKEPORT,
+    CD_NEXTHOP,
+    CD_CLIENT,
+    CD_UPDOWN,	/* last per-end */
+
+#   define CD_POLICY_FIRST  CD_PSK
+    CD_PSK,	/* same order as POLICY_* */
+    CD_RSASIG,	/* same order as POLICY_* */
+    CD_ENCRYPT,	/* same order as POLICY_* */
+    CD_AUTHENTICATE,	/* same order as POLICY_* */
+    CD_COMPRESS,	/* same order as POLICY_* */
+    CD_TUNNEL,	/* same order as POLICY_* */
+    CD_PFS,	/* same order as POLICY_* */
+    CD_DISABLEARRIVALCHECK,	/* same order as POLICY_* */
+    CD_FAIL_PASS,	/* same order as POLICY_* */
+    CD_FAIL_DROP,	/* same order as POLICY_* */
+    CD_DONT_REKEY,	/* same order as POLICY_* */
+    CD_TUNNELIPV4,
+    CD_TUNNELIPV6,
+    CD_CONNIPV4,
+    CD_CONNIPV6,
+
+    CD_IKELIFETIME,
+    CD_IPSECLIFETIME,
+    CD_RKMARGIN,
+    CD_RKFUZZ,
+    CD_KTRIES
+#   define CD_LAST CD_KTRIES	/* last connection description */
 
 #ifdef DEBUG	/* must be last so others are less than 32 to fit in lset_t */
+#   define DBGOPT_FIRST DBGOPT_NONE
     ,
-    OPT_DEBUG_NONE,
-    OPT_DEBUG_ALL,
+    DBGOPT_NONE,
+    DBGOPT_ALL,
 
-    OPT_DEBUG_RAW,	/* same order as DBG_* */
-    OPT_DEBUG_CRYPT,	/* same order as DBG_* */
-    OPT_DEBUG_PARSING,	/* same order as DBG_* */
-    OPT_DEBUG_EMITTING,	/* same order as DBG_* */
-    OPT_DEBUG_CONTROL,	/* same order as DBG_* */
-    OPT_DEBUG_KLIPS,	/* same order as DBG_* */
-    OPT_DEBUG_PRIVATE	/* same order as DBG_* */
+    DBGOPT_RAW,	/* same order as DBG_* */
+    DBGOPT_CRYPT,	/* same order as DBG_* */
+    DBGOPT_PARSING,	/* same order as DBG_* */
+    DBGOPT_EMITTING,	/* same order as DBG_* */
+    DBGOPT_CONTROL,	/* same order as DBG_* */
+    DBGOPT_LIFECYCLE,	/* same order as DBG_* */
+    DBGOPT_KLIPS,	/* same order as DBG_* */
+    DBGOPT_DNS,		/* same order as DBG_* */
+    DBGOPT_PRIVATE	/* same order as DBG_* */
+#   define DBGOPT_LAST DBGOPT_PRIVATE
 #endif
+
 };
 
 #define OPTION_OFFSET	256	/* to get out of the way of letter options */
@@ -242,31 +298,8 @@ static const struct option long_opts[] = {
     { "ctlbase", required_argument, NULL, OPT_CTLBASE + OO },
     { "name", required_argument, NULL, OPT_NAME + OO },
 
-    { "to", no_argument, NULL, OPT_TO + OO },
-
-    { "host", required_argument, NULL, OPT_HOST + OO },
-    { "id", required_argument, NULL, OPT_ID + OO },
-    { "ikeport", required_argument, NULL, OPT_IKEPORT + OO + NUMERIC_ARG },
-    { "nexthop", required_argument, NULL, OPT_NEXTHOP + OO },
-    { "client", required_argument, NULL, OPT_CLIENT + OO },
-    { "updown", required_argument, NULL, OPT_UPDOWN + OO },
-
-    { "psk", no_argument, NULL, OPT_PSK + OO },
-    { "rsasig", no_argument, NULL, OPT_RSASIG + OO },
-
-    { "encrypt", no_argument, NULL, OPT_ENCRYPT + OO },
-    { "authenticate", no_argument, NULL, OPT_AUTHENTICATE + OO },
-    { "tunnel", no_argument, NULL, OPT_TUNNEL + OO },
-    { "pfs", no_argument, NULL, OPT_PFS + OO },
-
-    { "ikelifetime", required_argument, NULL, OPT_IKELIFETIME + OO + NUMERIC_ARG },
-    { "ipseclifetime", required_argument, NULL, OPT_IPSECLIFETIME + OO + NUMERIC_ARG },
-    { "rekeymargin", required_argument, NULL, OPT_RKMARGIN + OO + NUMERIC_ARG },
-    { "rekeywindow", required_argument, NULL, OPT_RKMARGIN + OO + NUMERIC_ARG },	/* OBSOLETE */
-    { "rekeyfuzz", required_argument, NULL, OPT_RKFUZZ + OO + NUMERIC_ARG },
-    { "keyingtries", required_argument, NULL, OPT_KTRIES + OO + NUMERIC_ARG },
-
     { "keyid", required_argument, NULL, OPT_KEYID + OO },
+    { "addkey", no_argument, NULL, OPT_ADDKEY + OO },
     { "pubkeyrsa", required_argument, NULL, OPT_PUBKEYRSA + OO },
 
     { "route", no_argument, NULL, OPT_ROUTE + OO },
@@ -275,23 +308,65 @@ static const struct option long_opts[] = {
     { "initiate", no_argument, NULL, OPT_INITIATE + OO },
     { "terminate", no_argument, NULL, OPT_TERMINATE + OO },
     { "delete", no_argument, NULL, OPT_DELETE + OO },
+    { "deletestate", required_argument, NULL, OPT_DELETESTATE + OO + NUMERIC_ARG },
     { "listen", no_argument, NULL, OPT_LISTEN + OO },
     { "unlisten", no_argument, NULL, OPT_UNLISTEN + OO },
     { "status", no_argument, NULL, OPT_STATUS + OO },
     { "shutdown", no_argument, NULL, OPT_SHUTDOWN + OO },
 
+    { "oppohere", required_argument, NULL, OPT_OPPO_HERE + OO },
+    { "oppothere", required_argument, NULL, OPT_OPPO_THERE + OO },
+
     { "asynchronous", no_argument, NULL, OPT_ASYNC + OO },
 
+
+    /* options for a connection description */
+
+    { "to", no_argument, NULL, CD_TO + OO },
+
+    { "host", required_argument, NULL, CD_HOST + OO },
+    { "id", required_argument, NULL, CD_ID + OO },
+    { "ikeport", required_argument, NULL, CD_IKEPORT + OO + NUMERIC_ARG },
+    { "nexthop", required_argument, NULL, CD_NEXTHOP + OO },
+    { "client", required_argument, NULL, CD_CLIENT + OO },
+    { "updown", required_argument, NULL, CD_UPDOWN + OO },
+
+    { "psk", no_argument, NULL, CD_PSK + OO },
+    { "rsasig", no_argument, NULL, CD_RSASIG + OO },
+
+    { "encrypt", no_argument, NULL, CD_ENCRYPT + OO },
+    { "authenticate", no_argument, NULL, CD_AUTHENTICATE + OO },
+    { "compress", no_argument, NULL, CD_COMPRESS + OO },
+    { "tunnel", no_argument, NULL, CD_TUNNEL + OO },
+    { "tunnelipv4", no_argument, NULL, CD_TUNNELIPV4 + OO },
+    { "tunnelipv6", no_argument, NULL, CD_TUNNELIPV6 + OO },
+    { "pfs", no_argument, NULL, CD_PFS + OO },
+    { "disablearrivalcheck", no_argument, NULL, CD_DISABLEARRIVALCHECK + OO },
+    { "pass", no_argument, NULL, CD_FAIL_PASS + OO },
+    { "drop", no_argument, NULL, CD_FAIL_DROP + OO },
+    { "dontrekey", no_argument, NULL, CD_DONT_REKEY + OO },
+    { "ipv4", no_argument, NULL, CD_CONNIPV4 + OO },
+    { "ipv6", no_argument, NULL, CD_CONNIPV6 + OO },
+
+    { "ikelifetime", required_argument, NULL, CD_IKELIFETIME + OO + NUMERIC_ARG },
+    { "ipseclifetime", required_argument, NULL, CD_IPSECLIFETIME + OO + NUMERIC_ARG },
+    { "rekeymargin", required_argument, NULL, CD_RKMARGIN + OO + NUMERIC_ARG },
+    { "rekeywindow", required_argument, NULL, CD_RKMARGIN + OO + NUMERIC_ARG },	/* OBSOLETE */
+    { "rekeyfuzz", required_argument, NULL, CD_RKFUZZ + OO + NUMERIC_ARG },
+    { "keyingtries", required_argument, NULL, CD_KTRIES + OO + NUMERIC_ARG },
+
 #ifdef DEBUG
-    { "debug-none", no_argument, NULL, OPT_DEBUG_NONE + OO },
-    { "debug-all]", no_argument, NULL, OPT_DEBUG_ALL + OO },
-    { "debug-raw", no_argument, NULL, OPT_DEBUG_RAW + OO },
-    { "debug-crypt", no_argument, NULL, OPT_DEBUG_CRYPT + OO },
-    { "debug-parsing", no_argument, NULL, OPT_DEBUG_PARSING + OO },
-    { "debug-emitting", no_argument, NULL, OPT_DEBUG_EMITTING + OO },
-    { "debug-control", no_argument, NULL, OPT_DEBUG_CONTROL + OO },
-    { "debug-klips", no_argument, NULL, OPT_DEBUG_KLIPS + OO },
-    { "debug-private", no_argument, NULL, OPT_DEBUG_PRIVATE + OO },
+    { "debug-none", no_argument, NULL, DBGOPT_NONE + OO },
+    { "debug-all]", no_argument, NULL, DBGOPT_ALL + OO },
+    { "debug-raw", no_argument, NULL, DBGOPT_RAW + OO },
+    { "debug-crypt", no_argument, NULL, DBGOPT_CRYPT + OO },
+    { "debug-parsing", no_argument, NULL, DBGOPT_PARSING + OO },
+    { "debug-emitting", no_argument, NULL, DBGOPT_EMITTING + OO },
+    { "debug-control", no_argument, NULL, DBGOPT_CONTROL + OO },
+    { "debug-lifecycle", no_argument, NULL, DBGOPT_LIFECYCLE + OO },
+    { "debug-klips", no_argument, NULL, DBGOPT_KLIPS + OO },
+    { "debug-dns", no_argument, NULL, DBGOPT_DNS + OO },
+    { "debug-private", no_argument, NULL, DBGOPT_PRIVATE + OO },
 #endif
 #   undef OO
     { 0,0,0,0 }
@@ -325,28 +400,28 @@ pack_str(char **p)
 }
 
 static void
-check_life_time(time_t life, time_t limit, const char *name
+check_life_time(time_t life, time_t limit, const char *which
 , const struct whack_message *msg)
 {
     time_t mint = msg->sa_rekey_margin * (100 + msg->sa_rekey_fuzz) / 100;
 
     if (life > limit)
     {
-	char buf[200];
+	char buf[200];	/* arbitrary limit */
 
 	snprintf(buf, sizeof(buf)
 	    , "%s [%lu seconds] must be less than %lu seconds"
-	    , name, (unsigned long)life, (unsigned long)limit);
+	    , which, (unsigned long)life, (unsigned long)limit);
 	diag(buf);
     }
     if (life <= mint)
     {
-	char buf[200];
+	char buf[200];	/* arbitrary limit */
 
 	snprintf(buf, sizeof(buf)
 	    , "%s [%lu] must be greater than"
 	    " rekeymargin*(100+rekeyfuzz)/100 [%lu*(100+%lu)/100]"
-	    , name
+	    , which
 	    , (unsigned long)life
 	    , (unsigned long)msg->sa_rekey_margin
 	    , (unsigned long)msg->sa_rekey_fuzz);
@@ -354,6 +429,55 @@ check_life_time(time_t life, time_t limit, const char *name
     }
 }
 
+static void
+check_end(struct whack_end *this, struct whack_end *that
+, lset_t policy, bool default_nexthop, sa_family_t caf, sa_family_t taf)
+{
+    if (caf != addrtypeof(&this->host_addr))
+	diag("address family of host inconsistent");
+
+    if (default_nexthop)
+    {
+	if (isanyaddr(&that->host_addr))
+	    diag("our nexthop must be specified when other host is a %any or %opportunistic");
+	this->host_nexthop = that->host_addr;
+    }
+
+    if (caf != addrtypeof(&this->host_nexthop))
+	diag("address family of nexthop inconsistent");
+
+    if (this->has_client)
+    {
+	ip_address cn;
+
+	networkof(&this->client, &cn);
+
+	if (taf != addrtypeof(&cn))
+	    diag("address family of client subnet inconsistent");
+
+	if (isanyaddr(&cn) && subnetishost(&this->client))
+	{
+	    /* client is 0.0.0.0/32: Opportunism connection */
+	    if (!isanyaddr(&this->host_addr))
+		diag("normal client network must not be 0.0.0.0/32 or 0::0/128");
+	    if ((policy & (POLICY_PSK | POLICY_RSASIG)) != POLICY_RSASIG)
+		diag("only RSASIG is supported for opportunism");
+	    if ((policy & POLICY_PFS) == 0)
+		diag("PFS required for opportunism");
+	    if ((policy & POLICY_ENCRYPT) == 0)
+		diag("encryption required for opportunism");
+	}
+    }
+    else
+    {
+	/* fill in anyaddr-anyaddr as (missing) client subnet */
+	ip_address cn;
+
+	diagq(anyaddr(caf, &cn), NULL);
+	diagq(rangetosubnet(&cn, &cn, &this->client), NULL);
+    }
+}
+
 /* This is a hack for initiating ISAKMP exchanges. */
 
 int
@@ -362,8 +486,13 @@ main(int argc, char **argv)
     struct whack_message msg;
     lset_t
 	opts_seen = LEMPTY,
-	opts_seen_before_to;
-    memset(&msg, '\0', sizeof(msg));
+	cd_seen = LEMPTY,
+	cd_seen_before_to;
+    const char
+	*af_used_by = NULL,
+	*tunnel_af_used_by = NULL;
+
+    zero(&msg);
     msg.magic = WHACK_MAGIC;
     msg.right.host_port = IKE_UDP_PORT;
 
@@ -376,11 +505,14 @@ main(int argc, char **argv)
     msg.keyval.ptr = NULL;
 
     msg.sa_ike_life_seconds = OAKLEY_ISAKMP_SA_LIFETIME_DEFAULT;
-    msg.sa_ipsec_life_seconds = SA_LIFE_DURATION_DEFAULT;
+    msg.sa_ipsec_life_seconds = PLUTO_SA_LIFE_DURATION_DEFAULT;
     msg.sa_rekey_margin = SA_REPLACEMENT_MARGIN_DEFAULT;
     msg.sa_rekey_fuzz = SA_REPLACEMENT_FUZZ_DEFAULT;
     msg.sa_keying_tries = SA_REPLACEMENT_RETRIES_DEFAULT;
 
+    msg.addr_family = AF_INET;
+    msg.tunnel_addr_family = AF_INET;
+
     for (;;)
     {
 	int long_index;
@@ -404,24 +536,39 @@ main(int argc, char **argv)
 		diagq("badly formed numeric argument", optarg);
 	}
 
-	/* all of our non-letter flags except OPT_DEBUG_* get
-	 * added to opts_seen.  The OP_DEBUG_* exception is because
-	 * we have too many to fit in a 32-bit string!
-	 * Unless other code intervenes, we reject repeated options.
-	 */
-
-	if (0 <= c
-#ifdef DEBUG
-	&& c < OPT_DEBUG_NONE
-#endif
-	)
+	/* per-class option processing */
+	if (0 <= c && c < OPT_LAST)
 	{
+	    /* OPT_* options get added opts_seen.
+	     * Reject repeated options (unless later code intervenes).
+	     */
 	    lset_t f = LELEM(c);
 
 	    if (opts_seen & f)
 		diagq("duplicated flag", long_opts[long_index].name);
 	    opts_seen |= f;
 	}
+#ifdef DEBUG
+	else if (DBGOPT_FIRST <= c && c <= DBGOPT_LAST)
+	{
+	    /* DBGOPT_* options are treated separately to reduce
+	     * potential members of opts_seen.
+	     */
+	    msg.whack_options = TRUE;
+	}
+#endif
+	else if (CD_FIRST <= c && c <= CD_LAST)
+	{
+	    /* CD_* options are added to cd_seen.
+	     * Reject repeated options (unless later code intervenes).
+	     */
+	    lset_t f = LELEM(c - CD_FIRST);
+
+	    if (cd_seen & f)
+		diagq("duplicated flag", long_opts[long_index].name);
+	    cd_seen |= f;
+	    opts_seen |= LELEM(OPT_CD);
+	}
 
 	/* Note: "break"ing from switch terminates loop.
 	 * most cases should end with "continue".
@@ -444,7 +591,7 @@ main(int argc, char **argv)
 	    return 0;	/* GNU coding standards say to stop here */
 
 	case 'v' - OPTION_OFFSET:	/* --version */
-	    fprintf(stderr, "FreeS/WAN %s\n", freeswan_version);
+	    fprintf(stderr, "FreeS/WAN %s\n", ipsec_version_code());
 	    return 0;	/* GNU coding standards say to stop here */
 
 	case 'l' - OPTION_OFFSET:	/* --label <string> */
@@ -469,95 +616,28 @@ main(int argc, char **argv)
 	    msg.name = optarg;
 	    continue;
 
-	case OPT_HOST:	/* --host <ip-address> */
-	    diagq(atoaddr(optarg, 0, &msg.right.host_addr), optarg);
-	    continue;
-
-	case OPT_ID:	/* --id <identity> */
-	    msg.right.id = optarg;	/* decoded by Pluto */
-	    continue;
-
-	case OPT_IKEPORT:	/* --ikeport <port-number> */
-	    if (opt_whole<=0 || opt_whole >= 0x10000)
-		diagq("<port-number> must be a number between 1 and 65535", optarg);
-	    msg.right.host_port = opt_whole;
-	    continue;
-
-	case OPT_NEXTHOP:	/* --nexthop <ip-address> */
-	    diagq(atoaddr(optarg, 0, &msg.right.host_nexthop), optarg);
-	    continue;
-
-	case OPT_CLIENT:	/* --client <subnet> */
-	    diagq(atosubnet(optarg, 0, &msg.right.client_net, &msg.right.client_mask), optarg);
-	    msg.right.has_client = TRUE;
-	    msg.policy |= POLICY_TUNNEL;	/* client => tunnel */
-	    continue;
-
-	case OPT_UPDOWN:	/* --updown <updown> */
-	    msg.right.updown = optarg;
-	    continue;
-
-
-	case OPT_TO:		/* --to */
-	    /* process right end, move it to left, reset it */
-	    if ((opts_seen & (LELEM(OPT_HOST) | LELEM(OPT_ID))) == 0)
-		diag("connection missing both --host and --id (before --to)");
-	    msg.left = msg.right;
-	    memset(&msg.right, '\0', sizeof(msg.right));
-	    msg.right.id = NULL;
-	    msg.right.updown = NULL;
-	    msg.right.host_port = IKE_UDP_PORT;
-	    opts_seen_before_to = opts_seen;
-	    opts_seen = (opts_seen & ~LRANGE(OPT_HOST,OPT_UPDOWN));
-	    continue;
-
-	case OPT_PSK:		/* --psk */
-	case OPT_RSASIG:	/* --rsasig */
-	case OPT_ENCRYPT:	/* --encrypt */
-	case OPT_AUTHENTICATE:	/* --authenticate */
-	case OPT_TUNNEL:	/* --tunnel */
-	case OPT_PFS:	/* -- pfs */
-	    msg.policy |= LELEM(c - OPT_POLICY_FIRST);
-	    continue;
-
-	case OPT_IKELIFETIME:	/* --ikelifetime <seconds> */
-	    msg.sa_ike_life_seconds = opt_whole;
-	    continue;
-
-	case OPT_IPSECLIFETIME:	/* --ipseclifetime <seconds> */
-	    msg.sa_ipsec_life_seconds = opt_whole;
-	    continue;
-
-	case OPT_RKMARGIN:	/* --rekeymargin <seconds> */
-	    msg.sa_rekey_margin = opt_whole;
-	    continue;
-
-	case OPT_RKFUZZ:	/* --rekeyfuzz <percentage> */
-	    msg.sa_rekey_fuzz = opt_whole;
-	    continue;
-
-	case OPT_KTRIES:	/* --keyingtries <count> */
-	    msg.sa_keying_tries = opt_whole;
-	    continue;
-
 	case OPT_KEYID:	/* --keyid <identity> */
 	    msg.whack_key = TRUE;
 	    msg.keyid = optarg;	/* decoded by Pluto */
 	    continue;
 
-	case OPT_PUBKEYRSA:
+	case OPT_ADDKEY:	/* --addkey */
+	    msg.whack_addkey = TRUE;
+	    continue;
+
+	case OPT_PUBKEYRSA:	/* --pubkeyrsa <key> */
 	    {
 		const char *ugh;
-		static char keyspace[1024];
+		static char keyspace[1024 + 4];	/* room for 8K bit key */
 
 		msg.pubkey_alg = PUBKEY_ALG_RSA;
 		ugh = atobytes(optarg, 0, keyspace, sizeof(keyspace), &msg.keyval.len);
 		if (ugh != NULL)
 		{
-		    char ugh_space[60];	/* perhaps enough space */
+		    char ugh_space[80];	/* perhaps enough space */
 
 		    snprintf(ugh_space, sizeof(ugh_space)
-			, "PSK data malformed (%s)", ugh);
+			, "RSA public-key data malformed (%s)", ugh);
 		    diagq(ugh_space, optarg);
 		}
 		msg.keyval.ptr = keyspace;
@@ -584,6 +664,11 @@ main(int argc, char **argv)
 	    msg.whack_delete = TRUE;
 	    continue;
 
+	case OPT_DELETESTATE:	/* --deletestate <state_object_number> */
+	    msg.whack_deletestate = TRUE;
+	    msg.whack_deletestateno = opt_whole;
+	    continue;
+
 	case OPT_LISTEN:	/* --listen */
 	    msg.whack_listen = TRUE;
 	    continue;
@@ -600,30 +685,204 @@ main(int argc, char **argv)
 	    msg.whack_shutdown = TRUE;
 	    continue;
 
+	case OPT_OPPO_HERE:	/* --oppohere <ip-address> */
+	    af_used_by = long_opts[long_index].name;
+	    diagq(ttoaddr(optarg, 0, msg.addr_family, &msg.oppo_my_client), optarg);
+	    if (isanyaddr(&msg.oppo_my_client))
+		diagq("0.0.0.0 or 0::0 isn't a valid client address", optarg);
+	    continue;
+
+	case OPT_OPPO_THERE:	/* --oppohere <ip-address> */
+	    af_used_by = long_opts[long_index].name;
+	    diagq(ttoaddr(optarg, 0, msg.addr_family, &msg.oppo_peer_client), optarg);
+	    if (isanyaddr(&msg.oppo_peer_client))
+		diagq("0.0.0.0 or 0::0 isn't a valid client address", optarg);
+	    continue;
+
 	case OPT_ASYNC:
 	    msg.whack_async = TRUE;
 	    continue;
 
+
+	/* Connection Description options */
+
+	case CD_HOST:	/* --host <ip-address> */
+	    af_used_by = long_opts[long_index].name;
+	    if (streq(optarg, "%any"))
+	    {
+		diagq(anyaddr(msg.addr_family, &msg.right.host_addr), optarg);
+	    }
+	    else if (streq(optarg, "%opportunistic"))
+	    {
+		/* we also fill in --client to 0.0.0.0/32 or IPV6 equivalent */
+		ip_address  any;
+
+		if (cd_seen & LELEM(CD_CLIENT - CD_FIRST))
+		    diag("%opportunistic clashes with --client");
+
+		cd_seen |= LELEM(CD_CLIENT - CD_FIRST);
+		tunnel_af_used_by = optarg;
+		diagq(anyaddr(msg.addr_family, &msg.right.host_addr), optarg);
+		diagq(anyaddr(msg.tunnel_addr_family, &any), optarg);
+		diagq(rangetosubnet(&any, &any, &msg.right.client), optarg);
+		msg.right.has_client = TRUE;
+		/* always use tunnel mode; mark as opportunistic */
+		msg.policy |= POLICY_TUNNEL | POLICY_OPPO;
+	    }
+	    else
+	    {
+		diagq(ttoaddr(optarg, 0, msg.addr_family
+		    , &msg.right.host_addr), optarg);
+	    }
+	    continue;
+
+	case CD_ID:	/* --id <identity> */
+	    msg.right.id = optarg;	/* decoded by Pluto */
+	    continue;
+
+	case CD_IKEPORT:	/* --ikeport <port-number> */
+	    if (opt_whole<=0 || opt_whole >= 0x10000)
+		diagq("<port-number> must be a number between 1 and 65535", optarg);
+	    msg.right.host_port = opt_whole;
+	    continue;
+
+	case CD_NEXTHOP:	/* --nexthop <ip-address> */
+	    af_used_by = long_opts[long_index].name;
+	    if (streq(optarg, "%direct"))
+		diagq(anyaddr(msg.addr_family
+		    , &msg.right.host_nexthop), optarg);
+	    else
+		diagq(ttoaddr(optarg, 0, msg.addr_family
+		    , &msg.right.host_nexthop), optarg);
+	    continue;
+
+	case CD_CLIENT:	/* --client <subnet> */
+	    tunnel_af_used_by = long_opts[long_index].name;
+	    diagq(ttosubnet(optarg, 0, msg.tunnel_addr_family, &msg.right.client), optarg);
+	    msg.right.has_client = TRUE;
+	    msg.policy |= POLICY_TUNNEL;	/* client => tunnel */
+	    continue;
+
+	case CD_UPDOWN:	/* --updown <updown> */
+	    msg.right.updown = optarg;
+	    continue;
+
+
+	case CD_TO:		/* --to */
+	    /* process right end, move it to left, reset it */
+	    if ((cd_seen & LELEM(CD_HOST-CD_FIRST)) == 0)
+		diag("connection missing --host before --to");
+	    msg.left = msg.right;
+	    zero(&msg.right);
+	    msg.right.id = NULL;
+	    msg.right.updown = NULL;
+	    msg.right.host_port = IKE_UDP_PORT;
+	    cd_seen_before_to = cd_seen;
+	    cd_seen &= ~LRANGE(CD_HOST-CD_FIRST, CD_UPDOWN-CD_FIRST);
+	    continue;
+
+	case CD_PSK:		/* --psk */
+	case CD_RSASIG:	/* --rsasig */
+	case CD_ENCRYPT:	/* --encrypt */
+	case CD_AUTHENTICATE:	/* --authenticate */
+	case CD_COMPRESS:	/* --compress */
+	case CD_TUNNEL:	/* --tunnel */
+	case CD_PFS:	/* --pfs */
+	case CD_DISABLEARRIVALCHECK:	/* --disablearrivalcheck */
+	case CD_FAIL_PASS:	/* --pass */
+	case CD_FAIL_DROP:	/* --drop */
+	case CD_DONT_REKEY:	/* --donotrekey */
+	    msg.policy |= LELEM(c - CD_POLICY_FIRST);
+	    continue;
+
+	case CD_IKELIFETIME:	/* --ikelifetime <seconds> */
+	    msg.sa_ike_life_seconds = opt_whole;
+	    continue;
+
+	case CD_IPSECLIFETIME:	/* --ipseclifetime <seconds> */
+	    msg.sa_ipsec_life_seconds = opt_whole;
+	    continue;
+
+	case CD_RKMARGIN:	/* --rekeymargin <seconds> */
+	    msg.sa_rekey_margin = opt_whole;
+	    continue;
+
+	case CD_RKFUZZ:	/* --rekeyfuzz <percentage> */
+	    msg.sa_rekey_fuzz = opt_whole;
+	    continue;
+
+	case CD_KTRIES:	/* --keyingtries <count> */
+	    msg.sa_keying_tries = opt_whole;
+	    continue;
+
+	case CD_CONNIPV4:
+	    if (cd_seen & LELEM(CD_CONNIPV6 - CD_FIRST))
+		diag("--ipv4 conflicts with --ipv6");
+
+	    /* Since this is the default, the flag is redundant.
+	     * So we don't need to set msg.addr_family
+	     * and we don't need to check af_used_by
+	     * and we don't have to consider defaulting tunnel_addr_family.
+	     */
+	    continue;
+
+	case CD_CONNIPV6:
+	    if (cd_seen & LELEM(CD_CONNIPV4 - CD_FIRST))
+		diag("--ipv6 conflicts with --ipv4");
+
+	    if (af_used_by != NULL)
+		diagq("--ipv6 must precede", af_used_by);
+
+	    af_used_by = long_opts[long_index].name;
+	    msg.addr_family = AF_INET6;
+
+	    /* Consider defaulting tunnel_addr_family to AF_INET6.
+	     * Do so only if it hasn't yet been specified or used.
+	     */
+	    if ((cd_seen & (LELEM(CD_TUNNELIPV4 - CD_FIRST) | LELEM(CD_TUNNELIPV6 - CD_FIRST))) == 0
+	    && tunnel_af_used_by == NULL)
+		msg.tunnel_addr_family = AF_INET6;
+	    continue;
+
+	case CD_TUNNELIPV4:
+	    if (cd_seen & LELEM(CD_TUNNELIPV6 - CD_FIRST))
+		diag("--tunnelipv4 conflicts with --tunnelipv6");
+
+	    if (tunnel_af_used_by != NULL)
+		diagq("--tunnelipv4 must precede", af_used_by);
+
+	    msg.tunnel_addr_family = AF_INET;
+	    continue;
+
+	case CD_TUNNELIPV6:
+	    if (cd_seen & LELEM(CD_TUNNELIPV4 - CD_FIRST))
+		diag("--tunnelipv6 conflicts with --tunnelipv4");
+
+	    if (tunnel_af_used_by != NULL)
+		diagq("--tunnelipv6 must precede", af_used_by);
+
+	    msg.tunnel_addr_family = AF_INET6;
+	    continue;
+
 #ifdef DEBUG
-	case OPT_DEBUG_NONE:	/* --debug-none */
-	    msg.whack_options = TRUE;
+	case DBGOPT_NONE:	/* --debug-none */
 	    msg.debugging = DBG_NONE;
 	    continue;
 
-	case OPT_DEBUG_ALL:	/* --debug-all */
-	    msg.whack_options = TRUE;
+	case DBGOPT_ALL:	/* --debug-all */
 	    msg.debugging |= DBG_ALL;	/* note: does not include PRIVATE */
 	    continue;
 
-	case OPT_DEBUG_RAW:	/* --debug-raw */
-	case OPT_DEBUG_CRYPT:	/* --debug-crypt */
-	case OPT_DEBUG_PARSING:	/* --debug-parsing */
-	case OPT_DEBUG_EMITTING:	/* --debug-emitting */
-	case OPT_DEBUG_CONTROL:	/* --debug-control */
-	case OPT_DEBUG_KLIPS:	/* --debug-klips */
-	case OPT_DEBUG_PRIVATE:	/* --debug-private */
-	    msg.whack_options = TRUE;
-	    msg.debugging |= LELEM(c-OPT_DEBUG_RAW);
+	case DBGOPT_RAW:	/* --debug-raw */
+	case DBGOPT_CRYPT:	/* --debug-crypt */
+	case DBGOPT_PARSING:	/* --debug-parsing */
+	case DBGOPT_EMITTING:	/* --debug-emitting */
+	case DBGOPT_CONTROL:	/* --debug-control */
+	case DBGOPT_LIFECYCLE:	/* --debug-lifecycle */
+	case DBGOPT_KLIPS:	/* --debug-klips */
+	case DBGOPT_DNS:	/* --debug-dns */
+	case DBGOPT_PRIVATE:	/* --debug-private */
+	    msg.debugging |= LELEM(c-DBGOPT_RAW);
 	    continue;
 #endif
 	default:
@@ -640,30 +899,49 @@ main(int argc, char **argv)
      * required information was supplied.
      */
 
-    if (opts_seen & LRANGE(OPT_CD_FIRST, OPT_CD_LAST))
+    /* check opportunistic initiation simulation request */
+    switch (opts_seen & (LELEM(OPT_OPPO_HERE) | LELEM(OPT_OPPO_THERE)))
     {
-	if (!LALLIN(opts_seen, LELEM(OPT_TO)))
+    case LELEM(OPT_OPPO_HERE):
+    case LELEM(OPT_OPPO_THERE):
+	diag("--oppohere and --oppothere must be used together");
+	/*NOTREACHED*/
+    case LELEM(OPT_OPPO_HERE) | LELEM(OPT_OPPO_THERE):
+	msg.whack_oppo_initiate = TRUE;
+	break;
+    }
+
+    /* check connection description */
+    if (opts_seen & LELEM(OPT_CD))
+    {
+	if (!LALLIN(cd_seen, LELEM(CD_TO-CD_FIRST)))
 	    diag("connection description option, but no --to");
 
-	if ((opts_seen & (LELEM(OPT_HOST) | LELEM(OPT_ID))) == 0)
-	    diag("connection missing both --host and --id (after --to)");
+	if ((cd_seen & LELEM(CD_HOST-CD_FIRST)) == 0)
+	    diag("connection missing --host after --to");
 
-	if (is_NO_IP(msg.left.host_addr)
-	&& is_NO_IP(msg.right.host_addr))
-	    diag("hosts cannot both be 0.0.0.0");
+	if ((cd_seen & (CD_PSK | CD_RSASIG)) == 0)
+	    diag("connection must have --psk or --rsasig or both");
 
-	if ((opts_seen_before_to & LELEM(OPT_NEXTHOP)) == 0)
-	{
-	    if (is_NO_IP(msg.right.host_addr))
-		diag("left nexthop must be specified when right is Road Warrior");
-	    msg.left.host_nexthop = msg.right.host_addr;
-	}
-	if ((opts_seen & LELEM(OPT_NEXTHOP)) == 0)
-	{
-	    if (is_NO_IP(msg.left.host_addr))
-		diag("right nexthop must be specified when left is Road Warrior");
-	    msg.right.host_nexthop = msg.left.host_addr;
-	}
+	if (isanyaddr(&msg.left.host_addr)
+	&& isanyaddr(&msg.right.host_addr))
+	    diag("hosts cannot both be 0.0.0.0 or 0::0");
+
+	check_end(&msg.left, &msg.right, msg.policy, (cd_seen_before_to & LELEM(CD_NEXTHOP-CD_FIRST)) == 0
+	    , msg.addr_family, msg.tunnel_addr_family);
+
+	check_end(&msg.right, &msg.left, msg.policy, (cd_seen & LELEM(CD_NEXTHOP-CD_FIRST)) == 0
+	    , msg.addr_family, msg.tunnel_addr_family);
+
+	if (subnettypeof(&msg.left.client) != subnettypeof(&msg.right.client))
+	    diag("endpoints clash: one is IPv4 and the other is IPv6");
+
+	if ((msg.policy & POLICY_ID_AUTH_MASK) == LEMPTY)
+	    diag("must specify --rsasig or --psk for a connection");
+
+	if (!HAS_IPSEC_POLICY(msg.policy)
+	&& (msg.left.has_client || msg.right.has_client))
+	    diag("must not specify clients for ISAKMP-only connection");
 
 	msg.whack_connection = TRUE;
     }
@@ -671,7 +949,7 @@ main(int argc, char **argv)
     /* decide whether --name is mandatory or forbidden */
     if (opts_seen & (LELEM(OPT_ROUTE) | LELEM(OPT_UNROUTE)
     | LELEM(OPT_INITIATE) | LELEM(OPT_TERMINATE)
-    | LELEM(OPT_DELETE) | LELEM(OPT_TO)))
+    | LELEM(OPT_DELETE) | LELEM(OPT_CD)))
     {
 	if ((opts_seen & LELEM(OPT_NAME)) == 0)
 	    diag("missing --name <connection_name>");
@@ -682,14 +960,15 @@ main(int argc, char **argv)
 	    diag("no reason for --name");
     }
 
-    if (opts_seen & LELEM(OPT_PUBKEYRSA))
+    if (opts_seen & (LELEM(OPT_PUBKEYRSA) | LELEM(OPT_ADDKEY)))
     {
 	if ((opts_seen & LELEM(OPT_KEYID)) == 0)
-	    diag("--pubkeyrsa requires --keyid");
+	    diag("--addkey and --pubkeyrsa require --keyid");
     }
 
-    if (!(msg.whack_connection || msg.whack_key || msg.whack_delete
-    || msg.whack_initiate || msg.whack_terminate
+    if (!(msg.whack_connection || msg.whack_key
+    || msg.whack_delete || msg.whack_deletestate
+    || msg.whack_initiate || msg.whack_oppo_initiate || msg.whack_terminate
     || msg.whack_route || msg.whack_unroute
     || msg.whack_listen || msg.whack_unlisten || msg.whack_status
     || msg.whack_options || msg.whack_shutdown))
@@ -727,6 +1006,28 @@ main(int argc, char **argv)
     next_str += msg.keyval.len;
 
     /* send message to Pluto */
+    if (access(ctl_addr.sun_path, R_OK | W_OK) < 0)
+    {
+	int e = errno;
+
+	switch(e)
+	{
+	case EACCES:
+	    fprintf(stderr, "whack: no right to communicate with pluto (access(\"%s\"))\n"
+		, ctl_addr.sun_path);
+	    break;
+	case ENOENT:
+	    fprintf(stderr, "whack: Pluto is not running (no \"%s\")\n"
+		, ctl_addr.sun_path);
+	    break;
+	default:
+	    fprintf(stderr, "whack: access(\"%s\") failed with %d %s\n"
+		, ctl_addr.sun_path, errno, strerror(e));
+	    break;
+	}
+	exit(RC_WHACK_PROBLEM);
+    }
+    else
     {
 	int sock = socket(AF_UNIX, SOCK_STREAM, 0);
 	int exit_status = 0;
@@ -745,7 +1046,8 @@ main(int argc, char **argv)
 	{
 	    int e = errno;
 
-	    fprintf(stderr, "whack: connect() for %s failed (%d %s)\n"
+	    fprintf(stderr, "whack:%s connect() for \"%s\" failed (%d %s)\n"
+		, e == ECONNREFUSED? " is Pluto running? " : ""
 		, ctl_addr.sun_path, e, strerror(e));
 	    exit(RC_WHACK_PROBLEM);
 	}
@@ -761,29 +1063,29 @@ main(int argc, char **argv)
 	/* for now, just copy reply back to stdout */
 
 	{
-	    char buf[4097];
+	    char buf[4097];	/* arbitrary limit on log line length */
 	    char *be = buf;
 
 	    for (;;)
 	    {
 		char *ls = buf;
-		ssize_t len = read(sock, be, (buf + sizeof(buf)-1) - be);
+		ssize_t rl = read(sock, be, (buf + sizeof(buf)-1) - be);
 
-		if (len < 0)
+		if (rl < 0)
 		{
 		    int e = errno;
 
 		    fprintf(stderr, "whack: read() failed (%d %s)\n", e, strerror(e));
 		    exit(RC_WHACK_PROBLEM);
 		}
-		if (len == 0)
+		if (rl == 0)
 		{
 		    if (be != buf)
 			fprintf(stderr, "whack: last line from pluto too long or unterminated\n");
 		    break;
 		}
 
-		be += len;
+		be += rl;
 		*be = '\0';
 
 		for (;;)
