@@ -1,5 +1,5 @@
 /* Structure of messages from whack to Pluto proper.
- * Copyright (C) 1998, 1999  D. Hugh Redelmeier.
+ * Copyright (C) 1998-2001  D. Hugh Redelmeier.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -11,19 +11,21 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: whack.h,v 1.31 2000/04/15 08:56:04 dhr Exp $
+ * RCSID $Id: whack.h,v 1.41 2002/01/30 09:42:36 dhr Exp $
  */
 
+#include <freeswan.h>
+
 /*
  * Since the message remains on one host, native representation is used.
  * Think of this as horizontal microcode: all selected operations are
  * to be done (in the order declared here).
  *
- * MAGIC is used to help detect verion mismatches between whack and Pluto.
+ * MAGIC is used to help detect version mismatches between whack and Pluto.
  * Whenever the interface (i.e. this struct) changes in form or
  * meaning, change this value (probably by changing the last number).
  */
-#define WHACK_MAGIC (('w' << 24) + ('h' << 16) + ('k' << 8) + 10)
+#define WHACK_MAGIC (((((('w' << 8) + 'h') << 8) + 'k') << 8) + 18)
 
 /* struct whack_end is a lot like connection.h's struct end
  * It differs because it is going to be shipped down a socket
@@ -32,10 +34,10 @@
 struct whack_end {
     char *id;	/* id string (if any) -- decoded by pluto */
 
-    struct in_addr	/* network order */
+    ip_address
 	host_addr,
-	host_nexthop,
-	client_net, client_mask;
+	host_nexthop;
+    ip_subnet client;
 
     bool has_client;
     char *updown;	/* string */
@@ -71,42 +73,47 @@ struct whack_message {
     struct whack_end left;
     struct whack_end right;
 
+    /* note: if the client is the gateway, the following must be equal */
+    sa_family_t addr_family;	/* between gateways */
+    sa_family_t tunnel_addr_family;	/* between clients */
+
     /* for WHACK_KEY: */
     bool whack_key;
+    bool whack_addkey;
     char *keyid;	/* string 6 */
     enum pubkey_alg pubkey_alg;
     chunk_t keyval;	/* chunk */
 
     /* for WHACK_ROUTE: */
-
     bool whack_route;
 
     /* for WHACK_UNROUTE: */
-
     bool whack_unroute;
 
     /* for WHACK_INITIATE: */
-
     bool whack_initiate;
 
-    /* for WHACK_TERMINATE: */
+    /* for WHACK_OPINITIATE */
+    bool whack_oppo_initiate;
+    ip_address oppo_my_client, oppo_peer_client;
 
+    /* for WHACK_TERMINATE: */
     bool whack_terminate;
 
     /* for WHACK_DELETE: */
-
     bool whack_delete;
 
-    /* for WHACK_LISTEN: */
+    /* for WHACK_DELETESTATE: */
+    bool whack_deletestate;
+    so_serial_t whack_deletestateno;
 
+    /* for WHACK_LISTEN: */
     bool whack_listen, whack_unlisten;
 
     /* for WHACK_STATUS: */
-
     bool whack_status;
 
     /* for WHACK_SHUTDOWN */
-
     bool whack_shutdown;
 
     /* space for strings (hope there is enough room):
@@ -117,7 +124,7 @@ struct whack_message {
      * 4 right's name [left.host.name.len]
      * 5 right's updown
      * 6 keyid
-     * plus keyval, a chunk.
+     * plus keyval (limit: 8K bits + overhead), a chunk.
      */
     size_t str_size;
     char string[2048];
@@ -144,9 +151,9 @@ enum rc_type {
     /* improper request */
 
     RC_DUPNAME = 20,	/* attempt to reuse a connection name */
-    RC_UNKNOWN_NAME,	/* connection name unknown */
+    RC_UNKNOWN_NAME,	/* connection name unknown or state number */
     RC_ORIENT,	/* cannot orient connection: neither end is us */
-    RC_CLASH,	/* clash between two Road Warrior connections */
+    RC_CLASH,	/* clash between two Road Warrior connections OVERLOADED */
     RC_DEAF,	/* need --listen before --initiate */
     RC_ROUTE,	/* cannot route */
     RC_RTBUSY,	/* cannot unroute: route busy */
@@ -159,6 +166,7 @@ enum rc_type {
     RC_BADWHACKMESSAGE = 30,
     RC_NORETRANSMISSION,
     RC_INTERNALERR,
+    RC_OPPOFAILURE,	/* Opportunism failed */
 
     /* progress: start of range for successful state transition.
      * Actual value is RC_NEW_STATE plus the new state code.
