@@ -1,6 +1,6 @@
 /*
  * RSA signature key generation
- * Copyright (C) 1999, 2000  Henry Spencer.
+ * Copyright (C) 1999, 2000, 2001  Henry Spencer.
  * 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -12,7 +12,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: rsasigkey.c,v 1.12 2000/06/05 15:58:08 henry Exp $
+ * RCSID $Id: rsasigkey.c,v 1.19 2002/02/22 04:08:56 henry Exp $
  */
 
 #include <stdio.h>
@@ -29,7 +29,7 @@
 #define	DEVICE	"/dev/random"
 #endif
 #ifndef MAXBITS
-#define	MAXBITS	4096
+#define	MAXBITS	20000
 #endif
 
 /* the code in getoldkey() knows about this */
@@ -42,15 +42,20 @@ struct option opts[] = {
 	"random",	1,	NULL,	'r',
 	"rounds",	1,	NULL,	'p',
 	"oldkey",	1,	NULL,	'o',
+	"hostname",	1,	NULL,	'H',
+	"noopt",	0,	NULL,	'n',
 	"help",		0,	NULL,	'h',
 	"version",	0,	NULL,	'V',
 	0,		0,	NULL,	0,
 };
 int verbose = 0;		/* narrate the action? */
 char *device = DEVICE;		/* where to get randomness */
+int devicechanged = 0;		/* did user supply his own device? */
 int nrounds = 30;		/* rounds of prime checking; 25 is good */
 mpz_t prime1;			/* old key's prime1 */
 mpz_t prime2;			/* old key's prime2 */
+char outputhostname[1024];	/* hostname for output */
+int do_lcm = 1;			/* use lcm(p-1, q-1), not (p-1)*(q-1) */
 
 char me[] = "ipsec rsasigkey";	/* for messages */
 
@@ -76,6 +81,7 @@ char *argv[];
 	extern int optind;
 	extern char *optarg;
 	int errflg = 0;
+	int i;
 	int nbits;
 	char *oldkeyfile = NULL;
 
@@ -86,6 +92,7 @@ char *argv[];
 			break;
 		case 'r':	/* nonstandard /dev/random */
 			device = optarg;
+			devicechanged = 1;
 			break;
 		case 'p':	/* number of prime-check rounds */
 			nrounds = atoi(optarg);
@@ -97,6 +104,12 @@ char *argv[];
 		case 'o':	/* reformat old key */
 			oldkeyfile = optarg;
 			break;
+		case 'H':	/* set hostname for output */
+			strcpy(outputhostname, optarg);
+			break;
+		case 'n':	/* don't optimize the private key */
+			do_lcm = 0;
+			break;
 		case 'h':	/* help */
 			printf("Usage:\t%s\n", usage);
 			printf("\tor\n");
@@ -104,7 +117,7 @@ char *argv[];
 			exit(0);
 			break;
 		case 'V':	/* version */
-			printf("2\n");
+			printf("%s %s\n", me, ipsec_version_code());
 			exit(0);
 			break;
 		case '?':
@@ -119,6 +132,15 @@ char *argv[];
 		exit(2);
 	}
 
+	if (outputhostname[0] == '\0') {
+		i = gethostname(outputhostname, sizeof(outputhostname));
+		if (i < 0) {
+			fprintf(stderr, "%s: gethostname failed (%s)\n",
+						strerror(errno));
+			exit(1);
+		}
+	}
+
 	if (oldkeyfile == NULL) {
 		assert(argv[optind] != NULL);
 		nbits = atoi(argv[optind]);
@@ -150,7 +172,7 @@ getoldkey(filename)
 char *filename;
 {
 	FILE *f;
-	char line[10000];
+	char line[MAXBITS/2];
 	char *p;
 	char *value;
 	static char pube[] = "PublicExponent:";
@@ -257,6 +279,7 @@ int useoldkey;			/* take primes from old key? */
 	mpz_t n;
 	mpz_t e;
 	mpz_t d;
+	mpz_t q1;			/* temporary */
 	mpz_t m;			/* internal modulus, (p-1)*(q-1) */
 	mpz_t t;			/* temporary */
 	mpz_t exp1;
@@ -290,12 +313,15 @@ int useoldkey;			/* take primes from old key? */
 	mpz_init_set_ui(e, E);
 
 	/* internal modulus */
-	report("computing (p-1)*(q-1)...");
+	report("computing lcm(p-1, q-1)...");
 	mpz_init_set(m, p);
 	mpz_sub_ui(m, m, 1);
-	mpz_set(t, q);
-	mpz_sub_ui(t, t, 1);
-	mpz_mul(m, m, t);		/* m = (p-1)*(q-1) */
+	mpz_init_set(q1, q);
+	mpz_sub_ui(q1, q1, 1);
+	mpz_gcd(t, m, q1);		/* t = gcd(p-1, q-1) */
+	mpz_mul(m, m, q1);		/* m = (p-1)*(q-1) */
+	if (do_lcm)
+		mpz_divexact(m, m, t);		/* m = lcm(p-1, q-1) */
 	mpz_gcd(t, m, e);
 	assert(mpz_cmp_ui(t, 1) == 0);	/* m and e relatively prime */
 
@@ -303,7 +329,14 @@ int useoldkey;			/* take primes from old key? */
 	report("computing d...");
 	mpz_init(d);
 	success = mpz_invert(d, e, m);
-	assert(success);		/* e has an inverse mod m */
+	if (!success) {			/* e has no inverse mod m (!) */
+		fprintf(stderr, "%s: fatal error: unable to invert public "
+							"exponent!\n", me);
+		if (devicechanged)
+			fprintf(stderr, "\t(perhaps random bits are not very "
+								"random?)\n");
+		exit(1);
+	}
 	if (mpz_cmp_ui(d, 0) < 0)
 		mpz_add(d, d, m);
 	assert(mpz_cmp(d, m) < 0);
@@ -317,7 +350,15 @@ int useoldkey;			/* take primes from old key? */
 	mpz_sub_ui(t, q, 1);
 	mpz_mod(exp2, d, t);		/* exp2 = d mod q-1 */
 	mpz_init(coeff);
-	mpz_invert(coeff, q, p);	/* coeff = q^-1 mod p */
+	success = mpz_invert(coeff, q, p);	/* coeff = q^-1 mod p */
+	if (!success) {			/* q has no inverse mod p (!) */
+		fprintf(stderr, "%s: fatal error: unable to invert second "
+							"prime!\n", me);
+		if (devicechanged)
+			fprintf(stderr, "\t(perhaps random bits are not very "
+								"random?)\n");
+		exit(1);
+	}
 	if (mpz_cmp_ui(coeff, 0) < 0)
 		mpz_add(coeff, coeff, p);
 	assert(mpz_cmp(coeff, p) < 0);
@@ -325,10 +366,11 @@ int useoldkey;			/* take primes from old key? */
 	/* and the output */
 	/* note, getoldkey() knows about some of this */
 	report("output...\n");		/* deliberate extra newline */
-	printf("\t# %d bits, %s", nbits, ctime(&now));	/* ctime provides \n */
+	printf("\t# RSA %d bits   %s   %s", nbits, outputhostname, ctime(&now));
+							/* ctime provides \n */
 	printf("\t# for signatures only, UNSAFE FOR ENCRYPTION\n");
 	bundp = bundle(E, n, &bs);
-	printf("\t#pubkey=%s\n", conv(bundp, bs, 'x'));	/* RFC2537ish format */
+	printf("\t#pubkey=%s\n", conv(bundp, bs, 's'));	/* RFC2537ish format */
 	printf("\t#IN KEY 0x4200 4 1 %s\n",
 			conv(bundp, bs, 's')+2);	/* RFC2537 */
 	printf("\t# (0x4200 = auth-only host-level, 4 = IPSec, 1 = RSA)\n");
@@ -400,7 +442,7 @@ int nbits;			/* known to be a multiple of CHAR_BIT */
 	getrandom(nbytes, bitbuf);
 	bitbuf[0] |= 01 << (CHAR_BIT-1);	/* force high bit on */
 	bitbuf[nbytes-1] |= 01;			/* force low bit on */
-	if (bytestoa(bitbuf, nbytes, 'x', hexbuf, hsize) > hsize) {
+	if (datatot(bitbuf, nbytes, 'x', hexbuf, hsize) > hsize) {
 		fprintf(stderr, "%s: can't-happen buffer overflow\n", me);
 		exit(1);
 	}
@@ -493,7 +535,7 @@ size_t *sizep;
 	assert(e <= 255);
 	bundbuf[0] = 1;
 	bundbuf[1] = e;
-	er = atobytes(hexp, 0, bundbuf+2, sizeof(bundbuf)-2, &size);
+	er = ttodata(hexp, 0, 0, bundbuf+2, sizeof(bundbuf)-2, &size);
 	if (er != NULL) {
 		fprintf(stderr, "%s: can't-happen bundle convert error `%s'\n",
 								me, er);
@@ -516,12 +558,12 @@ char *				/* pointer to static buffer (ick) */
 conv(bits, nbytes, format)
 char *bits;
 size_t nbytes;
-int format;			/* bytestoa() code */
+int format;			/* datatot() code */
 {
 	static char convbuf[MAXBITS/4 + 50];	/* enough for hex */
 	size_t n;
 
-	n = bytestoa(bits, nbytes, format, convbuf, sizeof(convbuf));
+	n = datatot(bits, nbytes, format, convbuf, sizeof(convbuf));
 	if (n == 0) {
 		fprintf(stderr, "%s: can't-happen convert error\n", me);
 		exit(1);
